package com.chuang.tauceti.acme.v2;

import com.chuang.tauceti.acme.v2.exception.AcmeNetworkException;
import com.chuang.tauceti.tools.basic.FutureKit;

import java.io.IOException;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;
import java.util.function.BiPredicate;

public abstract class AbstractUpdatable<T extends IUpdatable<T>> implements IUpdatable<T> {

    private CompletableFuture<T> last = CompletableFuture.completedFuture(_this());

    @Override
    public synchronized CompletableFuture<T> update(long waitTime, int retryTimes, BiPredicate<T, Throwable> retry) {
        if(last.isDone()) {
            last = FutureKit.retry(
                    this::update,
                    retryTimes,
                    (int) waitTime,
                    TimeUnit.MILLISECONDS,
                    retry
            );
        }
        return last;
    }

    @SuppressWarnings("unchecked")
    protected T _this() {
        return (T) this;
    }

    @Override
    public synchronized void stopUpdate() {
        if(!this.last.isDone()) {
            AcmeNetworkException exception = new AcmeNetworkException(new IOException(this.getClass().getSimpleName() + "更新被强制中断"));
            this.last.completeExceptionally(exception);
        }
    }


}
