package com.chuang.tauceti.acme.v2;

import com.alibaba.fastjson.JSONObject;
import com.chuang.tauceti.acme.v2.model.Identifier;
import com.chuang.tauceti.acme.v2.model.enums.ChallengeType;
import com.chuang.tauceti.acme.v2.model.enums.OrderStatus;
import com.chuang.tauceti.acme.v2.utils.AcmeUtils;
import com.chuang.tauceti.httpclient.Response;
import com.chuang.tauceti.tools.basic.collection.CollectionKit;
import lombok.EqualsAndHashCode;
import lombok.Getter;

import java.util.*;
import java.util.concurrent.CompletableFuture;
/**
 * 吊销
 * https://datatracker.ietf.org/doc/html/rfc8555#section-7.6
 * */
@Getter
@EqualsAndHashCode(callSuper = true)
public class Authorization extends OrderStatusUpdatable<Authorization> {
    /** 认证信息查询地址，以下信息都是通过这个地址查询到的 */
    private final String url;
    private final Order order;

    /** 证书申请信息 */
    private Identifier identifier;
    /** 域名的认证状态 */
    private OrderStatus status;
    /** 和 order 的失效时间原则上应保持一致 */
    private String expires;
    /** 认证方式，每个认证方式都由各自的状态，选用一个就好，推荐http */
    private List<Challenge> challenges;


    public Authorization(String url, Order order) {
        this.url = url;
        this.order = order;
        this.status = OrderStatus.pending;
    }

    public Optional<Challenge> challenge(ChallengeType type) {
        return challenges.stream().filter(challenge -> challenge.getType().equals(type.getCode())).findFirst();
    }

    public Optional<Challenge> http() {
        return challenge(ChallengeType.HTTP);
    }

    public Optional<Challenge> dns() {
        return challenge(ChallengeType.DNS);
    }


    @Override
    public CompletableFuture<Authorization> update() {
//        return AcmeUtils.acmeRequest(session, account.getKeyPair(), null, session.getNonce(), this.url, account.getLocation())
        return AcmeUtils.acmeRequest(order.getAccount(), null, this.url)
                .thenApply(this::fullValue);
    }

    private Authorization fullValue(Response response) {
        if (CollectionKit.isNotEmpty(this.challenges)) {
            this.challenges.forEach(AbstractUpdatable::stopUpdate);
        }

        JSONObject json = JSONObject.parseObject(response.asString());
        this.status = OrderStatus.valueOf(json.getString("status"));
        this.expires = json.getString("expires");
        this.identifier = json.getObject("identifier", Identifier.class);
        this.challenges = json.getJSONArray("challenges").toJavaList(Challenge.class);
        this.challenges.forEach(challenge -> challenge.setAuthorization(this));
        return this;
    }


}


