package com.chuang.tauceti.acme.v2;

import com.chuang.tauceti.acme.v2.model.enums.OrderStatus;
import com.chuang.tauceti.acme.v2.store.AcmeStore;
import com.chuang.tauceti.acme.v2.utils.AcmeResponseUtils;
import com.chuang.tauceti.acme.v2.utils.AcmeUtils;
import com.chuang.tauceti.httpclient.Response;
import lombok.Getter;
import org.apache.http.HttpHeaders;

import java.security.cert.X509Certificate;
import java.util.Collection;
import java.util.Collections;
import java.util.Optional;
import java.util.concurrent.CompletableFuture;

public class Certificate {

    private final Order order;

    @Getter
    private Collection<String> alternate;

    @Getter
    private Collection<X509Certificate> certificateChains;


    public Certificate(Order order) {
        this.order = order;
    }

    public OrderStatus getStatus() {
        return this.order.getStatus();
    }

    public Optional<String> getDownloadURL() {
        return Optional.ofNullable(order.getCertificate());
    }

    public CompletableFuture<Certificate> download() {
        return getDownloadURL().map(s -> AcmeUtils.acmeRequest(order.getAccount(), null, s).thenApply(this::fullValue))
                .orElse(CompletableFuture.completedFuture(this));
    }

    private Certificate fullValue(Response response) {
        boolean success = response.getHeader(HttpHeaders.CONTENT_TYPE)
                .map(header -> header.equalsIgnoreCase(AcmeResponseUtils.MIME_CERTIFICATE_CHAIN))
                .orElse(false);
        if (!success) {
            return this;
        }
        this.alternate = Collections.unmodifiableCollection(AcmeResponseUtils.collectLinks(response, "alternate"));

        this.certificateChains = Collections.unmodifiableCollection(AcmeResponseUtils.readCertificates(response));

        return this;
    }


    public void save() {
        AcmeStore store = order.getAccount().getSession().getStore();
        store.saveOrderCertificate(order.getOrderRef(), this.certificateChains);
    }
}
