package com.chuang.tauceti.acme.v2;

import com.alibaba.fastjson.JSONObject;
import com.chuang.tauceti.acme.v2.model.ValidationRecord;
import com.chuang.tauceti.acme.v2.model.enums.OrderStatus;
import com.chuang.tauceti.acme.v2.utils.AcmeUtils;
import com.chuang.tauceti.httpclient.Response;
import com.chuang.tauceti.tools.basic.reflect.BeanKit;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.util.List;
import java.util.concurrent.CompletableFuture;

@Data
@EqualsAndHashCode(callSuper = true)
public class Challenge extends OrderStatusUpdatable<Challenge> {
    /** type =http-01、type=dns-01、type=tls-alpn-01,（若申请的域名符号为IP，则没有dns-01）*/
    private String type;
    /** 表示此种验证方式的状态 */
    private OrderStatus status;
    /** 为这条challenge手动呼唤 CA 供应商发起域名控制器校验的请求地址 */
    private String url;
    /** 为执行此验证规则的凭据（你申请某域名，跟别人申请同一域名的证书所需要填写/上传的凭据是不同的） */
    private String token;

    private Authorization authorization;

    // ===========
    private String validated;
    private List<ValidationRecord> validationRecord;


    @Override
    public CompletableFuture<Challenge> update() {
        JSONObject json = new JSONObject();
        return AcmeUtils.acmeRequest(authorization.getOrder().getAccount(), json, url)
                .thenApply(this::fullValue);
    }

    private Challenge fullValue(Response response) {
        Challenge challenge = JSONObject.parseObject(response.asString(), Challenge.class);
        BeanKit.copyProperties(challenge, this, BeanKit.CopyOptions.create().setIgnoreNullValue(true));
        return this;
    }
}

