package com.chuang.tauceti.acme.v2.exception;

import com.chuang.tauceti.acme.v2.Problem;

import javax.annotation.Nullable;
import java.net.URL;
import java.time.Instant;
import java.util.Collection;
import java.util.Collections;

public class AcmeRateLimitedException extends AcmeServerException {
    private static final long serialVersionUID = 4150484059796413069L;

    private final @Nullable Instant retryAfter;
    private final @Nullable Collection<URL> documents;

    /**
     * Creates a new {@link AcmeRateLimitedException}.
     *
     * @param problem
     *            {@link Problem} that caused the exception
     * @param retryAfter
     *            The moment the request is expected to succeed again, may be {@code null}
     *            if not known
     * @param documents
     *            URLs pointing to documents about the rate limit that was hit
     */
    public AcmeRateLimitedException(Problem problem, @Nullable Instant retryAfter,
                                    @Nullable Collection<URL> documents) {
        super(problem);
        this.retryAfter = retryAfter;
        this.documents =
                documents != null ? Collections.unmodifiableCollection(documents) : null;
    }

    /**
     * Returns the moment the request is expected to succeed again. {@code null} if this
     * moment is not known.
     */
    @Nullable
    public Instant getRetryAfter() {
        return retryAfter;
    }

    /**
     * Collection of URLs pointing to documents about the rate limit that was hit.
     * {@code null} if the server did not provide such URLs.
     */
    @Nullable
    public Collection<URL> getDocuments() {
        return documents;
    }

}
