package com.chuang.tauceti.acme.v2.exception;

import com.chuang.tauceti.acme.v2.Problem;
import com.chuang.tauceti.support.exception.BusinessException;

import javax.annotation.Nullable;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;

public class AcmeUserActionRequiredException extends AcmeServerException {
    private static final long serialVersionUID = 7719055447283858352L;

    @Nullable private final URI tosUri;

    /**
     * Creates a new {@link AcmeUserActionRequiredException}.
     *
     * @param problem
     *            {@link Problem} that caused the exception
     * @param tosUri
     *            {@link URI} of the terms-of-service document to accept
     */
    public AcmeUserActionRequiredException(Problem problem, @Nullable URI tosUri) {
        super(problem);
        this.tosUri = tosUri;
    }

    /**
     * Returns the {@link URI} of the terms-of-service document to accept, or {@code null}
     * if the server did not provide a link to such a document.
     */
    @Nullable
    public URI getTermsOfServiceUri() {
        return tosUri;
    }

    /**
     * Returns the {@link URL} of a document that gives instructions on the actions to
     * be taken by a human.
     */
    public URL getInstance() {
        URI instance = getProblem().getInstance()
                .orElseThrow(() -> new AcmeProtocolException("Instance URL required, but missing."));
        try {
            return instance.toURL();
        } catch (MalformedURLException ex) {
            throw new AcmeProtocolException("Bad instance URL: " + instance.toString(), ex);
        }
    }

}
