package com.chuang.tauceti.acme.v2.model;

import com.chuang.tauceti.acme.v2.exception.AcmeProtocolException;
import com.chuang.tauceti.acme.v2.model.enums.IdentifierType;
import lombok.Data;

import java.net.InetAddress;
import java.net.UnknownHostException;

/**
 * 证书申请信息
 */
@Data
public class Identifier {
    /** 申请方式 */
    private IdentifierType type;
    /** 需要申请证书的资源（域名） */
    private String value;

    public static Identifier dns(String domain) {
        Identifier id = new Identifier();
        id.type = IdentifierType.dns;
        id.value = domain;
        return id;
    }

    public static Identifier ip(InetAddress address) {
        Identifier id = new Identifier();
        id.type = IdentifierType.ip;
        id.value = address.getHostAddress();
        return id;
    }

    public String domain() {
        if (IdentifierType.dns != type) {
            throw new AcmeProtocolException("expected 'dns' identifier, but found '" + type + "'");
        }
        return value;
    }

    public InetAddress ip() {
        if (IdentifierType.ip != type) {
            throw new AcmeProtocolException("expected 'ip' identifier, but found '" + type + "'");
        }
        try {
            return InetAddress.getByName(value);
        } catch (UnknownHostException ex) {
            throw new AcmeProtocolException("bad ip identifier value", ex);
        }
    }

}
