package com.chuang.tauceti.acme.v2.provider;

import com.chuang.tauceti.acme.v2.Session;
import com.chuang.tauceti.acme.v2.connection.Connection;
import com.chuang.tauceti.acme.v2.connection.DefaultConnection;
import com.chuang.tauceti.acme.v2.utils.KeyPairUtils;
import com.chuang.tauceti.httpclient.Https;
import com.chuang.tauceti.httpclient.async.AsyncBuilder;

import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.security.KeyPair;
import java.util.ServiceLoader;

/**
 * acme 服务提供者
 * Provider 实现将通过 {@link ServiceLoader} 进行注册。
 */
public interface AcmeProvider {

    int KEY_SIZE = 2048;

    /**
     * 验证服务提供者 是否支持这个URI的接口
     */
    boolean support(URI serverUri);

    String name();

    default AsyncBuilder httpBuilder() {
        return Https.asyncBuilder();
    }

    default KeyPair genKeyPair() {
        return KeyPairUtils.createKeyPair(KEY_SIZE);
    }

    /**
     * 开始通讯
     */
    default Connection connect(Session session) {
        return new DefaultConnection(session, this::resolve);
    }

    default URL resolve(URI serverUri) {
        try {
            return new URL(serverUri.getScheme(), serverUri.getHost(), serverUri.getPort(), serverUri.getPath());
        } catch (MalformedURLException ex) {
            throw new IllegalArgumentException("Bad generic server URI", ex);
        }
    }
}
