package com.chuang.tauceti.acme.v2.provider;

import com.chuang.tauceti.acme.v2.exception.AcmeProtocolException;

import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;

public class LetsEncryptAcmeProvider implements AcmeProvider {

    private static final String V02_DIRECTORY_URL = "https://acme-v02.api.letsencrypt.org/directory";
    private static final String STAGING_DIRECTORY_URL = "https://acme-staging-v02.api.letsencrypt.org/directory";

    @Override
    public boolean support(URI serverUri) {
        return "acme".equals(serverUri.getScheme())
                && "letsencrypt.org".equals(serverUri.getHost());
    }

    @Override
    public String name() {
        return "letsencrypt";
    }

    @Override
    public URL resolve(URI serverUri) {
        String path = serverUri.getPath();
        String directoryUrl;
        if (path == null || "".equals(path) || "/".equals(path) || "/v02".equals(path)) {
            directoryUrl = V02_DIRECTORY_URL;
        } else if ("/staging".equals(path)) {
            directoryUrl = STAGING_DIRECTORY_URL;
        } else {
            throw new IllegalArgumentException("Unknown URI " + serverUri);
        }

        try {
            return new URL(directoryUrl);
        } catch (MalformedURLException ex) {
            throw new AcmeProtocolException(directoryUrl, ex);
        }
    }
}
