package com.chuang.tauceti.acme.v2.provider;

import com.chuang.tauceti.acme.v2.exception.AcmeException;
import com.chuang.tauceti.httpclient.Https;
import com.chuang.tauceti.httpclient.async.AsyncBuilder;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class PebbleAcmeProvider implements AcmeProvider {

    private static final Pattern HOST_PATTERN = Pattern.compile("^/([^:/]+)(?:\\:(\\d+))?/?$");

    @Override
    public boolean support(URI serverUri) {
        return "acme".equals(serverUri.getScheme()) && "pebble".equals(serverUri.getHost());
    }

    @Override
    public String name() {
        return "pebble";
    }

    @Override
    public AsyncBuilder httpBuilder() {
        FileInputStream input;
        try {
            input = new FileInputStream("com.chuang.tauceti.acme.v2.provider.pebble.truststore");
        } catch (FileNotFoundException e) {
            throw new AcmeException("pebble acme 读取truststore 失败");
        }
        return Https.asyncBuilder().setTrustKeyStore(input, "acme4j");
    }

    @Override
    public URL resolve(URI serverUri) {
        try {
            String path = serverUri.getPath();

            URL baseUrl = new URL("https://localhost:14000/dir");

            if (path != null && !path.isEmpty() && !"/".equals(path)) {
                baseUrl = parsePath(path);
            }

            return baseUrl;
        } catch (MalformedURLException ex) {
            throw new IllegalArgumentException("Bad server URI " + serverUri, ex);
        }
    }

    /**
     * Parses the server URI path and returns the server's base URL.
     *
     * @param path
     *            server URI path
     * @return URL of the server's base
     */
    private URL parsePath(String path) throws MalformedURLException {
        Matcher m = HOST_PATTERN.matcher(path);
        if (m.matches()) {
            String host = m.group(1);
            int port = 14000;
            if (m.group(2) != null) {
                port = Integer.parseInt(m.group(2));
            }
            return new URL("https", host, port, "/dir");
        } else {
            throw new IllegalArgumentException("Invalid Pebble host/port: " + path);
        }
    }
}
