import com.chuang.tauceti.acme.v2.*;
import com.chuang.tauceti.acme.v2.connection.Connection;
import com.chuang.tauceti.acme.v2.store.AcmeStore;
import com.chuang.tauceti.acme.v2.store.FileStore;
import com.chuang.tauceti.tools.basic.FutureKit;
import org.junit.jupiter.api.Test;

import java.util.List;
import java.util.Optional;
import java.util.concurrent.CompletableFuture;
import java.util.stream.Collectors;

public class ACMEClient {
    //acme-v02.api.letsencrypt.org
    private static final String STORE_PATH = "D://ACME_TEST";
    private static final String ACME_URL = "https://acme-v02.api.letsencrypt.org";
    private static final int retryTimes = 3;
    private static final long waiting = 3000L;
    private static final String userKeyRef = "anarres";
    private static final String ORDER_REF = "aidujingsnt.com";

    public static AcmeStore store() {
        return new FileStore(STORE_PATH);
    }

    @Test
    public void session() {
        String thumbprint = "";
        try {
            // 创建一个会话
            Session session = new Session(ACME_URL, store());

            // 开启一个连接
            Connection connection = session.connect();

            // 获取ACME 接口协议连接
            connection.directory().join();


            // 获取账号
            Account account = connection.loadOrNewAccount(userKeyRef, false, "mailto: anarres.work@gmail.com").join();


            thumbprint = account.thumbprint();
            // 通过账号申请域名订单，需要注意的是多个域名必须是同一个顶级域名。
            Order order = account.newDnsOrder(ORDER_REF, ORDER_REF, null, null, "", "www").join();

            order = order.updateAuthOneByOne().join();

            List<Challenge> httpChallenges = order.getAuthorizations().stream()
                    .map(Authorization::http)
                    .filter(Optional::isPresent)
                    .map(Optional::get)
                    .filter(challenge -> !challenge.getStatus().isDone())
                    .collect(Collectors.toList());


            // 等待所有域名的 http-challenge 就绪
            CompletableFuture<List<Challenge>> futures = FutureKit.oneByOne(
                    httpChallenges,
                    challenge -> challenge.update(waiting, retryTimes),
                    throwable -> null
            );

            futures.join();//同步等待就绪


            Certificate certificate = order.doFinalize().join();

            certificate.download().join();

            certificate.save();
        } catch (Exception e) {
            System.out.println("thumbprint >>>> " + thumbprint);
            e.printStackTrace();
        }
    }

    @Test
    public void fast() {
        // 创建一个会话
        Session session = new Session(ACME_URL, store());

        // 开启一个连接
        Connection connection = session.connect();


        // 获取ACME 接口协议连接
        connection.directory().join();

//        connection.newAccount(userKeyRef, )
//        connection.newAccount(userKeyRef, true);
        Account account = connection.loadAccount(userKeyRef).get();

        account = account.update().join();

        Order order = Order.loadWithUpdate(account, ORDER_REF).join();

    }

        @Test
    public void async_session() {
        // 创建一个会话
        Session session = new Session(ACME_URL, store());
        // 开启一个连接
        Connection connection = session.connect();

        // 获取ACME 接口协议连接
        connection.directory().join();


    }

}
