## 注意事项
1. Eth Client的实现中原本是打算重写 RawTransactionManager 的 nonce 来保证每次发送交易时 nonce都是正确的，但现在思路改变了，原因如下：
   1. RawTransactionManager的 nonce 是通过本地同步控制的，锁的应该是 address。同步后会在本地维护一个nonce缓存值。这样有两个问题：
      1. 这个不适用于分布式和负载均衡场景。nonce不会多台机器同步。
      2. 每个新的address都需要换成一个nonce值，且长期驻留。
   2. 对于第 1 点的问题，原本考虑过使用分布式锁来解决，但还是会有问题。因为一个address可能在系统以外进行交易。系统外的操作不会加入分布式锁。
   3. 目前的接口中都要求传入调用者自己传入nonce, 因此目前实现中每次都会先查询nonce，然后再发交易。虽然多一次请求，但是确保了不会有一致性问题。且没有nonce驻留内存。

2. sdk 中所有需要获取配置的地方，都通过 provider 接口提供注入点。
   1. GasProvider 的实现会给EthClient 提供交易时使用gas的策略。
   2. NonceProvider 的实现会给 EthClient提供交易时 nonce的获取方式。
   3. 

3. 监听
   1. 日志监听
      1. 特点：日志由链上的 event 产生. 代币协议的event只有成功后才会产生，因此监听log意味着其中的交易都是成功的.
      2. web3j 方法：`web3j.ethLogFlowable()`
         1. 实现步骤：
            1. 构建 `EthFilter` 过滤器，填入开始区块，结束区块等信息。 
            2. 调用 `eth_getNewFilter` 将过滤器附加到服务器。得到过滤器的 `filterId`
            3. 调用 `eth_getFilterLogs(filterId)` 获取符合条件的所有 Logs
            4. 循环调用 `eth_getFilterChanges(filterId)` 查询符合条件，且变更的内容。
         2. 注意：
            1. 以上监听常常让人误以为监听过程是一步步进行的，比如区块范围是从 10 到 10000时，常见的误解是监听器会从10到10000一点点逐步进行查询。而实际上不是这样，web3j 会一次性把10到10000的Logs全部进行查询返回。然后，每隔一段时间查询10-10000的变更。
            2. 如果 toBlock 设定为 LATEST 的话，因为 LATEST 会变更，所以每次 eth_getFilterChanges(filterId) 可能都会有新的内容，因此更容易让人产生误解1.
            3. 由于监听最终是不断的检查更新，所以监听可能永远不会完成，也就不会调用 complete
            4. 也正是因为是一次性抓取的，所以 fromBlock 到 toBlock 的区间必须小于5000
      3. 改进： 采用 `web3j.ethGetLogs()` 实现n个区块不断的查询。
      
   2. 区块监听
      1. 特点：监听区块，理论上应该能获取区块内的所有信息，但实际上只返回了区块信息和区块内的`交易`。其中不包含`回执`和 `日志`
      2. web3j 方法：`web3j.blockFlowable(true|false)`  参数表示是否查询完整信息(所有交易信息)
         1. 实现步骤：
            1. 调用 `eth_newBlockFilter` 将过滤器附加到服务器，得到过滤器的 `filterId`. (无需参数)
            2. 循环调用 `eth_getFilterChanges(filterId)` 获取最新的区块，第一次调用得到的是最近的区块高度。
         2. 注意：
            1. 对于区块的监听比较符合一般人对于监听的理解，就是不断监听最新的内容，而不是监听一段区域内的变化。
            2. 由于这个监听不会获取到 `回执` 和 `日志`. 因此，无法确定交易的状态
   3. 待确认交易监听
      1. 特点：可以监听到还未确认的交易
      2. web3j 方法：`web3j.pendingTransactionFlowable()`
         1. 实现步骤：
            1. 调用 `eth_newPendingTransactionFilter` 将过滤器附加到服务器，得到过滤器的 `filterId`. (无需参数)
            2. 循环调用 `eth_getFilterChanges(filterId)` 获取最新的区块，第一次调用得到的是最近的区块的新交易的 交易哈希。
            3. 将返回的交易哈希通过 `eth_getTransactionByHash(hash)` 进行查询，获的交易信息。
         2. 注意：
            1. 这种监听总是能马上获取到最新的待确认交易哈希，但是每一笔都需要重新发起一次单笔查询，相对来说效率非常低。
            2. 因为是待确认交易，很可能还没有区块哈希。
   4. 交易监听
      1. 特点：可以监听所有确认后的交易，实际上就是 `区块监听` 参数为true，然后 flatmap 交易信息流。
      2. web3j 方法：`web3j.transactionFlowable()`
         1. 实现步骤：
            1. 调用 `eth_newBlockFilter` 将过滤器附加到服务器，得到过滤器的 `filterId`. (无需参数)
            2. 循环调用 `eth_getFilterChanges(filterId)` 获取最新的区块哈希，第一次调用得到的是最近的区块。
            3. 遍历 区块哈希 并调用 `eth_getBlockByHash(hash, true)` 查询哈希详细信息，参数true表示查询完整信息，包含所有`交易`
            4. 将交易压入数据流。
4. 重放
   1. 重放区块信息
      1. web3j方法: `replayPastBlocksFlowable(start, end, fullTransaction)`
      2. 实现步骤：
         1. 遍历 start 到 end 的每一个区块number
         2. 根据 number 调用 `eth_getBlockByNumber(number, fullTransaction)`
   2. 


5. 总结数据抓取
   1. 抓本币，只能通过查区块信息中的交易获取，因为本币没有log. 同时，本币只要有记录肯定就是成功的。
   2. 抓token时，需要注意有可能存在失败的，只能通过抓回执来确认是否成功，或者监听log

倒叙问题
多次触发问题
