package com.chuang.tauceti.blockchain.payment;

import org.web3j.protocol.core.DefaultBlockParameter;
import org.web3j.protocol.core.DefaultBlockParameterName;
import org.web3j.utils.Numeric;

import java.math.BigInteger;

import static com.chuang.tauceti.blockchain.payment.BlockChainConst.HEX_PRE;

public class BlockChainUtils {

    public static BigInteger hexToInt(String hex) {
        if(hex.toLowerCase().startsWith(HEX_PRE)) {
            hex = hex.substring(2);
        }
        return new BigInteger(hex, 16);
    }

    public static BigInteger rawToInteger(String raw) {
        try {
            return Numeric.decodeQuantity(raw);
        } catch (Exception ignore) {
            return null;
        }
    }

    public static int compareTo(DefaultBlockParameter p1, DefaultBlockParameter p2) {
        if(p1 == p2) {
            return 0;
        }

        if (p2 == DefaultBlockParameterName.LATEST) {
            return -1;
        }

        if (p1 == DefaultBlockParameterName.LATEST) {
            return 1;
        }

        return hexToInt(p1.getValue()).compareTo(hexToInt(p2.getValue()));
    }

    public static String toAddress(String inputData) {
        if(inputData.toLowerCase().startsWith(HEX_PRE)) {
            return HEX_PRE + inputData.substring(26);
        } else {
            return HEX_PRE + inputData.substring(24);
        }

    }

    public static DefaultBlockParameter add(DefaultBlockParameter block, int offset) {
        return add(block, BigInteger.valueOf(offset));
    }

    public static DefaultBlockParameter add(DefaultBlockParameter block, BigInteger offset) {
        if(block == DefaultBlockParameterName.LATEST) {
            return block;
        }

        return DefaultBlockParameter.valueOf(hexToInt(block.getValue()).add(offset));
    }
}
