package com.chuang.tauceti.blockchain.payment.model;

import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.math.RoundingMode;

/**
 * 币
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@Accessors(chain = true)
public class Coin {

    /**
     * 币的符号
     */
    private String symbol;

    /**
     * 币的名称
     */
    private String name;

    /**
     * 总发行量
     */
    private BigInteger totalSupply;

    /**
     * 小数位
     */
    private Integer decimals;


    /**
     * 是否为20 代币 协议的币种
     */
    public boolean isToken20() {
        return false;
    }

    /**
     * 获取总额度，并转换成小数形式, 保留位数取决于 decimals
     * @return 总额的小数形式
     */
    public BigDecimal getTotalSupplyDecimal() {
        return intToDecimal(totalSupply, decimals, RoundingMode.DOWN);
    }


    /**
     * 获取总额度，并转换成小数形式
     * @return 总额的小数形式
     */
    public BigDecimal getTotalSupplyDecimal(int scale, RoundingMode roundingMode) {
        return intToDecimal(totalSupply, scale, roundingMode);
    }

    /**
     * 将额度转成链上真实额度
     * @param amount 展示额度
     * @return 真实额度（去小数点，链上没有小数）
     */
    public BigInteger decimalToInt(BigDecimal amount) {
        BigDecimal multi = BigDecimal.valueOf(Math.pow(10, decimals));
        return amount.multiply(multi).toBigInteger();
    }

    /**
     * 将大整数转成小数
     * @param value 大整数
     * @param scale 保留小数位
     * @param roundingMode 取整方式
     * @return 小数
     */
    public BigDecimal intToDecimal(BigInteger value, int scale, RoundingMode roundingMode) {
        BigDecimal multi = BigDecimal.valueOf(Math.pow(10, decimals));
        return new BigDecimal(value).divide(multi, scale, roundingMode);
    }

    /**
     * 将大整数转成小数. 小数保留decimals 位
     * @param value 大整数额度
     * @return 小数额度
     */
    public BigDecimal intToDecimal(BigInteger value) {
        return intToDecimal(value, decimals, RoundingMode.DOWN);
    }

}
