package com.chuang.tauceti.blockchain.payment.watch;

import com.chuang.tauceti.blockchain.payment.client.IBlockChainClient;
import com.chuang.tauceti.blockchain.payment.model.IBlock;
import com.chuang.tauceti.blockchain.payment.model.ITransaction;
import com.chuang.tauceti.blockchain.payment.watch.strategy.StorageStrategy;
import com.chuang.tauceti.blockchain.payment.watch.strategy.WatchRetryStrategy;
import com.chuang.tauceti.tools.basic.FutureKit;
import lombok.extern.slf4j.Slf4j;
import reactor.core.publisher.Flux;

import javax.annotation.Nullable;
import java.math.BigInteger;
import java.util.concurrent.CompletableFuture;

@Slf4j
public class BlockWatcher<B extends IBlock<T>, T extends ITransaction> extends BlockchainWatcher<B, B> implements IStorageWatcher<B> {

    private final IBlockChainClient<B, T> client;
    private StorageStrategy<B> strategy = StorageStrategy.empty();

    public BlockWatcher(IBlockChainClient<B, T> client, BlockRange range, @Nullable WatchRetryStrategy<B> retryStrategy, @Nullable Integer offset) {
        super(range, retryStrategy, offset);
        this.client = client;
    }

    @Override
    protected Flux<B> asFlux(CompletableFuture<B> future) {
        return FutureKit.toMono(future).flux();
    }

    @Override
    protected CompletableFuture<B> fetch(BigInteger number) {
        CompletableFuture<B> b =  strategy.load(client.getChainName(), number.toString(16))
                .map(CompletableFuture::completedFuture)
                .orElseGet(() -> queryBlock(number));

        // 保存一下
        b = b.thenApply(b1 -> {
            strategy.storage(client.getChainName(), b1);
            return b1;
        });

        return b;
    }

    private CompletableFuture<B> queryBlock(BigInteger number) {
        return client.getBlockInfoByNum(number);
    }

    @Override
    public String watchName() {
        return client.getChainName() + "-Transaction";
    }

    @Override
    public void setStorageStrategy(StorageStrategy<B> strategy) {
        this.strategy = strategy;
    }

    @Override
    public StorageStrategy<B> getStorageStrategy() {
        return strategy;
    }
}
