package com.chuang.tauceti.blockchain.payment.watch;

import com.chuang.tauceti.blockchain.payment.model.IStorage;
import com.chuang.tauceti.blockchain.payment.watch.strategy.StorageStrategy;

/**
 * 带存储器的监听器。
 * 主要作用是提供一个存储策略。在获取数据前会先加载存储器中的内容，如果存储器中已经存在，则直接使用存储器中的内容。
 * 当获取到新的数据时，也会将数据保存到存储器中，方便下次使用。
 * 其目的是为了减少获取数据的次数，提升性能。如果获取数据的开销很小，则没有必要使用这个接口。
 * 如果某个监听器获取数据的开销很小，但接口实现了的是 IStorageWatcher。可以选择使用 StorageStrategy.empty() 来忽略。
 * @param <T>
 */
public interface IStorageWatcher<T extends IStorage> extends IWatcher<T> {

    void setStorageStrategy(StorageStrategy<T> strategy);

    StorageStrategy<T> getStorageStrategy();
}
