package com.chuang.tauceti.blockchain.payment.watch.strategy;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.chuang.tauceti.blockchain.payment.model.IStorage;
import com.chuang.tauceti.tools.basic.FileKit;
import lombok.extern.slf4j.Slf4j;

import java.io.File;
import java.io.IOException;
import java.util.Optional;

/**
 * 文件存储器，
 * 文件存储器要求存储对象必须实现 IStorage 接口，用于获取存储对象的唯一key
 * @param <T>
 */
@Slf4j
public class FileStorageStrategy<T extends IStorage> implements StorageStrategy<T> {

    private final String dir;

    public FileStorageStrategy(String dir) {
        this.dir = dir;
    }

    @Override
    public void storage(String chainName, T t) {
        String blockJson = JSONObject.toJSONString(t);
        try {
            FileKit.writeUtf8String(blockJson, filePath(chainName, t.getStorageKey()));
        } catch (IOException e) {
            log.error("storage the {}.{}.json fail", chainName, t.getStorageKey(), e);
        }
    }

    @Override
    public Optional<T> load(String chainName, String key) {
        if(!FileKit.isEmpty(new File(filePath(chainName, key)))) {
            try {
                String content = FileKit.readUtf8String(filePath(chainName, key));
                T b = JSONObject.parseObject(content, new TypeReference<T>() {});
                return Optional.ofNullable(b);
            } catch (IOException e) {
                log.error("load the {}.{}.json fail", chainName, key, e);
            }
        }
        return Optional.empty();
    }

    private String filePath(String chainName, String hash) {
        return getDir() + "/" + chainName + "/" + hash + ".json";
    }

    public String getDir() {
        return dir;
    }
}
