package com.chuang.tauceti.blockchain;

import com.chuang.tauceti.support.exception.SystemException;
import com.chuang.tauceti.blockchain.watch.BlockRange;
import reactor.core.publisher.Flux;
import reactor.core.publisher.FluxSink;

import java.math.BigInteger;
import java.util.Iterator;
import java.util.concurrent.atomic.AtomicBoolean;

public class Fluxes {

    public static Flux<BigInteger> range(BlockRange range) {
        if(range.isAsc()) {
            return ((BlockRange.AscIteratorImpl)range.iterator()).nextFlux();
        } else {
            return Flux.fromIterable(range);
        }
    }

    public static Flux<BigInteger> iterator(Iterator<BigInteger> iterator) {
        if(iterator instanceof BlockRange.AscIteratorImpl) {
            return ((BlockRange.AscIteratorImpl) iterator).nextFlux();
        } else {
            return Flux.fromIterable(() -> iterator);
        }
    }

    public static Flux<BigInteger> backPressureIterator(Iterator<BigInteger> iterator) {
        if(iterator instanceof BlockRange.AscIteratorImpl) {
            return ((BlockRange.AscIteratorImpl) iterator).oneByOneBackPressureFlux();
        } else {
            throw new SystemException("not support");
        }
    }



    private static void rangeSinkLimit(FluxSink<BigInteger> emitter, BlockRange range) {
        AtomicBoolean cancel = new AtomicBoolean(false);

        emitter.onCancel(() -> cancel.set(true));

        for (BigInteger num: range) {
            if(!cancel.get()) {
                emitter.next(num);
            }
        }
        emitter.complete();
    }

}
