package com.chuang.tauceti.blockchain.client.contract;

import lombok.Data;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.Accessors;
import org.web3j.abi.TypeReference;
import org.web3j.abi.datatypes.Type;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

@Getter
public class Function<T, R extends Type<T>> {

    private List<Type> params = Collections.emptyList();
    private String function;
    private String caller;
    private String contract;

    /** writer 方法调用需要传入 privateKey 用于做签名，reader方法不需要，因为reader方法不需要签名 */
    private String privateKey;

    /** reader 方法调用需要传入 result 确定返回值， writer方法不需要，因为writer需要返回交易哈希，总是字符串 */
    private TypeReference<R> result;
    private java.util.function.Function<String, T> resultMap;


    public Function<T, R> params(Type<?>... params) {
        this.params = Arrays.asList(params);
        return this;
    }
    public Function<T, R> params(List<Type> params) {
        this.params = params;
        return this;
    }

    public Function<T, R> function(String function) {
        this.function = function;
        return this;
    }

    public Function<T, R> caller(String caller) {
        this.caller = caller;
        return this;
    }

    public Function<T, R> contract(String contract) {
        this.contract = contract;
        return this;
    }

    public Function<T, R> reader(TypeReference<R> result, java.util.function.Function<String, T> resultMap) {
        this.result = result;
        this.resultMap = resultMap;
        return this;
    }

    public Function<T, R> writer(String privateKey) {
        this.privateKey = privateKey;
        return this;
    }

}
