package com.chuang.tauceti.blockchain.model.eth;

import com.chuang.tauceti.blockchain.BlockChainUtils;
import com.chuang.tauceti.blockchain.model.ITransaction;
import com.chuang.tauceti.tools.basic.reflect.ConvertKit;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.web3j.protocol.core.methods.response.AccessListObject;
import org.web3j.protocol.core.methods.response.EthBlock;
import org.web3j.protocol.core.methods.response.Transaction;

import java.math.BigInteger;
import java.util.List;

@Data()
@EqualsAndHashCode(callSuper = true)
public class EthTransaction extends SimpleEthTransaction implements ITransaction {

    private Long chainId;
    private BigInteger gas;
    private BigInteger gasPrice;
    private BigInteger nonce;
    private String input;
    private String creates;
    private String publicKey;
    private String raw;
    private String r;
    private String s;
    private Long v;
    private String type;
    private BigInteger maxFeePerGas;
    private BigInteger maxPriorityFeePerGas;
    private List<AccessListObject> accessList;


    public static EthTransaction parseOf(EthBlock.TransactionResult<?> transactionResult) {
        if(transactionResult instanceof EthBlock.TransactionHash) {
            EthTransaction t = new EthTransaction();
            t.setTransactionHash(((EthBlock.TransactionHash) transactionResult).get());
            return t;
        } else {
            Transaction transaction = ((EthBlock.TransactionObject) transactionResult).get();
            return parseOf(transaction);
        }
    }

    public static EthTransaction parseOf(Transaction transaction) {

        if(null ==  transaction) {
            return null;
        }

        EthTransaction t = ConvertKit.toBean(transaction, EthTransaction::new,
                "value",
                "transactionIndex",
                "gas",
                "gasPrice",
                "nonce",
                "blockNumber",
                "maxFeePerGas",
                "maxPriorityFeePerGas");

        t.setValue(BlockChainUtils.rawToInteger(transaction.getValueRaw()));
        t.setTransactionIndex(BlockChainUtils.rawToInteger(transaction.getTransactionIndexRaw()));
        t.setGas(BlockChainUtils.rawToInteger(transaction.getGasRaw()));
        t.setGasPrice(BlockChainUtils.rawToInteger(transaction.getGasPriceRaw()));
        t.setNonce(BlockChainUtils.rawToInteger(transaction.getNonceRaw()));
        t.setBlockNumber(BlockChainUtils.rawToInteger(transaction.getBlockNumberRaw()));
        t.setMaxFeePerGas(BlockChainUtils.rawToInteger(transaction.getMaxFeePerGasRaw()));
        t.setMaxPriorityFeePerGas(BlockChainUtils.rawToInteger(transaction.getMaxPriorityFeePerGasRaw()));

        t.setTransactionHash(transaction.getHash());
        return t;
    }

}
