package com.chuang.tauceti.blockchain.model.eth;

import com.chuang.tauceti.blockchain.BlockChainConst;
import com.chuang.tauceti.blockchain.model.IStorage;
import com.chuang.tauceti.support.exception.SystemException;
import com.chuang.tauceti.tools.basic.collection.CollectionKit;
import com.chuang.tauceti.tools.basic.reflect.ConvertKit;
import lombok.Data;
import org.web3j.protocol.core.methods.response.EthLog;

import java.math.BigInteger;
import java.util.List;

@Data
public class Log implements IStorage {
    private Boolean removed;
    private BigInteger logIndex;
    private BigInteger transactionIndex;
    private String transactionHash;
    private String blockHash;
    private BigInteger blockNumber;
    private String address;
    private String data;
    private String type;
    private List<String> topics;

    public static Log parseOf(org.web3j.protocol.core.methods.response.Log log) {
        return ConvertKit.toBean(log, Log::new);
    }

    public static Log parseOf(org.web3j.protocol.websocket.events.Log log) {
        return ConvertKit.toBean(log, Log::new);
    }

    public static Log parseOf(EthLog.LogResult<?> logResult) {

        Object obj = logResult.get();
        if(obj instanceof org.web3j.protocol.websocket.events.Log) {
            return parseOf((org.web3j.protocol.websocket.events.Log) obj);
        } else if(obj instanceof EthLog.LogObject) {
            return parseOf(((EthLog.LogObject) obj).get());
        } else {
            throw new SystemException("", new ClassCastException("can not case " + obj.getClass() + " to " + Log.class.getName()));
        }
    }

    @Override
    public String getStorageKey() {
        return getTransactionHash();
    }

    public BigInteger getDataValue() {
        if(null == data) {
            return null;
        }
        if (data.toLowerCase().startsWith(BlockChainConst.HEX_PRE)) {
            data = data.substring(2);
        }

        return new BigInteger(data, 16);
    }

    public String getTopicHash() {
        if(CollectionKit.isNotEmpty(topics)) {
            return topics.get(0);
        } else {
            return "";
        }
    }

    public Token20Log asToken20() {
        return ConvertKit.toBean(this, Token20Log::new);
    }


}
