package com.chuang.tauceti.blockchain.model.eth;

import com.chuang.tauceti.blockchain.model.Coin;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.math.RoundingMode;

/**
 * RC20 代币
 */
@Data
@Accessors(chain = true)
@EqualsAndHashCode(callSuper = true)
public class Token extends Coin {

    /**
     * 创建地址
     */
    private String ownerAddress;

    /**
     * 合约地址
     */
    private String contractAddress;

    /**
     * 总发行量
     */
    private BigInteger totalSupply;

    /**
     * 获取总额度，并转换成小数形式, 保留位数取决于 decimals
     * @return 总额的小数形式
     */
    public BigDecimal getTotalSupplyDecimal() {
        return intToDecimal(totalSupply, getDecimals(), RoundingMode.DOWN);
    }


    /**
     * 获取总额度，并转换成小数形式
     * @return 总额的小数形式
     */
    public BigDecimal getTotalSupplyDecimal(int scale, RoundingMode roundingMode) {
        return intToDecimal(totalSupply, scale, roundingMode);
    }
}
