package com.chuang.tauceti.blockchain.watch.eth;

import com.chuang.tauceti.blockchain.client.contract.ether.EthClient;
import com.chuang.tauceti.blockchain.model.eth.Log;
import com.chuang.tauceti.blockchain.watch.BlockRange;
import com.chuang.tauceti.blockchain.watch.strategy.WatchRetryStrategy;
import com.chuang.tauceti.support.exception.SystemException;
import com.chuang.tauceti.tools.basic.collection.CollectionKit;
import org.jetbrains.annotations.Nullable;
import org.web3j.protocol.core.DefaultBlockParameter;
import org.web3j.protocol.core.methods.request.EthFilter;

import java.math.BigInteger;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.stream.Collectors;

public class BufferedLogWatcher extends LogWatcher {

    private final Integer buffer;
    private final Map<BigInteger, List<Log>> bufferCache = new HashMap<>();

    private BigInteger bufferFrom = BigInteger.valueOf(-1);
    private BigInteger bufferTo = BigInteger.valueOf(-1);

    public BufferedLogWatcher(EthClient client, BlockRange range, List<ITopic> topics, List<String> contracts, @Nullable WatchRetryStrategy<List<Log>> retryStrategy, @Nullable Integer offset, @Nullable Integer buffer) {
        super(client, range, topics, contracts, retryStrategy, offset);
        if(!range.isAsc()) {
            throw new SystemException("buffered watch can not support desc range");
        }
        this.buffer = buffer;

    }

    @Override
    protected CompletableFuture<List<Log>> fetch(BigInteger number) {
        try {
            if (bufferBetween(number)) {
                List<Log> list = bufferCache.remove(number);
                System.out.println("4 " + bufferFrom + " -> " + bufferTo + "=============================> " + bufferCache.size());
                return CompletableFuture.completedFuture(list);
            }

            BigInteger last = iteratorLast();

            BigInteger min = min(last, number.add(BigInteger.valueOf(buffer)));
            min = min(min, number);

            System.out.println("3.5 =============================> min -> " + min + " -> last -> " + last + " number -> " + number);

            return fetchCache(number, min);
        } catch ( Exception e) {
            e.printStackTrace();
            return CompletableFuture.completedFuture(Collections.emptyList());
        }
    }

    private boolean bufferBetween(BigInteger number) {
        return BlockRange.between(bufferFrom, bufferTo, number);
    }

    private CompletableFuture<List<Log>> fetchCache(BigInteger from, BigInteger to) {
        EthFilter filter = new EthFilter(DefaultBlockParameter.valueOf(from), DefaultBlockParameter.valueOf(to), contracts);

        if(topics.size() == 1) {
            filter.addSingleTopic(topics.get(0).topic());
        } else {
            String[] topics = CollectionKit.toArray(this.topics, ITopic::topic);
            filter.addOptionalTopics(topics);
        }

        this.bufferFrom = from;
        this.bufferTo = to;
        return client.queryLogs(filter).thenApply(logs -> {
           Map<BigInteger, List<Log>> map = logs.stream().collect(Collectors.groupingBy(Log::getBlockNumber));
           BlockRange.each(from, to, number -> {
                bufferCache.put(number, map.getOrDefault(number, Collections.emptyList()));
           });
           return bufferCache.remove(from);
        });
    }

    private BigInteger min(BigInteger a, BigInteger b) {
        return a.compareTo(b) <= 0 ? a : b;
    }



}
