package com.chuang.tauceti.blockchain.watch.eth;

import com.chuang.tauceti.blockchain.client.contract.ether.EthClient;
import com.chuang.tauceti.blockchain.model.eth.Log;
import com.chuang.tauceti.blockchain.watch.BlockRange;
import com.chuang.tauceti.blockchain.watch.BlockchainWatcher;
import com.chuang.tauceti.blockchain.watch.IWatcher;
import com.chuang.tauceti.blockchain.watch.strategy.WatchRetryStrategy;
import com.chuang.tauceti.tools.basic.FutureKit;
import com.chuang.tauceti.tools.basic.collection.CollectionKit;
import lombok.extern.slf4j.Slf4j;
import org.web3j.protocol.core.DefaultBlockParameter;
import org.web3j.protocol.core.methods.request.EthFilter;
import reactor.core.publisher.Flux;

import javax.annotation.Nullable;
import java.math.BigInteger;
import java.util.List;
import java.util.concurrent.CompletableFuture;

@Slf4j
public class LogWatcher extends BlockchainWatcher<Log, List<Log>> implements IWatcher<Log> {

    protected final List<ITopic> topics;
    protected final List<String> contracts;
    protected final EthClient client;

    public LogWatcher(EthClient client, BlockRange range, List<ITopic> topics, List<String> contracts,
                      @Nullable WatchRetryStrategy<List<Log>> retryStrategy, @Nullable Integer offset) {
        super(range, retryStrategy, offset);
        this.topics = topics;
        this.contracts = contracts;
        this.client = client;
    }

    @Override
    protected Flux<Log> asFlux(CompletableFuture<List<Log>> future) {
        return FutureKit.toFlux(future);
    }

    @Override
    protected CompletableFuture<List<Log>> fetch(BigInteger number) {
        DefaultBlockParameter blockNumber = DefaultBlockParameter.valueOf(number);

        EthFilter filter = new EthFilter(blockNumber, blockNumber, contracts);
        if(topics.size() == 1) {
            filter.addSingleTopic(topics.get(0).topic());
        } else {
            String[] topics = CollectionKit.toArray(this.topics, ITopic::topic);
            filter.addOptionalTopics(topics);
        }

        return client.queryLogs(filter);
    }



    @Override
    public String watchName() {
        return client.getChainName() + "-Log";
    }

}
