package com.chuang.tauceti.blockchain.watch.strategy;

import com.chuang.tauceti.blockchain.watch.IWatcher;
import com.chuang.tauceti.tools.basic.RetryStrategy;
import com.chuang.tauceti.tools.basic.ScheduleKit;
import lombok.EqualsAndHashCode;
import lombok.Getter;

import java.util.concurrent.CompletionException;
import java.util.concurrent.TimeUnit;
import java.util.function.BiFunction;
import java.util.function.BiPredicate;

@Getter
@EqualsAndHashCode(callSuper = true)
public class WatchRetryStrategy<T> extends RetryStrategy<T> {

    private final BiFunction<IWatcher<?>, Throwable, T> retryFail;

    public WatchRetryStrategy(int times, int delay, TimeUnit timeUnit, BiPredicate<T, Throwable> test, BiFunction<IWatcher<?>, Throwable, T> retryFail) {
        super(times, delay, timeUnit, test);
        this.retryFail = retryFail;
    }

    public static <T> WatchRetryStrategy<T> errRetryFailRestart(int times, int delay, TimeUnit timeUnit) {
        return new WatchRetryStrategy<>(times, delay, timeUnit, (t, throwable) -> null != throwable, (watcher, throwable) -> {
            // 延迟一分钟后重启，避免出现问题后疯狂重启
            ScheduleKit.schedule(watcher::restart, 1, TimeUnit.MINUTES);

            if(throwable instanceof CompletionException) {
                throw new CompletionException(throwable.getCause());
            } else {
                throw new CompletionException(throwable);
            }
        });
    }

    public static <T> WatchRetryStrategy<T> errRetryFailShutdown(int times, int delay, TimeUnit timeUnit) {
        return new WatchRetryStrategy<>(times, delay, timeUnit, (t, throwable) -> null != throwable, (watcher, throwable) -> {
            watcher.shutdown();
            return null;
        });
    }

    public static <T> WatchRetryStrategy<T> errRetry(int times, int delay, TimeUnit timeUnit, BiFunction<IWatcher<?>, Throwable, T> retryFail) {
        return new WatchRetryStrategy<>(times, delay, timeUnit, (t, throwable) -> null != throwable, retryFail);
    }
}
