package com.chuang.tauceti.generator.convert;

import com.baomidou.mybatisplus.generator.config.GlobalConfig;
import com.baomidou.mybatisplus.generator.config.ITypeConvert;
import com.baomidou.mybatisplus.generator.config.rules.DbColumnType;
import com.baomidou.mybatisplus.generator.config.rules.IColumnType;

import java.math.BigDecimal;

import static com.baomidou.mybatisplus.generator.config.rules.DbColumnType.*;

/**
 * Oracle 的类型相比于mysql来说比较少。因此代码生成时 boolean，byte这样的字段很难生成出来。
 * 默认的Oracle类型转换器过于简单
 * @author ATH
 */
public class MyOracleTypeConvert extends CustomTypeConvert implements ITypeConvert {

    private static final String BOOL = "number\\(1\\)";
    private static final String BYTE = "number\\([2-3]\\)";
    private static final String INTEGER = "number\\([4-8]\\)";
    private static final String DECIMAL = "number\\(.*,.*\\)";
    public static final MyOracleTypeConvert INSTANCE = new MyOracleTypeConvert();

    /**
     * 处理类型转换
     *
     * @param config    全局配置
     * @param fieldType 字段类型
     * @return 返回的对应的列类型
     */
    @Override
    public IColumnType processTypeConvert(GlobalConfig config, String fieldType) {
        return use(fieldType)
                .test(containsAny("char", "clob").then(STRING))
                .test(containsAny("date", "timestamp").then(p -> toDateType(config)))
                .test(contains("number").then(MyOracleTypeConvert::toNumberType))
                .test(contains("float").then(FLOAT))
                .test(contains("blob").then(BLOB))
                .test(containsAny("binary", "raw").then(BYTE_ARRAY))
                .or(STRING);
    }

    /**
     * 将对应的类型名称转换为对应的 java 类类型
     * <p>
     * String.valueOf(Integer.MAX_VALUE).length() == 10
     * Integer 不一定能装下 10 位的数字
     * <p>
     * String.valueOf(Long.MAX_VALUE).length() == 19
     * Long 不一定能装下 19 位的数字
     *
     * @param typeName 类型名称
     * @return 返回列类型
     */
    private static IColumnType toNumberType(String typeName) {

        if (typeName.matches(BOOL)) {
            return BOOLEAN;
        } else if (typeName.matches(BYTE)) {
            return DbColumnType.BYTE;
        } else if (typeName.matches(INTEGER)) {
            return DbColumnType.INTEGER;
        } else if (typeName.matches(DECIMAL)) {
            return DbColumnType.BIG_DECIMAL;
        }
        return LONG;
    }

    /**
     * 当前时间为字段类型，根据全局配置返回对应的时间类型
     *
     * @param config 全局配置
     * @return 时间类型
     * @see GlobalConfig#getDateType()
     */
    protected static IColumnType toDateType(GlobalConfig config) {
        switch (config.getDateType()) {
            case ONLY_DATE:
                return DbColumnType.DATE;
            case SQL_PACK:
                return DbColumnType.TIMESTAMP;
            case TIME_PACK:
                return DbColumnType.LOCAL_DATE_TIME;
            default:
                return STRING;
        }
    }
}
