package com.chuang.tauceti.httpclient;

import com.chuang.tauceti.httpclient.annotations.AsyncHttpClient;
import com.chuang.tauceti.httpclient.annotations.HttpClient;
import com.chuang.tauceti.httpclient.async.AsyncBuilder;
import com.chuang.tauceti.httpclient.async.IAsyncHttpClient;
import com.chuang.tauceti.httpclient.sync.IHttpClient;
import com.chuang.tauceti.httpclient.sync.SyncBuilder;
import com.google.inject.Module;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.nio.client.HttpAsyncClientBuilder;

/**
 * 目前httpclient只支持 http代理，socks由于是 TCP/IP 层的，不太好抽象。
 * apache httpclient的抽象中 request 是不能设置连接的。只能通过client设置。
 * 后续会尝试修改client的实现，让socks代理至少在client配置中可用。
 */
public class Https {

    private static IHttpClient defaultSyncClient = syncBuilder().setDefaultCharset("UTF-8")
            .setConnectTimeout(10000)
            .setSocketTimeout(10000)
            .setConnectionRequestTimeout(10000)
            .trustAll()
            .build();
    private static IAsyncHttpClient defaultAsyncClient;

    public static synchronized IAsyncHttpClient async() {
        if (null == defaultAsyncClient) {
            defaultAsyncClient = asyncBuilder().setDefaultCharset("UTF-8")
                    .workThreadFull()
                    .setConnectTimeout(10000)
                    .setSocketTimeout(10000)
                    .setConnectionRequestTimeout(10000)
                    .trustAll()
                    .build()
                    .init();
        }
        return defaultAsyncClient;
    }

    public static synchronized void setDefaultAsyncClient(IAsyncHttpClient client) {
        if (null != defaultAsyncClient) {
            defaultAsyncClient.shutdown();
        }
        defaultAsyncClient = client;
    }

    public static synchronized void setDefaultSyncClient(IHttpClient client) {
        defaultSyncClient = client;
    }

    public static IHttpClient sync() {
        return defaultSyncClient;
    }

    public static AsyncBuilder asyncBuilder() {
        return new AsyncBuilder();
    }

    public static SyncBuilder syncBuilder() {
        return new SyncBuilder();
    }

    public static AsyncBuilder asyncBuilder(HttpAsyncClientBuilder builder) {
        return new AsyncBuilder(builder);
    }

    public static SyncBuilder asyncBuilder(HttpClientBuilder builder) {
        return new SyncBuilder(builder);
    }


    public static Module asModule() {
        return binder -> {
            binder.bind(IHttpClient.class).annotatedWith(HttpClient.class).toInstance(sync());
            binder.bind(IAsyncHttpClient.class).annotatedWith(AsyncHttpClient.class).toInstance(async());
        };
    }
}
