package com.chuang.tauceti.httpclient;

import org.apache.http.ContentTooLongException;
import org.apache.http.Header;
import org.apache.http.HttpEntity;

import java.io.*;

public class NoLimitMultipartFormEntity implements HttpEntity {

    private final HttpEntity multipartFormEntity;

    public NoLimitMultipartFormEntity(HttpEntity multipartFormEntity) {
        this.multipartFormEntity = multipartFormEntity;
    }

    @Override
    public boolean isRepeatable() {
        return multipartFormEntity.isRepeatable();
    }

    @Override
    public boolean isChunked() {
        return multipartFormEntity.isChunked();
    }

    @Override
    public long getContentLength() {
        return multipartFormEntity.getContentLength();
    }

    @Override
    public Header getContentType() {
        return multipartFormEntity.getContentType();
    }

    @Override
    public Header getContentEncoding() {
        return multipartFormEntity.getContentEncoding();
    }

    @Override
    public InputStream getContent() throws IOException, UnsupportedOperationException {
        if (this.getContentLength() < 0) {
            throw new ContentTooLongException("Content length is unknown");
        }
        final ByteArrayOutputStream outStream = new ByteArrayOutputStream();
        writeTo(outStream);
        outStream.flush();
        return new ByteArrayInputStream(outStream.toByteArray());
    }

    @Override
    public void writeTo(OutputStream outStream) throws IOException {
        multipartFormEntity.writeTo(outStream);
    }

    @Override
    public boolean isStreaming() {
        return multipartFormEntity.isStreaming();
    }

    @Override
    public void consumeContent() throws IOException {
        multipartFormEntity.getContent();
    }
}
