package com.chuang.tauceti.rbac2.crud;

import com.chuang.tauceti.rbac2.crud.entity.Config;
import com.chuang.tauceti.rbac2.crud.service.IConfigService;
import com.chuang.tauceti.support.SystemConfigPolicy;
import com.chuang.tauceti.tools.basic.ScheduleKit;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.Optional;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;

/**
 * @author ath
 */
@Service
@Slf4j
public class CachedDBSystemSettingPolicy implements SystemConfigPolicy {

    private final IConfigService configService;
    private final ConcurrentHashMap<String, String> cached = new ConcurrentHashMap<>();

    public CachedDBSystemSettingPolicy(IConfigService configService) {
        this.configService = configService;
    }

    @PostConstruct()
    public void init() {
        ScheduleKit.schedule(this::clearCached, 10, TimeUnit.SECONDS);
    }

    @Override
    public boolean contains(String key) {
        if(cached.containsKey(key)) {
            return true;
        }
        return get(key).isPresent();
    }

    @Override
    public void clearCached() {
        log.debug("============= 清空配置缓存 ===========");
        cached.clear();
        ScheduleKit.schedule(this::clearCached, 10, TimeUnit.SECONDS);
    }


    @Override
    public Optional<String> get(String key) {
        if(cached.containsKey(key)) {
            return Optional.of(cached.get(key));
        }

        Optional<String> value = configService.findByCode(key).map(Config::getValue);
        value.ifPresent(v -> cached.put(key, v));
        return value;
    }

    @Override
    public void set(String key, String value, String regex) {
        Config sp = configService.lambdaQuery().eq(Config::getCode, key).one();
        Config entity = new Config();
        if (null != sp) {
            sp.setValue(value);
            configService.updateById(sp);
        } else {
            entity.setCode(key);
            entity.setRemark("");
            entity.setValue(value);
            entity.setValueRegex(regex);
            configService.save(entity);
        }
        cached.put(key, value);
    }
}
