package com.chuang.tauceti.rbac2.crud.entity;

import com.baomidou.mybatisplus.annotation.*;
import com.chuang.tauceti.rbac2.crud.enums.FieldAction;
import com.chuang.tauceti.tools.basic.StringKit;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

import java.io.Serializable;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * <p>
 * api字段权限
 * </p>
 *
 * @author chuang
 * @since 2021-04-30
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
@TableName("sys_api_field_acl")
@KeySequence(value = "SEQ_SYS_FIELD_ACL", dbType = DbType.OTHER)
public class ApiFieldAcl implements Serializable {

    private static final long serialVersionUID=1L;

    /**
     * id
     */
    @TableId(value = "id", type = IdType.NONE)
    private Integer id;

    /**
     * 接口编号
     */
    @TableField("api_code")
    private String apiCode;

    /**
     * 字段名
     */
    @TableField("field_el")
    private String fieldEl;

    /**
     * 字段标签
     */
    @TableField("field_label")
    private String fieldLabel;

    /**
     * 需要的角色
     */
    @TableField("roles")
    private String roles;

    /**
     * 需要的权限
     */
    @TableField("abilities")
    private String abilities;

    /**
     * 是否为入参
     */
    @TableField("income")
    private Boolean income;

    /**
     * 符合要求后的行为
     */
    @TableField("action")
    private FieldAction action;

    /**
     * 创建人
     */
    @TableField(value = "creator", fill = FieldFill.INSERT)
    private String creator;

    /**
     * 创建时间
     */
    @TableField("created_time")
    private LocalDateTime createdTime;

    /**
     * 更新人
     */
    @TableField(value = "updater", fill = FieldFill.INSERT_UPDATE)
    private String updater;

    /**
     * 更新时间
     */
    @TableField("updated_time")
    private LocalDateTime updatedTime;


    public List<String> abilities() {
        return new ArrayList<>(Arrays.asList(this.abilities.split(",")));
    }

    public void abilities(List<String> abilities) {
        this.abilities = StringKit.join(abilities, ",");
    }

    public List<String> roles() {
        return new ArrayList<>(Arrays.asList(this.roles.split(",")));
    }

    public void roles(List<String> roles) {
        this.roles = StringKit.join(roles, ",");
    }
}
