package com.chuang.tauceti.rowquery;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import java.util.List;

public class PageWrapper<T> extends Page<T> implements IPage<T> {

    private final Page<T> target;

    public PageWrapper(Page<T> page) {
        this.target = page;
    }

    public PageWrapper(IPage<T> page) {
        this((Page<T>) page);
    }

    /**
     * 是否存在上一页
     *
     * @return true / false
     */
    @Override
    public boolean hasPrevious() {
        return target.getCurrent() > 1;
    }

    /**
     * 是否存在下一页
     *
     * @return true / false
     */
    @Override
    public boolean hasNext() {
        return target.getCurrent() < this.getPages();
    }

    @Override
    public List<T> getRecords() {
        return target.getRecords();
    }

    @Override
    public PageWrapper<T> setRecords(List<T> records) {
        target.setRecords(records);
        return this;
    }

    @Override
    public long getTotal() {
        return target.getTotal();
    }

    @Override
    public PageWrapper<T> setTotal(long total) {
        target.setTotal(total);
        return this;
    }

    @Override
    public long getSize() {
        return target.getSize();
    }

    @Override
    public PageWrapper<T> setSize(long size) {
        target.setSize(size);
        return this;
    }

    @Override
    public long getCurrent() {
        return target.getCurrent();
    }

    @Override
    public PageWrapper<T> setCurrent(long current) {
        target.setCurrent(current);
        return this;
    }

    @Override
    public String countId() {
        return target.countId();
    }

    @Override
    public Long maxLimit() {
        return target.maxLimit();
    }



    /**
     * 添加新的排序条件，构造条件可以使用工厂：{@link OrderItem#build(String, boolean)}
     *
     * @param items 条件
     * @return 返回分页参数本身
     */
    @Override
    public PageWrapper<T> addOrder(OrderItem... items) {
        target.addOrder(items);
        return this;
    }

    /**
     * 添加新的排序条件，构造条件可以使用工厂：{@link OrderItem#build(String, boolean)}
     *
     * @param items 条件
     * @return 返回分页参数本身
     */
    @Override
    public PageWrapper<T> addOrder(List<OrderItem> items) {
        target.addOrder(items);
        return this;
    }

    @Override
    public List<OrderItem> orders() {
        return target.orders();
    }

    @Override
    public boolean optimizeCountSql() {
        return target.optimizeCountSql();
    }

    @Override
    public boolean searchCount() {
        return target.searchCount();
    }

    @Override
    public Page<T> setSearchCount(boolean searchCount) {
        target.setSearchCount(searchCount);
        return this;
    }

    @Override
    public PageWrapper<T> setOptimizeCountSql(boolean optimizeCountSql) {
        target.setOptimizeCountSql(optimizeCountSql);
        return this;
    }

    @Override
    public long getPages() {
        return target.getPages();
    }

}
