package com.chuang.tauceti.rowquery.handlers;

import com.baomidou.mybatisplus.core.handlers.MetaObjectHandler;
import com.chuang.tauceti.support.vg.ValueGetter;
import org.apache.ibatis.reflection.MetaObject;

import java.time.LocalDateTime;
import java.util.Date;

/**
 * @author ATH
 */
public class AutoTimeHandler implements MetaObjectHandler {

    private final ValueGetter<String> operatorGetter;

    public AutoTimeHandler(ValueGetter<String> operatorGetter) {
        this.operatorGetter = operatorGetter;
    }
    @Override
    public void insertFill(MetaObject metaObject) {
        LocalDateTime now = LocalDateTime.now();

        this.fullIfNull(metaObject, "createdTime", LocalDateTime.now(), new Date());
        this.fullIfNull(metaObject, "updatedTime", LocalDateTime.now(), new Date());
        this.fullIfNull(metaObject, "createTime", LocalDateTime.now(), new Date());
        this.fullIfNull(metaObject, "updateTime", LocalDateTime.now(), new Date());

        this.fullIfNull(metaObject, "deleted", false);
        this.fullIfNull(metaObject, "revision", 0, 0L, (short)0, (byte)0);
        this.fullIfNull(metaObject, "version", 0, 0L, (short)0, (byte)0);

        operatorGetter.get().ifPresent(operator -> {
            this.fullIfNull(metaObject, "creator", operator);
            this.fullIfNull(metaObject, "createBy", operator);
            this.fullIfNull(metaObject, "updater", operator);
            this.fullIfNull(metaObject, "updateBy", operator);
        });
    }

    private void fullIfNull(MetaObject metaObject, String field, Object... values) {
        for(Object value : values) {
            try {
                if(metaObject.hasGetter(field) && null == metaObject.getValue(field)) {
                    this.setFieldValByName(field, value, metaObject);
                }
                break;
            } catch (Exception ignore) {}
        }

    }

    private void full(MetaObject metaObject, String field, Object... values) {
        for(Object value : values) {
            try {
                if(metaObject.hasGetter(field)) {
                    this.setFieldValByName(field, value, metaObject);
                }
                break;
            } catch (Exception ignore) {}
        }
    }


    @Override
    public void updateFill(MetaObject metaObject) {
        this.full(metaObject, "updatedTime", LocalDateTime.now(), new Date());
        this.full(metaObject, "updateTime", LocalDateTime.now(), new Date());
        operatorGetter.get().ifPresent(operator -> {
            this.full(metaObject, "updater", operator);
            this.full(metaObject, "updateBy", operator);
        });
    }


}
