package com.chuang.tauceti.limit;

import com.chuang.tauceti.limit.annotations.Limit;
import com.chuang.tauceti.limit.group.IGroupMapping;
import com.chuang.tauceti.limit.quote.IQuote;
import com.chuang.tauceti.support.enums.Whether;
import com.chuang.tauceti.tools.basic.StringKit;
import lombok.Data;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.lang.Nullable;

/**
 *
 * @author ATH
 */
@Data
@ConfigurationProperties(prefix = "tau-ceti.limit")
public class LimitProperties {

    /** 是否启用 */
    private boolean enabled = true;

    private boolean defaultLimit = false;


    private String defaultGroupName = IGroupMapping.REQUEST;

    private int defaultMaxMulti = 1;

    private String defaultQuarantine = IQuote.USERNAME;


    /** 是否默认对所有的RequestMapping 做limit处理 */
    private boolean allController = false;



    public LimitInfo asDefault() {
        return new LimitInfo()
                .setLimit(enabled && defaultLimit)
                .setGroupName(defaultGroupName)
                .setQuarantine(defaultQuarantine)
                .setMaxMulti(defaultMaxMulti);
    }

    public LimitInfo asInfo(@Nullable Limit limit) {
        if(null == limit) {
            return asDefault();
        }


        return new LimitInfo()
                .setLimit(enabled && limit.value() == Whether.UNKNOWN ? defaultLimit : limit.value().isSuccess())
                .setGroup(limit.group())
                .setGroupName(StringKit.isBlank(limit.groupName()) ? defaultGroupName : limit.groupName())
                .setMaxMulti(limit.maxMulti() == 0 ? defaultMaxMulti : limit.maxMulti())
                .setQuarantine(StringKit.isBlank(limit.quarantine()) ? defaultQuarantine : limit.quarantine());
    }
}
