package com.chuang.tauceti.limit;

import com.alibaba.fastjson.JSONObject;
import com.chuang.tauceti.limit.quote.IQuote;
import com.chuang.tauceti.support.exception.SystemException;
import com.chuang.tauceti.tools.basic.ObjectKit;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.function.Function;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * @author ATH
 */
@Slf4j
public class QuarantineParser {

    private final static String VARS = "\\$\\{.+?}";

    private final String quarantine;

    private final List<IQuote> quotes;

    private final Map<String, Function<ProceedingJoinPoint, String>> refHandChain = new HashMap<>();

    public QuarantineParser(String quarantine, List<IQuote> quotes) {
        this.quarantine = quarantine;
        this.quotes = quotes;

        Pattern pattern = Pattern.compile(VARS, Pattern.DOTALL);
        Matcher matcher = pattern.matcher(quarantine);
        while (matcher.find()) {
            addRefHandChain(matcher.group());
        }
    }

    private void addRefHandChain(String ref) {
        String[] refs = ref.substring(2, ref.length() - 1).split("\\.");

        Map<String, IQuote> quoteMap = quotes.stream().collect(Collectors.toMap(IQuote::refer, o -> o));
        IQuote rootQuote = quoteMap.get(refs[0]);
        if (null == rootQuote) {
            throw new SystemException(ref + " 无法获取对象, " + refs[0] + " 会使用IQuote来获取对象，请检查是否有创建相关 IQuote");
        }

        Function<ProceedingJoinPoint, ?> function = rootQuote::quote;

        for(int i = 1; i < refs.length; i++) {
            String r = refs[i];
            function = function.andThen(o -> {
                Optional<Object> opt = ObjectKit.getValue(o, r);
                if(opt.isPresent()) {
                    return opt.get();
                } else {
                    throw new SystemException("{}.**.{} 为空, {}为: {}", refs[0], r, refs[0], JSONObject.toJSONString(rootQuote));
                }
            });
        }

        this.refHandChain.put(ref, function.andThen(Object::toString));
    }

    public String parser(ProceedingJoinPoint pjp) {
        String result = quarantine;
        for(String refTemp : refHandChain.keySet()) {
            Function<ProceedingJoinPoint, String> function = refHandChain.get(refTemp);
            String value = function.apply(pjp);

            if (value.contains(refTemp)) {
               if(log.isDebugEnabled()) {
                   log.debug("[严重警告]{} 得到的值 {} 包含了自身，为避免死循环，程序强制停止本次替换。", refTemp, value);
               }
               continue;
            }
            // 上面的if已经验证了 value 中不能包含 refTemp，避免死循环
            while(result.contains(refTemp)) {
                result = result.replace(refTemp, value);
            }
        }
        return result;
    }
}
