package com.chuang.tauceti.limit.aop;

import com.chuang.tauceti.limit.Limiter;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * @author ATH
 */
@Aspect
@Slf4j
@Component
@ConditionalOnProperty(name = "tau-ceti.limit.all-controller")
public class RequestAspect {

    @Resource private Limiter limiter;

    @Pointcut("@within(org.springframework.stereotype.Controller) || @within(org.springframework.web.bind.annotation.RestController)")
    public void controllerPointcut() {}


    @Pointcut("@annotation(org.springframework.web.bind.annotation.RequestMapping) || " +
            "@annotation(org.springframework.web.bind.annotation.GetMapping) || " +
            "@annotation(org.springframework.web.bind.annotation.PostMapping) || " +
            "@annotation(org.springframework.web.bind.annotation.PutMapping) || " +
            "@annotation(org.springframework.web.bind.annotation.DeleteMapping) )")
    public void requestMappingPointcut() {}

    @Around(value = "controllerPointcut() && requestMappingPointcut()")
    public Object handlerControllerMethod(ProceedingJoinPoint pjp) throws Throwable {
        return limiter.doLimit(pjp);
    }
}
