package com.chuang.tauceti.limit.counter;

import com.chuang.tauceti.limit.Runner;
import com.chuang.tauceti.limit.exception.LimitException;
import com.chuang.tauceti.tools.basic.collection.DoubleKeyMap;

import java.util.concurrent.atomic.AtomicInteger;

/**
 * @author ATH
 */
public class LocalCountRunner implements LimitCountRunner {

    private final DoubleKeyMap<String, String, AtomicInteger> counter = new DoubleKeyMap<>();

    @Override
    public <T> T countRun(String group, String quarantine, int maxMulti, Runner<T> getter) throws Throwable {
        AtomicInteger count = counter.getOrDefault(group, quarantine, (s, s2) -> new AtomicInteger(0));
        if(count.get() >= maxMulti) {
            throw new LimitException("group: " + group + ", quarantine: " + quarantine + " 超过了" + maxMulti + "次");
        }
        count.incrementAndGet();
        try {
            return getter.run();
        } finally {
            count.decrementAndGet();
        }
    }
}
