package com.chuang.tauceti.shiro.spring.web.jwt;

import com.chuang.tauceti.shiro.spring.web.jwt.filter.JwtAuthFilter;
import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.tools.third.servlet.HttpKit;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.mgt.SecurityManager;
import org.apache.shiro.subject.PrincipalCollection;

import java.util.Collection;
import java.util.Optional;

public class ShiroUtils {
    public static Optional<JwtPayload> jwt() {
        return HttpKit.getRequest().map(request -> (JwtPayload) request.getAttribute(JwtAuthFilter.JWT_PAYLOAD_REQUEST_ATTR));
    }

    public static Optional<String> jwtTokenStr() {
        return HttpKit.getRequest().map(request -> (String) request.getAttribute(JwtAuthFilter.JWT_STR_REQUEST_ATTR));
    }

    public static <T> Optional<T> loggedUser() {
        return Optional.ofNullable((T) SecurityUtils.getSubject().getPrincipal());
    }

    public static <T> T loggedUserNotNull() {
        Optional<T> opt = loggedUser();
        return opt.orElseThrow(() -> new BusinessException("没有登录~"));
    }


    public static boolean hasPermission(String permission) {
        return SecurityUtils.getSecurityManager().isPermitted(
                SecurityUtils.getSubject().getPrincipals(),
                permission
        );
    }

    public static boolean canAll(Collection<String> roles, Collection<String> abilities) {
        PrincipalCollection principals = SecurityUtils.getSubject().getPrincipals();

        boolean canRole = SecurityUtils.getSecurityManager().hasAllRoles(principals, roles);
        boolean canAbility = SecurityUtils.getSecurityManager().isPermittedAll(principals, abilities.toArray(new String[0]));

        return canRole && canAbility;
    }
    public static boolean canAny(Collection<String> roles, Collection<String> abilities) {
        PrincipalCollection principals = SecurityUtils.getSubject().getPrincipals();
        SecurityManager manager = SecurityUtils.getSecurityManager();
        boolean canRole = roles.stream().anyMatch(s -> manager.hasRole(principals, s));
        boolean canAbility = abilities.stream().anyMatch(s -> manager.isPermitted(principals, s));
        return canRole && canAbility;
    }
}
