package com.chuang.tauceti.support.page;

import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.experimental.Accessors;
import org.springframework.beans.BeanUtils;

import java.util.List;
import java.util.function.Function;
import java.util.function.Supplier;
import java.util.stream.Collectors;

@Getter
@Accessors(chain = true)
@EqualsAndHashCode(callSuper = true)
public class Page<T> extends Pageable {

    private final Long total;

    private final Long pages;

    private final List<T> records;

    public Page(Long current, Long size, Long total, Long pages, List<T> records) {
        super(current, size);
        this.total = total;
        this.pages = pages;
        this.records = records;
    }



    /**
     * 是否存在上一页
     *
     * @return true / false
     */
    public boolean hasPrevious() {
        return this.getCurrent() > 1;
    }

    /**
     * 是否存在下一页
     *
     * @return true / false
     */
    public boolean hasNext() {
        return this.getCurrent() < this.getPages();
    }

    public <R> Page<R> map(Function<T, R> map) {
        return new Page<>(
                this.getCurrent(),
                this.getSize(),
                this.total,
                this.pages,
                this.records.stream().map(map).collect(Collectors.toList())
        );
    }

    public <R> Page<R> map(Supplier<R> creator) {
        return map(t -> {
            if(null == t) {
                return creator.get();
            }
            R r = creator.get();
            BeanUtils.copyProperties(t, r);
            return r;
        });
    }
}
