### 关于 telegrambots 的代码说明
#### 整体设计
1. Update 消息
   1. 
2. Bot 机器人.  机器人承担消息的接受和发送业务. 它需要实现收到消息后怎么处理, 处理后发送哪些消息。机器人根据接受消息的方式被分为2大类， 这两类监听是互斥的，建议使用 WebHook
   1. LongPollingBot  长轮询机器人, 通过长轮询telegram某个地址来获取消息，并触发bot的 onUpdateReceived(Update update); 消息的监听依赖于 Session 来完成
      1. 需要说明的是长轮询的过滤条件在短时间内不会生效。
      2. 长轮询会查询重复的内容，需要自己过滤
      3. 长轮询拉取的内容保存在TG服务器上，只保留24小时。
      4. 如果设置了webhook，长轮询就无法使用。
   2. WebhookBot web钩子机器人, 通过向telegram发送一条回调地址的消息，然后进行web监听回调的方式来触发。消息的监听由 WebHook -> RestApi 来完成。
      1. 开启webhook后，长轮询将无法使用。因此每次使用完，记得给它关闭。
   
3. BotSession 会话. 类似于 HttpSession,只不过这里管理的是机器人与Telegram服务器的会话。会话的作用如下：
   1. 开启监听线程`readerThread`，不断到指定的地址拉取消息，然后回调 LongPollingBot 的  onUpdateReceived(Update update)。监听线程的消息监听依赖于 UpdatesSupplier, 默认使用 BotSession.getUpdatesFromServer() 实现。
   2. 开启处理线程`handlerThread` 对监听到的消息进行处理。
   3. 以上两个线程都是由 BotSession.start() 开启, BotSession.stop() 结束. 需要注意的是stop使用的是 interrupt() 进行中止通知. readerThread因为是IO线程可以立即停止响应。handlerThread则取决于回调中的实现。
4. WebHook web钩子,内置 RestApi 对request进行接收，并将接收到的消息转发给 WebHookBot
5. TelegramBotsApi 机器人管理类


#### 优缺点
1. 优点
   1. 模块非常清晰，代码质量也很高，使用了 Guice 来处理依赖注入。可以参考一下。
   2. 长轮询和WebHook都有实现，使用起来非常方便。
   3. 对线程的处理非常细致，业务处理相对安全。
2. 缺点
   1. BotSession 的线程管理不够灵活, handlerThread 和 readerThread都是单线程执行。 readerThread 无法被用户动态拓展。 handlerThread 虽然可以在回调中使用线程池处理，但接口不方便。
   2. 配置过于分散或者封装得不好理解，例如BotSession的 readerThread 使用的客户端超时时间都写在 CONST 中，不方便管理。
3. 改进
   1. 吸取其优点，使用 Guice 管理对象
   2. 参考其 WebHook 的实现
   3. http相关的配置由 HttpClient 来实现。
   4. 使用stream接口来实现输入和输出
