package com.chuang.telegram.bot;

import com.chuang.tauceti.httpclient.Https;
import com.chuang.telegram.bot.configs.TelegramBotConfig;
import com.chuang.telegram.bot.exception.TelegramApiException;
import com.chuang.telegram.bot.interfaces.Sender;
import com.chuang.telegram.bot.interfaces.WebHookBots;
import com.google.inject.Guice;
import com.google.inject.Injector;
import com.google.inject.Module;
import com.google.inject.Scopes;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.List;

@SuppressWarnings({"rawtypes", "unchecked"})
@Slf4j
public class ApiContext {

    private static final Object LOCK = new Object();
    private static Injector INJECTOR;
    private static final List<Module> MODULES = new ArrayList<>();

    public static <T> T getInstance(Class<T> type) {
        return getInjector().getInstance(type);
    }

    public static void register(Module module) {
        MODULES.add(module);
    }

    public static void register(Class<?>... clazz) {
        if(clazz.length % 2 != 0) {
            throw new TelegramApiException("class 数量必须是2的倍数");
        }
        MODULES.add(binder -> {
            for (int i = 0; i < clazz.length / 2; i++) {
                binder.bind(clazz[i * 2]).to((Class) clazz[i * 2 + 1]);
            }
        });
    }

    public static void registerInstance(Object... clazz) {
        if(clazz.length % 2 != 0) {
            throw new TelegramApiException("class 数量必须是2的倍数");
        }
        MODULES.add(binder -> {
            for (int i = 0; i < clazz.length / 2; i++) {
                binder.bind((Class)clazz[i * 2]).toInstance(clazz[i * 2 + 1]);
            }
        });
    }

    public static void registerConfig(TelegramBotConfig config) {
        registerInstance(TelegramBotConfig.class, config);
    }

    public static void registerSingleton(Class<?>... clazz) {
        if(clazz.length % 2 != 0) {
            throw new TelegramApiException("class 数量必须是2的倍数");
        }
        MODULES.add(binder -> {
            for (int i = 0; i < clazz.length / 2; i++) {
                binder.bind(clazz[i * 2]).to((Class) clazz[i * 2 + 1]).in(Scopes.SINGLETON);
            }
        });
    }

    private static Injector getInjector() {
        if (INJECTOR == null) {
            synchronized (LOCK) {
                if (INJECTOR == null) {
                    INJECTOR = Guice.createInjector(MODULES.toArray(new Module[0]));
                }
            }
        }
        return INJECTOR;
    }

    public static void init() {
        register(Https.asModule());
        register(Sender.class, DefaultSender.class, WebHookBots.class, DefaultWebHookBots.class);
    }

}
