package com.chuang.telegram.bot;

import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.tools.basic.IOKit;
import com.chuang.telegram.bot.interfaces.BotApiMethod;
import com.chuang.telegram.bot.model.InputFile;
import com.chuang.telegram.bot.model.media.InputMedia;
import com.chuang.telegram.bot.model.media.InputMediaAudio;
import com.chuang.telegram.bot.model.media.InputMediaDocument;
import com.chuang.telegram.bot.model.media.InputMediaVideo;
import com.fasterxml.jackson.core.JsonProcessingException;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.MultipartEntityBuilder;

import java.io.IOException;
import java.io.InputStream;
import java.util.List;

public class HttpUtils {

    public static MultipartEntityBuilder addInputFile(MultipartEntityBuilder builder, InputFile file, String fileField, boolean addField) {
        if (file.isNew()) {
            if (file.getNewMediaFile() != null) {
                return builder.addBinaryBody(fileField, file.getNewMediaFile(), ContentType.APPLICATION_OCTET_STREAM, file.getMediaName());
            } else if (file.getNewMediaStream() != null) {
                return builder.addBinaryBody(fileField, streamAsBytes(file.getNewMediaStream()), ContentType.APPLICATION_OCTET_STREAM, file.getMediaName());
            }
        }

        if (addField) {
            return builder.addTextBody(fileField, file.getAttachName(), BotApiMethod.TEXT_PLAIN_UTF8);
        }

        return builder;
    }

    public static void addInputData(MultipartEntityBuilder builder, List<InputMedia<?>> media, String mediaField) throws JsonProcessingException {
        for (InputMedia<?> inputMedia : media) {
            addInputData(builder, inputMedia, mediaField, false);
        }

        builder.addTextBody(mediaField, BotApiMethod.OBJECT_MAPPER.writeValueAsString(media), BotApiMethod.TEXT_PLAIN_UTF8);
    }

    public static void addInputData(MultipartEntityBuilder builder, InputMedia<?> media, String mediaField, boolean addField) throws JsonProcessingException {
        if (media.isNewMedia()) {
            if (media.getMediaFile() != null) {
                builder.addBinaryBody(mediaField, media.getMediaFile(), ContentType.APPLICATION_OCTET_STREAM, media.getMediaName());
            } else if (media.getNewMediaStream() != null) {
                builder.addBinaryBody(mediaField, streamAsBytes(media.getNewMediaStream()), ContentType.APPLICATION_OCTET_STREAM, media.getMediaName());
            }
        }

        if (media instanceof InputMediaAudio) {
            InputMediaAudio audio = (InputMediaAudio) media;
            if (audio.getThumb() != null) {
                addInputFile(builder, audio.getThumb(), InputMediaAudio.THUMB_FIELD, false);
            }
        } else if (media instanceof InputMediaDocument) {
            InputMediaDocument document = (InputMediaDocument) media;
            if (document.getThumb() != null) {
                addInputFile(builder, document.getThumb(), InputMediaDocument.THUMB_FIELD, false);
            }
        } else if (media instanceof InputMediaVideo) {
            InputMediaVideo video = (InputMediaVideo) media;
            if (video.getThumb() != null) {
                addInputFile(builder, video.getThumb(), InputMediaVideo.THUMB_FIELD, false);
            }
        }

        if (addField) {
            builder.addTextBody(mediaField, BotApiMethod.OBJECT_MAPPER.writeValueAsString(media), BotApiMethod.TEXT_PLAIN_UTF8);
        }
    }

    private static byte[] streamAsBytes(InputStream stream) {
        byte[] buf = new byte[0];
        try {
            return IOKit.readBytes(stream);
        } catch (IOException e) {
            throw new BusinessException("读取失败");
        }
    }
}
