package com.chuang.telegram.bot.interfaces;

import com.chuang.telegram.bot.model.Update;
import com.chuang.telegram.bot.model.hook.HookUpdate;
import com.chuang.telegram.bot.model.hook.WebHookBot;
import com.chuang.telegram.bot.model.hook.WebhookInfo;
import reactor.core.publisher.Flux;

import javax.annotation.Nullable;

public interface WebHookBots {

    /**
     * 触发一个更新消息
     * @param listenPath 监听地址
     * @param update     更新
     */
    void firePathUpdate(String listenPath, Update update, @Nullable Object payload);

    default void firePathUpdate(String listenPath, Update update) {
        firePathUpdate(listenPath, update, null);
    }

    /**
     * 触发一个更新消息
     * @param botKey     机器人key
     * @param update     更新
     */
    void fireKeyUpdate(String botKey, Update update, @Nullable Object payload);

    default void fireKeyUpdate(String botKey, Update update) {
        fireKeyUpdate(botKey, update, null);
    }

    /**
     * 开始监听指定的机器人
     * @param botKey        机器人key，唯一
     * @param token         机器人防伪token
     * @param listenPath    机器人监听地址，唯一
     * @param forceRestart  当webhook已经在监听，是否强制重启。
     */
    void listen(String botKey, String token, String listenPath, boolean forceRestart, boolean stopHistory);

    /**
     * 停止指定机器人的监听
     * @param botKey 机器人的key
     * @param stopHook  停止监听时是否通知tg移除webhook
     */
    void stopListen(String botKey, boolean stopHook);

    /**
     * 停止所有机器人的监听
     * @param stopHook  停止监听时是否通知tg移除webhook
     */
    void stopListen(boolean stopHook);

    /**
     * 获取指定机器人的监听信息
     * @param botKey 机器人key
     */
    WebhookInfo getWebhookInfo(String botKey);

    WebHookBot getBot(String botKey);
    WebHookBot getBotByPath(String path);

    /**
     * 转换成Flux
     */
    Flux<HookUpdate> asFlux();
}
