package com.chuang.telegram.bot.methods;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.BotApiMethod;
import com.chuang.telegram.bot.model.BotApiResponse;
import com.chuang.telegram.bot.model.UserProfilePhotos;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.type.TypeReference;
import lombok.Data;
import lombok.experimental.Accessors;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * @brief Use this method to get a list of profile pictures for a user. Returns a UserProfilePhotos object.
 * @date 20 of June of 2015
 */
@Data
@Accessors(chain = true)
public class GetUserProfilePhotos implements BotApiMethod<UserProfilePhotos> {
    public static final String PATH = "getuserprofilephotos";

    private static final String USER_ID_FIELD = "user_id";
    private static final String OFFSET_FIELD = "offset";
    private static final String LIMIT_FIELD = "limit";

	/**  Unique identifier of the target user */
    @JsonProperty(USER_ID_FIELD)
    private Long userId;
    /**
     * Sequential number of the first photo to be returned. By default, all photos are returned.
     */
    @JsonProperty(OFFSET_FIELD)
    private Long offset;
    /**
     * Optional. Limits the number of photos to be retrieved. Values between 1—100 are accepted. Defaults to 100.
     */
    @JsonProperty(LIMIT_FIELD)
    private Long limit;


    @Override
    public String getMethod() {
        return PATH;
    }


    @Override
    public void validate() throws TelegramApiValidationException {
        if (userId == null) {
            throw new TelegramApiValidationException("UserId parameter can't be empty", this);
        }
        if (offset == null) {
            throw new TelegramApiValidationException("Offset parameter can't be empty", this);
        }
    }

    @Override
    public TypeReference<BotApiResponse<UserProfilePhotos>> typeReference() {
        return new TypeReference<BotApiResponse<UserProfilePhotos>>() {};
    }

}
