
package com.chuang.telegram.bot.methods;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.BotApiMethod;
import com.chuang.telegram.bot.model.passport.dataerror.PassportElementError;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Ruben Bermudez
 * @version 4.0.0
 *
 * Informs a user that some Telegram Passport data contains errors.
 * The user will not be able to resend data, until the errors are fixed
 */
@Data
@Accessors(chain = true)
public class SetPassportDataErrors implements BotApiMethod<Boolean> {
    public static final String PATH = "setPassportDataErrors";

    private static final String USER_ID_FIELD = "user_id";
    private static final String ERRORS_FIELD = "errors";
    
	/**  User identifier */
    @JsonProperty(USER_ID_FIELD)
    private Long userId;
	/**  A JSON-serialized array describing the errors */
    @JsonProperty(ERRORS_FIELD)
    private List<PassportElementError> errors;

    public SetPassportDataErrors(Long userId, List<PassportElementError> errors) {
        super();
        this.userId = checkNotNull(userId);
        this.errors = checkNotNull(errors);
    }

    public SetPassportDataErrors() {
        super();
    }


    public SetPassportDataErrors addError(PassportElementError error) {
        error = checkNotNull(error);
        if (this.errors == null) {
            this.errors = new ArrayList<>();
        }
        this.errors.add(error);
        return this;
    }

    @Override
    public String getMethod() {
        return PATH;
    }

    @Override
    public void validate() throws TelegramApiValidationException {
        if (userId == null) {
            throw new TelegramApiValidationException("User ID can't be empty", this);
        }
        if (errors == null || errors.isEmpty()) {
            throw new TelegramApiValidationException("Errors can't be empty", this);
        }
    }
}
