
package com.chuang.telegram.bot.methods.groupadmin;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.BotApiMethod;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

/**
 * @author Ruben Bermudez
 * @version 3.1
 * Use this method to export an invite link to a supergroup or a channel. The bot must be an administrator in the
 * chat for this to work and must have the appropriate admin rights. Returns exported invite link as String on success.
 */
@Data
@Accessors(chain = true)
public class ExportChatInviteLink implements BotApiMethod<String> {
    public static final String PATH = "exportChatInviteLink";

    private static final String CHAT_ID_FIELD = "chat_id";

	/**  Unique identifier for the chat to send the message to (Or username for channels) */
    @JsonProperty(CHAT_ID_FIELD)
    private String chatId;

    @Override
    public String getMethod() {
        return PATH;
    }


    @Override
    public void validate() throws TelegramApiValidationException {
        if (chatId == null || chatId.isEmpty()) {
            throw new TelegramApiValidationException("ChatId can't be empty", this);
        }
    }
}
