package com.chuang.telegram.bot.methods.groupadmin;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.BotApiMethod;
import com.chuang.telegram.bot.model.BotApiResponse;
import com.chuang.telegram.bot.model.Chat;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.type.TypeReference;
import lombok.Data;
import lombok.experimental.Accessors;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * Use this method to get information about the chat. Returns Chat object on success.
 */
@Data
@Accessors(chain = true)
public class GetChat implements BotApiMethod<Chat> {
    public static final String PATH = "getChat";

    private static final String CHAT_ID_FIELD = "chat_id";

	/**  Unique identifier for the chat to send the message to (Or username for channels) */
    @JsonProperty(CHAT_ID_FIELD)
    private String chatId;

    public GetChat() {
        super();
    }

    public GetChat(String chatId) {
        super();
        this.chatId = checkNotNull(chatId);
    }

    public GetChat(Long chatId) {
        super();
        this.chatId = checkNotNull(chatId).toString();
    }

    @Override
    public String getMethod() {
        return PATH;
    }


    @Override
    public void validate() throws TelegramApiValidationException {
        if (chatId == null || chatId.isEmpty()) {
            throw new TelegramApiValidationException("ChatId can't be empty", this);
        }
    }

    @Override
    public TypeReference<BotApiResponse<Chat>> typeReference() {
        return new TypeReference<BotApiResponse<Chat>>() {};
    }
}
