
package com.chuang.telegram.bot.methods.groupadmin;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.BotApiMethod;
import com.chuang.telegram.bot.model.BotApiResponse;
import com.chuang.telegram.bot.model.ChatMember;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.type.TypeReference;
import lombok.Data;
import lombok.experimental.Accessors;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * @brief Use this method to get information about a member of a chat.
 * Returns a ChatMember object on success.
 * @date 20 of May of 2016
 */
@Data
@Accessors(chain = true)
public class GetChatMember implements BotApiMethod<ChatMember> {
    public static final String PATH = "getChatMember";

    private static final String CHAT_ID_FIELD = "chat_id";
    private static final String USER_ID_FIELD = "user_id";

	/**  Unique identifier for the chat to send the message to (Or username for channels) */
    @JsonProperty(CHAT_ID_FIELD)
    private String chatId;
	/**  Unique identifier of the target user */
    @JsonProperty(USER_ID_FIELD)
    private Long userId;


    @Override
    public String getMethod() {
        return PATH;
    }

    @Override
    public void validate() throws TelegramApiValidationException {
        if (chatId == null || chatId.isEmpty()) {
            throw new TelegramApiValidationException("ChatId can't be empty", this);
        }
        if (userId == null) {
            throw new TelegramApiValidationException("UserId can't be null", this);
        }
    }

    @Override
    public TypeReference<BotApiResponse<ChatMember>> typeReference() {
        return new TypeReference<BotApiResponse<ChatMember>>() {};
    }
}
