package com.chuang.telegram.bot.methods.groupadmin;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.BotApiMethod;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

import java.util.Objects;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * @brief Use this method for your bot to leave a group, supergroup or channel. Returns True on success.
 * @date 20 of May of 2016
 */
@Data
@Accessors(chain = true)
public class LeaveChat implements BotApiMethod<Boolean> {
    public static final String PATH = "leaveChat";

    private static final String CHAT_ID_FIELD = "chat_id";

	/**  Unique identifier for the chat to send the message to (Or username for channels) */
    @JsonProperty(CHAT_ID_FIELD)
    private String chatId;

    public LeaveChat() {
        super();
    }

    public String getChatId() {
        return chatId;
    }

    public LeaveChat setChatId(String chatId) {
        this.chatId = chatId;
        return this;
    }

    public LeaveChat setChatId(Long chatId) {
        Objects.requireNonNull(chatId);
        this.chatId = chatId.toString();
        return this;
    }

    @Override
    public String getMethod() {
        return PATH;
    }



    @Override
    public void validate() throws TelegramApiValidationException {
        if (chatId == null || chatId.isEmpty()) {
            throw new TelegramApiValidationException("ChatId can't be null", this);
        }
    }
}
