package com.chuang.telegram.bot.methods.groupadmin;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.BotApiMethod;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

import java.util.Objects;

/**
 * @author Ruben Bermudez
 * @version 3.1
 * Use this method to promote or demote a user in a supergroup or a channel.
 * The bot must be an administrator in the chat for this to work and must have the appropriate admin rights.
 * Pass False for all boolean parameters to demote a user. Returns True on success.
 *
 */
@Data
@Accessors(chain = true)
public class PromoteChatMember implements BotApiMethod<Boolean> {
    public static final String PATH = "promoteChatMember";

    private static final String CHAT_ID_FIELD = "chat_id";
    private static final String USER_ID_FIELD = "user_id";
    private static final String CAN_CHANGE_INFORMATION_FIELD = "can_change_info";
    private static final String CAN_POST_MESSAGES_FIELD = "can_post_messages";
    private static final String CAN_EDIT_MESSAGES_FIELD = "can_edit_messages";
    private static final String CAN_DELETE_MESSAGES_FIELD = "can_delete_messages";
    private static final String CAN_INVITE_USERS_FIELD = "can_invite_users";
    private static final String CAN_RESTRICT_MEMBERS_FIELD = "can_restrict_members";
    private static final String CAN_PIN_MESSAGES_FIELD = "can_pin_messages";
    private static final String CAN_PROMOTE_MEMBERS_FIELD = "can_promote_members";

	/**  Required. Unique identifier for the chat to send the message to (Or username for channels) */
    @JsonProperty(CHAT_ID_FIELD)
    private String chatId;
	/**  Required. Unique identifier of the target user */
    @JsonProperty(USER_ID_FIELD)
    private Long userId;
	/**  Pass True, if the administrator can change chat title, photo and other settings */
    @JsonProperty(CAN_CHANGE_INFORMATION_FIELD)
    private Boolean canChangeInformation;
	/**  Pass True, if the administrator can create channel posts, channels only */
    @JsonProperty(CAN_POST_MESSAGES_FIELD)
    private Boolean canPostMessages;
	/**  Pass True, if the administrator can edit messages of other users, channels only */
    @JsonProperty(CAN_EDIT_MESSAGES_FIELD)
    private Boolean canEditMessages;
	/**  Pass True, if the administrator can delete messages of other users */
    @JsonProperty(CAN_DELETE_MESSAGES_FIELD)
    private Boolean canDeleteMessages;
	/**  Pass True, if the administrator can invite new users to the chat */
    @JsonProperty(CAN_INVITE_USERS_FIELD)
    private Boolean canInviteUsers;
	/**  Pass True, if the administrator can restrict, ban or unban chat members */
    @JsonProperty(CAN_RESTRICT_MEMBERS_FIELD)
    private Boolean canRestrictMembers;
	/**  Pass True, if the administrator can pin messages */
    @JsonProperty(CAN_PIN_MESSAGES_FIELD)
    private Boolean canPinMessages;
	/**  Pass True, if the administrator can add new administrators with a subset of his own privileges or demote administrators that he has promoted, directly or indirectly (promoted by administators that were appointed by the him) */
    @JsonProperty(CAN_PROMOTE_MEMBERS_FIELD)
    private Boolean canPromoteMembers;

    public PromoteChatMember() {
        super();
    }

    public PromoteChatMember(String chatId, Long userId) {
        this.chatId = checkNotNull(chatId);
        this.userId = checkNotNull(userId);
    }

    public PromoteChatMember(Long chatId, Long userId) {
        this.chatId = checkNotNull(chatId).toString();
        this.userId = checkNotNull(userId);
    }


    public PromoteChatMember setChatId(String chatId) {
        this.chatId = chatId;
        return this;
    }

    public PromoteChatMember setChatId(Long chatId) {
        Objects.requireNonNull(chatId);
        this.chatId = chatId.toString();
        return this;
    }


    @Override
    public String getMethod() {
        return PATH;
    }


    @Override
    public void validate() throws TelegramApiValidationException {
        if (chatId == null || chatId.isEmpty()) {
            throw new TelegramApiValidationException("ChatId can't be empty", this);
        }
        if (userId == null) {
            throw new TelegramApiValidationException("UserId can't be null", this);
        }
    }

}
