package com.chuang.telegram.bot.methods.groupadmin;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.BotApiMethod;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

import java.util.Objects;

/**
 * @author Ruben Bermudez
 * @version 3.1
 * Use this method to change the description of a supergroup or channels.
 * The bot must be an administrator in the chat for this to work and must have the appropriate admin rights.
 * Returns True on success.
 */
@Data
@Accessors(chain = true)
public class SetChatDescription implements BotApiMethod<Boolean> {
    public static final String PATH = "setChatDescription";

    private static final String CHAT_ID_FIELD = "chat_id";
    private static final String DESCRIPTION_FIELD = "description";

	/**  Unique identifier for the target chat or username of the target channel (in the format @channelusername) */
    @JsonProperty(CHAT_ID_FIELD)
    private String chatId;
	/**  New chat description, 0-255 characters */
    @JsonProperty(DESCRIPTION_FIELD)
    private String description;

    public SetChatDescription() {
        super();
    }

    public SetChatDescription(String chatId, String description) {
        super();
        this.chatId = checkNotNull(chatId);
        this.description = checkNotNull(description);
    }

    public SetChatDescription(Long chatId, String description) {
        super();
        this.chatId = checkNotNull(chatId).toString();
        this.description = checkNotNull(description);
    }


    public SetChatDescription setChatId(String chatId) {
        this.chatId = chatId;
        return this;
    }

    public SetChatDescription setChatId(Long chatId) {
        Objects.requireNonNull(chatId);
        this.chatId = chatId.toString();
        return this;
    }

    @Override
    public String getMethod() {
        return PATH;
    }


    @Override
    public void validate() throws TelegramApiValidationException {
        if (chatId == null || chatId.isEmpty()) {
            throw new TelegramApiValidationException("ChatId can't be empty", this);
        }
        if (description == null) {
            throw new TelegramApiValidationException("Description can't be null", this);
        }
    }
}
