package com.chuang.telegram.bot.methods.groupadmin;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.BotApiMethod;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

import java.util.Objects;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * Use this method to set a new group sticker set for a supergroup.
 * The bot must be an administrator in the chat for this to work and must have the appropriate admin rights.
 * Use the field can_set_sticker_set optionally returned in getChat requests to check if the bot can use this method.
 * Returns True on success.
 */
@Data
@Accessors(chain = true)
public class SetChatStickerSet implements BotApiMethod<Boolean> {
    public static final String PATH = "setChatStickerSet";

    private static final String CHAT_ID_FIELD = "chat_id";
    private static final String STICKER_SET_NAME_FIELD = "sticker_set_name";

	/**  Unique identifier for the chat to send the message to (Or username for channels) */
    @JsonProperty(CHAT_ID_FIELD)
    private String chatId;
	/**  Name of the sticker set to be set as the group sticker set */
    @JsonProperty(STICKER_SET_NAME_FIELD)
    private String stickerSetName;

    public SetChatStickerSet() {
        super();
    }

    public SetChatStickerSet(String chatId, String stickerSetName) {
        super();
        this.chatId = checkNotNull(chatId);
        this.stickerSetName = checkNotNull(stickerSetName);
    }

    public SetChatStickerSet(Long chatId) {
        super();
        this.chatId = checkNotNull(chatId).toString();
    }


    public SetChatStickerSet setChatId(String chatId) {
        this.chatId = chatId;
        return this;
    }

    public SetChatStickerSet setChatId(Long chatId) {
        Objects.requireNonNull(chatId);
        this.chatId = chatId.toString();
        return this;
    }

    @Override
    public String getMethod() {
        return PATH;
    }


    @Override
    public void validate() throws TelegramApiValidationException {
        if (chatId == null || chatId.isEmpty()) {
            throw new TelegramApiValidationException("ChatId can't be empty", this);
        }
        if (stickerSetName == null || stickerSetName.isEmpty()) {
            throw new TelegramApiValidationException("StickerSetName can't be empty", this);
        }
    }

}
