
package com.chuang.telegram.bot.methods.pinnedmessage;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.BotApiMethod;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

import java.util.Objects;

/**
 * @author Ruben Bermudez
 * @version 3.1
 * Use this method to pin a message in a supergroup or channel.
 * The bot must be an administrator in the chat for this to work and must have the ‘can_pin_messages’
 * admin right in the supergroup or ‘can_edit_messages’ admin right in the channel.
 * Returns True on success.
 */
@Data
@Accessors(chain = true)
public class PinChatMessage implements BotApiMethod<Boolean> {
    public static final String PATH = "pinChatMessage";

    private static final String CHAT_ID_FIELD = "chat_id";
    private static final String MESSAGE_ID_FIELD = "message_id";
    private static final String DISABLE_NOTIFICATION_FIELD = "disable_notification";

	/**  Required. Unique identifier for the target chat or username of the target channel (in the format @channelusername) */
    @JsonProperty(CHAT_ID_FIELD)
    private String chatId;
	/**  Required. Identifier of a message to pin */
    @JsonProperty(MESSAGE_ID_FIELD)
    private Long messageId;
    @JsonProperty(DISABLE_NOTIFICATION_FIELD)
    /**
     * Pass True, if it is not necessary to send a notification to all chat members about the new pinned message.
     * Notifications are always disabled in channels.
     */
    private Boolean disableNotification;

    public PinChatMessage() {
        super();
    }

    public PinChatMessage(String chatId, Long messageId) {
        super();
        this.chatId = checkNotNull(chatId);
        this.messageId = checkNotNull(messageId);
    }

    public PinChatMessage(Long chatId, Long messageId) {
        super();
        this.chatId = checkNotNull(chatId).toString();
        this.messageId = checkNotNull(messageId);
    }

    public PinChatMessage setChatId(String chatId) {
        this.chatId = chatId;
        return this;
    }

    public PinChatMessage setChatId(Long chatId) {
        Objects.requireNonNull(chatId);
        this.chatId = chatId.toString();
        return this;
    }

    @Override
    public String getMethod() {
        return PATH;
    }


    @Override
    public void validate() throws TelegramApiValidationException {
        if (chatId == null || chatId.isEmpty()) {
            throw new TelegramApiValidationException("ChatId parameter can't be empty", this);
        }
        if (messageId == null) {
            throw new TelegramApiValidationException("MessageId parameter can't be null", this);
        }
    }

}
