
package com.chuang.telegram.bot.methods.pinnedmessage;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.BotApiMethod;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

import java.util.Objects;

/**
 * @author Ruben Bermudez
 * @version 3.1
 * Use this method to unpin a message in a supergroup or channel.
 * The bot must be an administrator in the chat for this to work and must have the appropriate admin rights.
 * Returns True on success.
 */
@Data
@Accessors(chain = true)
public class UnpinChatMessage implements BotApiMethod<Boolean> {
    public static final String PATH = "unpinChatMessage";

    private static final String CHAT_ID_FIELD = "chat_id";

	/**  Required. Unique identifier for the target chat or username of the target channel (in the format @channelusername) */
    @JsonProperty(CHAT_ID_FIELD)
    private String chatId;

    public UnpinChatMessage() {
        super();
    }

    public UnpinChatMessage(String chatId) {
        super();
        this.chatId = checkNotNull(chatId);
    }

    public UnpinChatMessage(Long chatId) {
        super();
        this.chatId = checkNotNull(chatId).toString();
    }

    public UnpinChatMessage setChatId(String chatId) {
        this.chatId = chatId;
        return this;
    }

    public UnpinChatMessage setChatId(Long chatId) {
        Objects.requireNonNull(chatId);
        this.chatId = chatId.toString();
        return this;
    }

    @Override
    public String getMethod() {
        return PATH;
    }

    @Override
    public void validate() throws TelegramApiValidationException {
        if (chatId == null || chatId.isEmpty()) {
            throw new TelegramApiValidationException("ChatId parameter can't be empty", this);
        }
    }

}
