
package com.chuang.telegram.bot.methods.send;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.AnyBotApiMethod;
import com.chuang.telegram.bot.model.replaykeyboard.ReplyKeyboard;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * Use this method to send point on the map. On success, the sent Message is returned.
 */
@Data
@Accessors(chain = true)
public class SendLocation implements AnyBotApiMethod {
    public static final String PATH = "sendlocation";

    private static final String CHAT_ID_FIELD = "chat_id";
    private static final String LATITUDE_FIELD = "latitude";
    private static final String LONGITUDE_FIELD = "longitude";
    private static final String DISABLE_NOTIFICATION_FIELD = "disable_notification";
    private static final String REPLY_TO_MESSAGE_ID_FIELD = "reply_to_message_id";
    private static final String REPLY_MARKUP_FIELD = "reply_markup";
    private static final String LIVE_PERIOD_FIELD = "live_period";

	/**  Unique identifier for the chat to send the message to (Or username for channels) */
    @JsonProperty(CHAT_ID_FIELD)
    private String chatId;
	/**  Latitude of location */
    @JsonProperty(LATITUDE_FIELD)
    private Float latitude;
	/**  Longitude of location */
    @JsonProperty(LONGITUDE_FIELD)
    private Float longitude;
	/**  Optional. Sends the message silently. Users will receive a notification with no sound. */
    @JsonProperty(DISABLE_NOTIFICATION_FIELD)
    private Boolean disableNotification;
	/**  Optional. If the message is a reply, ID of the original message */
    @JsonProperty(REPLY_TO_MESSAGE_ID_FIELD)
    private Long replyToMessageId;
	/**  Optional. JSON-serialized object for a custom reply keyboard */
    @JsonProperty(REPLY_MARKUP_FIELD)
    private ReplyKeyboard replyMarkup;
	/**  Optional. Period in seconds for which the location will be updated (see Live Locations), should be between 60 and 86400. */
    @JsonProperty(LIVE_PERIOD_FIELD)
    private Long livePeriod;

    public SendLocation() {
        super();
    }

    public SendLocation(Float latitude, Float longitude) {
        super();
        this.latitude = checkNotNull(latitude);
        this.longitude = checkNotNull(longitude);
    }

    public SendLocation setChatId(String chatId) {
        this.chatId = chatId;
        return this;
    }

    public SendLocation setChatId(Long chatId) {
        this.chatId = chatId.toString();
        return this;
    }

    @Override
    public String getMethod() {
        return PATH;
    }

    @Override
    public void validate() throws TelegramApiValidationException {
        if (chatId == null) {
            throw new TelegramApiValidationException("ChatId parameter can't be empty", this);
        }
        if (latitude == null) {
            throw new TelegramApiValidationException("Latitude parameter can't be empty", this);
        }
        if (longitude == null) {
            throw new TelegramApiValidationException("Longitude parameter can't be empty", this);
        }
        if (replyMarkup != null) {
            replyMarkup.validate();
        }
        if (livePeriod != null && (livePeriod < 60 || livePeriod > 86400)) {
            throw new TelegramApiValidationException("Live period parameter must be between 60 and 86400", this);
        }
    }

}
