package com.chuang.telegram.bot.methods.stickers;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.BotApiMethod;
import com.chuang.telegram.bot.model.BotApiResponse;
import com.chuang.telegram.bot.model.stickers.StickerSet;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.type.TypeReference;
import lombok.Data;
import lombok.experimental.Accessors;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * Use this method to get a sticker set. On success, a StickerSet object is returned.
 */
@Data
@Accessors(chain = true)
public class GetStickerSet implements BotApiMethod<StickerSet> {
    private static final String PATH = "getStickerSet";

    private static final String NAME_FIELD = "name";

	/**  Name of the sticker set */
    @JsonProperty(NAME_FIELD)
    private String name;

    public GetStickerSet(String name) {
        super();
        this.name = checkNotNull(name);
    }

    public GetStickerSet() {
        super();
    }

    @Override
    public String getMethod() {
        return PATH;
    }


    @Override
    public void validate() throws TelegramApiValidationException {
        if (name == null || name.isEmpty()) {
            throw new TelegramApiValidationException("Name can't be null", this);
        }
    }

    @Override
    public TypeReference<BotApiResponse<StickerSet>> typeReference() {
        return new TypeReference<BotApiResponse<StickerSet>>() {};
    }
}
