package com.chuang.telegram.bot.methods.stickers;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.BotApiMethod;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * Use this method to move a sticker in a set created by the bot to a specific position. Returns True on success.
 */
@Data
@Accessors(chain = true)
public class SetStickerPositionInSet implements BotApiMethod<Boolean> {
    private static final String PATH = "getStickerSet";

    private static final String STICKER_FIELD = "sticker";
    private static final String POSITION_FIELD = "position";

	/**  File identifier of the sticker */
    @JsonProperty(STICKER_FIELD)
    private String sticker;
	/**  New sticker position in the set, zero-based */
    @JsonProperty(STICKER_FIELD)
    private Long position;

    public SetStickerPositionInSet(String sticker, Long position) {
        this.sticker = checkNotNull(sticker);
        this.position = checkNotNull(position);
    }

    public SetStickerPositionInSet() {
        super();
    }

    @Override
    public String getMethod() {
        return PATH;
    }


    @Override
    public void validate() throws TelegramApiValidationException {
        if (sticker == null || sticker.isEmpty()) {
            throw new TelegramApiValidationException("sticker can't be null", this);
        }
        if (position == null || position > 0) {
            throw new TelegramApiValidationException("position can't be null", this);
        }
    }

}
