package com.chuang.telegram.bot.model;

import com.chuang.telegram.bot.interfaces.BotApiObject;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * This object represents a Telegram chat with an user or a group
 */
@Data
public class Chat implements BotApiObject {

    private static final String ID_FIELD = "id";
    private static final String TYPE_FIELD = "type";
    private static final String TITLE_FIELD = "title";
    private static final String USERNAME_FIELD = "username";
    private static final String FIRSTNAME_FIELD = "first_name";
    private static final String LASTNAME_FIELD = "last_name";
    private static final String ALL_MEMBERS_ARE_ADMINISTRATORS_FIELD = "all_members_are_administrators";
    private static final String PHOTO_FIELD = "photo";
    private static final String DESCRIPTION_FIELD = "description";
    private static final String INVITE_LINK_FIELD = "invite_link";
    private static final String PINNED_MESSAGE_FIELD = "pinned_message";
    private static final String STICKER_SET_NAME_FIELD = "sticker_set_name";
    private static final String CAN_SET_STICKER_SET_FIELD = "can_set_sticker_set";

    private static final String USER_CHAT_TYPE = "private";
    private static final String GROUP_CHAT_TYPE = "group";
    private static final String CHANNEL_CHAT_TYPE = "channel";
    private static final String SUPER_GROUP_CHAT_TYPE = "supergroup";

    /**
     * Unique identifier for this chat.
     * This number may be greater than 32 bits and some programming languages may
     * have difficulty/silent defects in interpreting it. But it smaller than 52 bits,
     * so a signed 64 bit Long or double-precision float type are safe for storing this identifier.
     */
    @JsonProperty(ID_FIELD)
    private Long id;

    /** < Type of the chat, one of “private”, “group” or “channel” */
    @JsonProperty(TYPE_FIELD)
    private String type;
    /** < Optional. Title of the chat, only for channels and group chat */
    @JsonProperty(TITLE_FIELD)
    private String title;
    /** < Optional. Username of the chat, only for private chats and channels if available */
    @JsonProperty(FIRSTNAME_FIELD)
    private String firstName;
    /** < Optional. Interlocutor's first name for private chats */
    @JsonProperty(LASTNAME_FIELD)
    private String lastName;
    /** < Optional. Interlocutor's last name for private chats */
    @JsonProperty(USERNAME_FIELD)
    private String userName;
    /** Optional. True if a group has ‘All Members Are Admins’ enabled.*/
    @JsonProperty(ALL_MEMBERS_ARE_ADMINISTRATORS_FIELD)
    private Boolean allMembersAreAdministrators;
    /** < Optional. Chat photo. Returned only in getChat. */
    @JsonProperty(PHOTO_FIELD)
    private ChatPhoto photo;
    /** < Optional. Description, for supergroups and channel chats. Returned only in getChat. */
    @JsonProperty(DESCRIPTION_FIELD)
    private String description;
    /** < Optional. Chat invite link, for supergroups and channel chats. Returned only in getChat. */
    @JsonProperty(INVITE_LINK_FIELD)
    private String inviteLink;
    /** < Optional. Pinned message, for supergroups. Returned only in getChat. */
    @JsonProperty(PINNED_MESSAGE_FIELD)
    private Message pinnedMessage;
    /** < Optional. For supergroups, name of Group sticker set. Returned only in getChat. */
    @JsonProperty(STICKER_SET_NAME_FIELD)
    private String stickerSetName;
    /** < Optional. True, if the bot can change group the sticker set. Returned only in getChat. */
    @JsonProperty(CAN_SET_STICKER_SET_FIELD)
    private Message canSetStickerSet;


    public Boolean isGroupChat() {
        return Chat.GROUP_CHAT_TYPE.equals(type);
    }

    public Boolean isChannelChat() {
        return CHANNEL_CHAT_TYPE.equals(type);
    }

    public Boolean isUserChat() {
        return USER_CHAT_TYPE.equals(type);
    }

    public Boolean isSuperGroupChat() {
        return SUPER_GROUP_CHAT_TYPE.equals(type);
    }
}
