package com.chuang.telegram.bot.model;

import com.chuang.telegram.bot.interfaces.BotApiObject;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;


/**
 * @author Ruben Bermudez
 * @version 1.0
 * @brief This object contains information about one member of the chat.
 * @date 20 of May of 2016
 */
@Data
public class ChatMember implements BotApiObject {
    private static final String USER_FIELD = "user";
    private static final String STATUS_FIELD = "status";
    private static final String UNTIL_DATE_FIELD = "until_date";
    private static final String CAN_BE_EDITED_FIELD = "can_be_edited";
    private static final String CAN_CHANGE_INFORMATION_FIELD = "can_change_information";
    private static final String CAN_POST_MESSAGES_FIELD = "can_post_messages";
    private static final String CAN_EDIT_MESSAGES_FIELD = "can_edit_messages";
    private static final String CAN_DELETE_MESSAGES_FIELD = "can_delete_messages";
    private static final String CAN_INVITE_USERS_FIELD = "can_invite_users";
    private static final String CAN_RESTRICT_USERS_FIELD = "can_restrict_users";
    private static final String CAN_PIN_MESSAGES_FIELD = "can_pin_messages";
    private static final String CAN_PROMOTE_MEMBERS_FIELD = "can_promote_members";
    private static final String CAN_SEND_MESSAGES_FIELD = "can_send_messages";
    private static final String CAN_SEND_MEDIA_MESSAGES_FIELD = "can_send_media_messages";
    private static final String CAN_SEND_OTHER_MESSAGES_FIELD = "can_send_other_messages";
    private static final String CAN_ADD_WEB_PAGE_PREVIEWS_FIELD = "can_add_web_page_previews";

    /**  Information about the user */
    @JsonProperty(USER_FIELD)
    private User user;
    /**  The member's status in the chat. Can be “creator”, “administrator”, “member”,  “restricted”, “left” or “kicked” */
    @JsonProperty(STATUS_FIELD)
    private String status;
    /**  Optional. Resticted and kicked only. Date when restrictions will be lifted for this user, unix time */
    @JsonProperty(UNTIL_DATE_FIELD)
    private Long untilDate;
    /** < Optional. Administrators only. True, if the bot is allowed to edit administrator privileges of that user */
    @JsonProperty(CAN_BE_EDITED_FIELD)
    private Boolean canBeEdited;
    /** < Optional. Administrators only. True, if the administrator can change the chat title, photo and other settings */
    @JsonProperty(CAN_CHANGE_INFORMATION_FIELD)
    private Boolean canChangeInformation;
    /** < Optional. Administrators only. True, if the administrator can post in the channel, channels only */
    @JsonProperty(CAN_POST_MESSAGES_FIELD)
    private Boolean canPostMessages;
    /** < Optional. Administrators only. True, if the administrator can edit messages of other users and can pin messages, channels only */
    @JsonProperty(CAN_EDIT_MESSAGES_FIELD)
    private Boolean canEditMessages;
    /** < Optional. Administrators only. True, if the administrator can delete messages of other users */
    @JsonProperty(CAN_DELETE_MESSAGES_FIELD)
    private Boolean canDeleteMessages;
    /** < Optional. Administrators only. True, if the administrator can invite new users to the chat */
    @JsonProperty(CAN_INVITE_USERS_FIELD)
    private Boolean canInviteUsers;
    /** < Optional. Administrators only. True, if the administrator can restrict, ban or unban chat members */
    @JsonProperty(CAN_RESTRICT_USERS_FIELD)
    private Boolean canRestrictUsers;
    /** < Optional. Administrators only. True, if the administrator can pin messages */
    @JsonProperty(CAN_PIN_MESSAGES_FIELD)
    private Boolean canPinMessages;
    /** < Optional. Administrators only. True, if the administrator can add new administrators with a subset of his own privileges or demote administrators that it has promoted, directly or indirectly (promoted by administators that were appointed by the bot) */
    @JsonProperty(CAN_PROMOTE_MEMBERS_FIELD)
    private Boolean canPromoteMembers;
    /**< Optional. Restricted only. True, if the user can send text messages, contacts, locations and venues */
    @JsonProperty(CAN_SEND_MESSAGES_FIELD)
    private Boolean canSendMessages;
    /**< Optional. Restricted only. True, if the user can send audios, documents, photos, videos, video notes and voice notes, implies can_send_messages */
    @JsonProperty(CAN_SEND_MEDIA_MESSAGES_FIELD)
    private Boolean canSendMediaMessages;
    /**< Optional. Restricted only. True, if the user can send animations, games, stickers and use inline bots, implies can_send_media_messages */
    @JsonProperty(CAN_SEND_OTHER_MESSAGES_FIELD)
    private Boolean canSendOtherMessages;
    /**< Optional. Restricted only. True, if user may add web page previews Э to his messages, implies can_send_messages */
    @JsonProperty(CAN_ADD_WEB_PAGE_PREVIEWS_FIELD)
    private Boolean canAddWebPagePreviews;


}
