package com.chuang.telegram.bot.model;

import com.chuang.telegram.bot.interfaces.BotApiObject;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * @brief This object represents a general file (as opposed to photos and audio files).
 * Telegram users can send files of any type of up to 1.5 GB in size.
 * @date 20 of June of 2015
 */
@Data
public class Document implements BotApiObject {

    private static final String FILE_ID_FIELD = "file_id";
    private static final String THUMB_FIELD = "thumb";
    private static final String FILENAME_FIELD = "file_name";
    private static final String MIMETYPE_FIELD = "mime_type";
    private static final String FILESIZE_FIELD = "file_size";

    /** Unique identifier for this file */
    @JsonProperty(FILE_ID_FIELD)
    private String fileId;
    /** Document thumbnail as defined by sender */
    @JsonProperty(THUMB_FIELD)
    private PhotoSize thumb;
    /** Optional. Original filename as defined by sender */
    @JsonProperty(FILENAME_FIELD)
    private String fileName;
    /** Optional. Mime type of a file as defined by sender */
    @JsonProperty(MIMETYPE_FIELD)
    private String mimeType;
    /** Optional. File size */
    @JsonProperty(FILESIZE_FIELD)
    private Long fileSize;

}
