package com.chuang.telegram.bot.model;

import com.chuang.telegram.bot.interfaces.BotApiObject;
import com.chuang.telegram.bot.model.games.Animation;
import com.chuang.telegram.bot.model.games.Game;
import com.chuang.telegram.bot.model.passport.PassportData;
import com.chuang.telegram.bot.model.payment.Invoice;
import com.chuang.telegram.bot.model.payment.SuccessfulPayment;
import com.chuang.telegram.bot.model.stickers.Sticker;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;

import java.util.List;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * This object represents a message.
 */
@Data
public class Message implements BotApiObject {
    private static final String MESSAGE_ID_FIELD = "message_id";
    private static final String FROM_FIELD = "from";
    private static final String DATE_FIELD = "date";
    private static final String CHAT_FIELD = "chat";
    private static final String FORWARD_FROM_FIELD = "forward_from";
    private static final String FORWARD_FROM_CHAT_FIELD = "forward_from_chat";
    private static final String FORWARD_DATE_FIELD = "forward_date";
    private static final String TEXT_FIELD = "text";
    private static final String ENTITIES_FIELD = "entities";
    private static final String CAPTION_ENTITIES_FIELD = "caption_entities";
    private static final String AUDIO_FIELD = "audio";
    private static final String DOCUMENT_FIELD = "document";
    private static final String PHOTO_FIELD = "photo";
    private static final String STICKER_FIELD = "sticker";
    private static final String VIDEO_FIELD = "video";
    private static final String CONTACT_FIELD = "contact";
    private static final String LOCATION_FIELD = "location";
    private static final String VENUE_FIELD = "venue";
    private static final String ANIMATION_FIELD = "animation";
    private static final String PINNED_MESSAGE_FIELD = "pinned_message";
    private static final String NEW_CHAT_MEMBERS_FIELD = "new_chat_members";
    private static final String LEFT_CHAT_MEMBER_FIELD = "left_chat_member";
    private static final String NEW_CHAT_TITLE_FIELD = "new_chat_title";
    private static final String NEW_CHAT_PHOTO_FIELD = "new_chat_photo";
    private static final String DELETE_CHAT_PHOTO_FIELD = "delete_chat_photo";
    private static final String GROUP_CHAT_CREATED_FIELD = "group_chat_created";
    private static final String REPLY_TO_MESSAGE_FIELD = "reply_to_message";
    private static final String VOICE_FIELD = "voice";
    private static final String CAPTION_FIELD = "caption";
    private static final String SUPER_GROUP_CREATED_FIELD = "supergroup_chat_created";
    private static final String CHANNEL_CHAT_CREATED_FIELD = "channel_chat_created";
    private static final String MIGRATE_TO_CHAT_FIELD = "migrate_to_chat_id";
    private static final String MIGRATE_FROM_CHAT_FIELD = "migrate_from_chat_id";
    private static final String EDIT_DATE_FIELD = "edit_date";
    private static final String GAME_FIELD = "game";
    private static final String FORWARD_FROM_MESSAGE_ID_FIELD = "forward_from_message_id";
    private static final String INVOICE_FIELD = "invoice";
    private static final String SUCCESSFUL_PAYMENT_FIELD = "successful_payment";
    private static final String VIDEO_NOTE_FIELD = "video_note";
    private static final String AUTHOR_SIGNATURE_FIELD = "author_signature";
    private static final String FORWARD_SIGNATURE_FIELD = "forward_signature";
    private static final String MEDIA_GROUP_ID_FIELD = "media_group_id";
    private static final String CONNECTED_WEBSITE_FIELD = "connected_website";
    private static final String PASSPORT_DATA_FIELD = "passport_data";

    /** Long	Unique message identifier */
    @JsonProperty(MESSAGE_ID_FIELD)
    private Long messageId;
    /** Optional. Sender, can be empty for messages sent to channels */
    @JsonProperty(FROM_FIELD)
    private User from;
    /** Optional. Date the message was sent in Unix time */
    @JsonProperty(DATE_FIELD)
    private Long date;
    /** Conversation the message belongs to */
    @JsonProperty(CHAT_FIELD)
    private Chat chat;
    /** Optional. For forwarded messages, sender of the original message */
    @JsonProperty(FORWARD_FROM_FIELD)
    private User forwardFrom;
    /** Optional. For messages forwarded from a channel, information about the original channel */
    @JsonProperty(FORWARD_FROM_CHAT_FIELD)
    private Chat forwardFromChat;
    /** Optional. For forwarded messages, date the original message was sent */
    @JsonProperty(FORWARD_DATE_FIELD)
    private Long forwardDate;
    /** Optional. For text messages, the actual UTF-8 text of the message */
    @JsonProperty(TEXT_FIELD)
    private String text;
    /**
     * Optional. For text messages, special entities like usernames, URLs,
     * bot commands, etc. that appear in the text
     */
    @JsonProperty(ENTITIES_FIELD)
    private List<MessageEntity> entities;
    /**
     * Optional. For messages with a caption, special entities like usernames,
     * URLs, bot commands, etc. that appear in the caption
     */
    @JsonProperty(CAPTION_ENTITIES_FIELD)
    private List<MessageEntity> captionEntities;
    /** Optional. Message is an audio file, information about the file */
    @JsonProperty(AUDIO_FIELD)
    private Audio audio;
    /** Optional. Message is a general file, information about the file */
    @JsonProperty(DOCUMENT_FIELD)
    private Document document;
    /** Optional. Message is a photo, available sizes of the photo */
    @JsonProperty(PHOTO_FIELD)
    private List<PhotoSize> photo;
    /** Optional. Message is a sticker, information about the sticker */
    @JsonProperty(STICKER_FIELD)
    private Sticker sticker;
    /** Optional. Message is a video, information about the video */
    @JsonProperty(VIDEO_FIELD)
    private Video video;
    /** Optional. Message is a shared contact, information about the contact */
    @JsonProperty(CONTACT_FIELD)
    private Contact contact;
    /** Optional. Message is a shared location, information about the location */
    @JsonProperty(LOCATION_FIELD)
    private Location location;
    /** Optional. Message is a venue, information about the venue */
    @JsonProperty(VENUE_FIELD)
    private Venue venue;
    /**
     * Optional. Message is an animation, information about the animation.
     * For backward compatibility, when this field is set, the document field will be also set
     */
    @JsonProperty(ANIMATION_FIELD)
    private Animation animation;
    /** Optional. Specified message was pinned. Note that the Message object in this field will not contain further reply_to_message fields even if it is itself a reply. */
    @JsonProperty(PINNED_MESSAGE_FIELD)
    private Message pinnedMessage;
    /** Optional. New members were added to the group or supergroup, information about them (the bot itself may be one of these members) */
    @JsonProperty(NEW_CHAT_MEMBERS_FIELD)
    private List<User> newChatMembers;
    /** Optional. A member was removed from the group, information about them (this member may be bot itself) */
    @JsonProperty(LEFT_CHAT_MEMBER_FIELD)
    private User leftChatMember;
    /** Optional. A chat title was changed to this value */
    @JsonProperty(NEW_CHAT_TITLE_FIELD)
    private String newChatTitle;
    /** Optional. A chat photo was change to this value */
    @JsonProperty(NEW_CHAT_PHOTO_FIELD)
    private List<PhotoSize> newChatPhoto;
    /** Optional. Informs that the chat photo was deleted */
    @JsonProperty(DELETE_CHAT_PHOTO_FIELD)
    private Boolean deleteChatPhoto;
    /** Optional. Informs that the group has been created */
    @JsonProperty(GROUP_CHAT_CREATED_FIELD)
    private Boolean groupchatCreated;
    @JsonProperty(REPLY_TO_MESSAGE_FIELD)
    private Message replyToMessage;
    /** Optional. Message is a voice message, information about the file */
    @JsonProperty(VOICE_FIELD)
    private Voice voice;
    /** Optional. Caption for the document, photo or video, 0-200 characters */
    @JsonProperty(CAPTION_FIELD)
    private String caption;
    /**
     * Optional. Service message: the supergroup has been created.
     * This field can‘t be received in a message coming through updates,
     * because bot can’t be a member of a supergroup when it is created.
     * It can only be found in reply_to_message
     * if someone replies to a very first message in a directly created supergroup.
     */
    @JsonProperty(SUPER_GROUP_CREATED_FIELD)
    private Boolean superGroupCreated;
    /**
     * Optional. Service message: the channel has been created.
     * This field can‘t be received in a message coming through updates,
     * because bot can’t be a member of a channel when it is created.
     * It can only be found in reply_to_message if someone
     * replies to a very first message in a channel.
     */
    @JsonProperty(CHANNEL_CHAT_CREATED_FIELD)
    private Boolean channelChatCreated;

    /**
     * Optional. The group has been migrated to a supergroup with the specified identifier.
     * This number may be greater than 32 bits and some programming languages
     * may have difficulty/silent defects in interpreting it.
     * But it smaller than 52 bits, so a signed 64 bit Long or double-precision
     * float type are safe for storing this identifier.
     */
    @JsonProperty(MIGRATE_TO_CHAT_FIELD)
    private Long migrateToChatId;
    /**
     * Optional. The supergroup has been migrated from a group with the specified identifier.
     * This number may be greater than 32 bits and some programming languages
     * may have difficulty/silent defects in interpreting it.
     * But it smaller than 52 bits, so a signed 64 bit Long or double-precision
     * float type are safe for storing this identifier.
     */
    @JsonProperty(MIGRATE_FROM_CHAT_FIELD)
    private Long migrateFromChatId;
    /** Optional. Date the message was last edited in Unix time */
    @JsonProperty(EDIT_DATE_FIELD)
    private Long editDate;
    /** Optional. Message is a game, information about the game */
    @JsonProperty(GAME_FIELD)
    private Game game;
    /** Optional. For forwarded channel posts, identifier of the original message in the channel */
    @JsonProperty(FORWARD_FROM_MESSAGE_ID_FIELD)
    private Long forwardFromMessageId;
    /** Optional. Message is an invoice for a payment, information about the invoice. */
    @JsonProperty(INVOICE_FIELD)
    private Invoice invoice;
    /** Optional. Message is a service message about a successful payment, information about the payment. */
    @JsonProperty(SUCCESSFUL_PAYMENT_FIELD)
    private SuccessfulPayment successfulPayment;
    /** Optional. Message is a video note, information about the video message */
    @JsonProperty(VIDEO_NOTE_FIELD)
    private VideoNote videoNote;
    /** Optional. Post author signature for posts in channel chats */
    @JsonProperty(AUTHOR_SIGNATURE_FIELD)
    private String authorSignature;
    /** Optional. Post author signature for messages forwarded from channel chats */
    @JsonProperty(FORWARD_SIGNATURE_FIELD)
    private String forwardSignature;
    /** Optional. The unique identifier of a media message group this message belongs to */
    @JsonProperty(MEDIA_GROUP_ID_FIELD)
    private String mediaGroupId;
    /** Optional. The domain name of the website on which the user has logged in */
    @JsonProperty(CONNECTED_WEBSITE_FIELD)
    private String connectedWebsite;
    /** Optional. Telegram Passport data */
    @JsonProperty(PASSPORT_DATA_FIELD)
    private PassportData passportData;


    public List<MessageEntity> getEntities() {
        if (entities != null) {
            entities.forEach(x -> x.computeText(text));
        }
        return entities;
    }

    public List<MessageEntity> getCaptionEntities() {
        if (captionEntities != null) {
            captionEntities.forEach(x -> x.computeText(caption));
        }
        return captionEntities;
    }


    public boolean hasSticker() {
        return sticker != null;
    }

    public boolean isGroupMessage() {
        return chat.isGroupChat();
    }

    public boolean isUserMessage() {
        return chat.isUserChat();
    }

    public boolean isChannelMessage() {
        return chat.isChannelChat();
    }

    public boolean isSuperGroupMessage() {
        return chat.isSuperGroupChat();
    }

    public Long getChatId() {
        return chat.getId();
    }

    public boolean hasText() {
        return text != null && !text.isEmpty();
    }

    public boolean isCommand() {
        if (hasText() && entities != null) {
            for (MessageEntity entity : entities) {
                if (entity != null && entity.getOffset() == 0 &&
                        EntityType.BOTCOMMAND.equals(entity.getType())) {
                    return true;
                }
            }
        }
        return false;
    }

    public boolean hasDocument() {
        return this.document != null;
    }

    public boolean hasVideo() {
        return this.video != null;
    }

    public boolean isReply() {
        return this.replyToMessage != null;
    }

    public boolean hasLocation() {
        return location != null;
    }

    private boolean hasGame() {
        return game != null;
    }

    public boolean hasEntities() {
        return entities != null && !entities.isEmpty();
    }

    public boolean hasPhoto() {
        return photo != null && !photo.isEmpty();
    }

    public boolean hasInvoice() {
        return invoice != null;
    }

    public boolean hasSuccessfulPayment() {
        return successfulPayment != null;
    }

    public boolean hasContact() {
        return contact != null;
    }


    public boolean hasVideoNote() {
        return videoNote != null;
    }

    public boolean hasPassportData() {
        return passportData != null;
    }

    public boolean hasAnimation() {
        return animation != null;
    }

}
