package com.chuang.telegram.bot.model;

import com.chuang.telegram.bot.interfaces.BotApiObject;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * This object represents one special entity in a text message. For example, hashtags,
 * usernames, URL.
 */
@Data
public class MessageEntity implements BotApiObject {
    private static final String TYPE_FIELD = "type";
    private static final String OFFSET_FIELD = "offset";
    private static final String LENGTH_FIELD = "length";
    private static final String URL_FIELD = "url";
    private static final String USER_FIELD = "user";
    /**
     * Type of the entity. One of
     * mention (@username),
     * hashtag,
     * cashtag
     * bot_command,
     * url,
     * email,
     * phone_number,
     * bold (bold text),
     * italic (italic text),
     * code (monowidth string),
     * pre (monowidth block),
     * text_link (for clickable text URLs)
     * text_mention (for users without usernames)
     */
    @JsonProperty(TYPE_FIELD)
    private String type;
    /** Length of the entity in UTF-16 code units */
    @JsonProperty(OFFSET_FIELD)
    private Integer offset;
    /** Optional. For “text_link” only, url that will be opened after user taps on the text */
    @JsonProperty(LENGTH_FIELD)
    private Integer length;
    /** Optional. For “text_mention” only, the mentioned user */
    @JsonProperty(URL_FIELD)
    private String url;
    /** Text present in the entity. Computed from offset and length */
    @JsonProperty(USER_FIELD)
    private User user;
    @JsonIgnore
    private String text;


    protected void computeText(String message) {
        if (message != null) {
            text = message.substring(offset, offset + length);
        }
    }
}
