package com.chuang.telegram.bot.model;

import com.chuang.telegram.bot.interfaces.BotApiObject;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * @brief Contains information about why a request was unsuccessfull.
 * @date 28 of September of 2016
 */
@Data
public class ResponseParameters implements BotApiObject {
    private static final String MIGRATE_TO_CHAT_ID_FIELD = "migrate_to_chat_id";
    private static final String RETRY_AFTER_FIELD = "retry_after";

    /**
     * Optional. The group has been migrated to a supergroup with the specified identifier.
     * This number may be greater than 32 bits and some programming languages may have
     * difficulty/silent defects in interpreting it. But it is smaller than 52 bits,
     * so a signed 64 bit Long or double-precision float type are safe for storing this identifier.
     */
    @JsonProperty(MIGRATE_TO_CHAT_ID_FIELD)
    private Long migrateToChatId;
    /**
     * Optional. In case of exceeding flood control a number of seconds to
     * wait before the request can be repeated
     */
    @JsonProperty(RETRY_AFTER_FIELD)
    private Long retryAfter;

}
