package com.chuang.telegram.bot.model;

import com.chuang.telegram.bot.interfaces.BotApiObject;
import com.chuang.telegram.bot.model.inlinequery.ChosenInlineQuery;
import com.chuang.telegram.bot.model.inlinequery.InlineQuery;
import com.chuang.telegram.bot.model.payment.PreCheckoutQuery;
import com.chuang.telegram.bot.model.payment.ShippingQuery;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;

/**
 * @author Ruben Bermudez
 * @version 1.0
 *
 * This object represents an incoming update.
 *
 * @apiNote Only one of the optional parameters can be present in any given update.
 */
@Data
public class Update implements BotApiObject {
    private static final String UPDATE_ID_FIELD = "update_id";
    private static final String MESSAGE_FIELD = "message";
    private static final String INLINE_QUERY_FIELD = "inline_query";
    private static final String CHOSEN_INLINE_QUERY_FIELD = "chosen_inline_result";
    private static final String CALLBACK_QUERY_FIELD = "callback_query";
    private static final String EDITED_MESSAGE_FIELD = "edited_message";
    private static final String CHANNEL_POST_FIELD = "channel_post";
    private static final String EDITED_CHANNEL_POST_FIELD = "edited_channel_post";
    private static final String SHIPPING_QUERY_FIELD = "shipping_query";
    private static final String PRE_CHECKOUT_QUERY_FIELD = "pre_checkout_query";

    @JsonProperty(UPDATE_ID_FIELD)
    private Long updateId;
    /**  Optional. New incoming message of any kind — text, photo, sticker, etc. */
    @JsonProperty(MESSAGE_FIELD)
    private Message message;
    /**  Optional. New incoming inline query */
    @JsonProperty(INLINE_QUERY_FIELD)
    private InlineQuery inlineQuery;
    /**  Optional. The result of a inline query that was chosen by a user and sent to their chat partner */
    @JsonProperty(CHOSEN_INLINE_QUERY_FIELD)
    private ChosenInlineQuery chosenInlineQuery;
    /**  Optional. New incoming callback query */
    @JsonProperty(CALLBACK_QUERY_FIELD)
    private CallbackQuery callbackQuery;
    /**  Optional. New version of a message that is known to the bot and was edited */
    @JsonProperty(EDITED_MESSAGE_FIELD)
    private Message editedMessage;
    /**  Optional. New incoming channel post of any kind — text, photo, sticker, etc. */
    @JsonProperty(CHANNEL_POST_FIELD)
    private Message channelPost;
    /**  Optional. New version of a channel post that is known to the bot and was edited */
    @JsonProperty(EDITED_CHANNEL_POST_FIELD)
    private Message editedChannelPost;
    /**  Optional. New incoming shipping query. Only for invoices with flexible price */
    @JsonProperty(SHIPPING_QUERY_FIELD)
    private ShippingQuery shippingQuery;
    /**  Optional. New incoming pre-checkout query. Contains full information about checkout */
    @JsonProperty(PRE_CHECKOUT_QUERY_FIELD)
    private PreCheckoutQuery preCheckoutQuery;

    public boolean hasMessage() {
        return message != null;
    }

    public boolean hasInlineQuery() {
        return inlineQuery != null;
    }

    public boolean hasChosenInlineQuery() {
        return chosenInlineQuery != null;
    }

    public boolean hasCallbackQuery() {
        return callbackQuery != null;
    }

    public boolean hasEditedMessage() {
        return editedMessage != null;
    }

    public boolean hasChannelPost() {
        return channelPost != null;
    }

    public boolean hasEditedChannelPost() {
        return editedChannelPost != null;
    }

    public boolean hasShippingQuery() {
        return shippingQuery != null;
    }

    public boolean hasPreCheckoutQuery() {
        return preCheckoutQuery != null;
    }

}
