package com.chuang.telegram.bot.model.games;

import com.chuang.telegram.bot.interfaces.BotApiObject;
import com.chuang.telegram.bot.model.PhotoSize;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;

/**
 * @author Ruben Bermudez
 * @version 2.4
 * This object represents an animation file (GIF or H.264/MPEG-4 AVC video without sound).
 */
@Data
public class Animation implements BotApiObject {
    private static final String FILE_ID_FIELD = "file_id";
    private static final String WIDTH_FIELD = "width";
    private static final String HEIGHT_FIELD = "height";
    private static final String DURATION_FIELD = "duration";
    private static final String THUMB_FIELD = "thumb";
    private static final String FILENAME_FIELD = "file_name";
    private static final String MIMETYPE_FIELD = "mime_type";
    private static final String FILESIZE_FIELD = "file_size";

    /** Unique file identifier */
    @JsonProperty(FILE_ID_FIELD)
    private String fileId;
    /** Video width as defined by sender */
    @JsonProperty(WIDTH_FIELD)
    private Integer width;
    /** Video height as defined by sender */
    @JsonProperty(HEIGHT_FIELD)
    private Integer height;
    /** Duration of the video in seconds as defined by sender */
    @JsonProperty(DURATION_FIELD)
    private Long duration;
    /** Optional. Animation thumbnail as defined by sender */
    @JsonProperty(THUMB_FIELD)
    private PhotoSize thumb;
    /** Optional. Original animation filename as defined by sender */
    @JsonProperty(FILENAME_FIELD)
    private String fileName;
    /** Optional. MIME type of the file as defined by sender */
    @JsonProperty(MIMETYPE_FIELD)
    private String mimetype;
    /** Optional. File size */
    @JsonProperty(FILESIZE_FIELD)
    private Long fileSize;

}
