package com.chuang.telegram.bot.model.games;

import com.chuang.telegram.bot.interfaces.BotApiObject;
import com.chuang.telegram.bot.model.MessageEntity;
import com.chuang.telegram.bot.model.PhotoSize;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;

import java.util.List;

/**
 * @author Ruben Bermudez
 * @version 2.4
 * @brief This object represents a game.
 * Use BotFather to create and edit games, their short names will act as unique identifiers.
 * @date 27 of September of 2016
 */
@Data
public class Game implements BotApiObject {

    private static final String TITLE_FIELD = "title";
    private static final String DESCRIPTION_FIELD = "description";
    private static final String PHOTO_FIELD = "photo";
    private static final String ANIMATION_FIELD = "animation";
    private static final String TEXT_FIELD = "text";
    private static final String TEXT_ENTITIES_FIELD = "text_entities";

    /** Title of the game */
    @JsonProperty(TITLE_FIELD)
    private String title;
    /** Description of the game */
    @JsonProperty(DESCRIPTION_FIELD)
    private String description;
    /** Photo */
    @JsonProperty(PHOTO_FIELD)
    private List<PhotoSize> photo;
    /**
     * Optional. Brief description of the game or high scores included in the game message.
     * Can be automatically edited to include current high scores for the game
     * when the bot calls setGameScore, or manually edited using editMessageText.
     * 0-4096 characters.
     */
    @JsonProperty(TEXT_FIELD)
    private String text;
    /**
     * Optional. Special entities that appear in text, such as usernames,
     * URLs, bot commands, etc.
     */
    @JsonProperty(TEXT_ENTITIES_FIELD)
    private List<MessageEntity> entities;
    /** Optional. Animation */
    @JsonProperty(ANIMATION_FIELD)
    private Animation animation;


    public boolean hasEntities() {
        return entities != null && !entities.isEmpty();
    }

}
