package com.chuang.telegram.bot.model.hook;

import com.chuang.telegram.bot.interfaces.BotApiObject;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;


import java.util.List;

/**
 * @author Ruben Bermudez
 * @version 2.4
 * @brief Contains information about the current status of a webhook.
 * @date 12 of August of 2016
 */
@Data
public class WebhookInfo implements BotApiObject {

    private static final String URL_FIELD = "url";
    private static final String HAS_CUSTOM_CERTIFICATE_FIELD = "has_custom_certificate";
    private static final String PENDING_UPDATE_COUNT_FIELD = "pending_update_count";
    private static final String MAX_CONNECTIONS_FIELD = "max_connections";
    private static final String ALLOWED_UPDATES_FIELD = "allowed_updates";
    private static final String LAST_ERROR_DATE_FIELD = "last_error_date";
    private static final String LAST_ERROR_MESSAGE_FIELD = "last_error_message";

    /** Webhook URL, may be empty if webhook is not set up */
    @JsonProperty(URL_FIELD)
    private String url;
    /** True, if a custom certificate was provided for webhook certificate checks */
    @JsonProperty(HAS_CUSTOM_CERTIFICATE_FIELD)
    private Boolean hasCustomCertificate;
    /** Number updates awaiting delivery */
    @JsonProperty(PENDING_UPDATE_COUNT_FIELD)
    private Long pendingUpdatesCount;
    /** Optional. Unix time for the most recent error that happened when trying to deliver an update via webhook */
    @JsonProperty(LAST_ERROR_DATE_FIELD)
    private Long lastErrorDate;
    /**  Optional. Error message in human-readable format for the most recent error that happened when trying to deliver an update via webhook */
    @JsonProperty(LAST_ERROR_MESSAGE_FIELD)
    private String lastErrorMessage;
    /** Optional. Maximum allowed number of simultaneous HTTPS connections to the webhook for update delivery */
    @JsonProperty(MAX_CONNECTIONS_FIELD)
    private Long maxConnections;
    /** Optional. A list of update types the bot is subscribed to. Defaults to all update types */
    @JsonProperty(ALLOWED_UPDATES_FIELD)
    private List<String> allowedUpdates;

}
