package com.chuang.telegram.bot.model.inlinequery;

import com.chuang.telegram.bot.interfaces.BotApiObject;
import com.chuang.telegram.bot.model.Location;
import com.chuang.telegram.bot.model.User;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * @brief Represents a result of an inline query that was chosen by the user and sent to their chat
 * partner.
 * @date 01 of January of 2016
 */
@Data
public class ChosenInlineQuery implements BotApiObject {
    private static final String RESULT_ID_FIELD = "result_id";
    private static final String FROM_FIELD = "from";
    private static final String LOCATION_FIELD = "location";
    private static final String INLINE_MESSAGE_ID_FIELD = "inline_message_id";
    private static final String QUERY_FIELD = "query";

    /** The unique identifier for the result that was chosen. */
    @JsonProperty(RESULT_ID_FIELD)
    private String resultId;
    /** The user that chose the result. */
    @JsonProperty(FROM_FIELD)
    private User from;
    /** Optional. Sender location, only for bots that require user location */
    @JsonProperty(LOCATION_FIELD)
    private Location location;
    /**
     * Optional.
     * Identifier of the sent inline message.
     * Available only if there is an inline keyboard attached to the message.
     * Will be also received in callback queries and can be used to edit the message.
     */
    @JsonProperty(INLINE_MESSAGE_ID_FIELD)
    private String inlineMessageId;
    /** The query that was used to obtain the result. */
    @JsonProperty(QUERY_FIELD)
    private String query;

}
