package com.chuang.telegram.bot.model.inlinequery;

import com.chuang.telegram.bot.interfaces.BotApiObject;
import com.chuang.telegram.bot.model.Location;
import com.chuang.telegram.bot.model.User;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * @brief This object represents an incoming inline query. When the user sends an empty query, your
 * bot could return some default or trending results.
 * @date 01 of January of 2016
 */
@Data
public class InlineQuery implements BotApiObject {
    private static final String ID_FIELD = "id";
    private static final String FROM_FIELD = "from";
    private static final String LOCATION_FIELD = "location";
    private static final String QUERY_FIELD = "query";
    private static final String OFFSET_FIELD = "offset";

    /** Unique identifier for this query */
    @JsonProperty(ID_FIELD)
    private String id;
    /** Sender */
    @JsonProperty(FROM_FIELD)
    private User from;
    /** Optional. Sender location, only for bots that request user location */
    @JsonProperty(LOCATION_FIELD)
    private Location location;
    /** Text of the query */
    @JsonProperty(QUERY_FIELD)
    private String query;
    /** Offset of the results to be returned, can be controlled by the bot */
    @JsonProperty(OFFSET_FIELD)
    private String offset;

}
