package com.chuang.telegram.bot.model.inlinequery.inputmessagecontent;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * Represents the content of a contact message to be sent as the result of an inline query
 * @apiNote  This will only work in Telegram versions released after 9 April, 2016. Older clients will
 * ignore them.
 */
@Data
@Accessors(chain = true)
public class InputContactMessageContent implements InputMessageContent {

    private static final String PHONE_NUMBER_FIELD = "phone_number";
    private static final String FIRST_NAME_FIELD = "first_name";
    private static final String LAST_NAME_FIELD = "last_name";
    private static final String VCARD_FIELD = "vcard";

    /** Contact's phone number */
    @JsonProperty(PHONE_NUMBER_FIELD)
    private String phoneNumber;
    /** Contact's first name */
    @JsonProperty(FIRST_NAME_FIELD)
    private String firstName;
    /** Optional. Contact's last name */
    @JsonProperty(LAST_NAME_FIELD)
    private String lastName;
    /** Optional. Additional data about the contact in the form of a vCard, 0-2048 bytes */
    @JsonProperty(VCARD_FIELD)
    private String vCard;

    @Override
    public void validate() throws TelegramApiValidationException {
        if (phoneNumber == null || phoneNumber.isEmpty()) {
            throw new TelegramApiValidationException("PhoneNumber parameter can't be empty", this);
        }
        if (firstName == null || firstName.isEmpty()) {
            throw new TelegramApiValidationException("FirstName parameter can't be empty", this);
        }
    }

}
