
package com.chuang.telegram.bot.model.inlinequery.inputmessagecontent;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * Represents the content of a location message to be sent as the result of an inline query.
 * @apiNote  This will only work in Telegram versions released after 9 April, 2016. Older clients will
 * ignore them.
 */
@Data
@Accessors(chain = true)
public class InputLocationMessageContent implements InputMessageContent {

    private static final String LATITUDE_FIELD = "latitude";
    private static final String LONGITUDE_FIELD = "longitude";
    private static final String LIVE_PERIOD_FIELD = "live_period";

    /** Latitude of the location in degrees */
    @JsonProperty(LATITUDE_FIELD)
    private Float latitude;
    /** Longitude of the location in degrees */
    @JsonProperty(LONGITUDE_FIELD)
    private Float longitude;
    /** Optional. Period in seconds for which the location can be updated, should be between 60 and 86400. */
    @JsonProperty(LIVE_PERIOD_FIELD)
    private Long livePeriod;

    public InputLocationMessageContent(Float latitude, Float longitude) {
        super();
        this.latitude = checkNotNull(latitude);
        this.longitude = checkNotNull(longitude);
    }

    @Override
    public void validate() throws TelegramApiValidationException {
        if (latitude == null) {
            throw new TelegramApiValidationException("Latitude parameter can't be empty", this);
        }
        if (longitude == null) {
            throw new TelegramApiValidationException("Longitude parameter can't be empty", this);
        }
        if (livePeriod != null && (livePeriod < 60 || livePeriod > 86400)) {
            throw new TelegramApiValidationException("Live period parameter must be between 60 and 86400", this);
        }
    }

}
