package com.chuang.telegram.bot.model.inlinequery.inputmessagecontent;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.enums.ParseMode;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * @brief Represents the content of a text message to be sent as the result of an inline query.
 * @date 10 of April of 2016
 */
@Data
@Accessors(chain = true)
public class InputTextMessageContent implements InputMessageContent {

    private static final String MESSAGE_TEXT_FIELD = "message_text";
    private static final String PARSE_MODE_FIELD = "parse_mode";
    private static final String DISABLE_WEB_PAGE_PREVIEW_FIELD = "disable_web_page_preview";

    /** Text of a message to be sent, 1-4096 characters */
    @JsonProperty(MESSAGE_TEXT_FIELD)
    private String messageText;
    /** Optional. Send Markdown or HTML, if you want Telegram apps to show bold, italic, fixed-width text or inline URLs in your bot's message. */
    @JsonProperty(PARSE_MODE_FIELD)
    private String parseMode;
    /** Optional. Disables link previews for links in the sent message */
    @JsonProperty(DISABLE_WEB_PAGE_PREVIEW_FIELD)
    private Boolean disableWebPagePreview;


    public InputTextMessageContent enableMarkdown(boolean enable) {
        if (enable) {
            this.parseMode = ParseMode.MARKDOWN;
        } else {
            this.parseMode = null;
        }
        return this;
    }

    public InputTextMessageContent enableHtml(boolean enable) {
        if (enable) {
            this.parseMode = ParseMode.HTML;
        } else {
            this.parseMode = null;
        }
        return this;
    }

    @Override
    public void validate() throws TelegramApiValidationException {
        if (messageText == null || messageText.isEmpty()) {
            throw new TelegramApiValidationException("MessageText parameter can't be empty", this);
        }
    }

}
