package com.chuang.telegram.bot.model.inlinequery.inputmessagecontent;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * Represents the content of a venue message to be sent as the result of an inline query.
 * @note This will only work in Telegram versions released after 9 April, 2016. Older clients will
 * ignore them.
 */
@Data
@Accessors(chain = true)
public class InputVenueMessageContent implements InputMessageContent {

    private static final String LATITUDE_FIELD = "latitude";
    private static final String LONGITUDE_FIELD = "longitude";
    private static final String TITLE_FIELD = "title";
    private static final String ADDRESS_FIELD = "address";
    private static final String FOURSQUARE_ID_FIELD = "foursquare_id";
    private static final String FOURSQUARE_TYPE_FIELD = "foursquare_type";

    /** Latitude of the venue in degrees */
    @JsonProperty(LATITUDE_FIELD)
    private Float latitude;
    /** Longitude of the venue in degrees */
    @JsonProperty(LONGITUDE_FIELD)
    private Float longitude;
    /** Name of the venue */
    @JsonProperty(TITLE_FIELD)
    private String title;
    /** Address of the venue */
    @JsonProperty(ADDRESS_FIELD)
    private String address;
    /** Optional. Foursquare identifier of the venue, if known */
    @JsonProperty(FOURSQUARE_ID_FIELD)
    private String foursquareId;
    /** Optional. Foursquare type of the venue, if known. */
    @JsonProperty(FOURSQUARE_TYPE_FIELD)
    private String foursquareType;


    @Override
    public void validate() throws TelegramApiValidationException {
        if (latitude == null) {
            throw new TelegramApiValidationException("Latitude parameter can't be empty", this);
        }
        if (longitude == null) {
            throw new TelegramApiValidationException("Longitude parameter can't be empty", this);
        }
        if (title == null || title.isEmpty()) {
            throw new TelegramApiValidationException("Title parameter can't be empty", this);
        }
        if (address == null || address.isEmpty()) {
            throw new TelegramApiValidationException("Address parameter can't be empty", this);
        }
    }
}
