package com.chuang.telegram.bot.model.inlinequery.result;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.model.inlinequery.inputmessagecontent.InputMessageContent;
import com.chuang.telegram.bot.model.replaykeyboard.InlineKeyboardMarkup;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * @brief Represents a link to an article or web page.
 * @date 01 of January of 2016
 */
@Data
@Accessors(chain = true)
public class InlineQueryResultArticle implements InlineQueryResult {

    private static final String TYPE_FIELD = "type";
    private static final String ID_FIELD = "id";
    private static final String TITLE_FIELD = "title";
    private static final String INPUT_MESSAGE_CONTENT_FIELD = "input_message_content";
    private static final String REPLY_MARKUP_FIELD = "reply_markup";
    private static final String URL_FIELD = "url";
    private static final String HIDE_URL_FIELD = "hide_url";
    private static final String DESCRIPTION_FIELD = "description";
    private static final String THUMB_URL_FIELD = "thumb_url";
    private static final String THUMB_WIDTH_FIELD = "thumb_width";
    private static final String THUMB_HEIGHT_FIELD = "thumb_height";

    /** Type of the result, must be “article” */
    @JsonProperty(TYPE_FIELD)
    private final String type = "article";
    /** Unique identifier of this result, 1-64 bytes */
    @JsonProperty(ID_FIELD)
    private String id;
    /** Title of the result */
    @JsonProperty(TITLE_FIELD)
    private String title;
    /** Content of the message to be sent */
    @JsonProperty(INPUT_MESSAGE_CONTENT_FIELD)
    private InputMessageContent inputMessageContent;
    /** Optional. Inline keyboard attached to the message */
    @JsonProperty(REPLY_MARKUP_FIELD)
    private InlineKeyboardMarkup replyMarkup;
    /** Optional. URL of the result */
    @JsonProperty(URL_FIELD)
    private String url;
    /** Optional. Pass True, if you don't want the URL to be shown in the message */
    @JsonProperty(HIDE_URL_FIELD)
    private Boolean hideUrl;
    /** Optional. Short description of the result */
    @JsonProperty(DESCRIPTION_FIELD)
    private String description;
    /** Optional. Url of the thumbnail for the result */
    @JsonProperty(THUMB_URL_FIELD)
    private String thumbUrl;
    /** Optional. Thumbnail width */
    @JsonProperty(THUMB_WIDTH_FIELD)
    private Integer thumbWidth;
    /** Optional. Thumbnail height */
    @JsonProperty(THUMB_HEIGHT_FIELD)
    private Integer thumbHeight;


    @Override
    public void validate() throws TelegramApiValidationException {
        if (id == null || id.isEmpty()) {
            throw new TelegramApiValidationException("ID parameter can't be empty", this);
        }
        if (title == null || title.isEmpty()) {
            throw new TelegramApiValidationException("Title parameter can't be empty", this);
        }
        if (inputMessageContent == null) {
            throw new TelegramApiValidationException("InputMessageContent parameter can't be null", this);
        }
        inputMessageContent.validate();
        if (replyMarkup != null) {
            replyMarkup.validate();
        }
    }
}
