package com.chuang.telegram.bot.model.inlinequery.result;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.model.inlinequery.inputmessagecontent.InputMessageContent;
import com.chuang.telegram.bot.model.replaykeyboard.InlineKeyboardMarkup;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * Represents a contact with a phone number. By default, this contact will be sent by the
 * user. Alternatively, you can use input_message_content to send a message with the specified
 * content instead of the contact.
 * @apiNote  This will only work in Telegram versions released after 9 April, 2016. Older clients will
 * ignore them.
 */
@Data
@Accessors(chain = true)
public class InlineQueryResultContact implements InlineQueryResult {

    private static final String TYPE_FIELD = "type";
    private static final String ID_FIELD = "id";
    private static final String PHONE_NUMBER_FIELD = "phone_number";
    private static final String FIRST_NAME_FIELD = "first_name";
    private static final String LAST_NAME_FIELD = "last_name";
    private static final String REPLY_MARKUP_FIELD = "reply_markup";
    private static final String INPUT_MESSAGE_CONTENT_FIELD = "input_message_content";
    private static final String THUMB_URL_FIELD = "thumb_url";
    private static final String THUMB_WIDTH_FIELD = "thumb_width";
    private static final String THUMB_HEIGHT_FIELD = "thumb_height";
    private static final String VCARD_FIELD = "vcard";

    /** Type of the result, must be "contact" */
    @JsonProperty(TYPE_FIELD)
    private final String type = "contact";
    /** Unique identifier of this result, 1-64 bytes */
    @JsonProperty(ID_FIELD)
    private String id;
    /** Contact's phone number */
    @JsonProperty(PHONE_NUMBER_FIELD)
    private String phoneNumber;
    /** Contact's first name */
    @JsonProperty(FIRST_NAME_FIELD)
    private String firstName;
    /** Contact's last name */
    @JsonProperty(LAST_NAME_FIELD)
    private String lastName;
    /** Optional. Inline keyboard attached to the message */
    @JsonProperty(REPLY_MARKUP_FIELD)
    private InlineKeyboardMarkup replyMarkup;
    /** Optional. Content of the message to be sent */
    @JsonProperty(INPUT_MESSAGE_CONTENT_FIELD)
    private InputMessageContent inputMessageContent;
    /** Optional. URL of the thumbnail (jpeg only) for the file */
    @JsonProperty(THUMB_URL_FIELD)
    private String thumbUrl;
    /** Optional. Thumbnail width */
    @JsonProperty(THUMB_WIDTH_FIELD)
    private Integer thumbWidth;
    /** Optional. Thumbnail height */
    @JsonProperty(THUMB_HEIGHT_FIELD)
    private Integer thumbHeight;
    /** Optional. Additional data about the contact in the form of a vCard, 0-2048 bytes */
    @JsonProperty(VCARD_FIELD)
    private String vCard;

    @Override
    public void validate() throws TelegramApiValidationException {
        if (id == null || id.isEmpty()) {
            throw new TelegramApiValidationException("ID parameter can't be empty", this);
        }
        if (phoneNumber == null || phoneNumber.isEmpty()) {
            throw new TelegramApiValidationException("PhoneNumber parameter can't be empty", this);
        }
        if (firstName == null || firstName.isEmpty()) {
            throw new TelegramApiValidationException("FirstName parameter can't be empty", this);
        }
        if (inputMessageContent != null) {
            inputMessageContent.validate();
        }
        if (replyMarkup != null) {
            replyMarkup.validate();
        }
    }

}
