package com.chuang.telegram.bot.model.inlinequery.result;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.model.inlinequery.inputmessagecontent.InputMessageContent;
import com.chuang.telegram.bot.model.replaykeyboard.InlineKeyboardMarkup;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;


/**
 * @author Ruben Bermudez
 * @version 1.0
 * Represents a link to an animated GIF file. By default, this animated GIF file will be sent
 * by the user with optional caption. Alternatively, you can use input_message_content to send a
 * message with the specified content instead of the animation.
 */
@Data
@Accessors(chain = true)
public class InlineQueryResultGif implements InlineQueryResult {
    private static final String TYPE_FIELD = "type";
    private static final String ID_FIELD = "id";
    private static final String GIF_URL_FIELD = "gif_url";
    private static final String GIF_WIDTH_FIELD = "gif_width";
    private static final String GIF_HEIGHT_FIELD = "gif_height";
    private static final String THUMB_URL_FIELD = "thumb_url";
    private static final String TITLE_FIELD = "title";
    private static final String CAPTION_FIELD = "caption";
    private static final String INPUT_MESSAGE_CONTENT_FIELD = "input_message_content";
    private static final String REPLY_MARKUP_FIELD = "reply_markup";
    private static final String GIF_DURATION_FIELD = "gif_duration";
    private static final String PARSE_MODE_FIELD = "parse_mode";

    /** Type of the result, must be "gif" */
    @JsonProperty(TYPE_FIELD)
    private final String type = "gif";
    /** Unique identifier of this result, 1-64 bytes */
    @JsonProperty(ID_FIELD)
    private String id;
    /** A valid URL for the GIF file. File size must not exceed 1MB */
    @JsonProperty(GIF_URL_FIELD)
    private String gifUrl;
    /** Optional. Width of the GIF */
    @JsonProperty(GIF_WIDTH_FIELD)
    private Integer gifWidth;
    /** Optional. Height of the GIF */
    @JsonProperty(GIF_HEIGHT_FIELD)
    private Integer gifHeight;
    /** Optional. URL of a static thumbnail for the result (jpeg or gif) */
    @JsonProperty(THUMB_URL_FIELD)
    private String thumbUrl;
    /** Optional. Title for the result */
    @JsonProperty(TITLE_FIELD)
    private String title;
    /** Optional. Caption of the GIF file to be sent */
    @JsonProperty(CAPTION_FIELD)
    private String caption;
    /** Optional. Content of the message to be sent instead of the GIF animation */
    @JsonProperty(INPUT_MESSAGE_CONTENT_FIELD)
    private InputMessageContent inputMessageContent;
    /** Optional. Inline keyboard attached to the message */
    @JsonProperty(REPLY_MARKUP_FIELD)
    private InlineKeyboardMarkup replyMarkup;
    /** Optional. Duration of the GIF */
    @JsonProperty(GIF_DURATION_FIELD)
    private Long gifDuration;
    /** Send Markdown or HTML, if you want Telegram apps to show bold, italic, fixed-width text or inline URLs in the media caption. */
    @JsonProperty(PARSE_MODE_FIELD)
    private String parseMode;

    @Override
    public void validate() throws TelegramApiValidationException {
        if (id == null || id.isEmpty()) {
            throw new TelegramApiValidationException("ID parameter can't be empty", this);
        }
        if (gifUrl == null || gifUrl.isEmpty()) {
            throw new TelegramApiValidationException("GifUrl parameter can't be empty", this);
        }
        if (inputMessageContent != null) {
            inputMessageContent.validate();
        }
        if (replyMarkup != null) {
            replyMarkup.validate();
        }
    }

}
