
package com.chuang.telegram.bot.model.inlinequery.result;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.model.inlinequery.inputmessagecontent.InputMessageContent;
import com.chuang.telegram.bot.model.replaykeyboard.InlineKeyboardMarkup;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * @brief Represents a link to a video animation (H.264/MPEG-4 AVC video without sound). By default,
 * this animated MPEG-4 file will be sent by the user with optional caption. Alternatively, you can
 * use input_message_content to send a message with the specified content instead of the animation.
 * @date 01 of January of 2016
 */
@Data
@Accessors(chain = true)
public class InlineQueryResultMpeg4Gif implements InlineQueryResult {

    private static final String TYPE_FIELD = "type";
    private static final String ID_FIELD = "id";
    private static final String MPEG4URL_FIELD = "mpeg4_url";
    private static final String MPEG4WIDTH_FIELD = "mpeg4_width";
    private static final String MPEG4HEIGHT_FIELD = "mpeg4_height";
    private static final String THUMB_URL_FIELD = "thumb_url";
    private static final String TITLE_FIELD = "title";
    private static final String CAPTION_FIELD = "caption";
    private static final String INPUT_MESSAGE_CONTENT_FIELD = "input_message_content";
    private static final String REPLY_MARKUP_FIELD = "reply_markup";
    private static final String MPEG4_DURATION_FIELD = "mpeg4_duration";
    private static final String PARSE_MODE_FIELD = "parse_mode";

    /** Type of the result, must be "mpeg4_gif" */
    @JsonProperty(TYPE_FIELD)
    private final String type = "mpeg4_gif";
    /** Unique identifier of this result, 1-64 bytes */
    @JsonProperty(ID_FIELD)
    private String id;
    /** A valid URL for the MP4 file. File size must not exceed 1MB */
    @JsonProperty(MPEG4URL_FIELD)
    private String mpeg4Url;
    /** Optional. Video width */
    @JsonProperty(MPEG4WIDTH_FIELD)
    private Integer mpeg4Width;
    /** Optional. Video height */
    @JsonProperty(MPEG4HEIGHT_FIELD)
    private Integer mpeg4Height;
    /** Optional. URL of the static thumbnail (jpeg or gif) for the result */
    @JsonProperty(THUMB_URL_FIELD)
    private String thumbUrl;
    /** Optional. Title for the result */
    @JsonProperty(TITLE_FIELD)
    private String title;
    /** Optional. Caption of the MPEG-4 file to be sent */
    @JsonProperty(CAPTION_FIELD)
    private String caption;
    /** Optional. Content of the message to be sent instead of the photo */
    @JsonProperty(INPUT_MESSAGE_CONTENT_FIELD)
    private InputMessageContent inputMessageContent;
    /** Optional. Inline keyboard attached to the message */
    @JsonProperty(REPLY_MARKUP_FIELD)
    private InlineKeyboardMarkup replyMarkup;
    /** Optional. Video duration */
    @JsonProperty(MPEG4_DURATION_FIELD)
    private Long mpeg4Duration;
    /** Send Markdown or HTML, if you want Telegram apps to show bold, italic, fixed-width text or inline URLs in the media caption. */
    @JsonProperty(PARSE_MODE_FIELD)
    private String parseMode;

    @Override
    public void validate() throws TelegramApiValidationException {
        if (id == null || id.isEmpty()) {
            throw new TelegramApiValidationException("ID parameter can't be empty", this);
        }
        if (mpeg4Url == null || mpeg4Url.isEmpty()) {
            throw new TelegramApiValidationException("Mpeg4Url parameter can't be empty", this);
        }
        if (inputMessageContent != null) {
            inputMessageContent.validate();
        }
        if (replyMarkup != null) {
            replyMarkup.validate();
        }
    }

}
