package com.chuang.telegram.bot.model.inlinequery.result;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.model.inlinequery.inputmessagecontent.InputMessageContent;
import com.chuang.telegram.bot.model.replaykeyboard.InlineKeyboardMarkup;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * Represents a venue. By default, the venue will be sent by the user. Alternatively, you can
 * use input_message_content to send a message with the specified content instead of the venue.
 * @apiNote  This will only work in Telegram versions released after 9 April, 2016. Older clients will
 * ignore them.
 */
@Data
@Accessors(chain = true)
public class InlineQueryResultVenue implements InlineQueryResult {

    private static final String TYPE_FIELD = "type";
    private static final String ID_FIELD = "id";
    private static final String TITLE_FIELD = "title";
    private static final String LATITUDE_FIELD = "latitude";
    private static final String LONGITUDE_FIELD = "longitude";
    private static final String ADDRESS_FIELD = "address";
    private static final String FOURSQUARE_ID_FIELD = "foursquare_id";
    private static final String REPLY_MARKUP_FIELD = "reply_markup";
    private static final String INPUT_MESSAGE_CONTENT_FIELD = "input_message_content";
    private static final String THUMB_URL_FIELD = "thumb_url";
    private static final String THUMB_WIDTH_FIELD = "thumb_width";
    private static final String THUMB_HEIGHT_FIELD = "thumb_height";
    private static final String FOURSQUARE_TYPE_FIELD = "foursquare_type";


    /** Type of the result, must be "venue" */
    @JsonProperty(TYPE_FIELD)
    private final String type = "venue";
    /** Unique identifier of this result, 1-64 bytes */
    @JsonProperty(ID_FIELD)
    private String id;
    /** Optional. Location title */
    @JsonProperty(TITLE_FIELD)
    private String title;
    /** Venue latitude in degrees */
    @JsonProperty(LATITUDE_FIELD)
    private Float latitude;
    /** Venue longitude in degrees */
    @JsonProperty(LONGITUDE_FIELD)
    private Float longitude;
    /** Address of the venue */
    @JsonProperty(ADDRESS_FIELD)
    private String address;
    /** Optional. Foursquare identifier of the venue if known */
    @JsonProperty(FOURSQUARE_ID_FIELD)
    private String foursquareId;
    /** Optional. Inline keyboard attached to the message */
    @JsonProperty(REPLY_MARKUP_FIELD)
    private InlineKeyboardMarkup replyMarkup;
    /** Optional. Content of the message to be sent */
    @JsonProperty(INPUT_MESSAGE_CONTENT_FIELD)
    private InputMessageContent inputMessageContent;
    /** Optional. URL of the thumbnail (jpeg only) for the file */
    @JsonProperty(THUMB_URL_FIELD)
    private String thumbUrl;
    /** Optional. Thumbnail width */
    @JsonProperty(THUMB_WIDTH_FIELD)
    private Integer thumbWidth;
    /** Optional. Thumbnail height */
    @JsonProperty(THUMB_HEIGHT_FIELD)
    private Integer thumbHeight;
    /** Optional. Foursquare type of the venue, if known. */
    @JsonProperty(FOURSQUARE_TYPE_FIELD)
    private String foursquareType;

    @Override
    public void validate() throws TelegramApiValidationException {
        if (id == null || id.isEmpty()) {
            throw new TelegramApiValidationException("ID parameter can't be empty", this);
        }
        if (title == null || title.isEmpty()) {
            throw new TelegramApiValidationException("Title parameter can't be empty", this);
        }
        if (latitude == null) {
            throw new TelegramApiValidationException("Latitude parameter can't be empty", this);
        }
        if (longitude == null) {
            throw new TelegramApiValidationException("Longitude parameter can't be empty", this);
        }
        if (address == null || address.isEmpty()) {
            throw new TelegramApiValidationException("Longitude parameter can't be empty", this);
        }
        if (inputMessageContent != null) {
            inputMessageContent.validate();
        }
        if (replyMarkup != null) {
            replyMarkup.validate();
        }

    }

}
