package com.chuang.telegram.bot.model.inlinequery.result;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.model.inlinequery.inputmessagecontent.InputMessageContent;
import com.chuang.telegram.bot.model.replaykeyboard.InlineKeyboardMarkup;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * Represents link to a page containing an embedded video player or a video file.
 * Alternatively, you can use input_message_content to send a message with the specified content
 * instead of the video.
 */
@Data
@Accessors(chain = true)
public class InlineQueryResultVideo implements InlineQueryResult {
    private static final String TYPE_FIELD = "type";
    private static final String ID_FIELD = "id";
    private static final String MIMETYPE_FIELD = "mime_type";
    private static final String VIDEO_URL_FIELD = "video_url";
    private static final String VIDEO_WIDTH_FIELD = "video_width";
    private static final String VIDEO_HEIGHT_FIELD = "video_height";
    private static final String VIDEO_DURATION_FIELD = "video_duration";
    private static final String THUMB_URL_FIELD = "thumb_url";
    private static final String TITLE_FIELD = "title";
    private static final String DESCRIPTION_FIELD = "description";
    private static final String CAPTION_FIELD = "caption";
    private static final String INPUT_MESSAGE_CONTENT_FIELD = "input_message_content";
    private static final String REPLY_MARKUP_FIELD = "reply_markup";
    private static final String PARSE_MODE_FIELD = "parse_mode";

    /** Type of the result, must be "video" */
    @JsonProperty(TYPE_FIELD)
    private final String type = "video";
    /** Unique identifier of this result */
    @JsonProperty(ID_FIELD)
    private String id;
    /** Mime type of the content of video url, i.e. “text/html” or “video/mp4” */
    @JsonProperty(MIMETYPE_FIELD)
    private String mimeType;
    /** A valid URL for the embedded video player or video file */
    @JsonProperty(VIDEO_URL_FIELD)
    private String videoUrl;
    /** Optional. Video width */
    @JsonProperty(VIDEO_WIDTH_FIELD)
    private Integer videoWidth;
    /** Optional. Video height */
    @JsonProperty(VIDEO_HEIGHT_FIELD)
    private Integer videoHeight;
    /** Optional. Video duration in seconds */
    @JsonProperty(VIDEO_DURATION_FIELD)
    private Long videoDuration;
    /** Optional. URL of the thumbnail (jpeg only) for the video */
    @JsonProperty(THUMB_URL_FIELD)
    private String thumbUrl;
    /** Optional. Title for the result */
    @JsonProperty(TITLE_FIELD)
    private String title;
    /** Optional. Short description of the result */
    @JsonProperty(DESCRIPTION_FIELD)
    private String description;
    /** Optional. Caption of the video to be sent, 0-200 characters */
    @JsonProperty(CAPTION_FIELD)
    private String caption;
    /** Optional. Content of the message to be sent instead of the photo */
    @JsonProperty(INPUT_MESSAGE_CONTENT_FIELD)
    private InputMessageContent inputMessageContent;
    /** Optional. Inline keyboard attached to the message */
    @JsonProperty(REPLY_MARKUP_FIELD)
    private InlineKeyboardMarkup replyMarkup;
    /** Send Markdown or HTML, if you want Telegram apps to show bold, italic, fixed-width text or inline URLs in the media caption. */
    @JsonProperty(PARSE_MODE_FIELD)
    private String parseMode;


    @Override
    public void validate() throws TelegramApiValidationException {
        if (id == null || id.isEmpty()) {
            throw new TelegramApiValidationException("ID parameter can't be empty", this);
        }
        if (videoUrl == null || videoUrl.isEmpty()) {
            throw new TelegramApiValidationException("VideoUrl parameter can't be empty", this);
        }
        if (inputMessageContent != null) {
            inputMessageContent.validate();
        }
        if (replyMarkup != null) {
            replyMarkup.validate();
        }
    }

}
