package com.chuang.telegram.bot.model.inlinequery.result.cached;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.model.inlinequery.inputmessagecontent.InputMessageContent;
import com.chuang.telegram.bot.model.inlinequery.result.InlineQueryResult;
import com.chuang.telegram.bot.model.replaykeyboard.InlineKeyboardMarkup;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * Represents a link to a video file stored on the Telegram servers. By default, this video
 * file will be sent by the user with an optional caption. Alternatively, you can use
 * input_message_content to send a message with the specified content instead of the video.
 */
@Data
@Accessors(chain = true)
public class InlineQueryResultCachedVideo implements InlineQueryResult {
    private static final String TYPE_FIELD = "type";
    private static final String ID_FIELD = "id";
    private static final String VIDEO_FILE_ID_FIELD = "video_file_id";
    private static final String TITLE_FIELD = "title";
    private static final String DESCRIPTION_FIELD = "description";
    private static final String CAPTION_FIELD = "caption";
    private static final String INPUT_MESSAGE_CONTENT_FIELD = "input_message_content";
    private static final String REPLY_MARKUP_FIELD = "reply_markup";
    private static final String PARSE_MODE_FIELD = "parse_mode";

    /** Type of the result, must be "video" */
    @JsonProperty(TYPE_FIELD)
    private final String type = "video";
    /** Unique identifier of this result */
    @JsonProperty(ID_FIELD)
    private String id;
    /** A valid file identifier for the video file */
    @JsonProperty(VIDEO_FILE_ID_FIELD)
    private String videoFileId;
    /** Optional. Title for the result */
    @JsonProperty(TITLE_FIELD)
    private String title;
    /** Optional. Short description of the result */
    @JsonProperty(DESCRIPTION_FIELD)
    private String description;
    /** Optional. Caption of the video to be sent, 0-200 characters */
    @JsonProperty(CAPTION_FIELD)
    private String caption;
    /** Optional. Content of the message to be sent instead of the photo */
    @JsonProperty(INPUT_MESSAGE_CONTENT_FIELD)
    private InputMessageContent inputMessageContent;
    /** Optional. Inline keyboard attached to the message */
    @JsonProperty(REPLY_MARKUP_FIELD)
    private InlineKeyboardMarkup replyMarkup;
    /** Send Markdown or HTML, if you want Telegram apps to show bold, italic, fixed-width text or inline URLs in the media caption. */
    @JsonProperty(PARSE_MODE_FIELD)
    private String parseMode;


    @Override
    public void validate() throws TelegramApiValidationException {
        if (id == null || id.isEmpty()) {
            throw new TelegramApiValidationException("ID parameter can't be empty", this);
        }
        if (videoFileId == null || videoFileId.isEmpty()) {
            throw new TelegramApiValidationException("VideoFileId parameter can't be empty", this);
        }
        if (inputMessageContent != null) {
            inputMessageContent.validate();
        }
        if (replyMarkup != null) {
            replyMarkup.validate();
        }
    }

}
