package com.chuang.telegram.bot.model.media;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.model.InputFile;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

/**
 * @author Ruben Bermudez
 * @version 4.0.0
 *
 * Represents an animation file (GIF or H.264/MPEG-4 AVC video without sound) to be sent.
 */
@SuppressWarnings("unused")
@Data
@EqualsAndHashCode(callSuper = true)
@Accessors(chain = true)
public class InputMediaAnimation extends InputMedia<InputMediaAnimation> {
    private static final String TYPE = "animation";

    public static final String WIDTH_FIELD = "width";
    public static final String HEIGHT_FIELD = "height";
    public static final String DURATION_FIELD = "duration";
    public static final String THUMB_FIELD = "thumb";

    /** Optional. Animation width */
    @JsonProperty(WIDTH_FIELD)
    private Integer width;
    /** Optional. Animation height */
    @JsonProperty(HEIGHT_FIELD)
    private Integer height;
    /** Optional. Animation duration */
    @JsonProperty(DURATION_FIELD)
    private Long duration;
    /**
     * Thumbnail of the file sent. The thumbnail should be in JPEG format and less than 200 kB in size.
     * A thumbnail‘s width and height should not exceed 90.
     * Ignored if the file is not uploaded using multipart/form-data.
     * Thumbnails can’t be reused and can be only uploaded as a new file, so you can pass “attach://<file_attach_name>”
     * if the thumbnail was uploaded using multipart/form-data under <file_attach_name>.
     */
    private InputFile thumb;

    public InputMediaAnimation() {
        super();
    }

    public InputMediaAnimation(String media, String caption) {
        super(media, caption);
    }


    @Override
    public String getType() {
        return TYPE;
    }


}
