package com.chuang.telegram.bot.model.media;

import com.chuang.telegram.bot.model.InputFile;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

/**
 * @author Ruben Bermudez
 * @version 4.0.0
 *
 * Represents an audio file to be treated as music to be sent.
 */
@Data
@EqualsAndHashCode(callSuper = true)
@Accessors(chain = true)
public class InputMediaAudio extends InputMedia<InputMediaAudio> {
    private static final String TYPE = "audio";

    public static final String DURATION_FIELD = "duration";
    public static final String PERFORMER_FIELD = "performer";
    public static final String TITLE_FIELD = "title";
    public static final String THUMB_FIELD = "thumb";

    /** Optional. Duration of the audio in seconds */
    @JsonProperty(DURATION_FIELD)
    private Long duration;
    /** Optional. Performer of the audio */
    @JsonProperty(PERFORMER_FIELD)
    private String performer;
    /** Optional. Title of the audio */
    @JsonProperty(TITLE_FIELD)
    private String title;
    /**
     * Optional. Thumbnail of the file sent. The thumbnail should be in JPEG format and less than 200 kB in size.
     * A thumbnail‘s width and height should not exceed 90.
     * Ignored if the file is not uploaded using multipart/form-data.
     * Thumbnails can’t be reused and can be only uploaded as a new file, so you can pass “attach://<file_attach_name>”
     * if the thumbnail was uploaded using multipart/form-data under <file_attach_name>.
     */
    private InputFile thumb;


    public InputMediaAudio() {
        super();
    }

    public InputMediaAudio(String media, String caption) {
        super(media, caption);
    }

    @Override
    public String getType() {
        return TYPE;
    }

}
