package com.chuang.telegram.bot.model.media;

import com.chuang.telegram.bot.model.InputFile;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

/**
 * @author Ruben Bermudez
 * @version 3.5
 *
 * Represents a video.
 */
@Data
@EqualsAndHashCode(callSuper = true)
@Accessors(chain = true)
public class InputMediaVideo extends InputMedia<InputMediaVideo> {
    private static final String TYPE = "video";

    public static final String WIDTH_FIELD = "width";
    public static final String HEIGHT_FIELD = "height";
    public static final String DURATION_FIELD = "duration";
    public static final String SUPPORTS_STREAMING_FIELD = "supports_streaming";
    public static final String THUMB_FIELD = "thumb";

    /**  Optional. Video width */
    @JsonProperty(WIDTH_FIELD)
    private Integer width;
    /**  Optional. Video height */
    @JsonProperty(HEIGHT_FIELD)
    private Integer height;
    /**  Optional. Video duration */
    @JsonProperty(DURATION_FIELD)
    private Long duration;
    /**  Optional. Pass True, if the uploaded video is suitable for streaming */
    @JsonProperty(SUPPORTS_STREAMING_FIELD)
    private Boolean supportsStreaming;
    /**
     * Thumbnail of the file sent. The thumbnail should be in JPEG format and less than 200 kB in size.
     * A thumbnail‘s width and height should not exceed 90.
     * Ignored if the file is not uploaded using multipart/form-data.
     * Thumbnails can’t be reused and can be only uploaded as a new file, so you can pass “attach://<file_attach_name>”
     * if the thumbnail was uploaded using multipart/form-data under <file_attach_name>.
     */
    @JsonProperty(THUMB_FIELD)
    private InputFile thumb;

    public InputMediaVideo() {
        super();
    }

    public InputMediaVideo(String media, String caption) {
        super(media, caption);
    }

    @Override
    public String getType() {
        return TYPE;
    }


}
