package com.chuang.telegram.bot.model.passport;

import com.chuang.telegram.bot.interfaces.BotApiObject;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;

/**
 * @author Ruben Bermudez
 * @version 4.0.0
 * Contains data required for decrypting and authenticating EncryptedPassportElement.
 * See the Telegram Passport Documentation for a complete description of the data decryption and authentication processes.
 */
@Data
public class EncryptedCredentials implements BotApiObject {
    private static final String DATA_FIELD = "data";
    private static final String HASH_FIELD = "hash";
    private static final String SECRET_FIELD = "secret";

    /**
     * Base64-encoded encrypted JSON-serialized data with unique user's payload,
     * data hashes and secrets required for EncryptedPassportElement decryption and authentication
     */
    @JsonProperty(DATA_FIELD)
    private String data;
    /** Base64-encoded data hash for data authentication */
    @JsonProperty(HASH_FIELD)
    private String hash;
    /** Base64-encoded secret, encrypted with the bot's public RSA key, required for data decryption */
    @JsonProperty(SECRET_FIELD)
    private String secret;

    public EncryptedCredentials() {
    }

    public EncryptedCredentials(String data, String hash, String secret) {
        this.data = data;
        this.hash = hash;
        this.secret = secret;
    }

}
