
package com.chuang.telegram.bot.model.passport.dataerror;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

/**
 * @author Ruben Bermudez
 * @version 4.0.0
 *
 * Represents an error in a field of data provided by a user.
 * The error is considered resolved when the field's value changes.
 */
@Data
@Accessors(chain = true)
public class PassportElementErrorDataField implements PassportElementError {
    private static final String SOURCE_FIELD = "source";
    private static final String TYPE_FIELD = "type";
    private static final String FIELD_NAME_FIELD = "field_name";
    private static final String DATA_HASH_FIELD = "data_hash";
    private static final String MESSAGE_FIELD = "message";

    /** Error source, must be data */
    @JsonProperty(SOURCE_FIELD)
    private final String source = "data";
    /**
     * Type of the Telegram Passport data with the error, one of “personal_details”, “passport”, “driver_license”,
     * “identity_card”, “internal_passport”, “address”
     */
    @JsonProperty(TYPE_FIELD)
    private String type;
    /** Name of the data field with the error */
    @JsonProperty(FIELD_NAME_FIELD)
    private String fieldName;
    /** Base64-encoded data hash */
    @JsonProperty(DATA_HASH_FIELD)
    private String dataHash;
    /** Error message */
    @JsonProperty(MESSAGE_FIELD)
    private String message;

    public PassportElementErrorDataField() {
        super();
    }

    public PassportElementErrorDataField(String type, String fieldName, String dataHash, String message) {
        super();
        this.type = checkNotNull(type);
        this.fieldName = checkNotNull(fieldName);
        this.dataHash = checkNotNull(dataHash);
        this.message = checkNotNull(message);
    }

    @Override
    public void validate() throws TelegramApiValidationException {
        if (fieldName == null || fieldName.isEmpty()) {
            throw new TelegramApiValidationException("Field Name parameter can't be empty", this);
        }
        if (dataHash == null || dataHash.isEmpty()) {
            throw new TelegramApiValidationException("Data hash parameter can't be empty", this);
        }
        if (message == null || message.isEmpty()) {
            throw new TelegramApiValidationException("Message parameter can't be empty", this);
        }
        if (type == null || type.isEmpty()) {
            throw new TelegramApiValidationException("Type parameter can't be empty", this);
        }
    }

}
