package com.chuang.telegram.bot.model.passport.dataerror;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Ruben Bermudez
 * @version 4.0.0
 *
 * Represents an error in a list of attached files.
 * The error is considered resolved when the file list changes.
 */
@Data
@Accessors(chain = true)
public class PassportElementErrorFiles implements PassportElementError {
    private static final String SOURCE_FIELD = "source";
    private static final String TYPE_FIELD = "type";
    private static final String FILE_HASHES_FIELD = "file_hashes";
    private static final String MESSAGE_FIELD = "message";

    /** Error source, must be file */
    @JsonProperty(SOURCE_FIELD)
    private final String source = "file";
    /**
     * Type of the Telegram Passport data with the error, one of “utility_bill”,
     * “bank_statement”, “rental_agreement”, “passport_registration”, “temporary_registration”
     */
    @JsonProperty(TYPE_FIELD)
    private String type;
    /** List of base64-encoded file hashes */
    @JsonProperty(FILE_HASHES_FIELD)
    private List<String> fileHashes;
    /** Error message */
    @JsonProperty(MESSAGE_FIELD)
    private String message;

    public PassportElementErrorFiles() {
        super();
    }

    public PassportElementErrorFiles(String type, List<String> fileHashes, String message) {
        super();
        this.type = checkNotNull(type);
        this.fileHashes = checkNotNull(fileHashes);
        this.message = checkNotNull(message);
    }


    public PassportElementErrorFiles addFileHash(String fileHash) {
        fileHash = checkNotNull(fileHash);
        if (fileHashes == null) {
            fileHashes = new ArrayList<>();
        }
        fileHashes.add(fileHash);
        return this;
    }


    @Override
    public void validate() throws TelegramApiValidationException {
        if (fileHashes == null || fileHashes.isEmpty()) {
            throw new TelegramApiValidationException("File hash parameter can't be empty", this);
        }
        if (message == null || message.isEmpty()) {
            throw new TelegramApiValidationException("Message parameter can't be empty", this);
        }
        if (type == null || type.isEmpty()) {
            throw new TelegramApiValidationException("Type parameter can't be empty", this);
        }
    }

}
