package com.chuang.telegram.bot.model.passport.dataerror;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

/**
 * @author Ruben Bermudez
 * @version 4.0.0
 *
 * Represents an error in a file with a document‘s front side.
 * The error is considered resolved when the file with the document’s front side changes.
 */
@Data
@Accessors(chain = true)
public class PassportElementErrorFrontSide implements PassportElementError {
    private static final String SOURCE_FIELD = "source";
    private static final String TYPE_FIELD = "type";
    private static final String FILE_HASH_FIELD = "file_hash";
    private static final String MESSAGE_FIELD = "message";

    /** Error source, must be front_side */
    @JsonProperty(SOURCE_FIELD)
    private final String source = "front_side";
    /**
     * Type of the Telegram Passport data with the error, one of “passport”,
     * “driver_license”, “identity_card”, “internal_passport”
     */
    @JsonProperty(TYPE_FIELD)
    private String type;
    /** Base64-encoded file with document's front side hash */
    @JsonProperty(FILE_HASH_FIELD)
    private String fileHash;
    /** Error message */
    @JsonProperty(MESSAGE_FIELD)
    private String message;

    public PassportElementErrorFrontSide() {
        super();
    }

    public PassportElementErrorFrontSide(String type, String fileHash, String message) {
        super();
        this.type = checkNotNull(type);
        this.fileHash = checkNotNull(fileHash);
        this.message = checkNotNull(message);
    }

    @Override
    public void validate() throws TelegramApiValidationException {
        if (fileHash == null || fileHash.isEmpty()) {
            throw new TelegramApiValidationException("File hash parameter can't be empty", this);
        }
        if (message == null || message.isEmpty()) {
            throw new TelegramApiValidationException("Message parameter can't be empty", this);
        }
        if (type == null || type.isEmpty()) {
            throw new TelegramApiValidationException("Type parameter can't be empty", this);
        }
    }
}
