package com.chuang.telegram.bot.model.payment;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.InputBotApiObject;
import com.chuang.telegram.bot.interfaces.Validable;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;

/**
 * @author Ruben Bermudez
 * @version 1.0
 * This object represents a portion of goods price.
 */
@Data
public class LabeledPrice implements InputBotApiObject, Validable {
    private static final String LABEL_FIELD = "label";
    private static final String AMOUNT_FIELD = "amount";

    /** Portion label */
    @JsonProperty(LABEL_FIELD)
    private String label;

    /**
     * Price of the product in the smallest units of the currency (Long, not float/double).
     * For example, for a price of US$ 1.45 pass amount = 145.
     */
    @JsonProperty(AMOUNT_FIELD)
    private Long amount;

    /**
     * Builds an empty LabeledPrice
     */
    public LabeledPrice() {
        super();
    }

    /**
     * Builds a LabeledPrice with mandatory parameters
     * @param label Portion label
     * @param amount Currency amount in minimal quantity of the currency
     */
    public LabeledPrice(String label, Long amount) {
        super();
        this.label = checkNotNull(label);
        this.amount = checkNotNull(amount);
    }
    @Override
    public void validate() throws TelegramApiValidationException {
        if (label == null || label.isEmpty()) {
            throw new TelegramApiValidationException("Label parameter can't be empty", this);
        }
        if (amount == null) {
            throw new TelegramApiValidationException("Amount parameter can't be empty", this);
        }
    }
}
