package com.chuang.telegram.bot.model.payment;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.InputBotApiObject;
import com.chuang.telegram.bot.interfaces.Validable;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.experimental.Accessors;

import java.util.List;

/**
 * @author Ruben Bermudez
 * @version 1.0
 *
 * This object represents one shipping option.
 */
@Data
@Accessors(chain = true)
public class ShippingOption implements InputBotApiObject, Validable {
    private static final String ID_FIELD = "id";
    private static final String TITLE_FIELD = "title";
    private static final String PRICES_FIELD = "prices";

    /** Shipping option identifier */
    @JsonProperty(ID_FIELD)
    private String id;
    /** Option title */
    @JsonProperty(TITLE_FIELD)
    private String title;
    /** List of price portions */
    @JsonProperty(PRICES_FIELD)
    private List<LabeledPrice> prices;

    /**
     * Creates an empty shipping option
     */
    public ShippingOption() {
        super();
    }

    /**
     * Creates a shipping option with mandatory fields
     * @param id Shipping option identifier
     * @param title Option title
     * @param prices List of price portions
     */
    public ShippingOption(String id, String title, List<LabeledPrice> prices) {
        this.id = checkNotNull(id);
        this.title = checkNotNull(title);
        this.prices = checkNotNull(prices);
    }


    @Override
    public void validate() throws TelegramApiValidationException {
        if (id == null || id.isEmpty()) {
            throw new TelegramApiValidationException("Id parameter can't be empty", this);
        }
        if (title == null || title.isEmpty()) {
            throw new TelegramApiValidationException("Title parameter can't be empty", this);
        }
        if (prices == null || prices.isEmpty()) {
            throw new TelegramApiValidationException("Prices parameter can't be empty", this);
        }
        for (LabeledPrice price : prices) {
            price.validate();
        }
    }
}
