package com.chuang.telegram.bot.model.payment;

import com.chuang.telegram.bot.interfaces.BotApiObject;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;

/**
 * @author Ruben Bermudez
 * @version 1.0
 *
 * This object contains basic information about a successful payment.
 */
@Data
public class SuccessfulPayment implements BotApiObject {
    private static final String CURRENCY_FIELD = "currency";
    private static final String TOTAL_AMOUNT_FIELD = "total_amount";
    private static final String INVOICE_PAYLOAD_FIELD = "invoice_payload";
    private static final String SHIPPING_OPTION_ID_FIELD = "shipping_option_id";
    private static final String ORDER_INFO_FIELD = "order_info";
    private static final String TELEGRAM_PAYMENT_CHARGE_ID_FIELD = "telegram_payment_charge_id";
    private static final String PROVIDER_PAYMENT_CHARGE_ID_FIELD = "provider_payment_charge_id";

    /** Three-letter ISO 4217 currency code */
    @JsonProperty(CURRENCY_FIELD)
    private String currency;

    /**
     * Total price in the smallest units of the currency (Long, not float/double).
     * For example, for a price of US$ 1.45 pass amount = 145.
     */
    @JsonProperty(TOTAL_AMOUNT_FIELD)
    private Long totalAmount;
    /** Bot specified invoice payload */
    @JsonProperty(INVOICE_PAYLOAD_FIELD)
    private String invoicePayload;
    /** Optional. Identifier of a chosen by user shipping option */
    @JsonProperty(SHIPPING_OPTION_ID_FIELD)
    private String shippingOptionId;
    /** Optional. Order info provided by the user */
    @JsonProperty(ORDER_INFO_FIELD)
    private OrderInfo orderInfo;
    /** Telegram payment identifier */
    @JsonProperty(TELEGRAM_PAYMENT_CHARGE_ID_FIELD)
    private String telegramPaymentChargeId;
    /** Provider payment identifier */
    @JsonProperty(PROVIDER_PAYMENT_CHARGE_ID_FIELD)
    private String providerPaymentChargeId;

}
