package com.chuang.telegram.bot.model.stickers;

import com.chuang.telegram.bot.exception.TelegramApiValidationException;
import com.chuang.telegram.bot.interfaces.InputBotApiObject;
import com.chuang.telegram.bot.interfaces.Validable;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;

/**
 * @author Ruben Bermudez
 * @version 3.2
 * This object describes the position on faces where a mask should be placed by default.
 */
@Data
public class MaskPosition implements InputBotApiObject, Validable {
    private static final String POINT_FIELD = "point";
    private static final String X_SHIFT_FIELD = "x_shift";
    private static final String Y_SHIFT_FIELD = "y_shift";
    private static final String SCALE_FIELD = "scale";

    /** The part of the face relative to which the mask should be placed. One of “forehead”, “eyes”, “mouth”, or “chin”. */
    @JsonProperty(POINT_FIELD)
    private String point;
    /** Shift by X-axis measured in widths of the mask scaled to the face size, from left to right. For example, choosing -1.0 will place mask just to the left of the default mask position. */
    @JsonProperty(X_SHIFT_FIELD)
    private Float xShift;
    /** Shift by Y-axis measured in heights of the mask scaled to the face size, from top to bottom. For example, 1.0 will place the mask just below the default mask position. */
    @JsonProperty(Y_SHIFT_FIELD)
    private Float yShift;
    /** Mask scaling coefficient. For example, 2.0 means double size. */
    @JsonProperty(SCALE_FIELD)
    private Float scale;


    @Override
    public void validate() throws TelegramApiValidationException {
        if (point == null || point.isEmpty()) {
            throw new TelegramApiValidationException("point can't be empty", this);
        }
        if (xShift == null) {
            throw new TelegramApiValidationException("xShift can't be empty", this);
        }
        if (yShift == null) {
            throw new TelegramApiValidationException("yShift can't be empty", this);
        }
        if (scale == null) {
            throw new TelegramApiValidationException("scale can't be empty", this);
        }
    }
}
