package com.chuang.test.tgbot;

import com.chuang.tauceti.httpclient.Request;
import com.chuang.tauceti.support.exception.BusinessException;
import org.apache.http.HttpHeaders;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.junit.jupiter.api.Test;

public class SendTest {

    private final String BOT = "5250897653:AAFa4Rl7QZI1Hqe4mFKln-FltR6ae2S1wVo";

    @Test
    public void testGetMe() {

        Request.Get("https://api.telegram.org/bot" + BOT + "/getMe")
                .build()
                .asyncExecuteAsString()
                .thenAccept(this::print)
                .join();
    }

    @Test
    public void testSendText() throws JSONException {
        JSONObject json = new JSONObject();
        json.put("chat_id", "-1001790810613");
        json.put("text", "<span class=\"tg-spoiler\">this is a text msg</span>");
        json.put("parse_mode", "html");

        Request.Post("https://api.telegram.org/bot" + BOT + "/sendMessage")
                .header(HttpHeaders.CONTENT_TYPE, "application/json")
                .body(json.toString())
                .build()
                .asyncExecuteAsString()
                .thenAccept(this::print)
                .join();
    }

    @Test
    public void testSendText2() throws JSONException {
        JSONObject json = new JSONObject();
        json.put("chat_id", "-1001790810613");
        json.put("text", "<span class=\"tg-spoiler\">this is a text msg</span>");
        json.put("parse_mode", "html");

        Request.Get("https://api.telegram.org/bot" + BOT + "/sendMessage")
                .parameter("chat_id", "-1001790810613")
                .parameter("text", "<span class=\"tg-spoiler\">this is a text msg</span>")
                .parameter("parse_mode", "html")
                .build()
                .asyncExecuteAsString()
                .thenAccept(this::print)
                .join();
    }


    @Test
    public void testEditMessage() throws JSONException {

        JSONObject json = new JSONObject();
        json.put("chat_id", "-1001790810613");
        json.put("message_id", "981");
        json.put("text", "aaaaaaa");
        json.put("parse_mode", "html");
        Request.Post("https://api.telegram.org/bot" + BOT + "/editMessageText")
                .header(HttpHeaders.CONTENT_TYPE, "application/json")
                .body(json.toString())
                .build()
                .asyncExecuteAsString()
                .thenAccept(this::print)
                .join();
    }

    @Test
    public void testDelMessage() throws JSONException {
        JSONObject json = new JSONObject();
        json.put("chat_id", "-1001790810613");
        json.put("message_id", "982");
        Request.Post("https://api.telegram.org/bot" + BOT + "/deleteMessage")
                .header(HttpHeaders.CONTENT_TYPE, "application/json")
                .body(json.toString())
                .build()
                .asyncExecuteAsString()
                .thenAccept(this::print)
                .join();
    }

    @Test
    public void testSendBtnText() throws JSONException {


        JSONObject json = new JSONObject();
        json.put("chat_id", "-1001790810613");
        json.put("text", "try alert");
        json.put("allow_sending_without_reply", true);
        json.put("reply_markup", inlineKeyboard());

        Request.Post("https://api.telegram.org/bot" + BOT + "/sendMessage")
                .header(HttpHeaders.CONTENT_TYPE, "application/json")
                .body(json.toString())
                .build()
                .asyncExecuteAsString()
                .thenAccept(this::print)
                .join();
    }


    @Test
    public void testSendBtnPhoto() throws JSONException {

        JSONObject json = new JSONObject();
        json.put("chat_id", "-1001790810613");
        json.put("photo", "https://imgpp.com/s1/2022/07/01/_.jpg");
        json.put("caption", "this is test photo");
        json.put("allow_sending_without_reply", true);
        json.put("reply_markup", inlineKeyboard());

        Request.Post("https://api.telegram.org/bot" + BOT + "/sendPhoto")
                .header(HttpHeaders.CONTENT_TYPE, "application/json")
                .body(json.toString())
                .build()
                .asyncExecuteAsString()
                .thenAccept(this::print)
                .join();
    }


    @Test
    public void testSendKeyBoardPhoto() throws JSONException {

        JSONObject json = new JSONObject();
        json.put("chat_id", "-1001790810613");
        json.put("photo", "https://imgpp.com/s1/2022/07/01/_.jpg");
        json.put("caption", "this is test photo");
        json.put("allow_sending_without_reply", true);
        json.put("reply_markup", replyKeyboard());

        Request.Post("https://api.telegram.org/bot" + BOT + "/sendPhoto")
                .header(HttpHeaders.CONTENT_TYPE, "application/json")
                .body(json.toString())
                .build()
                .asyncExecuteAsString()
                .thenAccept(this::print)
                .join();
    }

    @Test
    public void testDoubleMessage() throws JSONException {
        JSONObject markup = new JSONObject();
        markup.put("keyboard", replyKeyboard().getJSONArray("keyboard"));
        markup.put("inline_keyboard", inlineKeyboard().getJSONArray("inline_keyboard"));


        JSONObject json = new JSONObject();
        json.put("chat_id", "-1001790810613");
        json.put("photo", "https://imgpp.com/s1/2022/07/01/_.jpg");
        json.put("caption", "请下注");
        json.put("allow_sending_without_reply", true);
        json.put("reply_markup", markup);

        Request.Post("https://api.telegram.org/bot" + BOT + "/sendPhoto")
                .header(HttpHeaders.CONTENT_TYPE, "application/json")
                .body(json.toString())
                .build()
                .asyncExecuteAsString()
                .thenAccept(this::print)
                .join();
    }


    @Test
    public void testDoubleMessage2() throws JSONException {


        JSONObject json = new JSONObject();
        json.put("chat_id", "-1001790810613");
        json.put("photo", "https://imgpp.com/s1/2022/07/01/_.jpg");
        json.put("caption", "请下注");
        json.put("allow_sending_without_reply", true);
        json.put("reply_markup", inlineKeyboard());

        JSONObject response = Request.Post("https://api.telegram.org/bot" + BOT + "/sendPhoto")
                .header(HttpHeaders.CONTENT_TYPE, "application/json")
                .body(json.toString())
                .build()
                .asyncExecuteAsString()
                .thenApply(s -> {
                    try {
                        return new JSONObject(s);
                    } catch (JSONException e) {
                        e.printStackTrace();
                        return null;
                    }
                })
                .join();

        if(!response.getBoolean("ok")) {
            return;
        }

        String messageId = response.getJSONObject("result").getString("message_id");

        json = new JSONObject();
        json.put("chat_id", "-1001790810613");
        json.put("text", "请下注");
        json.put("reply_to_message_id", messageId);
        json.put("allow_sending_without_reply", true);
        json.put("reply_markup", replyKeyboard());

        Request.Post("https://api.telegram.org/bot" + BOT + "/sendMessage")
                .header(HttpHeaders.CONTENT_TYPE, "application/json")
                .body(json.toString())
                .build()
                .asyncExecuteAsString()
                .thenAccept(this::print)
                .join();
    }

    private void print(String msg) {
        System.out.println("\n\n\n\n====================================" + msg + "================================\n\n\n\n");
    }


    private JSONArray btns(String keyField, String valueField, Object... btns) throws JSONException {
        if(btns.length % 2 != 0) {
            throw new BusinessException("必须是2的倍数");
        }

        JSONArray array = new JSONArray();
        for (int i = 0; i < btns.length / 2; i++) {
            JSONObject btn = new JSONObject();
            btn.put(keyField, btns[i * 2]);
            btn.put(valueField, btns[i * 2 + 1]);
            array.put(btn);
        }

        return array;
    }

    private JSONObject inlineKeyboard() throws JSONException {
//        JSONArray row0 = btns("text", "url", "Payment", "tg://user?id=5250897653", "客服", "tg://user?id=2060157728");
        JSONArray row0 = btns("text", "url", "Payment", "https://telegram.me/anr_test01_bot", "客服", "https://telegram.me/Nonejks");
        JSONArray row1 = btns("text", "callback_data","500 U", "500 USDT", "1000 U", "1000 USDT", "1500 U", "1500 USDT");
        JSONArray row2 = btns("text", "callback_data","2000 U", "2000 USDT", "2500 U", "2500 USDT", "3500 U", "3500 USDT");

        JSONArray array = new JSONArray();
        array.put(row0);
        array.put(row1);
        array.put(row2);

        JSONObject keyboard = new JSONObject();
        keyboard.put("inline_keyboard", array);

        return keyboard;
    }

    private JSONObject replyKeyboard() throws JSONException {

        JSONArray row1 = btns("text", "request_contact","500 U", false, "1000 U", false, "1500 U", false);
        JSONArray row2 = btns("text", "request_contact","2000 U", false, "2500 U", false, "3500 U", false);

        JSONArray array = new JSONArray();
        array.put(row1);
        array.put(row2);

        JSONObject keyboard = new JSONObject();
        keyboard.put("keyboard", array);

        return keyboard;

    }
}
