package com.chuang.tauceti.tools.basic;

import com.chuang.tauceti.support.exception.SystemException;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Optional;

/**
 * @author ATH
 */
public class ExceptionKit {

    /**
     * 获取异常信息
     */
    public static String getTrace(Throwable t) {
        StringWriter stringWriter= new StringWriter();
        PrintWriter writer= new PrintWriter(stringWriter);
        t.printStackTrace(writer);
        StringBuffer buffer= stringWriter.getBuffer();
        return buffer.toString();
    }

    /**
     * 判断 e 或 e内部 中是否包含 类型为 errClazz 的错误
     * @param e 异常
     * @param errClazz 需要寻找的异常类型
     * @return 是否包含
     */
    public static boolean contains(Throwable e, Class<? extends Throwable> errClazz) {

        Throwable throwable = e;
        do {
            if (throwable.getClass() == errClazz) {
                return true;
            }
            throwable = e.getCause();
        } while (null != throwable);
        return false;
    }

    /**
     * 查找并获取 e 或 e内部 中含 类型为 errClazz 的异常对象
     * @param e 异常
     * @param errClazz 需要寻找的异常类型
     * @return 异常对象
     */
    public static <E extends Throwable> Optional<E> getError(Throwable e, Class<E> errClazz) {

        Throwable throwable = e;
        do {
            if (throwable.getClass() == errClazz) {
                return Optional.of((E) throwable);
            }
            throwable = e.getCause();
        } while (null != throwable);
        return Optional.empty();
    }

    /**
     * 查找并获取 e 或 e内部 中含 类型为 errClazz 的异常对象的错误信息
     * @param e 异常
     * @param errClazz 需要寻找的异常类型
     * @return 错误信息
     */
    public static <E extends Throwable> Optional<String> getErrorMessage(Throwable e, Class<E> errClazz) {
        return getError(e, errClazz).map(Throwable::getMessage);
    }
}
