package com.chuang.tauceti.tools.basic;

import java.util.concurrent.CompletableFuture;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.function.Supplier;

public class ScheduleKit {
    private final static ScheduledExecutorService EXECUTOR = Executors.newSingleThreadScheduledExecutor();

    public static void schedule(Runnable run, int time, TimeUnit timeUnit) {
        EXECUTOR.schedule(run, time, timeUnit);
    }

    public static <T> CompletableFuture<T> schedule(Supplier<T> run, int time, TimeUnit unit) {
        CompletableFuture<T> future = new CompletableFuture<T>();

        schedule(() -> {
            try {
                future.complete(run.get());
            } catch (Exception e) {
                future.completeExceptionally(e);
            }
        }, time, unit);

        return future;
    }

    public static <T> CompletableFuture<T> scheduleFuture(Supplier<CompletableFuture<T>> run, int time, TimeUnit unit) {
        CompletableFuture<T> future = new CompletableFuture<T>();

        schedule(() -> {
            run.get().whenComplete((t, throwable) -> {
                if(null == throwable) {
                    future.complete(t);
                } else {
                    future.completeExceptionally(throwable);
                }
            });
        }, time, unit);

        return future;
    }

    public static void scheduleAtFixedRate(Runnable run, long initialDelay, int delay, TimeUnit timeUnit) {
        EXECUTOR.scheduleAtFixedRate(run, initialDelay, delay, timeUnit);
    }


    public static void scheduleWithFixedRate(Runnable run, long initialDelay, int delay, TimeUnit timeUnit) {
        EXECUTOR.scheduleWithFixedDelay(run, initialDelay, delay, timeUnit);
    }
}
