package com.chuang.tauceti.tools.third.jackson;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonDeserializer;

import java.io.IOException;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeParseException;
import java.util.TimeZone;
import java.util.regex.Pattern;

public class LocalDateTimeDeserializer extends JsonDeserializer<LocalDateTime> {

    private static final Pattern PATTERN_TIMESTAMP = Pattern.compile("^\\d{10}$");

    private final String regex;

    public LocalDateTimeDeserializer() {
        this(null);
    }
    public LocalDateTimeDeserializer(String regex) {
        this.regex = regex;
    }

    @Override
    public LocalDateTime deserialize(JsonParser jsonParser, DeserializationContext deserializationContext) throws IOException, JsonProcessingException {
        String text = jsonParser.getValueAsString();
        if(null != regex) {
            return LocalDateTime.parse(jsonParser.getValueAsString(), DateTimeFormatter.ofPattern(this.regex, deserializationContext.getLocale()));
        }

        if (PATTERN_TIMESTAMP.matcher(text).matches()) {
            return LocalDateTime.ofInstant(Instant.ofEpochMilli(Long.parseLong(text)),
                    TimeZone.getDefault().toZoneId());
        }
        try {
            return LocalDateTime.parse(jsonParser.getValueAsString(), DateTimeFormatter.ISO_OFFSET_DATE_TIME);
        } catch (Exception ignore) {}
        try {
            return LocalDateTime.parse(jsonParser.getValueAsString(), DateTimeFormatter.ISO_LOCAL_DATE_TIME);
        } catch (Exception ignore) {}
        try {
            return LocalDateTime.parse(jsonParser.getValueAsString(), DateTimeFormatter.ISO_DATE_TIME);
        } catch (Exception ignore) {}
        try {
            return LocalDateTime.parse(jsonParser.getValueAsString(), DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss", deserializationContext.getLocale()));
        } catch (Exception ignore) {}
        try {
            return LocalDateTime.parse(jsonParser.getValueAsString(), DateTimeFormatter.ofPattern("yyyy/MM/dd HH:mm:ss", deserializationContext.getLocale()));
        } catch (Exception ignore) {}

        throw new DateTimeParseException("can not convert LocalDateTime", jsonParser.getValueAsString(), 0);
    }
}
