package com.chuang.urras.admin.configuration;

import com.chuang.tauceti.tools.basic.collection.CollectionKit;
import com.chuang.urras.admin.WsConst;
import com.chuang.urras.admin.ws.LoggedChannelInterceptor;
import com.chuang.urras.admin.ws.LoggedHandshakeInterceptor;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.Environment;
import org.springframework.messaging.simp.config.ChannelRegistration;
import org.springframework.messaging.simp.config.MessageBrokerRegistry;
import org.springframework.web.socket.config.annotation.EnableWebSocketMessageBroker;
import org.springframework.web.socket.config.annotation.StompEndpointRegistry;
import org.springframework.web.socket.config.annotation.WebSocketMessageBrokerConfigurer;

import javax.annotation.Resource;

/**
 * @author ATH
 */
@Configuration
@EnableWebSocketMessageBroker
public class WebsocketBrokerConfigurer implements WebSocketMessageBrokerConfigurer {

    @Resource private Environment environment;

    /**
     * 配置 WebSocket 进入点，及开启使用 SockJS，这些配置主要用配置连接端点，用于 WebSocket 连接
     *
     * @param registry STOMP 端点
     */
    @Override
    public void registerStompEndpoints(StompEndpointRegistry registry) {
        boolean prod = CollectionKit.contains(environment.getActiveProfiles(), "prod");

        if(prod) {
            registry.addEndpoint(WsConst.END_POINT)
                    .setAllowedOriginPatterns("*")
//                    .addInterceptors(loggedHandshakeInterceptor())
                    .withSockJS();
        } else {
            registry.addEndpoint(WsConst.END_POINT)
                    .setAllowedOriginPatterns("*")
//                    .addInterceptors(loggedHandshakeInterceptor())
                    .withSockJS();
        }

    }


    /**
     * 配置消息代理选项
     *
     * @param registry 消息代理注册配置
     */
    @Override
    public void configureMessageBroker(MessageBrokerRegistry registry) {
        // 客户端监听 服务端所发送消息的 destination 前缀，加入前缀是 /topic 则当服务端向 destination为 /test 发送消息时，会被客户端能通过监听 /topic/test 获取到消息
        registry.enableSimpleBroker(WsConst.LISTEN_PREFIX);
        // 客户端向服务端发送的消息的 destination 前缀，加入前缀是 /app 则当客户端向 /app/test 发送消息时，会被 @MessageMapping("/test") 收到
        registry.setApplicationDestinationPrefixes(WsConst.SEND_PREFIX);

        // 服务端通知特定用户客户端的前缀，可以不设置，默认为user
//        registry.setUserDestinationPrefix("/user");
    }

    @Override
    public void configureClientInboundChannel(ChannelRegistration registration) {
        registration.interceptors(loggedChannelInterceptor());
    }

    @Bean
    public LoggedChannelInterceptor loggedChannelInterceptor() {
        return new LoggedChannelInterceptor();
    }

    @Bean
    public LoggedHandshakeInterceptor loggedHandshakeInterceptor() {
        return new LoggedHandshakeInterceptor();
    }
}
