package com.chuang.urras.admin.ws;

import com.chuang.tauceti.rbac2.rest.model.ShiroUser;
import com.chuang.tauceti.shiro.spring.web.jwt.JwtManager;
import com.chuang.tauceti.shiro.spring.web.jwt.JwtPayload;
import org.springframework.messaging.Message;
import org.springframework.messaging.MessageChannel;
import org.springframework.messaging.simp.stomp.StompCommand;
import org.springframework.messaging.simp.stomp.StompHeaderAccessor;
import org.springframework.messaging.support.ChannelInterceptor;
import org.springframework.messaging.support.MessageHeaderAccessor;

import javax.annotation.Resource;
import java.util.Optional;

/**
 *
 */
public class LoggedChannelInterceptor implements ChannelInterceptor {

    @Resource private JwtManager jwtManager;

    @Override
    public Message<?> preSend(Message<?> message, MessageChannel channel) {
        StompHeaderAccessor accessor =
                MessageHeaderAccessor.getAccessor(message, StompHeaderAccessor.class);
        //1. 判断是否首次连接请求
        if (StompCommand.CONNECT.equals(accessor.getCommand())) {
            //2. 验证是否登录
            Optional<String> jwtTokenOpt = accessor.getNativeHeader("Authorization").stream().findFirst();
            if(jwtTokenOpt.isPresent()) {
                JwtPayload payload = jwtManager.parseNoErr(jwtTokenOpt.get());
                accessor.setUser((ShiroUser)payload.getBody());
            } else {
                return null;
            }
        }
        //不是首次连接，已经成功登陆
        return message;
    }
}
