package com.chuang.urras.admin.ws;

import com.chuang.tauceti.rbac2.rest.model.LoggedEvent;
import com.chuang.urras.admin.WsConst;
import com.chuang.urras.common.events.OfficeMsgEvent;
import com.chuang.urras.common.events.RedisEventManager;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.event.EventListener;
import org.springframework.messaging.simp.SimpMessagingTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * stomp 不支持 OnOpen 和 OnClose 事件。
 * 要监听连接和断开连接，我们通过stomp 的command实现
 * @author ATH
 */
@Slf4j
@Component
public class WebsocketManager {
    /** 消息发送工具对象 */
    @Resource private SimpMessagingTemplate brokerMessagingTemplate;
    @Resource private RedisEventManager redisEventManager;

    @EventListener
    public void listenOfficeWarnEvent(OfficeMsgEvent event) {
        brokerMessagingTemplate.convertAndSend(WsConst.OFFICE_WARN_TOPIC, event.getMessage());
    }

    @EventListener
    public void listenRbac2LoggedEvent(LoggedEvent event) {
        String msg = event.getUsername() + "@" + event.getIp() + "登录了系统";
        redisEventManager.fireAdminMsgEvent(new OfficeMsgEvent(event.getSource(), OfficeMsgEvent.MsgType.INFO, msg));
    }

}
