import { Inject, Injectable, Injector } from '@angular/core';
import { Router } from '@angular/router';
import { DA_SERVICE_TOKEN, ITokenService } from '@delon/auth';
import { _HttpClient } from '@delon/theme';
import { Observable } from 'rxjs';
import { tap } from 'rxjs/operators';

@Injectable({
  providedIn: 'root',
})
export class AuthService {
  constructor(
    private httpClient: _HttpClient,
    private injector: Injector,
    private router: Router,
    @Inject(DA_SERVICE_TOKEN) private tokenService: ITokenService,
  ) {}

  setToken(token: string): void {
    this.tokenService.set({
      token,
      expired: 3600000,
    });
  }

  requestToken(username: string, password: string): Observable<any> {
    return this.httpClient
      .post(
        '/api/auth/login?_allow_anonymous=true',
        `username=${username}&password=${password}`,
        {},
        { headers: { 'content-type': 'application/x-www-form-urlencoded' } },
      )
      .pipe(
        tap((res: any) => {
          this.setToken(res.data);
        }),
      );
  }

  logout(): void {
    this.httpClient.post(`/api/sys/auth/logout`).subscribe((value) => {
      this.tokenService.clear();
      this.router.navigateByUrl(this.tokenService.login_url!);
    });
  }

  changePwd(username: string, oldPwd: string, newPwd: string): Observable<any> {
    return this.httpClient.post(`/api/user/change-pwd/${username}/${oldPwd}/${newPwd}`);
  }

  changePwdForce(username: string, newPwd: string): Observable<any> {
    return this.httpClient.post(`/api/user/change-pwd/force/${username}/${newPwd}`);
  }

  changeCurrentUserPassword(oldPwd: string, newPwd: string): Observable<any> {
    return this.httpClient.post(`/api/user/change-pwd/current/${oldPwd}/${newPwd}`);
  }
}
