import {Inject, Injectable} from '@angular/core';
import {DA_SERVICE_TOKEN, ITokenService} from "@delon/auth";
import {ITokenModel} from "@delon/auth/src/token/interface";

import {NzMessageService} from "ng-zorro-antd/message";


declare const Stomp: any;
declare const SockJS: any;
@Injectable({
  providedIn: 'root'
})
export class StompService {

  private stompClient: any;

  private _alive: boolean = false;

  private officeWarnSub: any;


  constructor(@Inject(DA_SERVICE_TOKEN) private tokenService: ITokenService,
              private message: NzMessageService) {
  }

  public connect(url: string): void {
    this.disconnect();
    let ws = new SockJS(url);
    this.stompClient = Stomp.over(ws);

    this.stompClient.heartbeat.outgoing = 10000;
    this.stompClient.heartbeat.incoming = 10000;

    let token: ITokenModel | null = this.tokenService.get();
    this.stompClient.connect(
      {Authorization: 'Bearer ' + token!.token},
      (frame: any) => {
        console.log('Connected: ' + frame);
        this._alive = true;
        this.listenAll();
      },
      (err: any) => {
        this.message.info('ws连接断开，5秒后尝试重连');
        setTimeout(() => this.connect(url), 5000);
      }
    );
  }

  public disconnect() {
    this._alive = false;
    if(this.stompClient) {
      this.stompClient.disconnect();
    }
  }

  public alive(): boolean {
    return this._alive;
  }

  public listenOfficeWarn() {
    console.log('init listen: officeWarn');
    if(this.officeWarnSub) {
      this.officeWarnSub.unsubscribe();
      this.officeWarnSub = null;
    }
    this.stompClient.subscribe("/topic/officeWarn", (msg: any) => {
      this.message.info(`[系统警告]: ${msg.body}`)
    })
  }


  public listenAll() {
    this.listenOfficeWarn();
  }
}
