import { Component, Input, OnInit } from '@angular/core';
import { UserMenuService } from '@core';
import { ACLService } from '@delon/acl';
import { _HttpClient } from '@delon/theme';
import { ArrayService } from '@delon/util';
import { NzTreeNode } from 'ng-zorro-antd/core/tree';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzModalRef } from 'ng-zorro-antd/modal';
import { Observable, zip } from 'rxjs';

@Component({
  selector: 'app-user-fast-menu',
  templateUrl: './fast-menu.component.html',
})
export class SysFastMenuComponent implements OnInit {
  data!: any[];
  node!: NzTreeNode[];

  @Input() username!: string;

  constructor(
    private http: _HttpClient,
    private modal: NzModalRef,
    private arrSrv: ArrayService,
    private msgSrv: NzMessageService,
    private acl: ACLService,
    private userMenu: UserMenuService,
  ) {}

  ngOnInit(): void {
    this.queryMenu();
  }

  queryMenu(): void {
    const all$: Observable<any[]> = this.userMenu.all();
    const fast$: Observable<any[]> = this.userMenu.fastAll();

    zip(all$, fast$).subscribe(([all, fastCodes]) => {
      this.data = all.filter((value) => {
        try {
          let macl = JSON.parse(value.acl);
          if(Object.keys(macl).length === 0) {
            return this.acl.can(null);
          } else {
            return this.acl.can(macl);
          }
        } catch (s) {
          return true;
        }
      });

      this.data.forEach((value) => (value.shortcut = fastCodes.indexOf(value.code) !== -1));

      this.node = this.arrSrv.arrToTreeNode(this.data, {
        titleMapName: 'name',
        parentIdMapName: 'parentId',
        cb: (item, _parent, deep) => {
          item.expanded = deep <= 1;
        },
      });
    });
  }

  submit(): void {
    const codes = this.data.filter((value) => value.shortcut).map((value) => value.code);
    this.userMenu.settingFast(codes).subscribe((value) => {
      this.msgSrv.success(value ? '设置成功' : '设置失败');
      this.modal.close();
    });
  }
}
