import { ChangeDetectionStrategy, Component, Inject, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { AuthService } from '@core';
import { DA_SERVICE_TOKEN, ITokenService } from '@delon/auth';
import {SettingsService, _HttpClient} from '@delon/theme';
import { NzModalService } from 'ng-zorro-antd/modal';
import { NzNotificationService } from 'ng-zorro-antd/notification';
import { SysFastMenuComponent } from './fast-menu.component';
import {toggleSortable, sortable} from "@shared";

@Component({
  selector: 'layout-pro-user',
  templateUrl: 'user.component.html',
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class LayoutProWidgetUserComponent implements OnInit {
  oldPwd!: string;
  newPwd!: string;
  reference!: string;
  remark!: string;
  dataModule!: string;
  beginId: number = 0;
  delId: number = -1;
  module: any = {
    parentAbility: 'j9bc:page',
    parentMenu: 'pg-j9bc'
  };
  constructor(
    public settings: SettingsService,
    private router: Router,
    private modal: NzModalService,
    private notification: NzNotificationService,
    private auth: AuthService,
    private http: _HttpClient,
    @Inject(DA_SERVICE_TOKEN) private tokenService: ITokenService,
  ) {  }

  ngOnInit(): void {
    // mock
    const token = this.tokenService.get() || {
      token: 'nothing',
      name: 'Admin',
      avatar: './assets/logo-color.svg',
      email: 'cipchk@qq.com',
    };
    this.tokenService.set(token);
  }

  logout(): void {
    this.auth.logout();
  }

  refresh(): void {
    this.notification.warning('注意:', '暂未实现，要刷新用户相关的信息需要重新登录');
  }

  doChangePwd(): void {
    this.auth.changeCurrentUserPassword(this.oldPwd, this.newPwd).subscribe((value) => {
      this.notification.success('提示', '操作完成');
      this.modal.closeAll();
    });
  }

  showFastMenu(): void {
    this.modal
      .create({
        nzContent: SysFastMenuComponent,
        nzComponentParams: { username: this.settings.user['username'] },
        nzFooter: null,
        nzMaskClosable: false,
      })
      .afterClose.subscribe(() => {});
  }

  // testDubbo(): void {
  //   this.http.get(`/api/dubbo/test`).subscribe(value => {
  //     this.notification.success('result', value);
  //   });
  // }

  initNewModule(): void {
    this.http.post(`/api/admin/module/init`, this.module).subscribe(value => {
      if (value.success) {
        this.notification.success('success', 'success');
      }

    });
  }

  resetAdminAbility(): void {
    this.http.post(`/api/admin/ability/reset`).subscribe(value => {
      if (value.success) {
        this.notification.success('success', 'success');
      }
      this.modal.closeAll();
    });
  }

  encryptData() {
    this.http.post(`/api/repair/encrypt`).subscribe(value => {
      if (value.success) {
        this.notification.success('success', 'success');
      } else {
        this.notification.success('error', value.message);
      }
    });
  }

  checkEncryptData() {
    this.http.post(`/api/repair/check/encrypt-key/${this.oldPwd}`).subscribe(value => {
      if (value.success) {
        this.notification.success('success', `通过【${this.oldPwd}】密钥解密的第一个商户信息，您需要确认它的 privateKey：【${value.data.privateKey}】 是否已经解密成功！`);
      } else {
        this.notification.success('error', value.message);
      }
    });

  }

  doRepairTreasure(): void {
    this.http.post(`/api/repair/treasure/streaming?reference=${this.reference}&remark=${this.remark}` ).subscribe(value => {
      if (value.success) {
        this.notification.success('success', `通过【${this.oldPwd}】密钥解密的第一个商户信息，您需要确认它的 privateKey：【${value.data.privateKey}】 是否已经解密成功！`);
      } else {
        this.notification.success('error', value.message);
      }
    });
  }

  changeEncryptData(): void {
    this.http.post(`/api/repair/change/encrypt-key/${this.oldPwd}/${this.newPwd}`).subscribe(value => {
      if (value.success) {
        this.notification.success('success', 'success');
      } else {
        this.notification.success('error', value.message);
      }
    });

  }

  initUserServerSeed(): void {
    this.http.post(`/api/repair/init-server-seed`).subscribe(value => {
      if (value.success) {
        this.notification.success('success', 'success');
      } else {
        this.notification.success('error', value.message);
      }
    });
  }

  repairNickname(): void {
    this.http.post(`/api/repair/nickname`).subscribe(value => {
      if (value.success) {
        this.notification.success('success', 'success');
      } else {
        this.notification.success('error', value.message);
      }
    });
  }

  repairTokenCache(): void {
    this.http.post(`/api/repair/clear-token`).subscribe(value => {
      if (value.success) {
        this.notification.success('success', 'success');
      } else {
        this.notification.success('error', value.message);
      }
    });
  }

  doTransfer() {
    this.http.post(`/api/admin/data-transfer/${this.dataModule}/${this.beginId}/${this.delId}`).subscribe(value => {
      if (value.success) {
        this.notification.success('success', 'success');
      } else {
        this.notification.success('error', value.message);
      }
    });
  }

  onSortable() {
    toggleSortable();
    location.reload();
  }
}
