import { _HttpClient } from '@delon/theme';
import { _HttpHeaders } from '@delon/theme/src/services/http/http.client';
import {GridStatistics, IFilter, IGridDataSource, IPage, IRangeFilter, IRowQuery, StatisticsRow} from '@shared';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';

export class DataSourceUtils {
  static rowQuery<T, R>(http: _HttpClient, queryUrl: string, queryMap: (t: T) => R): IGridDataSource<R> {
    return ((rowQuery: IRowQuery) => DataSourceUtils.asObs(http, queryUrl, rowQuery, queryMap)) as IGridDataSource<R>;
  }

  private static asObs<T, R>(http: _HttpClient, queryUrl: string, rowQuery: IRowQuery, queryMap: (t: T) => R): Observable<IPage<R>> {
    const headers: _HttpHeaders = { 'Content-Type': 'application/row-query' };
    return http
      .post(`${queryUrl}`, JSON.stringify(rowQuery), {}, { headers })
      .pipe(map((value) => value as IPage<T>))

      .pipe(
        map((page) => {
          return {
            records: page.records.map(queryMap),
            total: page.total,
            size: page.size,
            current: page.current,
            statisticRows: page.statisticRows,
          } as IPage<R>;
        }),
      );
  }

  // private static statisticsParse(statistics: any[]): StatisticsRow<any>[] {
  //   if(!statistics) {
  //     return [];
  //   }
  //
  //   const s: GridStatistics = {
  //     func: 'sum',
  //     label: '总计',
  //     skipExport: false,
  //     className: '',
  //     fields: []
  //   };
  //   for (let statistic of statistics) {
  //     s.fields.push({
  //       field: statistic['field'],
  //       label: statistic['field'],
  //       value: statistic['value'],
  //       className: ''
  //     });
  //   }
  //
  //   return [s];
  // }

  static of<T>(query: (rowQuery: IRowQuery) => Observable<T[]>): IGridDataSource<T> {
    return ((rq) =>
      query(rq).pipe(
        map((records: T[]) => {
          return {
            records,
            total: records.length,
            size: records.length,
            current: 1,
          };
        }),
      )) as IGridDataSource<T>;
  }

  public static filtersToObj(filters: IFilter[], forceEq: boolean): any {

    const obj: any = {};
    for (const filter of filters) {
      if (forceEq && filter.option !== 'equals') {
        continue;
      }
      obj[filter.field] = filter.value;
    }

    return obj;
  }

  public static upperCaseFirstChar(field: string): string {
    return field.substr(0, 1).toUpperCase() + field.substring(1);
  }

  public static asHx1Page(rowQuery: IRowQuery, req: any): any {
    return {
      size: rowQuery.pageSize,
      current: rowQuery.pageNum,
      queries: req,
      orders: rowQuery.sorts ? rowQuery.sorts.map(value => ({
        column: value.field,
        asc: value.sort === 'asc'
      })) : []
    } as any;
  }

  public static rowQueryToHx1Page(rowQuery: IRowQuery): any {
    const req: any = {};
    rowQuery.filters.forEach(value => {
      if (value.option === 'inRange' && value.filterType === 'date') {
        const v2 = value as IRangeFilter;
        const upper = DataSourceUtils.upperCaseFirstChar(value.field);
        req['begin' + upper] = v2.value;
        req['end' + upper] = v2.valueTo;
      } else {
        req[value.field] = value.value;
      }
    });

    return DataSourceUtils.asHx1Page(rowQuery, req);

  }
}
