import { HttpClient } from '@angular/common/http';
import { Component, OnInit } from '@angular/core';
import { AbilityVO, RoleVO } from '@core';
import { SFSchema } from '@delon/form';
import { _HttpClient, Menu } from '@delon/theme';
import { ArrayService } from '@delon/util';
import { NzContextMenuService, NzDropdownMenuComponent } from 'ng-zorro-antd/dropdown';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzSelectOptionInterface } from 'ng-zorro-antd/select/select.types';
import { NzFormatBeforeDropEvent, NzFormatEmitEvent, NzTreeNode } from 'ng-zorro-antd/tree';
import { NzUploadXHRArgs } from 'ng-zorro-antd/upload';
import { mergeMap, Observable, of } from 'rxjs';
import { map, tap } from 'rxjs/operators';

import { UserMenuService } from '../../../core/menu/user-menu.service';

@Component({
  selector: 'app-sys-menu',
  templateUrl: './menu.component.html'
})
export class SysMenuComponent implements OnInit {
  private menuEvent!: NzFormatEmitEvent;

  data: NzTreeNode[] = [];
  op!: string;
  item!: Menu;
  delDisabled = false;

  roles!: NzSelectOptionInterface[];
  abilities!: NzSelectOptionInterface[];

  objectOptions = [
    {
      id: 'role',
      title: '角色',
      maxMultipleCount: 8,
      maxTagCount: 2,
      mode: 'multiple',
      options: () => of(this.roles)
    },
    {
      id: 'ability',
      title: '权限',
      maxMultipleCount: 8,
      maxTagCount: 2,
      mode: 'multiple',
      options: () => of(this.abilities)
    }
  ];
  schema: SFSchema = {
    properties: {
      text: { type: 'string', title: '名称', minLength: 1 },
      code: { type: 'string', title: '编号', minLength: 4 },
      i18n: { type: 'string', title: '国际化', minLength: 1 },
      acl: {
        type: 'string',
        title: 'ACL',
        ui: {
          widget: 'object-select',
          options: this.objectOptions
        }
      },
      reuse: { type: 'boolean', title: '复用', default: false },
      link: { type: 'string', title: '路由', minLength: 1, default: '#' },
      target: {
        type: 'string',
        title: '跳转方式',
        default: '_self',
        enum: [
          { label: '_self', title: '_self', value: '_self', checked: true },
          { label: '_blank', title: '_blank', value: '_blank' },
          { label: '_parent', title: '_parent', value: '_parent' },
          { label: '_top', title: '_top', value: '_top' }
        ]
      },
      enabled: { type: 'boolean', title: '启用', default: true },
      hideInBreadcrumb: { type: 'boolean', title: '隐藏导航', default: false },
      shortcutRoot: { type: 'boolean', title: '根快捷菜单', default: false },
      icon: { type: 'string', title: '图标', default: '' },
      sortRank: { type: 'number', title: '排序值', default: 1 }
    },
    required: ['text'],
    ui: { grid: { md: 24, lg: 12 }, spanLabelFixed: 100 }
  };

  objectSelect = {};

  uploading = false;
  // fileList: NzUploadFile[] = []

  constructor(
    private http: _HttpClient,
    private http0: HttpClient,
    private ccSrv: NzContextMenuService,
    private arrSrv: ArrayService,
    private msg: NzMessageService,
    private userMenu: UserMenuService
  ) {}

  ngOnInit(): void {
    this.getData();

    this.http
      .get(`/api/role/menus`)
      .pipe(map(value => value.data))
      .subscribe((roles: RoleVO[]) => {
        this.roles = roles.map(role => ({ label: role.name, value: role.role }));
      });
    this.http
      .get(`/api/sys/ability/menus`)
      .pipe(map(value => value.data))
      .subscribe((abilities: AbilityVO[]) => {
        this.abilities = abilities.map(ability => ({ label: ability.name, value: ability.ability }));
      });
  }

  objectSelectChange(value: any): void {
    console.log(value);
  }

  private getData(): void {
    // https://ng-alain.com/util/array/zh?#arrToTree
    this.userMenu.all().subscribe((res: Menu[]) => {
      res = res.map(item => {
        if (item.acl) {
          // ).replace('{ability', '{"ability"').replaceAll('\'', '"')
          item.acl = JSON.parse(item.acl as string);
        } else {
          item.acl = {
            role: undefined,
            ability: undefined
          };
        }
        return item;
      });
      this.data = this.arrSrv.arrToTreeNode(res, {
        titleMapName: 'text',
        parentIdMapName: 'parentId',
        cb: (item, _parent, deep) => {
          item.expanded = deep <= 1;
        }
      });
    });
  }

  add(item: any): void {
    this.closeContextMenu();
    this.op = 'edit';
    this.item = {
      id: 0,
      text: '',
      parentId: item ? item.id : 0
    };
  }

  edit(): void {
    this.closeContextMenu();
    this.op = 'edit';
  }

  save(item: any): void {
    item.externalLink = '';

    // 先转为对象，后面用于过滤无效的权限
    if (typeof item.acl === 'string') {
      item.acl = JSON.parse(item.acl);
    }

    item.acl.role = this.filterRole(item.acl.role);
    item.acl.ability = this.filterAbility(item.acl.ability);

    if (typeof item.acl === 'object') {
      item.acl = JSON.stringify(item.acl);
    }

    this.http.post(`/api/sys/menu/${item.id ? 'update' : 'create'}`, item).subscribe(next => {
      if (!next.success) {
        this.msg.error(next.message);
        return;
      }
      if (item.id <= 0) {
        // next.title = next.text;
        // this.menuEvent.node!.addChildren([next]);
        this.getData();
        this.op = '';
      } else {
        this.item = item;
        this.menuEvent.node!.title = item.text;
        this.menuEvent.node!.origin = item;
        this.op = 'view';
      }
    });
  }

  del(): void {
    this.closeContextMenu();
    this.http.delete(`/api/sys/menu/delete/${this.item['id']}`).subscribe(() => {
      this.menuEvent.node!.remove();
      this.op = '';
      this.getData();
    });
  }

  get delMsg(): string {
    if (!this.menuEvent) {
      return '';
    }
    const childrenLen = this.menuEvent.node!.children.length;
    if (childrenLen === 0) {
      return `确认删除【${this.menuEvent.node!.title}】吗？`;
    }
    return `确认删除【${this.menuEvent.node!.title}】以及所有子菜单吗？`;
  }

  move = (e: NzFormatBeforeDropEvent) => {
    // if (e.pos !== 0) {
    //   this.msg.warning(`只支持菜单不同类目的移动，且无法移动至顶层`);
    //   return of(false);
    // }

    if (e.dragNode.origin['parent_id'] === e.node.origin['id']) {
      return of(false);
    }
    const from = e.dragNode.origin['id'];
    const to = e.node.origin['id'];
    return this.http
      .post('/api/sys/menu/move', {
        from,
        to,
        pos: e.pos
      })
      .pipe(
        tap(() => {
          this.op = '';
        }),
        map(data => data)
      );
    // tslint:disable-next-line: semicolon
  };

  show(e: NzFormatEmitEvent): void {
    this.op = e.node!.isSelected ? 'view' : '';
    this.item = e.node!.origin as any;
  }

  showContextMenu(e: NzFormatEmitEvent, menu: NzDropdownMenuComponent): void {
    this.menuEvent = e;

    // 如果不希望直接批量删除子节点，则解除这个注释
    // this.delDisabled = e.node!.children.length !== 0;
    this.ccSrv.create(e.event!, menu);
  }

  closeContextMenu(): void {
    this.ccSrv.close();
  }

  private filterRole(array: any[]): any[] {
    if (array && array.length > 0) {
      const v: string[] = this.roles.map(value => value.value);
      return array.filter(value => v.indexOf(value) !== -1);
    } else {
      return array;
    }
  }

  private filterAbility(array: any[]): any[] {
    if (array && array.length > 0) {
      const v: string[] = this.abilities.map(value => value.value);
      return array.filter(value => v.indexOf(value) !== -1);
    } else {
      return array;
    }
  }

  export(item: Menu): void {
    this.http.get(`/api/sys/menu/export/${item.key}`, {}).subscribe(value => {
      // console.log(JSON.parse(value.data));
      this.download(item.text!, value.data);
    });
  }

  exportAll(): void {
    this.http.get(`/api/sys/menu/export`, {}).subscribe(value => {
      this.download('root', value.data);
    });
  }

  download(filename: string, txt: string): void {
    const file = new File([txt], `${filename}.json`, {
      type: 'text/plain'
    });
    const tmpLink = document.createElement('a');
    const objectUrl = URL.createObjectURL(file);

    tmpLink.href = objectUrl;
    tmpLink.download = file.name;
    document.body.appendChild(tmpLink);
    tmpLink.click();

    document.body.removeChild(tmpLink);
    URL.revokeObjectURL(objectUrl);
  }

  nzCustomRequest(id: string = '') {
    return (item: NzUploadXHRArgs) => {
      this.uploading = true;

      const observable = new Observable(sub => {
        const reader = new FileReader();
        reader.onload = ev => {
          sub.next(reader.result);
          sub.complete();
        };
        reader.readAsText(item.file as any);
      });

      return observable.pipe(mergeMap(value => this.http.post(item.action!, { id, json: value }))).subscribe(
        value => {
          this.uploading = false;
          if (value.success) {
            this.msg.success('上传成功');
          } else {
            this.msg.error(value.message);
          }
          this.getData();
        },
        () => {
          this.uploading = false;
        }
      );
    };
  }

  rebuildPath(item: any) {
    this.http.post(`/api/sys/menu/rebuild-path/${item.id}`, {}).subscribe(value => {
      if (value.success) {
        this.msg.success('重建成功');
        this.getData();
      } else {
        this.msg.error(value.message);
      }
    });
  }
}
