import { Component, Input, OnInit, ViewChild } from '@angular/core';
import { AbilityCheckInfo, AbilityVO, RoleAbilities, RoleVO } from '@core';
import { ACLService } from '@delon/acl';
import { _HttpClient } from '@delon/theme';

import { ArrayService } from '@delon/util';
import { NzTreeNode, NzTreeNodeOptions } from 'ng-zorro-antd/core/tree/nz-tree-base-node';
import { NzTreeComponent } from 'ng-zorro-antd/tree';
import { of } from 'rxjs';
import {map} from "rxjs/operators";

@Component({
  selector: 'app-sys-role-assign-ability-tree',
  templateUrl: './assign-ability-tree.component.html',
  styleUrls: ['./assign-ability-tree.component.less'],
})
export class SysRoleAssignAbilityTreeComponent implements OnInit {
  nodes!: NzTreeNodeOptions[] | NzTreeNode[];

  @Input()
  role!: RoleVO;

  defaultCheckedKeys: string[] = [];
  defaultExpandedKeys: string[] = [];
  defaultSelectedKeys: string[] = [];

  abilities: Array<AbilityVO> = [];

  @ViewChild('tree', { static: true }) tree!: NzTreeComponent;

  constructor(private http: _HttpClient, private aclService: ACLService, private arrayService: ArrayService) {}

  ngOnInit(): void {
    this.refresh();
  }

  refresh(): void {
    const can = this.aclService.canAbility('role:assign');

    const roleAbilities$ = can ? this.http.get(`/api/role/${this.role.id}/abilities/full`).pipe(map(value => value.data)) : of([]);

    roleAbilities$.subscribe((abilities) => {
      this.abilities = abilities;
      this.nodes = this.asNode(abilities);
    });
  }

  asNode(abilities: AbilityVO[]): NzTreeNodeOptions[] | NzTreeNode[] {
    return this.arrayService.arrToTreeNode(abilities, {
      titleMapName: 'name',
      parentIdMapName: 'parentId',
      checkedMapname: 'checked',
      disabledMapname: 'disableOpt',
      cb: (item, _parent, deep) => {
        item.expanded = deep <= 1;
      },
    });
  }

  getNewData(): RoleAbilities {
    const ids: number[] = this.arrayService.getKeysByTreeNode(this.nodes as NzTreeNode[]);

    const abilities = this.abilities
      .filter((value) => ids.indexOf(value.id) !== -1)
      .map(
        (value) =>
          ({
            abilityId: value.id,
            licensable: value.licensable,
            checked: true,
            v: value
          } as AbilityCheckInfo),
      );

    return {
      roleId: this.role.id,
      abilities,
    };
  }

  getChildren(node: NzTreeNode): Array<NzTreeNode> {
    let nodes: Array<NzTreeNode> = [];
    node.children.forEach((item: any) => {
      nodes.push(item);
      if (item.children && item.children.length) {
        nodes = nodes.concat(this.getChildren(item));
      }
    });
    return nodes;
  }
}
