import { Component, OnInit } from '@angular/core';
import { SFSchema } from '@delon/form';
import { _HttpClient } from '@delon/theme';
import { ScheduleStatus } from '@shared';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzModalRef } from 'ng-zorro-antd/modal';

@Component({
  selector: 'app-create-scheduler',
  templateUrl: './create.component.html'
})
export class SchedulerCreateComponent implements OnInit {
  rolePrefix = 'scheduler:';

  // test : {
  //   type: 'string',
  //   title: '测试',
  //   pattern: '^\\w+$',
  //   ui: {
  //     widget: 'string',
  //     placeholder: '唯一标识,不可重复,由字母,下划线,数字组合',
  //     errors: {
  //       pattern: '仅支持输入字母、下划线、数字'
  //     }
  //   }
  // },

  schema: SFSchema = {
    properties: {
      jobName: { type: 'string', title: '任务名称' },
      jobGroup: { type: 'string', title: '分组名称' },
      jobClass: { type: 'string', title: '任务类全名' },
      cron: { type: 'string', title: 'Cron表达式' },
      description: {
        type: 'string',
        title: '任务描述',
        ui: {
          widget: 'textarea',
          autosize: { minRows: 2, maxRows: 10 },
          placeholder: '添加描述'
        }
      },
      params: { type: 'string', title: '执行参数' },
      // 所有未知的复杂类型都当作枚举处理
      state: {
        type: 'string',
        title: '任务状态',
        enum: ScheduleStatus,
        ui: { placeholder: '任务状态', widget: 'select' }
      }
    },
    required: ['jobName', 'jobGroup', 'jobClass', 'cron'],
    ui: {
      spanLabelFixed: 100,
      grid: { span: 24 }
    }
  };

  constructor(private modal: NzModalRef, private msgSrv: NzMessageService, public http: _HttpClient) {}

  ngOnInit(): void {}

  save(value: any): void {
    this.http.post(`/api/sys/scheduler/create`, value).subscribe(res => {
      this.modal.close(res);
    });
  }

  close(): void {
    this.modal.destroy();
  }
}
