import { Component, OnInit } from '@angular/core';
import { SFSchema } from '@delon/form';
import { _HttpClient } from '@delon/theme';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzModalRef } from 'ng-zorro-antd/modal';

@Component({
  selector: 'app-create-acme-account',
  templateUrl: './create.component.html'
})
export class AcmeAccountCreateComponent implements OnInit {
  rolePrefix = 'acme-account:';

  schema: SFSchema = {
    properties: {
      account: { type: 'string', title: 'ACME账号' },
      contactEmail: { type: 'string', title: '联系email' },
      remark: {
        type: 'string',
        title: '备注说明',
        ui: {
          widget: 'textarea',
          autosize: { minRows: 2, maxRows: 10 },
          placeholder: '添加描述'
        }
      }
    },
    required: ['account', 'contactEmail', 'remark'],
    ui: {
      spanLabelFixed: 100,
      grid: { span: 24 }
    }
  };

  constructor(
    private modal: NzModalRef,
    private msgSrv: NzMessageService,

    public http: _HttpClient
  ) {}

  ngOnInit(): void {}

  save(value: any): void {
    this.http.post(`/api/star-ring/acme-account/create`, value).subscribe(res => {
      this.msgSrv.success('添加成功');
      this.modal.close(res);
    });
  }

  close(): void {
    this.modal.destroy();
  }
}
