import { Component, Input, OnInit } from '@angular/core';
import { SFSchema } from '@delon/form';
import { NzModalRef } from 'ng-zorro-antd/modal';
import { ACME_ACCOUNT_STATUS } from '../../seo.enums';

@Component({
  selector: 'app-view-acme-account',
  templateUrl: './view.component.html',
})
export class AcmeAccountViewComponent implements OnInit {
  @Input()
  record: any;

  schema: SFSchema = {
    properties: {
      account: { type: 'string', title: 'ACME账号', readOnly: true },
      initialIp: { type: 'string', title: '初始化IP', readOnly: true },
      provider: { type: 'string', title: 'ACME提供商', readOnly: true },
      // 所有未知的复杂类型都当作枚举处理
      status: {
        type: 'string',
        title: '状态',
        enum: ACME_ACCOUNT_STATUS,
        readOnly: true,
        ui: { placeholder: '状态', widget: 'select' },
      },
      location: { type: 'string', title: 'location', readOnly: true },
      contactEmail: { type: 'string', title: '联系email', readOnly: true },
      remark: {
        type: 'string',
        title: '备注说明',
        readOnly: true,
        ui: {
          widget: 'textarea',
          autosize: { minRows: 2, maxRows: 10 },
          placeholder: '添加描述',
        },
      },
    },
    ui: {
      spanLabelFixed: 100,
      grid: { span: 24 },
    },
  };

  constructor(private modal: NzModalRef) {}

  ngOnInit(): void {}

  close(): void {
    this.modal.destroy();
  }
}
