import { Component, OnInit } from '@angular/core';
import { SFSchema } from '@delon/form';
import { _HttpClient } from '@delon/theme';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzModalRef } from 'ng-zorro-antd/modal';
import { ACME_ORDER_STATUS } from '../../seo.enums';

@Component({
  selector: 'app-batch-create-acme-order',
  templateUrl: './batch-create.component.html',
})
export class AcmeOrderBatchCreateComponent implements OnInit {
  rolePrefix = 'acme-order:';

  schema: SFSchema = {
    properties: {
      account: { type: 'string', title: 'acme账号' },
      domains: {
        type: 'string',
        title: '顶级域名',
        ui: {
          widget: 'textarea',
          autosize: { minRows: 4, maxRows: 10 },
          placeholder: '顶级域名，一行一个',
        },
      },
      secs: { type: 'string', title: '二级域名', ui: { placeholder: `二级域名,如'www'多个用逗号隔开` } },
    },
    required: ['account', 'orderReference', 'domain', 'sec'],
    ui: {
      spanLabelFixed: 100,
      grid: { span: 24 },
    },
  };

  constructor(
    private modal: NzModalRef,
    private msgSrv: NzMessageService,

    public http: _HttpClient,
  ) {}

  ngOnInit(): void {}

  save(value: any): void {
    const body = {
      account: value.account,
      domains: value.domains.replace('\r', '').split('\n'),
      secs: value.secs.replace('，', ',').split(','),
    };
    this.http.post(`/api/star-ring/acme/order/batch-create`, body).subscribe((res) => {
      this.msgSrv.success('添加成功');
      this.modal.close(res);
    });
  }

  close(): void {
    this.modal.destroy();
  }
}
