import { Component, Input, OnInit } from '@angular/core';
import { SFSchema } from '@delon/form';
import { _HttpClient } from '@delon/theme';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzModalRef } from 'ng-zorro-antd/modal';
import { ACME_ORDER_STATUS } from '../../seo.enums';

@Component({
  selector: 'app-edit-acme-order',
  templateUrl: './edit.component.html',
})
export class AcmeOrderEditComponent implements OnInit {
  rolePrefix = 'acme-order:';

  @Input()
  record: any;

  schema: SFSchema = {
    properties: {
      account: { type: 'string', title: 'acme账号' },
      // 所有未知的复杂类型都当作枚举处理
      status: {
        type: 'string',
        title: '状态',
        enum: ACME_ORDER_STATUS,
        ui: { placeholder: '状态', widget: 'select' },
      },
      provider: { type: 'string', title: 'acme提供者' },
      orderReference: { type: 'string', title: '订单(可以用域名)' },
      domain: { type: 'string', title: '顶级域名' },
      sec: { type: 'string', title: '二级域名' },
    },
    required: ['account', 'status', 'provider', 'orderReference', 'domain', 'sec'],
    ui: {
      spanLabelFixed: 100,
      grid: { span: 24 },
    },
  };

  constructor(
    private modal: NzModalRef,
    private msgSrv: NzMessageService,

    public http: _HttpClient,
  ) {}

  ngOnInit(): void {}

  save(value: any): void {
    this.http.post(`/api/star-ring/acme/order/update`, value).subscribe((res) => {
      this.msgSrv.success('修改成功');
      this.modal.close(res);
    });
  }

  close(): void {
    this.modal.destroy();
  }
}
