import { Component, Input, OnInit } from '@angular/core';
import { SFSchema } from '@delon/form';
import { _HttpClient } from '@delon/theme';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzModalRef } from 'ng-zorro-antd/modal';
import { DOMAIN_TYPE } from '../../seo.enums';

@Component({
  selector: 'app-edit-article',
  templateUrl: './edit.component.html',
})
export class ArticleEditComponent implements OnInit {
  rolePrefix = 'article:';

  @Input()
  record: any;

  schema: SFSchema = {
    properties: {
      title: {
        type: 'string',
        title: '标题',
      },

      used: {
        type: 'boolean',
        title: '是否使用',
        enum: [
          { label: '是', value: true },
          { label: '否', value: false },
        ],
        default: true,
        ui: { placeholder: '是否启用', widget: 'select' },
      },
      owner: {
        type: 'string',
        title: '所属网站',
      },
      menu: {
        type: 'boolean',
        title: '是否为菜单',
        enum: [
          { label: '是', value: true },
          { label: '否', value: false },
        ],
        default: true,
        ui: { placeholder: '是否启用', widget: 'select' },
      },

      // 所有未知的复杂类型都当作枚举处理
      type: {
        type: 'string',
        title: '类型',
        enum: DOMAIN_TYPE,
        ui: { placeholder: '类型', widget: 'select' },
      },
      article: {
        type: 'string',
        title: '文章',
        ui: {
          widget: 'textarea',
          autosize: { minRows: 2, maxRows: 10 },
          placeholder: '文章',
        },
      },
    },
    required: ['title', 'article', 'used', 'menu', 'type'],
    ui: {
      spanLabelFixed: 100,
      grid: { span: 24 },
    },
  };

  constructor(
    private modal: NzModalRef,
    private msgSrv: NzMessageService,

    public http: _HttpClient,
  ) {}

  ngOnInit(): void {}

  save(value: any): void {
    this.http.post(`/api/star-ring/article/update`, value).subscribe((res) => {
      this.msgSrv.success('修改成功');
      this.modal.close(res);
    });
  }

  close(): void {
    this.modal.destroy();
  }
}
