import { Component, Input, OnInit } from '@angular/core';
import { SFSchema } from '@delon/form';
import { NzModalRef } from 'ng-zorro-antd/modal';
import { DOMAIN_TYPE } from '../../seo.enums';

@Component({
  selector: 'app-view-article',
  templateUrl: './view.component.html',
})
export class ArticleViewComponent implements OnInit {
  @Input()
  record: any;

  schema: SFSchema = {
    properties: {
      title: {
        type: 'string',
        title: '标题',
        readOnly: true,
      },
      used: {
        type: 'boolean',
        title: '是否使用',
        enum: [
          { label: '是', value: true },
          { label: '否', value: false },
        ],
        default: true,
        readOnly: true,
        ui: { placeholder: '是否启用', widget: 'select' },
      },
      owner: {
        type: 'string',
        title: '所属网站',
        pattern: '^\\w+$',
        readOnly: true,
        ui: {
          widget: 'string',
        },
      },
      menu: {
        type: 'boolean',
        title: '是否为菜单',
        enum: [
          { label: '是', value: true },
          { label: '否', value: false },
        ],
        default: true,
        readOnly: true,
        ui: { placeholder: '是否启用', widget: 'select' },
      },
      // 所有未知的复杂类型都当作枚举处理
      type: {
        type: 'array',
        title: '类型',
        enum: DOMAIN_TYPE,
        default: 0,
        readOnly: true,
        ui: { placeholder: '类型', widget: 'select' },
      },
      md5: {
        type: 'string',
        title: 'md5',
        pattern: '^\\w+$',
        readOnly: true,
        ui: {
          widget: 'string',
        },
      },
      article: {
        type: 'string',
        title: '文章',
        pattern: '^\\w+$',
        readOnly: true,
        ui: {
          widget: 'textarea',
          autosize: { minRows: 2, maxRows: 10 },
        },
      },
    },
    ui: {
      spanLabelFixed: 100,
      grid: { span: 24 },
    },
  };

  constructor(private modal: NzModalRef) {}

  ngOnInit(): void {}

  close(): void {
    this.modal.destroy();
  }
}
