import { Component, Input, OnInit } from '@angular/core';
import { SFSchema } from '@delon/form';
import { _HttpClient } from '@delon/theme';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzModalRef } from 'ng-zorro-antd/modal';

@Component({
  selector: 'app-edit-domain-fetch-log',
  templateUrl: './edit.component.html',
})
export class DomainFetchLogEditComponent implements OnInit {
  rolePrefix = 'domain-fetch-log:';

  @Input()
  record: any;

  schema: SFSchema = {
    properties: {
      domain: {
        type: 'string',
        title: '域名',
      },
      times: { type: 'number', title: '抓取次数' },
      logDate: {
        type: 'string',
        title: '更新时间',
        format: 'date',
      },
      internal: {
        type: 'boolean',
        title: '内部网站',
        enum: [
          { label: '是', value: true },
          { label: '否', value: false },
        ],
        default: true,
        ui: { placeholder: '是否启用', widget: 'select' },
      },
    },
    required: ['domain', 'times', 'log_date', 'internal'],
    ui: {
      spanLabelFixed: 100,
      grid: { span: 24 },
    },
  };

  constructor(
    private modal: NzModalRef,
    private msgSrv: NzMessageService,

    public http: _HttpClient,
  ) {}

  ngOnInit(): void {}

  save(value: any): void {
    this.http.post(`/api/star-ring/domain-fetch-log/update`, value).subscribe((res) => {
      this.msgSrv.success('修改成功');
      this.modal.close(res);
    });
  }

  close(): void {
    this.modal.destroy();
  }
}
