import { Component, OnInit } from '@angular/core';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzModalRef } from 'ng-zorro-antd/modal';
import { _HttpClient } from '@delon/theme';
import { SFSchema } from '@delon/form';
import { TASK_STATUS } from '../../seo.enums';

@Component({
  selector: 'app-create-download-task',
  templateUrl: './create.component.html',
})
export class DownloadTaskCreateComponent implements OnInit {
  rolePrefix = 'download-task:';

  schema: SFSchema = {
    properties: {
      website: {
        type: 'string',
        title: '资源站点',
      },
      url: {
        type: 'string',
        title: '资源url',
      },
      savePath: {
        type: 'string',
        title: '资源保存地址',
      },
      // 所有未知的复杂类型都当作枚举处理
      state: {
        type: 'string',
        title: '状态',
        enum: TASK_STATUS,
        default: 0,
        ui: { placeholder: '状态', widget: 'select' },
      },
      error: {
        type: 'string',
        title: '错误',
      },
      failTimes: { type: 'number', title: '下载错误次数' },
    },
    required: ['website', 'url', 'save_path', 'state', 'error', 'fail_times'],
    ui: {
      spanLabelFixed: 100,
      grid: { span: 24 },
    },
  };

  constructor(
    private modal: NzModalRef,
    private msgSrv: NzMessageService,

    public http: _HttpClient,
  ) {}

  ngOnInit(): void {}

  save(value: any) {
    this.http.post(`/api/star-ring/download-task/create`, value).subscribe((res) => {
      this.msgSrv.success('添加成功');
      this.modal.close(res);
    });
  }

  close() {
    this.modal.destroy();
  }
}
