import { Component, Input, OnInit } from '@angular/core';
import { SFSchema } from '@delon/form';
import { _HttpClient } from '@delon/theme';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzModalRef } from 'ng-zorro-antd/modal';
import { PLATFORM_TYPE } from '../../seo.enums';

@Component({
  selector: 'app-edit-platform-check-content',
  templateUrl: './edit.component.html',
})
export class PlatformCheckContentEditComponent implements OnInit {
  rolePrefix = 'platform-check-content:';

  @Input() record: any;

  schema: SFSchema = {
    properties: {
      // 所有未知的复杂类型都当作枚举处理
      platformType: {
        type: 'string',
        title: '平台',
        enum: PLATFORM_TYPE,
        default: 0,
        ui: { placeholder: '平台', widget: 'select' },
      },
      domain: {
        type: 'string',
        title: '域名',
      },
      checkContent: {
        type: 'string',
        title: '检测内容',
      },
      username: {
        type: 'string',
        title: '账号',
      },
      submitted: {
        type: 'boolean',
        title: '提交成功',
        enum: [
          { label: '是', value: true },
          { label: '否', value: false },
        ],
        default: true,
        ui: { placeholder: '是否启用', widget: 'select' },
      },
      result: {
        type: 'string',
        title: '返回结果',
      },
      temp: {
        type: 'string',
        title: '缓存1',
      },
      temp2: {
        type: 'string',
        title: '缓存2',
      },
      linkPublished: {
        type: 'boolean',
        title: '已发布',
        enum: [
          { label: '是', value: true },
          { label: '否', value: false },
        ],
        default: true,
        ui: { placeholder: '是否启用', widget: 'select' },
      },
    },
    required: ['platform_type', 'domain', 'username', 'submitted', 'link_published'],
    ui: {
      spanLabelFixed: 100,
      grid: { span: 24 },
    },
  };

  constructor(
    private modal: NzModalRef,
    private msgSrv: NzMessageService,

    public http: _HttpClient,
  ) {}

  ngOnInit(): void {}

  save(value: any): void {
    this.http.post(`/api/star-ring/platform-check-content/update`, value).subscribe((res) => {
      this.msgSrv.success('修改成功');
      this.modal.close(res);
    });
  }

  close(): void {
    this.modal.destroy();
  }
}
