import { Component, Input, OnInit } from '@angular/core';
import { SFSchema } from '@delon/form';
import { NzModalRef } from 'ng-zorro-antd/modal';
import { PLATFORM_TYPE } from '../../seo.enums';

@Component({
  selector: 'app-view-platform-check-content',
  templateUrl: './view.component.html',
})
export class PlatformCheckContentViewComponent implements OnInit {
  @Input()
  record: any;

  schema: SFSchema = {
    properties: {
      // 所有未知的复杂类型都当作枚举处理
      platformType: {
        type: 'string',
        title: '平台',
        enum: PLATFORM_TYPE,
        default: 0,
        readOnly: true,
        ui: { placeholder: '平台', widget: 'select' },
      },
      domain: {
        type: 'string',
        title: '域名',
        readOnly: true,
      },
      checkContent: {
        type: 'string',
        title: '检测内容',
        readOnly: true,
      },
      username: {
        type: 'string',
        title: '账号',
        readOnly: true,
      },
      submitted: {
        type: 'boolean',
        title: '提交成功',
        enum: [
          { label: '是', value: true },
          { label: '否', value: false },
        ],
        default: true,
        readOnly: true,
        ui: { placeholder: '是否启用', widget: 'select' },
      },
      result: {
        type: 'string',
        title: '返回结果',
        readOnly: true,
      },
      temp: {
        type: 'string',
        title: '缓存1',
        readOnly: true,
      },
      temp2: {
        type: 'string',
        title: '缓存2',
        readOnly: true,
      },
      linkPublished: {
        type: 'boolean',
        title: '已发布',
        enum: [
          { label: '是', value: true },
          { label: '否', value: false },
        ],
        default: true,
        readOnly: true,
        ui: { placeholder: '是否启用', widget: 'select' },
      },
    },
    ui: {
      spanLabelFixed: 100,
      grid: { span: 24 },
    },
  };

  constructor(private modal: NzModalRef) {}

  ngOnInit(): void {}

  close(): void {
    this.modal.destroy();
  }
}
