import { Component, OnInit } from '@angular/core';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzModalRef } from 'ng-zorro-antd/modal';
import { _HttpClient } from '@delon/theme';
import { SFSchema } from '@delon/form';
import { PLATFORM_TYPE } from '../../seo.enums';

@Component({
  selector: 'app-create-platform-cookie',
  templateUrl: './create.component.html',
})
export class PlatformCookieCreateComponent implements OnInit {
  rolePrefix = 'platform-cookie:';

  schema: SFSchema = {
    properties: {
      // 所有未知的复杂类型都当作枚举处理
      platformType: {
        type: 'string',
        title: '平台',
        enum: PLATFORM_TYPE,
        default: 0,
        ui: { placeholder: '平台', widget: 'select' },
      },
      cookie: {
        type: 'string',
        title: 'cookie',
      },
      username: {
        type: 'string',
        title: '账号',
      },
      valid: {
        type: 'boolean',
        title: '是否检验',
        enum: [
          { label: '是', value: true },
          { label: '否', value: false },
        ],
        default: true,
        ui: { placeholder: '是否启用', widget: 'select' },
      },
      auto: {
        type: 'boolean',
        title: '是否自动',
        enum: [
          { label: '是', value: true },
          { label: '否', value: false },
        ],
        default: true,
        ui: { placeholder: '是否启用', widget: 'select' },
      },
    },
    required: ['platformType', 'cookie', 'username', 'valid', 'auto'],
    ui: {
      spanLabelFixed: 100,
      grid: { span: 24 },
    },
  };

  constructor(
    private modal: NzModalRef,
    private msgSrv: NzMessageService,

    public http: _HttpClient,
  ) {}

  ngOnInit(): void {}

  save(value: any) {
    this.http.post(`/api/star-ring/platform-cookie/create`, value).subscribe((res) => {
      this.msgSrv.success('添加成功');
      this.modal.close(res);
    });
  }

  close() {
    this.modal.destroy();
  }
}
