import { Component, Input, OnInit } from '@angular/core';
import { NzModalRef } from 'ng-zorro-antd/modal';
import { SFSchema } from '@delon/form';
import { PLATFORM_TYPE } from '../../seo.enums';

@Component({
  selector: 'app-view-platform-cookie',
  templateUrl: './view.component.html',
})
export class PlatformCookieViewComponent implements OnInit {
  @Input()
  record: any;

  schema: SFSchema = {
    properties: {
      // 所有未知的复杂类型都当作枚举处理
      platformType: {
        type: 'string',
        title: '平台',
        enum: PLATFORM_TYPE,
        default: 0,
        readOnly: true,
        ui: { placeholder: '平台', widget: 'select' },
      },
      cookie: {
        type: 'string',
        title: '是否有效',
        readOnly: true,
      },
      username: {
        type: 'string',
        title: '账号',
        readOnly: true,
      },
      valid: {
        type: 'boolean',
        title: '是否检验',
        enum: [
          { label: '是', value: true },
          { label: '否', value: false },
        ],
        default: true,
        readOnly: true,
        ui: { placeholder: '是否启用', widget: 'select' },
      },
      auto: {
        type: 'boolean',
        title: '是否自动',
        enum: [
          { label: '是', value: true },
          { label: '否', value: false },
        ],
        default: true,
        readOnly: true,
        ui: { placeholder: '是否启用', widget: 'select' },
      },
    },
    ui: {
      spanLabelFixed: 100,
      grid: { span: 24 },
    },
  };

  constructor(private modal: NzModalRef) {}

  ngOnInit(): void {}

  close(): void {
    this.modal.destroy();
  }
}
