import { Component, Input, OnInit } from '@angular/core';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzModalRef } from 'ng-zorro-antd/modal';
import { _HttpClient } from '@delon/theme';
import { SFSchema } from '@delon/form';
import { DOMAIN_TYPE, PROPOSAL_STATUS } from '../../seo.enums';

@Component({
  selector: 'app-edit-seo-domain-proposal',
  templateUrl: './edit.component.html',
})
export class SeoDomainProposalEditComponent implements OnInit {
  rolePrefix = 'seo-domain-proposal:';

  @Input()
  record: any;

  schema: SFSchema = {
    properties: {
      reference: {
        type: 'string',
        title: '订单',
        readOnly: true,
      },
      domain: {
        type: 'string',
        title: '域名',
      },
      templateWebsite: {
        type: 'string',
        title: '站点模板',
      },
      title: {
        type: 'string',
        title: 'title',
      },
      keywords: {
        type: 'string',
        title: '关键词列表',
      },
      listPageTitle: {
        type: 'string',
        title: 'list页面的title',
      },
      otherMenus: {
        type: 'string',
        title: '其他菜单名，用逗号隔开',
      },
      mainKey: {
        type: 'string',
        title: '主关键词',
      },
      // 所有未知的复杂类型都当作枚举处理
      type: {
        type: 'string',
        title: '类型',
        enum: DOMAIN_TYPE,
        default: 0,
        ui: { placeholder: '类型', widget: 'select' },
      },
      description: {
        type: 'string',
        title: '网站描述',
        ui: {
          widget: 'textarea',
          autosize: { minRows: 2, maxRows: 10 },
          placeholder: '添加描述',
        },
      },
      // 所有未知的复杂类型都当作枚举处理
      state: {
        type: 'string',
        title: '状态',
        enum: PROPOSAL_STATUS,
        default: 0,
        ui: { placeholder: '状态', widget: 'select' },
      },
      remark: {
        type: 'string',
        title: '审核备注',
        ui: {
          widget: 'textarea',
          autosize: { minRows: 2, maxRows: 10 },
          placeholder: '添加描述',
        },
      },
      serverTag: {
        type: 'string',
        title: '服务器标记',
      },
    },
    required: [
      'domain',
      'template_website',
      'title',
      'keywords',
      'list_page_title',
      'other_menus',
      'main_key',
      'type',
      'state',
      'server_tag',
    ],
    ui: {
      spanLabelFixed: 100,
      grid: { span: 24 },
    },
  };

  constructor(
    private modal: NzModalRef,
    private msgSrv: NzMessageService,

    public http: _HttpClient,
  ) {}

  ngOnInit(): void {}

  save(value: any): void {
    this.http.post(`/api/star-ring/seo-domain-proposal/update`, value).subscribe((res) => {
      this.msgSrv.success('修改成功');
      this.modal.close(res);
    });
  }

  close(): void {
    this.modal.destroy();
  }
}
