import { Component, Input, OnInit } from '@angular/core';
import { NzModalRef } from 'ng-zorro-antd/modal';
import { SFSchema } from '@delon/form';

@Component({
  selector: 'app-view-seo-domain-proposal',
  templateUrl: './view.component.html',
})
export class SeoDomainProposalViewComponent implements OnInit {
  @Input()
  record: any;

  schema: SFSchema = {
    properties: {
      reference: {
        type: 'string',
        title: '',
        pattern: '^\\w+$',
        readOnly: true,
        ui: {
          widget: 'string',
        },
      },
      domain: {
        type: 'string',
        title: '域名',
        pattern: '^\\w+$',
        readOnly: true,
        ui: {
          widget: 'string',
        },
      },
      templateWebsite: {
        type: 'string',
        title: '站点模板',
        pattern: '^\\w+$',
        readOnly: true,
        ui: {
          widget: 'string',
        },
      },
      title: {
        type: 'string',
        title: 'title',
        pattern: '^\\w+$',
        readOnly: true,
        ui: {
          widget: 'string',
        },
      },
      keywords: {
        type: 'string',
        title: '关键词列表',
        pattern: '^\\w+$',
        readOnly: true,
        ui: {
          widget: 'string',
        },
      },
      listPageTitle: {
        type: 'string',
        title: 'list页面的title',
        pattern: '^\\w+$',
        readOnly: true,
        ui: {
          widget: 'string',
        },
      },
      otherMenus: {
        type: 'string',
        title: '其他菜单名，用逗号隔开',
        pattern: '^\\w+$',
        readOnly: true,
        ui: {
          widget: 'string',
        },
      },
      mainKey: {
        type: 'string',
        title: '主关键词',
        pattern: '^\\w+$',
        readOnly: true,
        ui: {
          widget: 'string',
        },
      },
      // 所有未知的复杂类型都当作枚举处理
      type: {
        type: 'array',
        title: '类型',
        enum: [
          { label: 'v0', value: 0 },
          { label: 'v1', value: 1 },
        ],
        default: 0,
        readOnly: true,
        ui: { placeholder: '类型', widget: 'select' },
      },
      description: {
        type: 'string',
        title: '网站描述',
        pattern: '^\\w+$',
        readOnly: true,
        ui: {
          widget: 'string',
        },
      },
      // 所有未知的复杂类型都当作枚举处理
      state: {
        type: 'array',
        title: '状态',
        enum: [
          { label: 'v0', value: 0 },
          { label: 'v1', value: 1 },
        ],
        default: 0,
        readOnly: true,
        ui: { placeholder: '状态', widget: 'select' },
      },
      auditor: {
        type: 'string',
        title: '审核人',
        pattern: '^\\w+$',
        readOnly: true,
        ui: {
          widget: 'string',
        },
      },
      auditedTime: {
        type: 'string',
        title: '审核时间',
        format: 'date-time',
        readOnly: true,
      },
      remark: {
        type: 'string',
        title: '审核备注',
        readOnly: true,
        ui: {
          widget: 'textarea',
          autosize: { minRows: 2, maxRows: 10 },
          placeholder: '添加描述',
        },
      },
      serverTag: {
        type: 'string',
        title: '服务器标记',
        pattern: '^\\w+$',
        readOnly: true,
        ui: {
          widget: 'string',
        },
      },
    },
    ui: {
      spanLabelFixed: 100,
      grid: { span: 24 },
    },
  };

  constructor(private modal: NzModalRef) {}

  ngOnInit(): void {}

  close(): void {
    this.modal.destroy();
  }
}
