import { Component, Input, OnInit } from '@angular/core';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzModalRef } from 'ng-zorro-antd/modal';
import { _HttpClient } from '@delon/theme';
import { SFSchema } from '@delon/form';
import { DOMAIN_TYPE } from '../../seo.enums';

@Component({
  selector: 'app-edit-seo-domain',
  templateUrl: './edit.component.html',
})
export class SeoDomainEditComponent implements OnInit {
  rolePrefix = 'seo-domain:';

  @Input()
  record: any;

  schema: SFSchema = {
    properties: {
      templateWebsite: {
        type: 'string',
        title: '模板网址',
      },
      domain: {
        type: 'string',
        title: '网站域名',
      },
      title: {
        type: 'string',
        title: '标题',
      },
      keywords: {
        type: 'string',
        title: '关键词列表',
      },
      menus: {
        type: 'string',
        title: '菜单列表',
      },
      mainKey: {
        type: 'string',
        title: '主关键词',
      },
      // 所有未知的复杂类型都当作枚举处理
      type: {
        type: 'string',
        title: '类型',
        enum: DOMAIN_TYPE,
        default: 0,
        ui: { placeholder: '类型', widget: 'select' },
      },
      description: {
        type: 'string',
        title: '网站说明',
        ui: {
          widget: 'textarea',
          autosize: { minRows: 2, maxRows: 10 },
          placeholder: '添加描述',
        },
      },
      serverTag: {
        type: 'string',
        title: '服务器标记',
      },
      suffix: {
        type: 'string',
        title: '网页后缀',
      },
      dir: {
        type: 'string',
        title: '目录',
      },
      dailyArticle: { type: 'number', title: '每天最多更新多少篇' },
      // 所有未知的复杂类型都当作枚举处理
      lastArticleTime: {
        type: 'string',
        title: '最后一次文章更新时间',
        format: 'date-time',
      },
      newSite: {
        type: 'boolean',
        title: '是否为新站',
        enum: [
          { label: '是', value: true },
          { label: '否', value: false },
        ],
        default: true,
        ui: { placeholder: '是否启用', widget: 'select' },
      },
      jsGroup: {
        type: 'string',
        title: 'js分组',
      },
      proposalReference: {
        type: 'string',
        title: '提案编号',
      },
    },
    required: [
      'template_website',
      'domain',
      'title',
      'keywords',
      'menus',
      'main_key',
      'type',
      'description',
      'server_tag',
      'suffix',
      'dir',
      'daily_article',
      'article_time',
      'last_article_time',
      'new_site',
      'js_group',
      'proposal_reference',
    ],
    ui: {
      spanLabelFixed: 100,
      grid: { span: 24 },
    },
  };

  constructor(
    private modal: NzModalRef,
    private msgSrv: NzMessageService,

    public http: _HttpClient,
  ) {}

  ngOnInit(): void {}

  save(value: any) {
    this.http.post(`/api/star-ring/seo/domain/update`, value).subscribe((res) => {
      this.msgSrv.success('修改成功');
      this.modal.close(res);
    });
  }

  close() {
    this.modal.destroy();
  }
}
