import { Component, Input, OnInit } from '@angular/core';
import { NzModalRef } from 'ng-zorro-antd/modal';
import { SFSchema } from '@delon/form';
import { DOMAIN_TYPE } from '../../seo.enums';

@Component({
  selector: 'app-view-seo-domain',
  templateUrl: './view.component.html',
})
export class SeoDomainViewComponent implements OnInit {
  @Input()
  record: any;

  schema: SFSchema = {
    properties: {
      templateWebsite: {
        type: 'string',
        title: '模板网址',
        pattern: '^\\w+$',
        readOnly: true,
        ui: {
          widget: 'string',
        },
      },
      domain: {
        type: 'string',
        title: '网站域名',
        pattern: '^\\w+$',
        readOnly: true,
        ui: {
          widget: 'string',
        },
      },
      title: {
        type: 'string',
        title: '标题',
        pattern: '^\\w+$',
        readOnly: true,
        ui: {
          widget: 'string',
        },
      },
      keywords: {
        type: 'string',
        title: '关键词列表',
        pattern: '^\\w+$',
        readOnly: true,
        ui: {
          widget: 'string',
        },
      },
      menus: {
        type: 'string',
        title: '菜单列表',
        pattern: '^\\w+$',
        readOnly: true,
        ui: {
          widget: 'string',
        },
      },
      mainKey: {
        type: 'string',
        title: '主关键词',
        pattern: '^\\w+$',
        readOnly: true,
        ui: {
          widget: 'string',
        },
      },
      // 所有未知的复杂类型都当作枚举处理
      type: {
        type: 'string',
        title: '类型',
        enum: DOMAIN_TYPE,
        default: 0,
        readOnly: true,
        ui: { placeholder: '类型', widget: 'select' },
      },
      description: {
        type: 'string',
        title: '网站说明',
        pattern: '^\\w+$',
        readOnly: true,
        ui: {
          widget: 'string',
        },
      },
      serverTag: {
        type: 'string',
        title: '服务器标记',
        pattern: '^\\w+$',
        readOnly: true,
        ui: {
          widget: 'string',
        },
      },
      suffix: {
        type: 'string',
        title: '网页后缀',
        pattern: '^\\w+$',
        readOnly: true,
        ui: {
          widget: 'string',
        },
      },
      dir: {
        type: 'string',
        title: '目录',
        pattern: '^\\w+$',
        readOnly: true,
        ui: {
          widget: 'string',
        },
      },
      dailyArticle: { type: 'number', title: '每天最多更新多少篇', readOnly: true },
      // 所有未知的复杂类型都当作枚举处理
      articleTime: {
        type: 'array',
        title: '文章更新时间',
        enum: [
          { label: 'v0', value: 0 },
          { label: 'v1', value: 1 },
        ],
        default: 0,
        readOnly: true,
        ui: { placeholder: '文章更新时间', widget: 'select' },
      },
      lastArticleTime: {
        type: 'string',
        title: '最后一次文章更新时间',
        format: 'date-time',
        readOnly: true,
      },
      newSite: {
        type: 'boolean',
        title: '是否为新站',
        enum: [
          { label: '是', value: true },
          { label: '否', value: false },
        ],
        default: true,
        readOnly: true,
        ui: { placeholder: '是否启用', widget: 'select' },
      },
      jsGroup: {
        type: 'string',
        title: 'js分组',
        pattern: '^\\w+$',
        readOnly: true,
        ui: {
          widget: 'string',
        },
      },
      proposalReference: {
        type: 'string',
        title: '提案编号',
        pattern: '^\\w+$',
        readOnly: true,
        ui: {
          widget: 'string',
        },
      },
    },
    ui: {
      spanLabelFixed: 100,
      grid: { span: 24 },
    },
  };

  constructor(private modal: NzModalRef) {}

  ngOnInit(): void {}

  close(): void {
    this.modal.destroy();
  }
}
