import { Component, Input, OnInit } from '@angular/core';
import { SFSchema } from '@delon/form';
import { _HttpClient } from '@delon/theme';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzModalRef } from 'ng-zorro-antd/modal';

import { CHECKED_STATUS } from '../../seo.enums';

@Component({
  selector: 'app-edit-seo-template',
  templateUrl: './edit.component.html'
})
export class SeoTemplateEditComponent implements OnInit {
  rolePrefix = 'seo-template:';

  @Input()
  record: any;

  schema: SFSchema = {
    properties: {
      indexSubKeySize: { type: 'number', title: 'index(ALT)' },
      listSubKeySize: { type: 'number', title: 'list(ALT)' },
      articleSubKeySize: { type: 'number', title: 'article(ALT)' },
      indexATagSize: { type: 'number', title: 'index(A)' },
      listATagSize: { type: 'number', title: 'list(A)' },
      articleATagSize: { type: 'number', title: '文章页(A)' },
      interlinkSize: { type: 'number', title: '内链数' },
      indexHSize: { type: 'number', title: 'index(H)' },
      listHSize: { type: 'number', title: 'list(H)' },
      articleHSize: { type: 'number', title: 'article(H)' },
      tpType: { type: 'string', title: '模板类型' },
      checked: {
        type: 'string',
        title: '审核状态',
        enum: CHECKED_STATUS,
        ui: { placeholder: '状态', widget: 'select' }
      }
    },
    required: [
      'indexSubKeySize',
      'listSubKeySize',
      'articleSubKeySize',
      'indexATagSize',
      'listATagSize',
      'articleATagSize',
      'interlinkSize',
      'indexHSize',
      'listHSize',
      'articleHSize',
      'checked'
    ],
    ui: {
      spanLabelFixed: 100,
      grid: { span: 24 }
    }
  };

  constructor(
    private modal: NzModalRef,
    private msgSrv: NzMessageService,

    public http: _HttpClient
  ) {}

  ngOnInit(): void {}

  save(value: any): void {
    this.http.post(`/api/star-ring/seo-template/update`, value).subscribe(res => {
      this.msgSrv.success('修改成功');
      this.modal.close(res);
    });
  }

  close(): void {
    this.modal.destroy();
  }
}
