import { Component, Input, OnInit } from '@angular/core';
import { SFSchema } from '@delon/form';
import { NzModalRef } from 'ng-zorro-antd/modal';
import { CHECKED_STATUS, TASK_STATUS } from '../../seo.enums';

@Component({
  selector: 'app-view-seo-template',
  templateUrl: './view.component.html',
})
export class SeoTemplateViewComponent implements OnInit {
  @Input()
  record: any;

  schema: SFSchema = {
    properties: {
      website: {
        type: 'string',
        title: '模板域名',
        readOnly: true,
      },
      indexSubKeySize: { type: 'number', title: 'index(ALT)', readOnly: true },
      listSubKeySize: { type: 'number', title: 'list(ALT)', readOnly: true },
      articleSubKeySize: { type: 'number', title: 'article(ALT)', readOnly: true },
      indexATagSize: { type: 'number', title: 'index(A)', readOnly: true },
      listATagSize: { type: 'number', title: 'list(A)', readOnly: true },
      articleATagSize: { type: 'number', title: '文章页(A)', readOnly: true },
      interlinkSize: { type: 'number', title: '内链数', readOnly: true },
      // 所有未知的复杂类型都当作枚举处理
      checked: {
        type: 'string',
        title: '审核状态',
        enum: CHECKED_STATUS,
        default: 0,
        readOnly: true,
        ui: { placeholder: '审核状态', widget: 'select' },
      },
      // 所有未知的复杂类型都当作枚举处理
      indexState: {
        type: 'string',
        title: 'index页面的抓取状态',
        enum: TASK_STATUS,
        readOnly: true,
        ui: { placeholder: 'index页面的抓取状态', widget: 'select' },
      },
      // 所有未知的复杂类型都当作枚举处理
      listState: {
        type: 'string',
        title: 'list页面的抓取状态',
        enum: TASK_STATUS,
        readOnly: true,
        ui: { placeholder: 'list页面的抓取状态', widget: 'select' },
      },
      // 所有未知的复杂类型都当作枚举处理
      articleState: {
        type: 'string',
        title: '文章页面的抓取状态',
        enum: TASK_STATUS,
        readOnly: true,
        ui: { placeholder: '文章页面的抓取状态', widget: 'select' },
      },
      indexHSize: { type: 'number', title: 'index(H)', readOnly: true },
      listHSize: { type: 'number', title: 'list(H)', readOnly: true },
      articleHSize: { type: 'number', title: 'article(H)', readOnly: true },
      groupId: { type: 'number', title: '分组', readOnly: true },
      auditor: {
        type: 'string',
        title: '审核人',
        readOnly: true,
      },
      remark: {
        type: 'string',
        title: '审核备注',
        readOnly: true,
        ui: {
          widget: 'textarea',
          autosize: { minRows: 2, maxRows: 10 },
          placeholder: '添加描述',
        },
      },
      useTimes: { type: 'number', title: '使用次数', readOnly: true },
      auditedTime: {
        type: 'string',
        title: '审核时间',
        format: 'date-time',
        readOnly: true,
      },
    },
    ui: {
      spanLabelFixed: 100,
      grid: { span: 24 },
    },
  };

  constructor(private modal: NzModalRef) {}

  ngOnInit(): void {}

  close(): void {
    this.modal.destroy();
  }
}
