import { Component, OnInit } from '@angular/core';
import { SFSchema } from '@delon/form';
import { _HttpClient } from '@delon/theme';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzModalRef } from 'ng-zorro-antd/modal';

@Component({
  selector: 'app-create-sprocket',
  templateUrl: './create.component.html',
})
export class SprocketCreateComponent implements OnInit {
  rolePrefix = 'sprocket:';

  // test : {
  //   type: 'string',
  //   title: '测试',
  //   pattern: '^\\w+$',
  //   ui: {
  //     widget: 'string',
  //     placeholder: '唯一标识,不可重复,由字母,下划线,数字组合',
  //     errors: {
  //       pattern: '仅支持输入字母、下划线、数字'
  //     }
  //   }
  // },

  schema: SFSchema = {
    properties: {
      domain: { type: 'string', title: '域名' },
      targetDomain: { type: 'string', title: '跳转域名' },
      ring: {
        type: 'boolean',
        title: '是否为链轮',
        enum: [
          { label: '是', value: true },
          { label: '否', value: false },
        ],
        default: true,
        ui: { placeholder: '是否启用', widget: 'select' },
      },
    },
    required: ['domain', 'targetDomain', 'ring'],
    ui: {
      spanLabelFixed: 100,
      grid: { span: 24 },
    },
  };

  constructor(
    private modal: NzModalRef,
    private msgSrv: NzMessageService,

    public http: _HttpClient,
  ) {}

  ngOnInit(): void {}

  save(value: any): void {
    this.http.post(`/api/star-ring/sprocket/create`, value).subscribe((res) => {
      this.msgSrv.success('添加成功');
      this.modal.close(res);
    });
  }

  close(): void {
    this.modal.destroy();
  }
}
