import { Component, Input, OnInit } from '@angular/core';
import { SFSchema } from '@delon/form';
import { _HttpClient } from '@delon/theme';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzModalRef } from 'ng-zorro-antd/modal';

@Component({
  selector: 'app-edit-sprocket',
  templateUrl: './edit.component.html',
})
export class SprocketEditComponent implements OnInit {
  rolePrefix = 'sprocket:';

  @Input()
  record: any;

  schema: SFSchema = {
    properties: {
      domain: { type: 'string', title: '域名' },
      targetDomain: { type: 'string', title: '跳转域名' },
      ring: {
        type: 'boolean',
        title: '是否为链轮',
        enum: [
          { label: '是', value: true },
          { label: '否', value: false },
        ],
        default: true,
        ui: { placeholder: '是否启用', widget: 'select' },
      },
    },
    required: ['domain', 'targetDomain', 'ring'],
    ui: {
      spanLabelFixed: 100,
      grid: { span: 24 },
    },
  };

  constructor(
    private modal: NzModalRef,
    private msgSrv: NzMessageService,

    public http: _HttpClient,
  ) {}

  ngOnInit(): void {}

  save(value: any): void {
    this.http.post(`/api/star-ring/sprocket/update`, value).subscribe((res) => {
      this.msgSrv.success('修改成功');
      this.modal.close(res);
    });
  }

  close(): void {
    this.modal.destroy();
  }
}
