import { Component, OnInit } from '@angular/core';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzModalRef } from 'ng-zorro-antd/modal';
import { _HttpClient } from '@delon/theme';
import { SFSchema } from '@delon/form';

@Component({
  selector: 'app-create-target-domain',
  templateUrl: './create.component.html',
})
export class TargetDomainCreateComponent implements OnInit {
  rolePrefix = 'target-domain:';

  schema: SFSchema = {
    properties: {
      targetDomain: {
        type: 'string',
        title: '目标站',
      },
      xinhuan: {
        type: 'boolean',
        title: '星环特有',
        enum: [
          { label: '是', value: true },
          { label: '否', value: false },
        ],
        default: true,
        ui: { placeholder: '是否启用', widget: 'select' },
      },
      used: {
        type: 'boolean',
        title: '是否使用',
        enum: [
          { label: '是', value: true },
          { label: '否', value: false },
        ],
        default: true,
        ui: { placeholder: '是否启用', widget: 'select' },
      },
    },
    required: ['targetDomain', 'xinhuan', 'used'],
    ui: {
      spanLabelFixed: 100,
      grid: { span: 24 },
    },
  };

  constructor(
    private modal: NzModalRef,
    private msgSrv: NzMessageService,

    public http: _HttpClient,
  ) {}

  ngOnInit(): void {}

  save(value: any): void {
    this.http.post(`/api/star-ring/target-domain/create`, value).subscribe((res) => {
      this.msgSrv.success('添加成功');
      this.modal.close(res);
    });
  }

  close(): void {
    this.modal.destroy();
  }
}
