package com.chuang.urras.common.crud;

import com.baomidou.mybatisplus.core.toolkit.AES;
import com.chuang.tauceti.tools.basic.HexKit;
import com.chuang.tauceti.tools.basic.StringKit;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.type.BaseTypeHandler;
import org.apache.ibatis.type.JdbcType;
import org.apache.ibatis.type.MappedJdbcTypes;
import org.apache.ibatis.type.MappedTypes;

import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.sql.CallableStatement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * @author ATH
 */
@Slf4j
@MappedJdbcTypes(JdbcType.VARCHAR)
@MappedTypes(String.class)
public class AESEncryptHandler extends BaseTypeHandler<String> {

    public static volatile String ENCRYPT_KEY;
    private static final Charset CHARSET = StandardCharsets.UTF_8;

    @Override
    public void setNonNullParameter(PreparedStatement ps, int i, String parameter, JdbcType jdbcType) throws SQLException {
        ps.setString(i, encrypt(parameter));
    }

    @Override
    public String getNullableResult(ResultSet rs, String columnName) throws SQLException {
        String columnValue = rs.getString(columnName);
        return decrypt(columnValue);
    }

    @Override
    public String getNullableResult(ResultSet rs, int columnIndex) throws SQLException {
        String columnValue = rs.getString(columnIndex);
        return decrypt(columnValue);
    }

    @Override
    public String getNullableResult(CallableStatement cs, int columnIndex) throws SQLException {
        String columnValue = cs.getString(columnIndex);
        return decrypt(columnValue);
    }

    public static String encrypt(String value, String pass) {
        if (StringKit.isBlank(pass) || StringKit.isBlank(value)) {
            return value;
        }
        return HexKit.binary2Hex(AES.encrypt(value.getBytes(CHARSET), pass.getBytes(CHARSET)));
    }

    public static String encrypt(String value) {
        return encrypt(value, ENCRYPT_KEY);
    }

    public static String decrypt(String value) {
        return decrypt(value, ENCRYPT_KEY);
    }

    public static String decrypt(String value, String pass) {
        if (StringKit.isBlank(pass) || StringKit.isBlank(value)) {
            return value;
        }
        try {
            return new String(AES.decrypt(HexKit.decodeHex(value.toCharArray()), pass.getBytes(CHARSET)), CHARSET);
        } catch (Exception err) {
            return value;
        }

    }

}
