package com.chuang.urras.common.crud;

import com.alibaba.druid.pool.DruidDataSource;
import com.alibaba.druid.spring.boot.autoconfigure.DruidDataSourceBuilder;
import com.baomidou.mybatisplus.annotation.DbType;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.MybatisPlusInterceptor;
import com.baomidou.mybatisplus.extension.plugins.inner.BlockAttackInnerInterceptor;
import com.baomidou.mybatisplus.extension.plugins.inner.OptimisticLockerInnerInterceptor;
import com.baomidou.mybatisplus.extension.plugins.inner.PaginationInnerInterceptor;
import com.chuang.tauceti.rowquery.handlers.AutoTimeHandler;
import com.chuang.tauceti.support.vg.OperatorManager;
import com.chuang.urras.common.crud.properties.CrudProperties;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.context.event.ApplicationStartedEvent;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.*;
import org.springframework.context.event.EventListener;
import org.springframework.context.support.PropertySourcesPlaceholderConfigurer;
import org.springframework.transaction.annotation.EnableTransactionManagement;

import javax.annotation.Resource;

@Slf4j
@Configuration
@PropertySource({
        "classpath:druid.properties", "classpath:mybatis.properties"
})
@EnableConfigurationProperties(CrudProperties.class)
@EnableAspectJAutoProxy
//@EnableTransactionManagement(mode = AdviceMode.ASPECTJ)
@ComponentScan("com.chuang.urras.common.crud")

public class CrudAutoConfiguration {

    @Resource CrudProperties crudProperties;

    @Bean
    @ConditionalOnMissingBean
    public static PropertySourcesPlaceholderConfigurer propertySourcesPlaceholderConfigurer() {
        return new PropertySourcesPlaceholderConfigurer();
    }

    /**
     * 配置数据源
     */
    @Bean("dataSource")
    @ConfigurationProperties(prefix = "spring.datasource.druid")
    public DruidDataSource dataSource() {
        return DruidDataSourceBuilder.create().build();
    }

//    @Bean
//    public IKeyGenerator keyGenerator() {
//        return new OracleKeyGenerator();
//    }

    @Bean
    public MybatisPlusInterceptor mybatisPlusInterceptor() {
        MybatisPlusInterceptor interceptor = new MybatisPlusInterceptor();
        interceptor.addInnerInterceptor(new PaginationInnerInterceptor(DbType.MYSQL));
        // 乐观锁
        interceptor.addInnerInterceptor(new OptimisticLockerInnerInterceptor());
        // 防止全表更新与删除
        interceptor.addInnerInterceptor(new BlockAttackInnerInterceptor());
        return interceptor;
    }

    @Bean
    public AutoTimeHandler autoTimeHandler(OperatorManager operatorManager) {
        return new AutoTimeHandler(operatorManager);
    }

//    @Bean
//    public MybatisPlusPropertiesCustomizer mybatisPlusPropertiesCustomizer() {
//        return this::handProperties;
//    }
//
//    private void handProperties(MybatisPlusProperties properties) {
//        // 默认将 ENUM_PACKAGE 添加到配置中
//        if (StringKit.isBlank(properties.getTypeEnumsPackage())) {
//            properties.setTypeEnumsPackage(PAYMENT_ENUM_PACKAGE);
//        } else {
//            List<String> typeEnumsPackage = Arrays.stream(properties.getTypeEnumsPackage().split(","))
//                    .map(String::trim)
//                    .filter(s -> !s.equals(PAYMENT_ENUM_PACKAGE))
//                    .collect(Collectors.toList());
//
//            properties.setTypeEnumsPackage(StringKit.join(typeEnumsPackage, ",") + "," + PAYMENT_ENUM_PACKAGE);
//        }
//
//        // 默认将 ALIASES_PACKAGE 添加到配置中
//        if (StringKit.isBlank(properties.getTypeAliasesPackage())) {
//            properties.setTypeAliasesPackage(PAYMENT_ALIASES_PACKAGE);
//        } else {
//            List<String> typeAliasesPackage = Arrays.stream(properties.getTypeAliasesPackage().split(","))
//                    .map(String::trim)
//                    .filter(s -> !s.equals(PAYMENT_ALIASES_PACKAGE))
//                    .collect(Collectors.toList());
//            properties.setTypeAliasesPackage(StringKit.join(typeAliasesPackage, ",") + "," + PAYMENT_ALIASES_PACKAGE);
//        }
//    }

    @EventListener
    public void listenAppStarted(ApplicationStartedEvent event) {
        CrudProperties.Reference reference = crudProperties.getReference();
        IdWorker.initSequence(reference.getWorkId(), reference.getDataCenterId());

        AESEncryptHandler.ENCRYPT_KEY = crudProperties.getEncryptKey();
        log.info("设置了密钥，长度为：{}， 第一个字符：{}， 最后一个字符：{}", crudProperties.getEncryptKey().length(), crudProperties.getEncryptKey().charAt(0), crudProperties.getEncryptKey().charAt(crudProperties.getEncryptKey().length() - 1));
    }
}
