package com.chuang.urras.common.crud;

import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.tools.third.spring.SpringKit;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.MessageSource;
import org.springframework.context.i18n.LocaleContextHolder;

@Slf4j
public class Errors {


    public static BusinessException business(int code, String pattern, Object... args) {
        return new BusinessException(code, pattern, args);
    }

    public static BusinessException business(String pattern, Object... args) {
        return new BusinessException(pattern, args);
    }

    public static BusinessException business(int code, Throwable e, String pattern, Object... args) {
        return new BusinessException(code, e, pattern, args);
    }

    public static BusinessException business(Throwable e, String pattern, Object... args) {
        return new BusinessException(e, pattern, args);
    }

    public static BusinessException businessI18n(int code, String i18n, Object... args) {

        return new BusinessException(code, getMessage(i18n, args));
    }

    public static BusinessException businessI18n(String i18n, Object... args) {
        return new BusinessException(getMessage(i18n, args));
    }

    public static BusinessException businessI18n(int code, Throwable e, String i18n, Object... args) {
        return new BusinessException(code, e, getMessage(i18n, args));
    }

    public static BusinessException businessI18n(Throwable e, String i18n, Object... args) {
        return new BusinessException(e, getMessage(i18n, args));
    }

    private static final class MessageSourceHolder {
        static final MessageSource MESSAGE_SOURCE = SpringKit.getBean(MessageSource.class);
    }

    private static MessageSource messageSource() {
        return MessageSourceHolder.MESSAGE_SOURCE;
    }

    private static String getMessage(String i18n, Object... args) {
        try {
            return messageSource().getMessage(i18n, args, LocaleContextHolder.getLocale());
        } catch (Exception e) {
            log.warn("", e);
            return i18n;
        }
    }
}
