package com.chuang.urras.common.crud;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.support.SFunction;
import com.baomidou.mybatisplus.extension.service.IService;
import com.chuang.tauceti.tools.basic.collection.CollectionKit;
import com.chuang.tauceti.tools.basic.reflect.BeanKit;

import javax.annotation.Nullable;
import java.util.*;
import java.util.function.BiFunction;
import java.util.function.Supplier;
import java.util.stream.Collectors;

public class JoinUtils {

    public static <J, L, R, LF, RF, JC extends Collection<J>> JC joinOne2One(Collection<L> data,
                                                                             IService<R> service,
                                                                             SFunction<L, LF> leftFiled,
                                                                             SFunction<R, RF> rightField,
                                                                             BiFunction<L, R, J> join,
                                                                             Supplier<JC> collectionFactory,
                                                                             @Nullable Supplier<QueryWrapper<R>> where) {
        if(CollectionKit.isEmpty(data)) {
            return collectionFactory.get();
        }
        Set<LF> lfs = data.stream().map(leftFiled).collect(Collectors.toSet());

        List<R> rList;

        if(null != where) {
            LambdaQueryWrapper<R> wrapper = where.get().lambda().in(rightField, lfs);
            rList = service.list(wrapper);
        } else {
            rList = service.lambdaQuery().in(rightField, lfs).list();
        }

        Map<RF, R> rMap = rList.stream().collect(Collectors.toMap(rightField, o->o));

        return data.stream().map(left -> {
            LF lf = leftFiled.apply(left);
            if(null == lf) {
                return join.apply(left, null);
            } else {
                return join.apply(left, rMap.get(lf));
            }
        }).collect(Collectors.toCollection(collectionFactory));
    }

    public static <J, L, R, LF, RF, JC extends Collection<J>> JC joinOne2One(Collection<L> data,
                                                                             IService<R> service,
                                                                             SFunction<L, LF> leftFiled,
                                                                             SFunction<R, RF> rightField,
                                                                             BiFunction<L, R, J> join,
                                                                             Supplier<JC> collectionFactory) {
        return joinOne2One(data, service, leftFiled, rightField, join, collectionFactory, null);
    }

    public static <J, L, R, LF, RF> List<J> joinOne2One(Collection<L> data,
                                                        IService<R> service,
                                                        SFunction<L, LF> leftFiled,
                                                        SFunction<R, RF> rightField,
                                                        BiFunction<L, R, J> join) {

        return joinOne2One(data, service, leftFiled, rightField, join, ArrayList::new);
    }

    public static <J, L, R, LF, RF, JC extends Collection<J>> JC joinOne2One(Collection<L> data,
                                                                             IService<R> service,
                                                                             SFunction<L, LF> leftFiled,
                                                                             SFunction<R, RF> rightField,
                                                                             Supplier<J> jFactory,
                                                                             Supplier<JC> collectionFactory) {
        return joinOne2One(data, service, leftFiled, rightField, (left, right) -> join(left, right, jFactory), collectionFactory, null);
    }

    public static <J, L, R, LF, RF> List<J> joinOne2One(Collection<L> data,
                                                        IService<R> service,
                                                        SFunction<L, LF> leftFiled,
                                                        SFunction<R, RF> rightField,
                                                        Supplier<J> jFactory) {
        return joinOne2One(data, service, leftFiled, rightField, (left, right) -> join(left, right, jFactory), ArrayList::new);
    }

    private static <J, L, R> J join(L left, R right, Supplier<J> jFactory) {
        J j = jFactory.get();
        if(null != right) {
            BeanKit.copyProperties(right, j);
        }
        if(null != left) {
            BeanKit.copyProperties(left, j);
        }
        return j;
    }
}
