package com.chuang.urras.common.crud.annotations;

import com.baomidou.mybatisplus.autoconfigure.MybatisPlusPropertiesCustomizer;
import com.chuang.tauceti.tools.basic.StringKit;
import org.springframework.aop.config.AopConfigUtils;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.BeanDefinitionRegistry;
import org.springframework.context.annotation.ImportBeanDefinitionRegistrar;
import org.springframework.core.annotation.MergedAnnotation;
import org.springframework.core.type.AnnotationMetadata;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

public class MybatisPlusPackageRegistrar implements ImportBeanDefinitionRegistrar {

    @Override
    public void registerBeanDefinitions(AnnotationMetadata importingClassMetadata, BeanDefinitionRegistry registry) {

        AopConfigUtils.registerAspectJAnnotationAutoProxyCreatorIfNecessary(registry);

        MergedAnnotation<MybatisPlusPackage> ma = importingClassMetadata.getAnnotations().get(MybatisPlusPackage.class);

        String[] enums = ma.getStringArray("enums");
        String[] aliases = ma.getStringArray("aliases");
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.genericBeanDefinition(MybatisPlusPropertiesCustomizer.class, () -> {
            return properties -> {
                for (String anEnum : enums) {
                    // 默认将 ENUM_PACKAGE 添加到配置中
                    if (StringKit.isBlank(properties.getTypeEnumsPackage())) {
                        properties.setTypeEnumsPackage(anEnum);
                    } else {
                        List<String> typeEnumsPackage = Arrays.stream(properties.getTypeEnumsPackage().split(","))
                                .map(String::trim)
                                .filter(s -> !s.equals(anEnum))
                                .collect(Collectors.toList());

                        properties.setTypeEnumsPackage(StringKit.join(typeEnumsPackage, ",") + "," + anEnum);
                    }
                }

                for (String alias : aliases) {
                    // 默认将 ALIASES_PACKAGE 添加到配置中
                    if (StringKit.isBlank(properties.getTypeAliasesPackage())) {
                        properties.setTypeAliasesPackage(alias);
                    } else {
                        List<String> typeAliasesPackage = Arrays.stream(properties.getTypeAliasesPackage().split(","))
                                .map(String::trim)
                                .filter(s -> !s.equals(alias))
                                .collect(Collectors.toList());
                        properties.setTypeAliasesPackage(StringKit.join(typeAliasesPackage, ",") + "," + alias);
                    }
                }
            };
        });
        registry.registerBeanDefinition("mybatisPlusPropertiesCustomizer#" + importingClassMetadata.getClassName(), builder.getBeanDefinition());

    }
}
