package com.chuang.urras.common.crud.aop;

import com.chuang.tauceti.rowquery.exception.OptimisticLockerException;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

@Aspect
@Component
@Slf4j
@Order(Ordered.HIGHEST_PRECEDENCE)
public class OptimisticLockerAspect {

    @Around(value = "@annotation(retry)")
    public Object handlerControllerMethod(ProceedingJoinPoint pjp, NeedRetry retry) throws Throwable {
        int maxRetry = retry.maxTimes();
        OptimisticLockerException last = new OptimisticLockerException("乐观锁异常");
        for (int i = 0; i < maxRetry; i++) {
            try {
                return pjp.proceed();
            } catch (OptimisticLockerException e) {
                log.warn(e.getMessage(), e);
                last = e;
            }
        }

        throw last;
    }
}
