package com.chuang.urras.common.events;

import com.chuang.tauceti.support.exception.BusinessException;
import org.redisson.api.RTopic;
import org.redisson.api.RedissonClient;
import org.springframework.context.ApplicationContext;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;

public class RedisEventManager {

    @Resource private RedissonClient redissonClient;

    @Resource private ApplicationContext applicationContext;

    private final Map<String, RTopic> topics = new HashMap<>();

    private static final String OFFICE_WARN_TOPIC = "office-warn-event-topic";

    @PostConstruct
    public void init() {
        addEventTopic(OFFICE_WARN_TOPIC, OfficeMsgEvent.class);
    }

    public <T> void addEventTopic(String topic, Class<T> eventClass) {
        if(topics.containsKey(topic)) {
            throw new BusinessException("topic contains");
        }
        RTopic t = redissonClient.getTopic(topic);
        topics.put(topic, t);

        t.addListener(eventClass, (channel, msg) -> applicationContext.publishEvent(msg));
    }

    public <T> void fireEvent(String topic, T event) {
        topics.get(topic).publish(event);
    }

    public <T> void fireAdminMsgEvent(OfficeMsgEvent event) {
        fireEvent(OFFICE_WARN_TOPIC, event);
    }

}
