package com.chuang.urras.common.task.controller;

import com.chuang.tauceti.rowquery.controller.basic.IRetrieveController;
import com.chuang.tauceti.support.Result;
import com.chuang.urras.common.task.enums.JobStatus;
import com.chuang.urras.common.task.enums.SchedulerStatus;
import com.chuang.urras.common.task.model.Scheduler;
import com.chuang.urras.common.task.model.co.SchedulerCO;
import com.chuang.urras.common.task.model.ro.SchedulerRO;
import com.chuang.urras.common.task.model.uo.SchedulerUO;
import com.chuang.urras.common.task.service.IQuartzSchedulerService;
import com.chuang.urras.common.task.service.ISchedulerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.SneakyThrows;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;

/**
 * <p>
 * 调度任务信息表 前端控制器
 * </p>
 *
 * @author Ives.L
 * @since 2021-12-09
 */
@Validated
@RestController
@RequestMapping("/sys/scheduler")
@Api(tags = {"任务调度"})
public class SchedulerController implements IRetrieveController<SchedulerRO, Scheduler, ISchedulerService> {

	@Resource private IQuartzSchedulerService schedulerService;
	@Resource private ISchedulerService service;

	@Override
	public ISchedulerService service() {
		return service;
	}


	@PostMapping(value = "/create")
	@ApiOperation(value = "创建一条记录")
	public Result<Void> save(@Valid @RequestBody SchedulerCO schedulerCO) {
		if (null == schedulerCO.getState()) {
			schedulerCO.setState(SchedulerStatus.NORMAL);
		}
		this.schedulerService.addJob(schedulerCO);
		return Result.success();
	}

	@PutMapping(value = "/update")
	@ApiOperation(value = "修改任务参数", notes = "根据 ID 修改任务参数")
	public Result<Void> update(@Valid @RequestBody SchedulerUO schedulerUO) {
		if (null == schedulerUO.getState()) {
			schedulerUO.setState(SchedulerStatus.NORMAL);
		}
		this.schedulerService.updateJob(schedulerUO);
		return Result.success();
	}

	@DeleteMapping(value = "/delete/{jobId}")
	@ApiOperation(value = "删除任务", notes = "根据 ID 删除任务")
	public Result<Void> remove(@PathVariable Long jobId) {
		this.schedulerService.removeJob(jobId);
		return Result.success();
	}

	@PutMapping(value = "/state/{jobId}/{status}")
	@ApiOperation(value = "变更任务状态", notes = "根据 ID 变更任务状态")
	public Result<Void> pause(@PathVariable Long jobId,
	                          @PathVariable JobStatus status) {
		switch (status) {
			case RUN:
				this.schedulerService.runOnce(jobId);
				break;
			case PAUSE:
				this.schedulerService.pauseJob(jobId);
				break;
			case RESUME:
				this.schedulerService.resumeJob(jobId);
				break;
			case INTERRUPT:
				this.schedulerService.interruptJob(jobId);
				break;
			default:
		}

		return Result.success();
	}

	@Override
	public String basePermission() {
		return "scheduler:";
	}

}

