package com.chuang.urras.common.task.model.co;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.validation.constraints.NotBlank;
import java.io.Serializable;
import java.util.Map;

/**
 * 创建任务CO
 *
 * @author Ives.l
 * @since 2021/11/26 15:15
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@ApiModel(value = "创建新任务VO")
public class QuartzJobCO implements Serializable {
	private static final long serialVersionUID = 2364707704762811080L;

	@ApiModelProperty(value = "任务名称", required = true, example = "TestJob(任务名称必须是jobClass对应的类名)")
	@NotBlank(message = "任务名字不能为空")
	private String jobName;

	@ApiModelProperty(value = "任务组名称", required = true, example = "GROUP1")
	@NotBlank(message = "任务组名称不能为空")
	private String jobGroup;

	@ApiModelProperty(value = "任务类的全路径名", required = true, example = "com.pn.TestJob")
	@NotBlank(message = "任务类的全路径名不能为空")
	private String jobClass;

	@ApiModelProperty(value = "cron表达式", required = true, example = "0 0/1 * * * ?")
	@NotBlank(message = "cron表达式不能为空")
	private String cronExpression;

	@ApiModelProperty(value = "触发器有效期开始时间", example = "2019-11-03 14:44:02")
	private String startAt;

	@ApiModelProperty(value = "触发器有效期结束时间", example = "2019-11-05 14:44:02")
	private String endAt;

	@ApiModelProperty(value = "任务描述", example = "任务描述")
	private String description;

	@ApiModelProperty(value = "执行任务的指定参数", example = "{param1:'123',param2:'456'}")
	private Map<String, Object> jobDataMap;

}
