package com.chuang.urras.common.task.service;

import com.chuang.urras.common.task.model.co.SchedulerCO;
import com.chuang.urras.common.task.model.ro.QuartzJobRO;
import com.chuang.urras.common.task.model.uo.SchedulerUO;
import org.quartz.SchedulerException;

import javax.transaction.Transactional;
import java.util.List;

/**
 * 任务调用服务
 *
 * @author Ives.l
 * @since 2021/11/26 15:13
 */
public interface IQuartzSchedulerService {

	/**
	 * 查询任务列表
	 *
	 * @param jobName   任务名称
	 * @param groupName 分组名称
	 * @return List<QuartzJobRO>
	 */
	List<QuartzJobRO> findJob(String jobName, String groupName);

	/**
	 * 创建任务
	 *
	 * @param quartzJobCO 任务参数
	 */
	void addJob(SchedulerCO quartzJobCO);

	/**
	 * 暂停任务
	 *
	 * @param id 任务Id
	 */

	void pauseJob(Long id);

	/**
	 * 恢复任务
	 *
	 * @param id 任务Id
	 */
	void resumeJob(Long id);

	/**
	 * 立即运行一次定时任务
	 *
	 * @param id 任务Id
	 */
	void runOnce(Long id);

	/**
	 * 更新任务
	 *
	 * @param schedulerUO 更细任务参数
	 */
	void updateJob(SchedulerUO schedulerUO);

	/**
	 * 删除任务
	 *
	 * @param id 任务Id
	 */
	void removeJob(Long id);

	/**
	 * 中断任务
	 *
	 * @param id 任务Id
	 */
	void interruptJob(Long id);

	/**
	 * 启动所有任务
	 *
	 * @param delayed 延迟执行（秒）
	 */
	void startAllJobs(Integer delayed);

	/**
	 * 暂停所有任务
	 */
	void pauseAllJobs();

	/**
	 * 恢复所有任务
	 */
	void resumeAllJobs();

	/**
	 * 关闭所有任务
	 * <p>
	 * 需谨慎操作关闭 scheduler 容器
	 * scheduler 生命周期结束，无法再 start() 启动scheduler
	 *
	 * @param waitForJobsToComplete 是否等待任务执行完成
	 */
	void shutdownAllJobs(boolean waitForJobsToComplete);

}
