package com.chuang.urras.common.web.aop;

import com.chuang.tauceti.support.vg.OperatorManager;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

@Aspect
@Slf4j
@Component
public class OperatorAspect {
    @Resource private OperatorManager operatorManager;

    @Pointcut("@within(org.springframework.stereotype.Controller) || @within(org.springframework.web.bind.annotation.RestController)")
    public void controllerPointcut() {}


    @Pointcut("@annotation(org.springframework.web.bind.annotation.RequestMapping) || " +
            "@annotation(org.springframework.web.bind.annotation.GetMapping) || " +
            "@annotation(org.springframework.web.bind.annotation.PostMapping) || " +
            "@annotation(org.springframework.web.bind.annotation.PutMapping) || " +
            "@annotation(org.springframework.web.bind.annotation.DeleteMapping) )")
    public void requestMappingPointcut() {}

    @Around(value = "requestMappingPointcut()")
    public Object handlerControllerMethod(ProceedingJoinPoint pjp) throws Throwable {
        try {
            operatorManager.init();
            return pjp.proceed();
        } finally {
            operatorManager.release();
        }
    }
}
