package com.chuang.urras.common.web.configuration;

import com.chuang.tauceti.tools.basic.DESKit;
import com.chuang.tauceti.tools.basic.RegexKit;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.env.OriginTrackedMapPropertySource;
import org.springframework.boot.env.YamlPropertySourceLoader;
import org.springframework.core.env.PropertySource;
import org.springframework.core.io.Resource;

import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/***
 * 重写 Yaml 配置
 *
 * @author: Ives.l
 * @since: 2021/10/11 11:36
 */
@Slf4j
public class CustomizeYamlPropertySourceLoader extends YamlPropertySourceLoader {

	private static final String ENCRYPT_PASSWORD_KEY = "38d5d3bad5";
	private static final String REGEX = "DECRYPT\\((.*)\\)";

	@Override
	public List<PropertySource<?>> load(String name, Resource resource) throws IOException {
		List<PropertySource<?>> propertySources = super.load(name, resource);
		List<PropertySource<?>> newPropertySources = new ArrayList<>(propertySources.size());

		for (PropertySource<?> propertySource : propertySources) {
			if (propertySource instanceof OriginTrackedMapPropertySource) {
				Map<?, ?> source = (Map<?, ?>) propertySource.getSource();
				Map<Object, Object> newSource = new LinkedHashMap<>((int) (source.size() / 0.75 + 1));
				for (Object key : source.keySet()) {
					Object value = source.get(key);

					if (RegexKit.isMatch(REGEX, value.toString())) {
						try {
							String v = RegexKit.get(REGEX, value.toString(), 1);
							log.info("{} 配置文件密码解密, 密文长度为:{}, 首字母:{}, 尾字母:{}", key, v.length(), v.charAt(0), v.charAt(v.length() - 1));
							value = DESKit.decrypt(v, ENCRYPT_PASSWORD_KEY);
						} catch (Exception e) {
							log.error("解密失败", e);
						}
					}
					newSource.put(key, value);
				}
				propertySource = new OriginTrackedMapPropertySource(propertySource.getName(),
						newSource, true);
			}
			newPropertySources.add(propertySource);
		}
		return newPropertySources;
	}
}
