package com.chuang.urras.common.web.configuration;

import com.chuang.urras.common.web.properties.SpringWebProperties;
import org.springframework.boot.autoconfigure.web.WebProperties;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.DispatcherServlet;
import org.springframework.web.servlet.LocaleResolver;
import org.springframework.web.servlet.i18n.FixedLocaleResolver;

import javax.annotation.Resource;
import java.util.Locale;

@Configuration
@EnableConfigurationProperties(value = {
        SpringWebProperties.class
})
@ComponentScan("com.chuang.urras.common.web")
public class WebMvcConfiguration {

    @Resource private WebProperties webProperties;
    @Resource private SpringWebProperties springWebProperties;

    @Bean(value = DispatcherServlet.LOCALE_RESOLVER_BEAN_NAME)
    public LocaleResolver localeResolver() {
        if (this.webProperties.getLocaleResolver() == WebProperties.LocaleResolver.FIXED) {
            return new FixedLocaleResolver(this.webProperties.getLocale());
        }
        CustomizeAcceptHeaderLocaleResolver localeResolver = new CustomizeAcceptHeaderLocaleResolver();
        Locale locale = (this.webProperties.getLocale() != null) ? this.webProperties.getLocale()
                : Locale.CHINA;
        localeResolver.setDefaultLocale(locale);
        localeResolver.setSupportedLocales(springWebProperties.getSupportedLocales());
        return localeResolver;
    }

}
