package com.chuang.urras.generator.impl.angular;

import com.baomidou.mybatisplus.generator.config.po.TableField;
import com.baomidou.mybatisplus.generator.config.po.TableInfo;
import com.baomidou.mybatisplus.generator.config.rules.NamingStrategy;
import com.chuang.tauceti.generator.Generator;
import com.chuang.tauceti.generator.config.GenConfig;
import com.chuang.tauceti.tools.basic.StringKit;
import com.chuang.urras.generator.GenConst;

import java.util.HashMap;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * angular Component 生成器
 * @author ATH
 */
public abstract class AngularComponentGenerator implements Generator {

    public static String BASE_PATH = GenConst.CURREN.getAngularPath();

    @Override
    public Map<String, Object> initTableMap(GenConfig config, TableInfo tableInfo) {
        Map<String, Object> map = new HashMap<>();
        map.put("angularCompSelectorName", angularCompSelectorName(config, tableInfo));
        map.put("angularCompName", angularCompName(config, tableInfo));
        map.put("angularCompClassName", angularCompClassName(config, tableInfo));
        map.put("angularApiPath", angularApiPath(config, tableInfo));
        map.put("angularCompSfRequires", angularCompSfRequires(config, tableInfo));
        return map;
    }

    protected String angularCompName(GenConfig config, TableInfo tableInfo) {
        String name = tableNameWithoutPrefix(config, tableInfo);
        return name.replaceAll("_", "-").toLowerCase();
    }

    protected String angularCompSelectorName(GenConfig config, TableInfo tableInfo) {
        String name = tableNameWithoutPrefix(config, tableInfo);
        return name.replaceAll("_", "-").toLowerCase();
    }
    protected String angularCompClassName(GenConfig config, TableInfo tableInfo) {
        String name = tableNameWithoutPrefix(config, tableInfo);
        String camel = NamingStrategy.removePrefixAndCamel(name, config.getStrategy().getTablePrefix());
        return StringKit.firstCharToUpperCase(camel);
    }
    protected String angularApiPath(GenConfig config, TableInfo tableInfo) {
        String name = tableNameWithoutPrefix(config, tableInfo);
        return "/api/" + name.replaceAll("_", "/").toLowerCase();
    }
    protected String angularCompSfRequires(GenConfig config, TableInfo tableInfo) {
        String requires = tableInfo.getFields().stream()
                .filter(field ->
                        StringKit.isBlank(field.getFill()) && !field.isKeyFlag()
                )
                .map(TableField::getPropertyName)
                .collect(Collectors.joining("', '"));
        return  "'" + requires + "'";
    }
}
