package com.chuang.urras.sr.admin.controller;

import com.chuang.tauceti.acme.v2.store.AcmeStore;
import com.chuang.tauceti.rowquery.controller.basic.IRetrieveController;
import com.chuang.tauceti.support.Result;
import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.tools.basic.FutureKit;
import com.chuang.tauceti.tools.basic.reflect.ConvertKit;
import com.chuang.urras.sr.admin.model.co.AcmeAccountCO;
import com.chuang.urras.sr.admin.model.ro.AcmeAccountRO;
import com.chuang.urras.sr.core.acme.AcmeManager;
import com.chuang.urras.sr.core.entity.AcmeAccount;
import com.chuang.urras.sr.core.service.single.IAcmeAccountService;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Mono;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.concurrent.CompletableFuture;
/**
 * <p>
 * ACME账号 前端控制器
 * </p>
 *
 * @author chuang
 * @since 2021-05-29
 */
@RestController
@RequestMapping("/star-ring/acme-account")
public class AcmeAccountController implements IRetrieveController<AcmeAccountRO, AcmeAccount, IAcmeAccountService> {

    @Resource private IAcmeAccountService service;
    @Resource private AcmeManager acmeManager;
    @Resource private AcmeStore store;

    @PostMapping(value = "/create")
    @ApiOperation("创建一条数据")
    @RequiresPermissions("acme-account:create")
    public Mono<Result<AcmeAccountRO>> create(@RequestBody @ApiParam @Valid AcmeAccountCO co) {
        service.findByAccount(co.getAccount()).ifPresent(acmeAccount -> {
            throw new BusinessException("此账号已经存在");
        });
        CompletableFuture<Result<AcmeAccountRO>> r = acmeManager.createAccount(co.getAccount(), co.getContactEmail(), co.getRemark())
                .thenApply(acc -> Result.success(ConvertKit.toBean(acc, AcmeAccountRO::new)));
        return FutureKit.toMono(r);
    }

    @DeleteMapping(value = "/delete/{id}")
    @ApiOperation("删除以条记录")
    @RequiresPermissions("acme-account:delete")
    public Result<Void> delete(@PathVariable("id") String id) {
        AcmeAccount account = service.findById(id).orElseThrow(() -> new BusinessException("不存在这个记录"));
        store.setProvider(acmeManager.newConnection().getSession().getProvider());
        store.delete(account.getAccount());
        return Result.whether(service().removeById(id));
    }

    @Override
    public IAcmeAccountService service() {
        return service;
    }

    @Override
    public String basePermission() {
        return "acme-account:";
    }

}

