package com.chuang.urras.sr.admin.controller;

import com.chuang.tauceti.rowquery.controller.basic.ICrudController;
import com.chuang.tauceti.support.Result;
import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.tools.basic.collection.CollectionKit;
import com.chuang.urras.sr.admin.model.co.JsGroupCO;
import com.chuang.urras.sr.admin.model.ro.JsGroupRO;
import com.chuang.urras.sr.admin.model.uo.JsGroupUO;
import com.chuang.urras.sr.core.entity.JsGroup;
import com.chuang.urras.sr.core.entity.SeoDomain;
import com.chuang.urras.sr.core.service.single.IJsGroupService;
import com.chuang.urras.sr.core.service.single.ISeoDomainService;
import io.swagger.annotations.ApiOperation;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author chuang
 * @since 2021-05-26
 */
@RestController
@RequestMapping("/star-ring/js-group")
public class JsGroupController implements ICrudController<JsGroupCO, JsGroupRO, JsGroupUO, JsGroup, IJsGroupService> {

    @Resource private IJsGroupService service;
    @Resource private ISeoDomainService domainService;

    @PostMapping("/group/{id}")
    @ApiOperation("分组")
    @RequiresPermissions("js-group:group")
    public Result<Set<String>> group(@PathVariable @Valid @NotEmpty Integer id, @RequestBody List<String> domains) {
        JsGroup group = service.findById(id)
                .orElseThrow(() -> new BusinessException(-1, "无法找到该记录"));

        List<SeoDomain> list = domainService.lambdaQuery()
                .in(SeoDomain::getDomain, domains)
                .list();

        Set<String> strs = list.stream()
                .map(seoDomain -> seoDomain.setJsGroup(group.getName()))
                .map(SeoDomain::getDomain)
                .collect(Collectors.toSet());

        domainService.updateBatchById(list);

        strs = CollectionKit.subtract(domains, strs, HashSet::new);

        if(strs.isEmpty()) {
            return Result.success();
        } else {
            return Result.success(strs);
        }
    }

    @Override
    public IJsGroupService service() {
        return service;
    }

    @Override
    public String basePermission() {
        return "js-group:";
    }

}

