package com.chuang.urras.sr.admin.controller;

import com.chuang.tauceti.rowquery.controller.basic.ICrudController;
import com.chuang.tauceti.support.BiValue;
import com.chuang.tauceti.support.Result;
import com.chuang.tauceti.tools.basic.IOKit;
import com.chuang.urras.sr.admin.model.co.SeoDomainCO;
import com.chuang.urras.sr.admin.model.ro.SeoDomainRO;
import com.chuang.urras.sr.admin.model.uo.SeoDomainUO;
import com.chuang.urras.sr.core.entity.SeoDomain;
import com.chuang.urras.sr.core.entity.Server;
import com.chuang.urras.sr.core.service.single.ISeoDomainService;
import com.chuang.urras.sr.core.service.single.IServerService;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

import static java.util.stream.Collectors.*;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author chuang
 * @since 2021-05-26
 */
@Slf4j
@RestController
@RequestMapping("/star-ring/seo-domain")
public class SeoDomainController implements ICrudController<SeoDomainCO, SeoDomainRO, SeoDomainUO, SeoDomain, ISeoDomainService> {

    @Resource private ISeoDomainService service;
    @Resource private IServerService serverService;

    @PostMapping("/upload")
    @ApiOperation("上传对比文件，每行格式为  domain,服务器编号 ")
    @RequiresPermissions("seo-domain:create")
    public Result<Void> upload(@RequestParam("file") MultipartFile file) {

        List<Server> servers = serverService.list();
        Set<String> serverNames = servers.stream().map(Server::getServerName).collect(Collectors.toSet());
        try {

            Map<String, List<String>> lines = IOKit.readLines(file.getInputStream(), "gbk", new ArrayList<>())
                    .stream().map(s -> {
                        String[] strs = s.split(",");
                        if(strs.length < 2) {
                            return null;
                        }
                        return new BiValue<>(strs[0], strs[1]);
                    })
                    .filter(Objects::nonNull)
                    .filter(bi -> serverNames.contains(bi.getTwo()))
                    .collect(groupingBy(BiValue::getTwo, mapping(BiValue::getOne, toList())));

            lines.forEach(service::updateServer);

            return Result.success();
        } catch (IOException e) {
            log.error("读取文件失败, lines");
            return Result.fail("文件读取失败");
        }
    }

    @Override
    public ISeoDomainService service() {
        return service;
    }

    @Override
    public String basePermission() {
        return "seo-domain:";
    }

}

