package com.chuang.urras.sr.admin.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.chuang.tauceti.rbac2.rest.OfficeUtils;
import com.chuang.tauceti.rowquery.controller.basic.ICrudController;
import com.chuang.tauceti.support.Result;
import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.support.rowquery.RowQuery;
import com.chuang.tauceti.tools.basic.FileKit;
import com.chuang.tauceti.tools.basic.HashKit;
import com.chuang.tauceti.tools.basic.reflect.ConvertKit;
import com.chuang.urras.sr.admin.ControllerUtil;
import com.chuang.urras.sr.admin.model.GroupParam;
import com.chuang.urras.sr.admin.model.TemplateAuditParam;
import com.chuang.urras.sr.admin.model.co.SeoTemplateCO;
import com.chuang.urras.sr.admin.model.ro.SeoTemplateRO;
import com.chuang.urras.sr.admin.model.uo.SeoTemplateUO;
import com.chuang.urras.sr.core.entity.SeoTemplate;
import com.chuang.urras.sr.core.entity.TemplateGroup;
import com.chuang.urras.sr.core.enums.CheckedStatus;
import com.chuang.urras.sr.core.enums.TaskStatus;
import com.chuang.urras.sr.core.service.single.ISeoTemplateService;
import com.chuang.urras.sr.core.service.single.ITemplateGroupService;
import io.swagger.annotations.ApiOperation;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.io.File;
import java.util.Arrays;
import java.util.List;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author chuang
 * @since 2021-05-26
 */
@RestController
@RequestMapping("/star-ring/seo-template")
public class SeoTemplateController implements ICrudController<SeoTemplateCO, SeoTemplateRO, SeoTemplateUO, SeoTemplate, ISeoTemplateService> {

    @Resource private ITemplateGroupService templateGroupService;
    @Resource private ISeoTemplateService service;

    @Value("${eden.fetch.save-path}") private String savePath;


    @PostMapping("/query")
    @ApiOperation("根据RowQuery对象进行查询")
    @RequiresPermissions("seo-template:query")
    @Override
    public IPage<SeoTemplateRO> query(@RequestBody RowQuery rowQuery) {
        boolean hasAudit = OfficeUtils.hasPermission("seo-template:audit");
        if(!hasAudit) {
            ControllerUtil.querySelf(rowQuery, OfficeUtils.shiroUserNotNull().getUsername());
        }
        return service.pageByRowQuery(rowQuery).convert(temp -> ConvertKit.toBean(temp, SeoTemplateRO::new));
    }


    @PostMapping("/audit")
    @ApiOperation("审核")
    @RequiresPermissions("seo-template:audit")
    public Result<Void> audit(@RequestBody @Valid TemplateAuditParam param) {
        boolean success = service.audit(param.getId(), param.getLike(), param.getRemark(), OfficeUtils.shiroUserNotNull().getUsername());
        return Result.whether(success);
    }

    @PostMapping("/re-fetch/{id}")
    @ApiOperation("重抓")
    @RequiresPermissions("seo-template:re-fetch")
    public Result<Void> reFetch(@PathVariable("id") Integer id) {
        SeoTemplate before = checkAndGet(id);

        before.setChecked(CheckedStatus.NO_CHECK)
                .setArticleHSize(0)
                .setArticleSubKeySize(0)
                .setArticleATagSize(0)
                .setArticleState(TaskStatus.NONE)
                .setIndexHSize(0)
                .setIndexATagSize(0)
                .setIndexSubKeySize(0)
                .setIndexState(TaskStatus.NONE)
                .setListHSize(0)
                .setListATagSize(0)
                .setListSubKeySize(0)
                .setListState(TaskStatus.NONE);
        File f = new File(savePath + "/" + HashKit.md5(before.getWebsite()));
        FileKit.del(f);
        service.updateById(before);
        return Result.success();
    }

    @PostMapping("/hidden/{id}")
    @ApiOperation("隐藏")
    @RequiresPermissions("seo-template:hidden")
    public Result<Void> hidden(@PathVariable Integer id) {
        SeoTemplate before = checkAndGet(id);
        before.setChecked(CheckedStatus.HIDDEN);
        service.updateById(before);
        return Result.success();
    }

    private SeoTemplate checkAndGet(Integer id) {
        String loggedInUser = OfficeUtils.shiroUserNotNull().getUsername();
        SeoTemplate before = this.service.findById(id).orElseThrow(() ->
                new BusinessException(-1, "记录本身就不存在")
        );
        if(before.getChecked() == CheckedStatus.PASS) {
            throw new BusinessException(-1, "已经通过的不能重抓");
        }
        if(!before.getCreator().trim().equalsIgnoreCase(loggedInUser.trim())) {
            throw new BusinessException(-1, "这条记录不属于你,[" + before.getCreator() + "],[" + loggedInUser + "]");
        }
        return before;
    }

    @PostMapping("/re-fetch/{page}/{id}")
    @ApiOperation("重抓")
    @RequiresPermissions("seo-template:re-fetch")
    public Result<Void> reFetch(@PathVariable String page, @PathVariable("id") Integer id) {
        SeoTemplate before = checkAndGet(id);

        before.setChecked(CheckedStatus.NO_CHECK);

        if("article".equalsIgnoreCase(page)) {
            before.setArticleHSize(0)
                    .setArticleSubKeySize(0)
                    .setArticleATagSize(0)
                    .setArticleState(TaskStatus.NONE);
        }
        if("index".equalsIgnoreCase(page)) {
            before.setIndexHSize(0)
                    .setIndexATagSize(0)
                    .setIndexSubKeySize(0)
                    .setIndexState(TaskStatus.NONE);
        }

        if("list".equalsIgnoreCase(page)) {
            before.setListHSize(0)
                    .setListATagSize(0)
                    .setListSubKeySize(0)
                    .setListState(TaskStatus.NONE);
        }

        File f = new File(savePath + "/" + HashKit.md5(before.getWebsite()) + "/" + page + ".ftl");
        FileKit.del(f);
        service.updateById(before);
        return Result.success();
    }


    @DeleteMapping({"/delete/{id}"})
    @ApiOperation("根据id删除一条记录")
    @RequiresPermissions("seo-template:delete")
    @Override
    public Result<Void> delete(@PathVariable String id) {
        SeoTemplate before = this.service.findById(id).orElseThrow(() ->
                new BusinessException(2, "记录本身就不存在")
        );
        if(before.getChecked().isDone()) {
            throw new BusinessException(1, "已经审核过的记录不能删除");
        }
        File f = new File(savePath + "/" + HashKit.md5(before.getWebsite()));
        FileKit.del(f);
        return Result.whether(this.service.removeById(id));
    }

    @PostMapping({"/deleteByIds"})
    @ApiOperation("根据id删除一条记录")
    @RequiresPermissions("seo-template:delete")
    public Result<Void> deleteByKeys(String ids) {
        List<String> idList = Arrays.asList(ids.split(","));

        List<SeoTemplate> templates = this.service.lambdaQuery().in(SeoTemplate::getId, idList).list();
        templates.forEach(template -> {
            File f = new File(savePath + "/" + HashKit.md5(template.getWebsite()));
            FileKit.del(f);
        });

        boolean deleted = this.service.removeByIds(idList);
        return Result.whether(deleted);
    }

    @PostMapping("/group")
    @ApiOperation("分组")
    @RequiresPermissions("seo-template:group")
    public Result<Void> group(@RequestBody @Valid GroupParam param) {

        TemplateGroup group = templateGroupService.findById(param.getGroupId())
                .orElseThrow(() -> new BusinessException(-1, "这个分组不存在"));

        List<SeoTemplate> templates = service.lambdaQuery().in(SeoTemplate::getId, param.getIds()).list();

        templates.forEach(template -> template.setGroupId(group.getId()) );

        service.updateBatchById(templates);
        return Result.success();
    }

    @Override
    public ISeoTemplateService service() {
        return service;
    }

    @Override
    public String basePermission() {
        return "seo-template:";
    }

}

