package com.chuang.urras.sr.admin.controller;

import com.chuang.tauceti.rowquery.controller.basic.ICrudController;
import com.chuang.tauceti.support.Result;
import com.chuang.tauceti.tools.basic.reflect.ConvertKit;
import com.chuang.urras.sr.admin.model.co.ServerCO;
import com.chuang.urras.sr.admin.model.ro.ServerRO;
import com.chuang.urras.sr.admin.model.uo.ServerUO;
import com.chuang.urras.sr.core.ProcessUtils;
import com.chuang.urras.sr.core.entity.Server;
import com.chuang.urras.sr.core.service.single.IServerService;
import io.swagger.annotations.ApiOperation;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;

/**
 * <p>
 * 服务器 前端控制器
 * </p>
 *
 * @author chuang
 * @since 2021-06-11
 */
@RestController
@RequestMapping("/star-ring/server")
public class ServerController implements ICrudController<ServerCO, ServerRO, ServerUO, Server, IServerService> {

    @Resource private IServerService service;

    @Override
    @PostMapping(value = "/create")
    @ApiOperation("创建一条数据")
    @RequiresPermissions("server:create")
    public Result<ServerRO> create(@Valid ServerCO serverCO) {
        Server server = ConvertKit.toBean(serverCO, Server::new);

        service.save(server);

        String cmd = "mkdir -p /opt/acme/";

        ProcessUtils.runAndThrow(server, cmd);

        return Result.success(ConvertKit.toBean(server, ServerRO::new));
    }

    @PostMapping(value = "/update")
    @ApiOperation("修改一条数据")
    @RequiresPermissions("server:update")
    @Override
    public Result<Void> update(@Valid ServerUO serverUO) {
        Server server = ConvertKit.toBean(serverUO, Server::new);

        service.updateById(server);

        String cmd = "mkdir -p /opt/acme/";

        ProcessUtils.runAndThrow(server, cmd);

        return Result.success();
    }

    @Override
    public IServerService service() {
        return service;
    }

    @Override
    public String basePermission() {
        return "server:";
    }

}

