package com.chuang.urras.sr.admin.controller;


import com.chuang.tauceti.rowquery.controller.basic.ICrudController;
import com.chuang.tauceti.support.Result;
import com.chuang.tauceti.tools.basic.IOKit;
import com.chuang.tauceti.tools.basic.StringKit;
import com.chuang.tauceti.tools.basic.collection.CollectionKit;
import com.chuang.urras.sr.admin.model.co.SprocketCO;
import com.chuang.urras.sr.admin.model.ro.SprocketRO;
import com.chuang.urras.sr.admin.model.uo.SprocketUO;
import com.chuang.urras.sr.core.entity.Sprocket;
import com.chuang.urras.sr.core.service.ISprocketService;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.RandomUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author chuang
 * @since 2021-09-18
 */
@Slf4j
@RestController
@RequestMapping("/star-ring/sprocket")
public class SprocketController implements ICrudController<SprocketCO, SprocketRO, SprocketUO, Sprocket, ISprocketService> {

    @Resource private ISprocketService service;



    @PostMapping("/upload")
    @ApiOperation("上传对比文件，每行格式为  domain,服务器编号 ")
    @RequiresPermissions("sprocket:create")
    public Result<Void> upload(@RequestParam("file") MultipartFile file) {

        try {
            List<String> lines = IOKit.readLines(file.getInputStream(), "gbk", new ArrayList<>())
                    .stream().filter(StringKit::isNotBlank).collect(Collectors.toList());
            if(lines.size() != 2) {
                return Result.fail("必须是两行，目录前是" + lines.size() + "行");
            }
            Set<String> in = Arrays.stream(lines.get(0).split(",")).filter(StringKit::isNotBlank).collect(Collectors.toSet());
            Set<String> out = Arrays.stream(lines.get(1).split(",")).filter(StringKit::isNotBlank).collect(Collectors.toSet());

            Set<Sprocket> set = in.stream().flatMap(s ->
                    CollectionKit.random(out, RandomUtils.nextInt(2, 3)).stream()
                            .map(s1 -> new Sprocket().setRing(false).setDomain(s).setTargetDomain(s1))
            ).collect(Collectors.toSet());

            service.saveBatch(set);
            return Result.success();
        } catch (IOException e) {
            log.error("读取文件失败, lines");
            return Result.fail("文件读取失败");
        }
    }

    @Override
    public ISprocketService service() {
        return service;
    }

    @Override
    public String basePermission() {
        return "sprocket:";
    }

}

