package com.chuang.urras.sr.admin.controller;

import com.chuang.tauceti.httpclient.Request;
import com.chuang.tauceti.rbac2.rest.OfficeUtils;
import com.chuang.tauceti.rowquery.controller.basic.ICrudController;
import com.chuang.tauceti.support.Result;
import com.chuang.tauceti.support.SystemConfigPolicy;
import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.urras.sr.admin.model.co.TargetDomainBatchAddCO;
import com.chuang.urras.sr.admin.model.co.TargetDomainCO;
import com.chuang.urras.sr.admin.model.ro.TargetDomainRO;
import com.chuang.urras.sr.admin.model.uo.TargetDomainUO;
import com.chuang.urras.sr.core.entity.TargetDomain;
import com.chuang.urras.sr.core.service.single.ITargetDomainService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Mono;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.net.MalformedURLException;
import java.net.URL;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author chuang
 * @since 2021-05-26
 */
@Slf4j
@RestController
@RequestMapping("/star-ring/target-domain")
public class TargetDomainController implements ICrudController<TargetDomainCO, TargetDomainRO, TargetDomainUO, TargetDomain, ITargetDomainService> {

    @Resource private ITargetDomainService service;

    @Resource private ITargetDomainService targetDomainService;
    @Resource private SystemConfigPolicy systemConfigPolicy;

    @PostMapping("/use/xinhuan/{id}")
    public Result<Boolean> useXinhuan(@PathVariable @Valid @NotNull Integer id) {
        return Result.success(targetDomainService.useXinhuan(OfficeUtils.shiroUserNotNull().getUsername(), id));
    }

    @PostMapping(value = "/add/batch")
    public Result<Void> setting(@RequestBody @Valid TargetDomainBatchAddCO co) {
        targetDomainService.add(co.getUrls(), co.getXinHuanOnly());
        return Result.success();
    }

    @PostMapping(value = "/add/spider")
    public Mono<Result<Void>> settingSpider(@RequestBody @Valid @NotEmpty List<String> urls) {
        return this.setting0(urls, "SPIDER_FETCH_URL", "spider");
    }

    @PostMapping(value = "/add/order")
    public Mono<Result<Void>> settingOrder(@RequestBody @Valid @NotEmpty List<String> urls) {
        return this.setting0(urls, "ORDER_FETCH_URL", "order");
    }

    private Mono<Result<Void>> setting0(List<String> urls, String urlKey, String userBy) {
        String fetchUrl = systemConfigPolicy.get(urlKey)
                .orElseThrow(() -> new BusinessException(-1, "系统没有配置蜘蛛池服务地址"));

        Map<String, String> urlMap = new HashMap<>(urls.size());
        for(String url: urls) {
            urlMap.put(getUrlSec(url), url);
        }

        Set<String> inserts = targetDomainService.add(urlMap.keySet(), false);
        if (inserts.isEmpty()) {
            return Mono.just(Result.success());
        }

        String str = inserts.stream().map(urlMap::get).reduce((s, s2) -> s + "," + s2).orElse("");
        log.info("url:->" + inserts);
        log.info("str:->" + str);
        return Mono.create(sink -> Request.Post(fetchUrl)
                .parameter("urls", str)
                .build()
                .asyncExecuteAsString()
                .whenComplete((s, throwable) -> {
                    if(null != throwable) {
                        sink.success(Result.fail(-1, throwable.getMessage()));
                        return;
                    }
                    log.info("spider result:" + s);
                    targetDomainService.lambdaUpdate()
                            .set(TargetDomain::getUsed, true)
                            .set(TargetDomain::getUsedBy, userBy)
                            .set(TargetDomain::getUsedTime, LocalDateTime.now())
                            .in(TargetDomain::getTargetDomain, inserts)
                            .update();
                    sink.success(Result.success());
                }));
    }

    private String getUrlSec(String url) {
        url = url.toLowerCase();
        if(!url.startsWith("http://") && !url.startsWith("https://")) {
            url = "http://" + url;
        }

        String host;
        try {
            host = new URL(url).getHost();
        } catch (MalformedURLException e) {
            throw new BusinessException(-1, url + "是错误地址");
        }
        return host.replace("www.", "");
    }

    @Override
    public ITargetDomainService service() {
        return service;
    }

    @Override
    public String basePermission() {
        return "target-domain:";
    }

}

