package com.chuang.urras.sr.admin.controller;

import com.chuang.tauceti.rowquery.controller.basic.ICrudController;
import com.chuang.tauceti.support.Result;
import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.tools.basic.IOKit;
import com.chuang.tauceti.tools.basic.collection.CollectionKit;
import com.chuang.urras.sr.admin.model.co.TitleCO;
import com.chuang.urras.sr.admin.model.ro.TitleRO;
import com.chuang.urras.sr.admin.model.uo.TitleUO;
import com.chuang.urras.sr.core.entity.Title;
import com.chuang.urras.sr.core.service.single.ITitleService;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author chuang
 * @since 2021-06-10
 */
@Slf4j
@RestController
@RequestMapping("/star-ring/title")
public class TitleController implements ICrudController<TitleCO, TitleRO, TitleUO, Title, ITitleService> {

    @Resource private ITitleService service;

    @PostMapping("/upload")
    @ApiOperation("上传文本文件，一行一条，gbk编码")
    @RequiresPermissions("sentence:create")
    public Result<Void> upload(@RequestParam("file") MultipartFile file) {

        Set<Title> lines = new HashSet<>();
        try {
            lines = IOKit.readLines(file.getInputStream(), "gbk", new ArrayList<>())
                    .stream().distinct().map(s -> new Title().setTitle(s)).collect(Collectors.toSet());
        } catch (Exception e) {
            log.error("读取异常", e);
        }

        if(lines.isEmpty()) {
            throw new BusinessException(-1, "一条句子都没成功");
        }
        CollectionKit.split(lines, 1000, HashSet::new)
                .forEach(titles -> service.saveBatchSafe(titles));


        return Result.success();
    }

    @Override
    public ITitleService service() {
        return service;
    }

    @Override
    public String basePermission() {
        return "title:";
    }


}

