package com.chuang.urras.sr.admin.model.co;

import com.chuang.urras.sr.core.enums.DomainType;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.time.LocalDateTime;
import java.time.LocalTime;

/**
 * <p>
 * 网站记录
 * </p>
 *
 * @author chuang
 * @since 2021-05-26
 */
@Data
@Accessors(chain = true)
@ApiModel(value="SeoDomainCO对象", description="网站记录")
public class SeoDomainCO implements Serializable {

    @ApiModelProperty(value = "模板网址")
    @NotBlank(message = "模板网址不能为空")
    private String templateWebsite;

    @ApiModelProperty(value = "网站域名")
    @NotBlank(message = "网站域名不能为空")
    private String domain;

    @ApiModelProperty(value = "标题")
    @NotBlank(message = "标题不能为空")
    private String title;

    @ApiModelProperty(value = "关键词列表")
    @NotBlank(message = "关键词列表不能为空")
    private String keywords;

    @ApiModelProperty(value = "菜单列表")
    @NotBlank(message = "菜单列表不能为空")
    private String menus;

    @ApiModelProperty(value = "主关键词")
    @NotBlank(message = "主关键词不能为空")
    private String mainKey;

    @ApiModelProperty(value = "类型")
    @NotNull(message = "类型不能为空")
    private DomainType type;

    @ApiModelProperty(value = "网站说明")
    @NotBlank(message = "网站说明不能为空")
    private String description;

    @ApiModelProperty(value = "服务器标记")
    @NotBlank(message = "服务器标记不能为空")
    private String serverTag;

    @ApiModelProperty(value = "网页后缀")
    @NotBlank(message = "网页后缀不能为空")
    private String suffix;

    @ApiModelProperty(value = "文章目录")
    @NotBlank(message = "文章目录不能为空")
    private String dir;

    @ApiModelProperty(value = "句子目录")
    @NotBlank(message = "句子目录不能为空")
    private String sentenceDir;

    @ApiModelProperty(value = "每天最多更新多少篇")
    @NotNull(message = "每天最多更新多少篇不能为空")
    private Integer dailyArticle;

    @ApiModelProperty(value = "文章更新时间")
    @NotNull(message = "文章更新时间不能为空")
    private LocalTime articleTime;

    @ApiModelProperty(value = "首页的句子数")
    @NotNull(message = "首页句子数不能为空")
    private Integer indexSentenceSize;

    @ApiModelProperty(value = "list页的句子数")
    @NotNull(message = "list页句子数不能为空")
    private Integer listSentenceSize;

    @ApiModelProperty(value = "文章页面的句子数")
    @NotNull(message = "文章页句子数不能为空")
    private Integer articleSentenceSize;

    @ApiModelProperty(value = "最后一次文章更新时间")
    @NotNull(message = "最后一次文章更新时间不能为空")
    private LocalDateTime lastArticleTime;

    @ApiModelProperty(value = "是否为新站")
    @NotNull(message = "是否为新站不能为空")
    private Boolean newSite;

    @ApiModelProperty(value = "js分组")
    @NotBlank(message = "js分组不能为空")
    private String jsGroup;

    @ApiModelProperty(value = "提案编号")
    @NotBlank(message = "提案编号不能为空")
    private String proposalReference;

}
