package com.chuang.urras.sr.admin.task.acme;

import com.chuang.tauceti.support.SystemConfigPolicy;
import com.chuang.tauceti.support.vg.OperatorManager;
import com.chuang.tauceti.tools.basic.FutureKit;
import com.chuang.tauceti.tools.basic.collection.CollectionKit;
import com.chuang.urras.sr.core.acme.AcmeManager;
import com.chuang.urras.sr.core.cache.CacheManager;
import com.chuang.urras.sr.core.entity.AcmeOrder;
import com.chuang.urras.sr.core.service.single.IAcmeOrderService;
import lombok.extern.slf4j.Slf4j;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.scheduling.quartz.QuartzJobBean;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.concurrent.CompletableFuture;

@Slf4j
@Component
public class ApplyTask extends QuartzJobBean {

    private static final String OPERATOR = "Acme-Apply-Task";
    @Resource private OperatorManager operatorManager;

    @Resource private CacheManager cacheManager;
    @Resource private IAcmeOrderService acmeOrderService;
    @Resource private SystemConfigPolicy systemConfigPolicy;
    @Resource private AcmeManager acmeManager;
    private volatile boolean applying = false;

    @Override
    protected void executeInternal(JobExecutionContext context) throws JobExecutionException {
        operatorManager.temp(OPERATOR, this::apply);
    }

//    @Scheduled(cron = "0 */1 * * * ?")
    public void apply() {
        int thread = systemConfigPolicy.getInteger("APPLY_THREAD").orElse(4);
        if(thread < 1) {
            log.info("applying 线程小于1，不做任何处理");
            this.applying = false;
            return;
        }
        log.info("准备更新未完成的所有证书");
        if(applying) {
            log.info("有个任务正在申请证书，直接退出");
            return;
        }

        this.applying = true;

        Set<String> domains = cacheManager.getAcmeApplyDomains().removeRandom(1000);

        if(domains.isEmpty()) {
            this.applying = false;
            return;
        }

        List<CompletableFuture<List<AcmeOrder>>> futures = new ArrayList<>();

        List<AcmeOrder> orders = acmeOrderService.lambdaQuery().in(AcmeOrder::getDomain, domains).list();
        // 分线程处理
        CollectionKit.split(orders, Math.min(orders.size(), orders.size() / thread + 1), HashSet::new).forEach(strings -> {
            CompletableFuture<List<AcmeOrder>> f;
            try {
                f = acmeManager.batchApply(strings, false);
            } catch (Exception e) {
                f = FutureKit.error(e);
            }
            futures.add(f);
        });

        // 当一切完成，把renew改为 false
        FutureKit.allOf(futures).whenComplete((lists, error) -> this.applying = false);
    }


}
