package com.chuang.urras.sr.admin.task.acme;

import com.chuang.tauceti.support.SystemConfigPolicy;
import com.chuang.tauceti.support.vg.OperatorManager;
import com.chuang.urras.common.events.OfficeMsgEvent;
import com.chuang.urras.common.events.RedisEventManager;
import com.chuang.urras.sr.core.cache.CacheManager;
import com.chuang.urras.sr.core.entity.AcmeOrder;
import com.chuang.urras.sr.core.service.single.IAcmeOrderService;
import lombok.extern.slf4j.Slf4j;
import org.quartz.JobExecutionContext;
import org.springframework.scheduling.quartz.QuartzJobBean;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Set;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.stream.Collectors;

@Slf4j
@Component
public class CheckRenewTask extends QuartzJobBean {

    public static final AtomicBoolean RENEWING = new AtomicBoolean(false);

    private static final String OPERATOR = "CheckRenewTask";

    @Resource private OperatorManager operatorManager;
    @Resource private SystemConfigPolicy systemConfigPolicy;
    @Resource private RedisEventManager redisEventManager;
    @Resource private IAcmeOrderService acmeOrderService;
    @Resource private CacheManager cacheManager;

    @Override
    protected void executeInternal(JobExecutionContext context) {
        operatorManager.temp(OPERATOR, this::checkRenew);
    }

    /**
     * 每天中午11点
     */
//    @Scheduled(cron = "0 0 11 * * ?")
    public void checkRenew() {
        log.info("开始更新快过期的证书");
        int thread = systemConfigPolicy.getInteger("RENEW_THREAD").orElse(4);
        if(thread < 1) {
            log.info("renew 线程小于1，不做任何处理");
            RENEWING.set(false);
            return;
        }

        LocalDateTime time = LocalDate.now().plusDays(-25).atStartOfDay();
        Set<String> orders = acmeOrderService.lambdaQuery()
                .select(AcmeOrder::getDomain)
                .le(AcmeOrder::getLastRenewTime, time).list()
                .stream()
                .map(AcmeOrder::getDomain)
                .collect(Collectors.toSet());


        String timeStr = time.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        log.info("发现{}条记录小于{}，需要renew", orders.size(), timeStr);

        redisEventManager.fireAdminMsgEvent(new OfficeMsgEvent("renew-task", OfficeMsgEvent.MsgType.INFO, "系统开始寻找需要renew的证书，发现" + orders.size() + "条记录小于[" + timeStr + "]的记录"));
        cacheManager.getAcmeRenewDomains().addAll(orders);
    }

}
