package com.chuang.urras.sr.admin.task.acme;

import com.chuang.tauceti.support.SystemConfigPolicy;
import com.chuang.tauceti.support.vg.OperatorManager;
import com.chuang.tauceti.tools.basic.FutureKit;
import com.chuang.tauceti.tools.basic.collection.CollectionKit;
import com.chuang.urras.sr.core.acme.AcmeManager;
import com.chuang.urras.sr.core.cache.CacheManager;
import com.chuang.urras.sr.core.entity.AcmeOrder;
import com.chuang.urras.sr.core.service.single.IAcmeOrderService;
import lombok.extern.slf4j.Slf4j;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.scheduling.quartz.QuartzJobBean;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.concurrent.CompletableFuture;

import static com.chuang.urras.sr.admin.task.acme.CheckRenewTask.RENEWING;

@Slf4j
@Component
public class RenewTasks extends QuartzJobBean {

    private static final String OPERATOR = "RenewTask";
    @Resource private OperatorManager operatorManager;

    @Resource private CacheManager cacheManager;
    @Resource private IAcmeOrderService acmeOrderService;
    @Resource private AcmeManager acmeManager;
    @Resource private SystemConfigPolicy systemConfigPolicy;

    @Override
    protected void executeInternal(JobExecutionContext context) throws JobExecutionException {
        operatorManager.temp(OPERATOR, this::renew);
    }

//    @Scheduled(cron = "0 */1 * * * ?")
//    @Scheduled(cron = "*/10 * * * * ?")
    public void renew() {
        int thread = systemConfigPolicy.getInteger("RENEW_THREAD").orElse(4);
        if(thread < 1) {
            log.info("renew 线程小于1，不做任何处理");
            RENEWING.set(false);
            return;
        }
        log.info("准备更新未完成的所有证书");
        if(RENEWING.get()) {
            log.info("有个任务正在更新证书，直接退出");
            return;
        }

        RENEWING.set(true);

        try {
            Set<String> domains = cacheManager.getAcmeRenewDomains().removeRandom(1000);

            if (domains.isEmpty()) {
                RENEWING.set(false);
                return;
            }

            List<CompletableFuture<List<AcmeOrder>>> futures = new ArrayList<>();

            List<AcmeOrder> orders = acmeOrderService.lambdaQuery().in(AcmeOrder::getDomain, domains).list();
            // 分线程处理
            CollectionKit.split(orders, Math.min(orders.size(), orders.size() / thread + 1), HashSet::new).forEach(strings -> {
                CompletableFuture<List<AcmeOrder>> f;
                try {
                    f = acmeManager.batchApply(strings, true);
                } catch (Exception e) {
                    f = FutureKit.error(e);
                }
                futures.add(f);
            });

            // 当一切完成，把renew改为 false
            FutureKit.allOf(futures).whenComplete((lists, throwable) -> RENEWING.set(false));
        } catch (Exception e) {
            log.error("", e);

            RENEWING.set(false);
        }
    }


}
