package com.chuang.urras.sr.admin.task.spider;

import com.chuang.tauceti.support.vg.OperatorManager;
import com.chuang.urras.common.events.OfficeMsgEvent;
import com.chuang.urras.common.events.RedisEventManager;
import com.chuang.urras.sr.core.cache.CacheManager;
import com.chuang.urras.sr.core.entity.SpiderLog;
import com.chuang.urras.sr.core.enums.PlatformType;
import com.chuang.urras.sr.core.service.single.ISpiderLogService;
import lombok.extern.slf4j.Slf4j;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.scheduling.quartz.QuartzJobBean;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.time.LocalDate;

@Component
@Slf4j
public class ClearSpiderTask extends QuartzJobBean {

    private static final String OPERATOR = "ClearSpiderTask";
    @Resource private OperatorManager operatorManager;

    @Resource private ISpiderLogService spiderLogService;
    @Resource private RedisEventManager redisEventManager;
    @Resource private CacheManager cacheManager;

    @Override
    protected void executeInternal(JobExecutionContext context) throws JobExecutionException {
        operatorManager.temp(OPERATOR, this::clearSpider);
    }

//    @Scheduled(cron = "0 0 0 * * ?")
    public void clearSpider() {
        log.info("记录蜘蛛");
        log(LocalDate.now().plusDays(-1));
        spiderLogService.saveOrIgnore(
                new SpiderLog().setLogDate(LocalDate.now())
                        .setBaidu(0)
                        .setHaosou(0)
                        .setOther(0)
                        .setSogou(0)
                        .setYisou(0)
        );
        redisEventManager.fireAdminMsgEvent(new OfficeMsgEvent("spider-task", OfficeMsgEvent.MsgType.INFO, "记录蜘蛛，并创建新一天的蜘蛛记录"));
    }


    private void log(LocalDate date) {
        if (null == date) {
            return;
        }

        int baidu = (int) cacheManager.getSpiderCache(PlatformType.BAIDU.name()).getAndSet(0);
        int _360 = (int) cacheManager.getSpiderCache(PlatformType.M360.name()).getAndSet(0);
        int sogou = (int) cacheManager.getSpiderCache(PlatformType.SOGOU.name()).getAndSet(0);
        int toutiao = (int) cacheManager.getSpiderCache(PlatformType.TOU_TIAO.name()).getAndSet(0);
        int google = (int) cacheManager.getSpiderCache(PlatformType.GOOGLE.name()).getAndSet(0);
        int shenma = (int) cacheManager.getSpiderCache(PlatformType.SHENMA.name()).getAndSet(0);
        int others = (int) cacheManager.getSpiderCache(PlatformType.OTHERS.name()).getAndSet(0);

        spiderLogService.addSpider(baidu, _360, sogou, shenma, others + google + toutiao, date);
    }
}
