package com.chuang.urras.sr.admin.task.spider;

import com.chuang.tauceti.support.vg.OperatorManager;
import com.chuang.urras.common.events.OfficeMsgEvent;
import com.chuang.urras.common.events.RedisEventManager;
import com.chuang.urras.sr.core.cache.CacheManager;
import com.chuang.urras.sr.core.enums.PlatformType;
import com.chuang.urras.sr.core.service.single.ISpiderLogService;
import lombok.extern.slf4j.Slf4j;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.scheduling.quartz.QuartzJobBean;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.time.LocalDate;

@Component
@Slf4j
public class LogSpiderTask extends QuartzJobBean {

    private static final String OPERATOR = "LogSpiderTask";
    @Resource private OperatorManager operatorManager;

    @Resource private ISpiderLogService spiderLogService;
    @Resource private RedisEventManager redisEventManager;
    @Resource private CacheManager cacheManager;

    @Override
    protected void executeInternal(JobExecutionContext context) throws JobExecutionException {
        operatorManager.temp(OPERATOR, this::log);
    }


//    @Scheduled(cron = "15 */5 * * * ?")
    public void log() {
        log.info("记录蜘蛛");
        doLog(LocalDate.now());
        redisEventManager.fireAdminMsgEvent(new OfficeMsgEvent("spider-task", OfficeMsgEvent.MsgType.INFO, "记录蜘蛛"));
    }

    private void doLog(LocalDate date) {
        if (null == date) {
            return;
        }

        int baidu = (int) cacheManager.getSpiderCache(PlatformType.BAIDU.name()).getAndSet(0);
        int _360 = (int) cacheManager.getSpiderCache(PlatformType.M360.name()).getAndSet(0);
        int sogou = (int) cacheManager.getSpiderCache(PlatformType.SOGOU.name()).getAndSet(0);
        int toutiao = (int) cacheManager.getSpiderCache(PlatformType.TOU_TIAO.name()).getAndSet(0);
        int google = (int) cacheManager.getSpiderCache(PlatformType.GOOGLE.name()).getAndSet(0);
        int shenma = (int) cacheManager.getSpiderCache(PlatformType.SHENMA.name()).getAndSet(0);
        int others = (int) cacheManager.getSpiderCache(PlatformType.OTHERS.name()).getAndSet(0);

        spiderLogService.addSpider(baidu, _360, sogou, shenma, others + google + toutiao, date);
    }
}
