package com.chuang.urras.sr.admin.task.template;

import com.chuang.tauceti.support.vg.OperatorManager;
import com.chuang.urras.sr.core.entity.DownloadTask;
import com.chuang.urras.sr.core.enums.TaskStatus;
import com.chuang.urras.sr.core.service.single.IDownloadTaskService;
import lombok.extern.slf4j.Slf4j;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.scheduling.quartz.QuartzJobBean;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.List;

@Component
@Slf4j
public class DownloadTasks extends QuartzJobBean {
    private volatile boolean downloading = false;

    private static final String OPERATOR = "DownloadTask";
    @Resource private OperatorManager operatorManager;

    @Resource private IDownloadTaskService downloadTaskService;

    @Override
    protected void executeInternal(JobExecutionContext context) throws JobExecutionException {
        operatorManager.temp(OPERATOR, this::repair);
    }

//    @Scheduled(cron = "0 */1 * * * ?")
    public void repair() {
        if(downloading) {
            return;
        } else {
            downloading = true;
        }
        log.info("开始修复所有没有执行的下载任务");

        LocalDateTime end = LocalDateTime.now().plusMinutes(-1);
        LocalDateTime begin = LocalDateTime.now().plusMonths(-1);
        List<DownloadTask> tasks = downloadTaskService.lambdaQuery()
                .eq(DownloadTask::getState, TaskStatus.CREATED)
                .lt(DownloadTask::getFailTimes, 5)
                .between(DownloadTask::getCreatedTime, begin, end)
                .last(" limit 50")
                .list();

        tasks.forEach(task -> {
            try {
                downloadTaskService.download(task).join();
            }catch (Exception e){
                log.error("下载失败："+ task.getId(), e);
            }

        });
        log.info("修复下载任务完成");
        downloading = false;
    }

}
