package com.chuang.urras.sr.core;

import com.chuang.urras.sr.core.entity.Server;
import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.tools.basic.IOKit;
import lombok.extern.slf4j.Slf4j;

import java.text.MessageFormat;
import java.util.function.Consumer;

@Slf4j
public class ProcessUtils {

    public static int run(String cmd) {
        return run(cmd, throwable -> log.error("{} 命令执行失败", cmd, throwable));
    }

    public static int run(Server server, String cmd) {
        String temp = "sshpass -p {0} ssh {1}@{2} \"{3}\"";

        return run(
                MessageFormat.format(temp, server.getSshPort(), server.getServerUser(), server.getIntranetIp(), cmd)
        );
    }

    public static int runAndThrow(Server server, String cmd) {
        String temp = "sshpass -p {0} ssh -p {1} {2}@{3} \"{4}\"";

        return run(
                MessageFormat.format(temp, server.getServerPass(), server.getSshPort() + "", server.getServerUser(), server.getIntranetIp(), cmd),
                throwable -> { throw throwable; }
        );
    }

    public static int run(String cmd, Consumer<BusinessException> whenError) {
        log.info("cmd:->" + cmd);
        try {
            Process process = Runtime.getRuntime().exec(cmd);
            int status = process.waitFor();
            if (0 == status) {
                log.info("命令: {} 执行成功， 结果->{}", cmd, IOKit.read(process.getInputStream(), "utf-8"));
            } else {
                String err = IOKit.read(process.getErrorStream() , "utf-8");
                log.warn(err);
                whenError.accept(new BusinessException("命令执行结果结果->" + err));
            }
            IOKit.close(process.getInputStream());
            IOKit.close(process.getOutputStream());
            IOKit.close(process.getErrorStream());
            process.destroy();
            return status;
        } catch (Throwable e) {
            log.error("口令执行失败", e);
            whenError.accept(new BusinessException("口令执行失败"));
            return -1;
        }
    }
}
