package com.chuang.urras.sr.core.cache;

import org.redisson.api.RAtomicLong;
import org.redisson.api.RMapCache;
import org.redisson.api.RSet;
import org.redisson.api.RedissonClient;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;

@Component
public class CacheManager {

    @Resource private RedissonClient redissonClient;

    private RSet<String> acmeRenewDomains;
    private RSet<String> acmeApplyDomains;
    private RSet<String> publishDomains;
    private RMapCache<String, String> thumbprintCache;

    private final Map<String, RAtomicLong> spiderCache = new HashMap<>();

    @PostConstruct
    public void init() {
        thumbprintCache = redissonClient.getMapCache("thumbprint-cache");
        acmeApplyDomains = redissonClient.getSet("acme-apply-domains");
        acmeRenewDomains = redissonClient.getSet("acme-renew-domains");
        publishDomains = redissonClient.getSet("publish-ssl-domains");
    }

    public RMapCache<String, String> getThumbprintCache() {
        return thumbprintCache;
    }

    public RAtomicLong getSpiderCache(String platform) {
        return spiderCache.computeIfAbsent(platform, p -> redissonClient.getAtomicLong("spider-" + p));
    }

    public RSet<String> getAcmeApplyDomains() {
        return acmeApplyDomains;
    }
    public RSet<String> getAcmeRenewDomains() {
        return acmeRenewDomains;
    }

    public RSet<String> getPublishDomains() {
        return publishDomains;
    }

    public long addSpider(String platform) {
        return getSpiderCache(platform).incrementAndGet();
    }

    public long addSpider(String platform, long times) {
        return getSpiderCache(platform).addAndGet(times);
    }
}
