package com.chuang.urras.sr.core.entity;

import com.baomidou.mybatisplus.annotation.*;
import com.chuang.urras.sr.core.enums.CheckedStatus;
import com.chuang.urras.sr.core.enums.TaskStatus;
import com.chuang.tauceti.support.exception.BusinessException;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

import java.io.Serializable;
import java.time.LocalDateTime;

/**
 * <p>
 * 网站模板
 * </p>
 *
 * @author chuang
 * @since 2021-05-26
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
@TableName("t_seo_template")
public class SeoTemplate implements Serializable {

    private static final long serialVersionUID=1L;

    /**
     * id
     */
    @TableId(value = "id", type = IdType.AUTO)
    private Integer id;

    /**
     * 模板域名
     */
    @TableField("website")
    private String website;

    /**
     * index(ALT)
     */
    @TableField("index_sub_key_size")
    private Integer indexSubKeySize;

    /**
     * list(ALT)
     */
    @TableField("list_sub_key_size")
    private Integer listSubKeySize;

    /**
     * article(ALT)
     */
    @TableField("article_sub_key_size")
    private Integer articleSubKeySize;

    @TableField("tp_type")
    private String tpType;

    /**
     * index(A)
     */
    @TableField("index_a_tag_size")
    private Integer indexATagSize;

    /**
     * list(A)
     */
    @TableField("list_a_tag_size")
    private Integer listATagSize;

    /**
     * 文章页(A)
     */
    @TableField("article_a_tag_size")
    private Integer articleATagSize;

    /**
     * 内链数
     */
    @TableField("interlink_size")
    private Integer interlinkSize;


    @TableField("index_sentence_size")
    private Integer indexSentenceSize;

    @TableField("list_sentence_size")
    private Integer listSentenceSize;

    @TableField("article_sentence_size")
    private Integer articleSentenceSize;

    /**
     * 审核状态
     */
    @TableField("checked")
    private CheckedStatus checked;

    /**
     * 创建时间
     */
    @TableField(value = "created_time", fill = FieldFill.INSERT)
    private LocalDateTime createdTime;

    /**
     * 创建人
     */
    @TableField(value = "creator", fill = FieldFill.INSERT)
    private String creator;

    /**
     * 最后更新时间
     */
    @TableField(value = "updated_time", fill = FieldFill.INSERT_UPDATE)
    private LocalDateTime updatedTime;

    /**
     * 最后更新人
     */
    @TableField(value = "updater", fill = FieldFill.INSERT_UPDATE)
    private String updater;

    /**
     * index页面的抓取状态
     */
    @TableField("index_state")
    private TaskStatus indexState;

    /**
     * list页面的抓取状态
     */
    @TableField("list_state")
    private TaskStatus listState;

    /**
     * 文章页面的抓取状态
     */
    @TableField("article_state")
    private TaskStatus articleState;

    /**
     * index(H)
     */
    @TableField("index_h_size")
    private Integer indexHSize;

    /**
     * list(H)
     */
    @TableField("list_h_size")
    private Integer listHSize;

    /**
     * article(H)
     */
    @TableField("article_h_size")
    private Integer articleHSize;

    /**
     * 分组
     */
    @TableField("group_id")
    private Integer groupId;

    /**
     * 审核人
     */
    @TableField("auditor")
    private String auditor;

    /**
     * 审核备注
     */
    @TableField("remark")
    private String remark;

    /**
     * 使用次数
     */
    @TableField("use_times")
    private Integer useTimes;

    /**
     * 审核时间
     */
    @TableField("audited_time")
    private LocalDateTime auditedTime;

    public void changeState(String pageTag, TaskStatus state) {
        if("index".equalsIgnoreCase(pageTag)) {
            this.indexState = state;
        } else if("list".equalsIgnoreCase(pageTag)) {
            this.listState = state;
        } else if("article".equalsIgnoreCase(pageTag)) {
            this.articleState = state;
        } else {
            throw new BusinessException(-1, "错误的page tag:" + pageTag);
        }
    }

    public TaskStatus state(String pageTag) {
        if("index".equalsIgnoreCase(pageTag)) {
            return this.indexState;
        } else if("list".equalsIgnoreCase(pageTag)) {
            return this.listState;
        } else if("article".equalsIgnoreCase(pageTag)) {
            return this.articleState;
        } else {
            throw new BusinessException(-1, "错误的page tag:" + pageTag);
        }
    }
}
