package com.chuang.urras.sr.core.service;

import com.baomidou.mybatisplus.core.toolkit.support.SFunction;
import com.chuang.urras.sr.core.Utils;
import com.chuang.tauceti.rowquery.IRowQueryService;
import com.chuang.tauceti.support.BiValue;
import org.apache.commons.lang3.RandomUtils;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

public interface IRandomService<R, T> extends IRowQueryService<T> {

    Map<Class<?>, BiValue<Long, Long>> countCache = new ConcurrentHashMap<>();

    default List<R> random(int count) {
        if(count == 0) {
            return new ArrayList<>();
        }

        Set<Integer> ids = new HashSet<>();
        long dataCount = this.cacheCount();

        //小于count，大于
        while(ids.size() < count && ids.size() < dataCount) {
            ids.add(RandomUtils.nextInt(1, (int) dataCount));
        }

        List<R> rs;
        if(randomMapper() == null) {
            rs = lambdaQuery()
                    .in(idMapper(), ids)
                    .list().stream()
                    .map(t -> (R) t)
                    .collect(Collectors.toList());
        } else {
            rs = lambdaQuery()
                    .select(randomMapper())
                    .in(idMapper(), ids)
                    .list().stream()
                    .map(randomMapper())
                    .collect(Collectors.toList());
        }

        Utils.full(rs, count);
        return rs;
    }

    default long cacheCount() {
        Class<?> key = getEntityClass();
        BiValue<Long, Long> timeWithCnt = countCache.computeIfAbsent(
                key,
                aClass -> new BiValue<>(System.currentTimeMillis(), this.count())
        );

        if(System.currentTimeMillis() - timeWithCnt.getOne() > 60000) {
            long count = this.count();
            timeWithCnt = new BiValue<>(System.currentTimeMillis(), count);
            countCache.put(key, timeWithCnt);
        }

        return timeWithCnt.getTwo();
    }

    default SFunction<T, R> randomMapper() {
        return null;
    }
    SFunction<T, Integer> idMapper();
}
