package com.chuang.urras.sr.core.service.single;

import com.chuang.urras.sr.core.entity.SentenceArticle;
import com.chuang.urras.sr.core.enums.PageType;
import com.chuang.tauceti.rowquery.IRowQueryService;

import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * <p>
 * 句子表 服务类
 * </p>
 *
 * @author chuang
 * @since 2021-06-10
 */
public interface ISentenceArticleService extends IRowQueryService<SentenceArticle> {
    default List<SentenceArticle> findSentences(String domain, PageType type, Integer total) {
        return lambdaQuery().eq(SentenceArticle::getDomain, domain)
                .eq(SentenceArticle::getPage, type)
                .orderByDesc(SentenceArticle::getId)
                .last(" limit " + total)
                .list();
    }

    default Set<Integer> findIdByDomain(String domain) {
        return lambdaQuery().select(SentenceArticle::getId).eq(SentenceArticle::getDomain, domain).list().stream()
                .map(SentenceArticle::getId).collect(Collectors.toSet());
    }

    default boolean deleteByIds(Set<Integer> ids) {
        return lambdaUpdate().in(SentenceArticle::getId, ids).remove();
    }

}
