package com.chuang.urras.sr.restapp;

import com.chuang.eden.template.fetch.FetchProcessor;
import com.chuang.eden.template.fetch.WebsiteMapping;
import com.chuang.eden.template.fetch.handler.ISelectorFinder;
import com.chuang.eden.template.fetch.properties.FetchProperties;
import com.chuang.tauceti.support.vg.OperatorManager;
import com.chuang.tauceti.tools.basic.HashKit;
import com.chuang.tauceti.tools.third.spring.SpringKit;
import com.chuang.urras.sr.core.configuration.CoreConfiguration;
import com.chuang.urras.sr.restapp.configuration.RequestInterceptor;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Import;
import org.springframework.http.MediaType;
import org.springframework.mobile.device.DeviceHandlerMethodArgumentResolver;
import org.springframework.mobile.device.DeviceResolverHandlerInterceptor;
import org.springframework.web.method.support.HandlerMethodArgumentResolver;
import org.springframework.web.servlet.config.annotation.*;

import javax.annotation.Resource;
import java.util.List;
import java.util.Optional;

@Import(CoreConfiguration.class)
@SpringBootApplication(scanBasePackages = "com.chuang.urras.sr.restapp")
public class Application implements WebMvcConfigurer {

    @Resource private FetchProperties fetchProperties;

    @Override
    public void addResourceHandlers(ResourceHandlerRegistry registry) {

        registry.addResourceHandler("/" + fetchProperties.getSaveMapping() + "/**")
                .addResourceLocations("file:///" + fetchProperties.getSavePath() + "/");

    }

    @Bean
    public OperatorManager operatorManager() {
        return OperatorManager.createOpt(Optional::empty);
    }


    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        registry.addInterceptor(requestInterceptor()).addPathPatterns("/**");
        registry.addInterceptor(new DeviceResolverHandlerInterceptor());
    }

    @Bean
    public RequestInterceptor requestInterceptor() {
        return new RequestInterceptor();
    }

    @Bean
    public WebsiteMapping websiteMapping() {
        return HashKit::md5;
    }


    @Bean
    public ISelectorFinder selectorFinder() {
        return new CachedSelectorFinder();
    }

    @Bean
    public FetchProcessor fetchProcessor() {
        return new FetchProcessor();
    }


    @Override
    public void addArgumentResolvers(List<HandlerMethodArgumentResolver> argumentResolvers) {
        argumentResolvers.add(new DeviceHandlerMethodArgumentResolver());
    }


    @Bean
    public SpringKit springKit() {
        return new SpringKit();
    }

//    /**
//     * -设置url后缀模式匹配规则,该设置匹配所有的后缀，使用.do或.action都可以
//     * 目前已经不支持具体看https://docs.spring.io/spring-framework/docs/current/reference/html/web.html#mvc-config-content-negotiation
//     */
//    @Override
//    public void configurePathMatch(PathMatchConfigurer configurer) {
//        configurer.setUseSuffixPatternMatch(true);	//设置是否是后缀模式匹配,即:/test.*
//    }

    @Override
    public void configureContentNegotiation(ContentNegotiationConfigurer configurer) {
        configurer.mediaType("*", MediaType.ALL);
    }

    public static void main(String[] args) {
        SpringApplication.run(Application.class, args);
    }

}
