package com.chuang.urras.sr.restapp;

import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.tools.basic.RegexKit;
import com.chuang.tauceti.tools.basic.StringKit;
import com.chuang.tauceti.tools.basic.URLKit;
import com.chuang.tauceti.tools.third.servlet.HttpKit;
import lombok.extern.slf4j.Slf4j;

@Slf4j
public class HttpUtils {

    public static String getServer() {
        return HttpKit.getRequest().map(request -> {
            try {
                String domain = URLKit.getDomainName("http://" + request.getServerName());
                if(RegexKit.isTrue(RegexKit.regExp_ip, domain)) {
                    throw new BusinessException(404, "没有绑定域名");
                }
                return domain;
            } catch (Exception e) {
                throw new BusinessException(-1, "系统无法解析域名");
            }
        }).orElse("");
    }

    public static String getScheme() {
        return HttpKit.getRequest().map(request -> {
            String scheme = request.getHeader("X-Forwarded-Proto");
            if(null == scheme) {
                return request.getScheme();
            }

            return scheme;
        }).orElse("http");

    }

    public static void response(String contentType, String txt) {
        HttpKit.getResponse().ifPresent(response -> {
            try {
//            log.info("js->" + txt);
                response.setCharacterEncoding("UTF-8");
                response.setContentType(contentType);
                response.getWriter().println(txt);
                response.getWriter().flush();
                response.getWriter().close();
            } catch (Exception e) {
                log.error("", e);
            }
        });

    }

    public static String getRefererServerName() {
        return HttpKit.getRequest().map(request -> {
            try {
                String referer = request.getHeader("referer");
                if(StringKit.isBlank(referer)) {
                    return "no_have";
                }
                String domain = URLKit.getDomainName(referer);
                if(RegexKit.isTrue(RegexKit.regExp_ip, domain)) {
                    throw new BusinessException(404, "没有绑定域名");
                }
                return domain;
            } catch (Exception e) {
                throw new BusinessException(-1, "系统无法解析域名");
            }
        }).orElse("");
    }
}
