package com.chuang.urras.sr.restapp.configuration;

import org.springframework.boot.autoconfigure.freemarker.FreeMarkerProperties;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.View;
import org.springframework.web.servlet.view.RedirectView;
import org.springframework.web.servlet.view.freemarker.FreeMarkerViewResolver;

import javax.annotation.Resource;
import java.util.Locale;
import java.util.Objects;

@Component
public class MyUrlBasedViewResolver extends FreeMarkerViewResolver {

    private static final String REDIRECT_301_URL_PREFIX = "redirect 301:";

    @Resource private FreeMarkerProperties properties;

    @Override
    protected View createView(String viewName, Locale locale) throws Exception {
        if (!canHandle(viewName, locale)) {
            return null;
        }
        if (viewName.startsWith(REDIRECT_301_URL_PREFIX)) {
            String redirectUrl = viewName.substring(REDIRECT_301_URL_PREFIX.length());
            RedirectView view = new RedirectView(redirectUrl, isRedirectContextRelative(), isRedirectHttp10Compatible(), false);
            view.setStatusCode(HttpStatus.MOVED_PERMANENTLY);
            return (View) Objects.requireNonNull(getApplicationContext()).getAutowireCapableBeanFactory().initializeBean(view, viewName);
        }
        return super.createView(viewName, locale);
    }

    @Override
    protected String getContentType() {
        return properties.getContentType().toString();
    }

    @Override
    protected String getSuffix() {
        return properties.getSuffix();
    }
}
