package com.chuang.urras.sr.restapp.configuration;

import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.tools.third.servlet.HttpKit;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.RandomUtils;
import org.springframework.context.annotation.Primary;
import org.springframework.http.HttpStatus;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.validation.ConstraintViolationException;
import java.util.Objects;
import java.util.UUID;

/**
 * 控制器建言，由于内部有@Component注解，自动会被spring当做组件管理。
 * 该类用来控制器全局配置,
 * 该类要再WebMvcConfig Import进入才有效。
 * Created by urras on 2017/3/17.
 */
@ControllerAdvice
@Primary
@Slf4j
public class SrControllerAdvice {
    private String getScheme(HttpServletRequest request) {
        String scheme = request.getHeader("X-Forwarded-Proto");
        if(null == scheme) {
            return request.getScheme();
        }

        return scheme;
    }
    @ExceptionHandler(value = Throwable.class)
    public ModelAndView exception(HttpServletRequest request, Throwable exception) {

        try {
            if (exception instanceof BusinessException && ((BusinessException) exception).getCode() == 404) {
                ModelAndView view = new ModelAndView();
                view.addObject("error", exception.getMessage());
                view.addObject("title", "404");
                view.setViewName("/404/" + RandomUtils.nextInt(0, 9));
                view.setStatus(HttpStatus.NOT_FOUND);
                return view;
            }

            ModelAndView view = new ModelAndView();
            view.setViewName("/500/" + RandomUtils.nextInt(0, 2));
            view.setStatus(HttpStatus.INTERNAL_SERVER_ERROR);

            boolean containsError = true;
            if (exception instanceof BusinessException) {
                view.addObject("error", exception.getMessage());
            } else if (exception instanceof HttpRequestMethodNotSupportedException |
                    exception instanceof MethodArgumentNotValidException) {
                view.addObject("error", exception.getMessage());
            } else if (exception instanceof ConstraintViolationException) {
                ConstraintViolationException ex = (ConstraintViolationException) exception;
                view.addObject("error", ex.getMessage());
            } else {
                containsError = false;
            }

            view.addObject("title", Objects.requireNonNull(view.getStatus()).value());

            if (!containsError) {
                String err = "Ex:" + UUID.randomUUID();
                log.error("controller异常, 错误号:" + err + ", request:\r\n" + HttpKit.requestDebugText() + "\r\n", exception);
                view.addObject("error", "系统异常,请联系开发人员!异常号:\r\n" + err);
            }
            log.info("500 page===>" + exception.getMessage());
            return view;
        } catch (Exception e) {
            ModelAndView view = new ModelAndView();
            view.setViewName("/500/1");
            view.setStatus(HttpStatus.INTERNAL_SERVER_ERROR);
            view.addObject("error", "错误跳转错误:" + e.getMessage());
            log.info("500 page===>" + e.getMessage());
            return view;
        }
    }

//    @ModelAttribute
//    public void addAttributes(Model m) {
//        m.addAttribute("staticDomain", staticDomain);
//    }
//
//    @InitBinder
//    public void initBinder(WebDataBinder binder) {
//        binder.setDisallowedFields("id");
//    }


}
