package com.chuang.urras.sr.restapp.controller;

import com.chuang.eden.template.fetch.WebsiteMapping;
import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.tools.basic.StringKit;
import com.chuang.urras.sr.core.entity.*;
import com.chuang.urras.sr.core.enums.PageType;
import com.chuang.urras.sr.core.enums.PlatformType;
import com.chuang.urras.sr.core.service.IExemptDomainService;
import com.chuang.urras.sr.core.service.ISentenceArticleContentService;
import com.chuang.urras.sr.core.service.ISprocketService;
import com.chuang.urras.sr.core.service.single.*;
import com.chuang.urras.sr.restapp.HttpUtils;
import com.chuang.urras.sr.restapp.monitor.RestMetrics;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.RandomUtils;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.servlet.ModelAndView;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author Ath
 */
@Slf4j
@Controller
@RequestMapping(ApiController.STAR_RING_API_PATH)
public class ApiController extends BaseController {

    public static final String STAR_RING_API_PATH = "/star-ring/--api--";

    @Resource private ISeoDomainService seoDomainService;
    @Resource private ISeoTemplateService seoTemplateService;
    @Resource private IArticleService articleService;
    @Resource private WebsiteMapping websiteMapping;
    @Resource private ISentenceArticleService sentenceArticleService;
    @Resource private ISentenceArticleContentService sentenceArticleContentService;
    @Resource private IPlatformCheckContentService platformCheckContentService;
    @Resource private RestMetrics restMetrics;

    @Resource private ISprocketService sprocketService;
    @Resource private IExemptDomainService exemptDomainService;


    @GetMapping(value = "/{menu:^(?!index).+$}")
    public ModelAndView exec(@PathVariable String menu) {
        if(menu.contains(".")) {
            menu = menu.split("\\.")[0];
        }

        String server = getServer();
        SeoDomain domain = seoDomainService.findByDomain(server)
                .orElseThrow(() -> new BusinessException(404, "服务器暂时没有绑定UI"));

        checkAndGetTemplate(domain);

        int id = getMenuId(domain, menu);
        if(-1 == id) { // list
            return list();
        } else { // 其他菜单
            return page(menu, id + "");
        }
    }


    @GetMapping(value = "/{menu:^(?!index).+$}/{id}")
    public ModelAndView exec(@PathVariable String menu, @PathVariable String id) {
        if(id.contains(".")) {
            id = id.split("\\.")[0];
        }
        try {
            Integer.parseInt(id);
        } catch (Exception ignore) {
            throw new BusinessException(404, "无法找到页面");
        }

        String server = getServer();
        SeoDomain domain = seoDomainService.findByDomain(server)
                .orElseThrow(() -> new BusinessException(404, "服务器暂时没有绑定UI"));

        checkAndGetTemplate(domain);

        // 判断是不是文章目录
        if(menu.equalsIgnoreCase(domain.getDir())) {
            return page(menu, id);
        }

        // 判断是不是句子目录
        if(StringKit.isNotBlank(domain.getSentenceDir()) && menu.equalsIgnoreCase(domain.getSentenceDir())) {
            return sentence(menu, id);
        }

        throw new BusinessException(404, "无法找到页面");// 只有list页面才能有id
    }


    @GetMapping(value = {"", "/", "/index"})
    public ModelAndView index() {
        restMetrics.addIndexCounter();
//        log.info("进入 Index");

        String server = getServer();
        SeoDomain domain = seoDomainService.findByDomain(server)
                .orElseThrow(() -> new BusinessException(404, "服务器暂时没有绑定UI"));

        checkAndGetTemplate(domain);

        // 如果顶级域名，正好就是访问的域名。则将其引导到www下
        if(domain.getDomain().equalsIgnoreCase(request.getServerName())) {
            ModelAndView view = new ModelAndView();
            log.info("蜘蛛 301 to >redirect 301:" + HttpUtils.getScheme()  + "://www." + domain.getDomain());
            view.setViewName("redirect 301:" + HttpUtils.getScheme()  + "://www." + domain.getDomain());
            return view;
        }


        SeoTemplate template = seoTemplateService.findByWebsite(domain.getTemplateWebsite())
                .orElseThrow(() -> new BusinessException(404, "网站模板有点问题~~~"));

        String html = "fetch/" + websiteMapping.mapping(domain.getTemplateWebsite()) + "/index";
//        log.info("html: " + html);
        ModelAndView view = new ModelAndView();
        view.setViewName(html);
        addDomain(view, domain);

        platformCheckContentService.findOne(PlatformType.M360, domain.getDomain()).ifPresent(s -> {
            if(!s.getSubmitted()) {
                view.addObject("headBegin",
                        "<meta name=\"360-site-verification\" content=\"" + s.getCheckContent() + "\"/>");
                view.addObject("headEnd",
                        "<meta name=\"360-site-verification\" content=\"" + s.getCheckContent() + "\"/>");
            }
        });

        view.addObject("title", domain.getTitle()); // 直接使用title
        view.addObject("subKey", subKeys(domain, template.getIndexSubKeySize()));
        view.addObject("randomKey", randomKeys(domain, template.getIndexHSize()));
        view.addObject("articles", articles(domain, template.getIndexATagSize()));
        view.addObject("sentences", sentences(domain, PageType.INDEX, template.getIndexSentenceSize()));
        sprocketHTML(view, domain);
        return view;
    }

    /**
     * 链轮html
     */
    private void sprocketHTML(ModelAndView view, SeoDomain domain) {

        boolean openOut = systemSettingPolicy.getBoolean("OPEN_LINK_OUT_SITE")
                .orElse(false);
        boolean openIn = systemSettingPolicy.getBoolean("OPEN_LINK_IN_SITE")
                .orElse(false);
        // 如果in 和 out 都打开了，是否都显示。如果为false，则只显示 out
        boolean both = systemSettingPolicy.getBoolean("BOTH_IN_AND_OUT_SITE")
                .orElse(false);

        // 两个都不显示，就算了。
        if(!openOut && !openIn) {
            return;
        }

        // 是否被豁免
        if (exemptDomainService.isExempt(domain.getDomain())) {
            return;
        }

        Map<Boolean, List<Sprocket>> map = sprocketService.findByDomain(domain.getDomain())
                .stream().collect(Collectors.groupingBy(Sprocket::getRing));

        List<Sprocket> inSprockets = map.getOrDefault(true, Collections.emptyList());
        List<Sprocket> outSprockets = map.getOrDefault(false, Collections.emptyList());

        if (inSprockets.isEmpty()) {
            // 如果开放了内链 链轮，则随机生成3-5个，其他服务器的域名作为跳转域名. 否则直接返回
            if(openIn) {
                inSprockets = randomInRing(domain, RandomUtils.nextInt(3, 5));
                sprocketService.saveBatch(inSprockets);
            }
        }


        StringBuilder builder = new StringBuilder("<div class=\"_friend\">");

        // 如果外链不为空，则加入外链，并判断是否同时展示内链，如果是，则内链一起展示
        if(!outSprockets.isEmpty()) {
            builder.append(outSprocketHtml(outSprockets));
            if(both) {
                builder.append(inSprocketHtml(inSprockets));
            }
        } else {
            builder.append(inSprocketHtml(inSprockets));
        }

        // 手动外链-先删掉链轮，然后加入外链。
        builder.append("</div>");
        view.addObject("bodyEnd", builder.toString());

    }

    /**
     * 随机创建 num 个内链
     */
    private List<Sprocket> randomInRing(SeoDomain domain, int num) {
        List<Sprocket> list = new ArrayList<>();

        while(list.size() < num) {
            List<SeoDomain> domains = seoDomainService.random(num * 5);
            for (SeoDomain target : domains) {
                if (target.getServerTag().equalsIgnoreCase(domain.getServerTag())) {
                    continue;
                }
                list.add(new Sprocket().setDomain(domain.getDomain()).setRing(true).setTargetDomain(target.getDomain()));
                if (list.size() >= num) {
                    return list;
                }
            }
        }
        return list;
    }

    /**
     * 内链 html
     */
    private String inSprocketHtml(List<Sprocket> list) {
        boolean openIn = systemSettingPolicy.getBoolean("OPEN_LINK_IN_SITE")
                .orElse(false);
        if (!openIn) {
            return "";
        }

        StringBuilder builder = new StringBuilder("<div class=\"in-f\">");
        list.forEach(sprocket -> builder.append("<a href=")
                .append("https://www.")
                .append(sprocket.getTargetDomain())
                .append(">")
                .append(sprocket.getTargetDomain())
                .append("</a>   "));
        return builder.append("</div>").toString();
    }

    /**
     * 外链 html
     */
    private String outSprocketHtml(List<Sprocket> list) {
        boolean openOut = systemSettingPolicy.getBoolean("OPEN_LINK_OUT_SITE")
                .orElse(false);
        if (!openOut) {
            return "";
        }

        StringBuilder builder = new StringBuilder("<div class=\"out-f\">");
        list.forEach(sprocket -> builder.append("<a href=")
                .append("https://www.")
                .append(sprocket.getTargetDomain())
                .append(">")
                .append(sprocket.getTargetDomain())
                .append("</a>   "));
        return builder.append("</div>").toString();
    }

    /**
     * list页面
     */
    private ModelAndView list() {
//        log.info("进入List");
        restMetrics.addListCounter();
        String server = getServer();
        SeoDomain domain = seoDomainService.findByDomain(server)
                .orElseThrow(() -> new BusinessException(404, "服务器暂时没有绑定UI"));

        SeoTemplate template = seoTemplateService.findByWebsite(domain.getTemplateWebsite())
                .orElseThrow(() -> new BusinessException(404, "网站模板有点问题~~~"));

        ModelAndView view = new ModelAndView();
        view.setViewName("fetch/" + websiteMapping.mapping(domain.getTemplateWebsite()) + "/list");

        addDomain(view, domain);
        view.addObject("title", getMenuTitle(domain, domain.getDir()) + " - " + domain.getTitle());
        view.addObject("subKey", subKeys(domain, template.getListSubKeySize()));
        view.addObject("articles", articles(domain, template.getListATagSize()));
        view.addObject("randomKey", randomKeys(domain, template.getListHSize()));
        view.addObject("sentences", sentences(domain, PageType.LIST, template.getListSentenceSize()));

        return view;
    }


    /**
     * page页面，包含除list之外的菜单页和所有文章页面
     */
    private ModelAndView page(String menu, String sid) {
//        log.info("进入 Page");
        restMetrics.addArticleCounter();
        Integer id = parseId(sid);

        String server = getServer();

        Article article = articleService.findById(id)
                .orElseThrow(() -> new BusinessException(404, "文章已丢失"));

        if(!server.equalsIgnoreCase(article.getOwner())) {
            throw new BusinessException(-1, "文章不属于这个网站");
        }

        SeoDomain domain = seoDomainService.findByDomain(server)
                .orElseThrow(() -> new BusinessException(404, "服务器暂时没有绑定UI"));
        SeoTemplate template = seoTemplateService.findByWebsite(domain.getTemplateWebsite())
                .orElseThrow(() -> new BusinessException(404, "网站模板有点问题~~~"));

        ModelAndView view = new ModelAndView();
        view.setViewName("fetch/" + websiteMapping.mapping(domain.getTemplateWebsite()) + "/article");

        addDomain(view, domain);


        if(menu.equalsIgnoreCase(domain.getDir())) {
            view.addObject("title", article.getTitle() + " - " + domain.getTitle());
        } else {
            view.addObject("title", getMenuTitle(domain, menu) + " - " + domain.getTitle());
        }
        view.addObject("subKey", subKeys(domain, template.getArticleSubKeySize()));
        view.addObject("randomKey", randomKeys(domain, template.getArticleHSize()));
        view.addObject("articles", articles(domain, template.getArticleATagSize()));
        view.addObject("sentences", sentences(domain, PageType.ARTICLE, template.getArticleSentenceSize()));

        view.addObject("interlinks", interlinks(domain, article.getId(), template));
        view.addObject("articleTitle", article.getTitle());
        view.addObject("articleContent", article.getArticle());
        return view;
    }


    /**
     * 句子页面
     */
    private ModelAndView sentence(String menu, String sid) {
        log.info("进入 sentence");
        restMetrics.addSentenceCounter();
        Integer id = parseId(sid);

        String server = getServer();

        SentenceArticle article = sentenceArticleService.findById(id)
                .orElseThrow(() -> new BusinessException(404, "文章已丢失"));

        SentenceArticleContent content = sentenceArticleContentService.findBySentenceId(article.getId())
                .orElseThrow(() -> new BusinessException(404, "句子已丢失"));

        if(!server.equalsIgnoreCase(article.getDomain())) {
            throw new BusinessException(-1, "文章不属于这个网站");
        }

        SeoDomain domain = seoDomainService.findByDomain(server)
                .orElseThrow(() -> new BusinessException(404, "服务器暂时没有绑定UI"));
        SeoTemplate template = seoTemplateService.findByWebsite(domain.getTemplateWebsite())
                .orElseThrow(() -> new BusinessException(404, "网站模板有点问题~~~"));

        ModelAndView view = new ModelAndView();
        view.setViewName("fetch/" + websiteMapping.mapping(domain.getTemplateWebsite()) + "/article");

        addDomain(view, domain);


        if(menu.equalsIgnoreCase(domain.getSentenceDir())) {
            view.addObject("title", article.getTitle() + " - " + domain.getTitle());
        } else {
            view.addObject("title", getMenuTitle(domain, menu) + " - " + domain.getTitle());
        }

        view.addObject("subKey", subKeys(domain, template.getArticleSubKeySize()));
        view.addObject("randomKey", randomKeys(domain, template.getArticleHSize()));
        view.addObject("articles", articles(domain, template.getArticleATagSize()));
        view.addObject("sentences", sentences(domain, PageType.ARTICLE, template.getArticleSentenceSize()));

        view.addObject("interlinks", interlinks(domain, article.getId(), template));
        view.addObject("articleTitle", article.getTitle());
        view.addObject("articleContent", content.getContent());
        return view;
    }


    private void addDomain(ModelAndView view, SeoDomain domain) {
        view.addObject("mainKey", domain.getMainKey());
        view.addObject("keywords", domain.getKeywords());
        view.addObject("description", domain.getDescription());
        view.addObject("menus", menus(domain));
    }


    /**
     * 引导页
     */
    @RequestMapping("/landing")
    public void landing() {
        String server = getServer();
        restMetrics.addLandCounter();
        try {
            if(server.equalsIgnoreCase(request.getServerName())) {
                ModelAndView view = new ModelAndView();
                log.info("落地页 301 to >redirect 301:" + HttpUtils.getScheme()  + "://www." + server);
                view.setViewName("redirect 301:" + HttpUtils.getScheme()  + "://www." + server);
                response.sendRedirect(HttpUtils.getScheme()  + "://www." + server);
                return;
            }
            response.setCharacterEncoding("UTF-8");
            response.setContentType("text/html; charset=UTF-8");
            response.getWriter().println("<html><head>");
            Optional<PlatformCheckContent> opt = platformCheckContentService.findOne(PlatformType.M360, getServer());
            if(opt.isPresent()) {
                if(!opt.get().getSubmitted()) {
                    response.getWriter().println("<meta name=\"360-site-verification\" content=\"" + opt.get().getCheckContent() + "\"/>");
                }
            }
            response.getWriter().println("<meta name=\"viewport\" content=\"width=device-width,initial-scale=1,maximum-scale=1,minimum-scale=1,user-scalable=0\">");
            if(systemSettingPolicy.getBoolean("JS_OPEN").orElse(true)) {
                response.getWriter().println("<script src=\"/public/js/" + RandomStringUtils.randomAlphabetic(5, 8) + ".js\" da=\"da\"></script>");
            }
            response.getWriter().println("</head><body></body></html>");
            response.getWriter().flush();
            response.getWriter().close();
        } catch (Exception e) {
            log.error("", e);
        }
    }


//   String a=document.writeln("<script async src=\"https://www.googletagmanager.com/gtag/js?id=G-3LJBY7DD1F\"></script>");
//            document.writeln("<script>" );
//            document.writeln("  window.dataLayer = window.dataLayer || [];" );
//            document.writeln("          function gtag(){dataLayer.push(arguments);}");
//            document.writeln("          gtag('js', new Date());" );
//            document.writeln("" );
//            document.writeln("          gtag('config', 'G-3LJBY7DD1F');" );
//            document.writeln("</script>");

    private Integer parseId(String id) {
        if(id.contains(".")) {
            id = id.split("\\.")[0];
        }
        try {
            return Integer.parseInt(id);
        } catch (Exception ignore) {
            log.info("id错误-> {} -> {}", getServer(), request.getServletPath());
            throw new BusinessException(404, "文章不存在");
        }
    }

}


