package com.chuang.urras.sr.restapp.controller;

import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.urras.sr.core.entity.Article;
import com.chuang.urras.sr.core.entity.SeoDomain;
import com.chuang.urras.sr.core.service.single.IArticleService;
import com.chuang.urras.sr.core.service.single.ISeoDomainService;
import com.chuang.urras.sr.restapp.HttpUtils;
import com.chuang.urras.sr.restapp.model.MenuItem;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.time.format.DateTimeFormatter;
import java.util.List;

@RestController
public class PlatformController extends BaseController {

    @Resource private ISeoDomainService seoDomainService;
    @Resource private IArticleService articleService;

    @RequestMapping(value = "/robots")
    public String robots() {
        response.setCharacterEncoding("UTF-8");
        response.setContentType("text/plain");
        String server = getServer();
        return "User-agent: *\n" +
                "Disallow: \n" +
                "Disallow: /bin/\n" +
                "Sitemap: " + HttpUtils.getScheme() + "://www." + server + "/sitemap.xml";
    }

    @RequestMapping(value = "/sitemap")
    public void siteMap() {

        String server = getServer();
        SeoDomain domain = seoDomainService.findByDomain(server)
                .orElseThrow(() -> new BusinessException(404, "服务器暂时没有绑定UI"));

        HttpUtils.response("application/xml;charset=UTF-8", sitemap(domain));
    }

    private String sitemap(SeoDomain domain) {
        String http = HttpUtils.getScheme() + "://www." + getServer();
        List<MenuItem> menus =  menus(domain);

        List<Article> articles = articleService.lambdaQuery()
                .eq(Article::getOwner, domain.getDomain())
                .last(" limit 500 ")
                .list();


        String lastTimeStr = domain.getLastArticleTime().format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));


        StringBuilder xml = new StringBuilder("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
        xml.append("<urlset>\n");
        for (MenuItem menu: menus) {
            xml.append("<url>\n");
            xml.append("\t<loc>").append(http).append(menu.getHref()).append("</loc>\n");
            xml.append("\t<lastmod>").append(lastTimeStr).append("</lastmod>\n");
            xml.append("\t<changefreq>daily</changefreq>\n");
            if(menu.getHref().equals("/")) {
                xml.append("\t<priority>1</priority>\n");
            } else {
                xml.append("\t<priority>0.6</priority>\n");
            }
            xml.append("</url>\n");
        }
        for (Article article: articles) {
            xml.append("<url>\n");
            xml.append("\t<loc>").append(http).append("/").append(domain.getDir()).append("/").append(article.getId()).append(domain.getSuffix()).append("</loc>\n");
            xml.append("\t<lastmod>").append(article.getUpdatedTime()).append("</lastmod>\n");
            xml.append("\t<changefreq>weekly</changefreq>\n");
            xml.append("\t<priority>0.3</priority>\n");
            xml.append("</url>\n");
        }
        xml.append("</urlset>");
        return xml.toString();

    }
}
