package com.chuang.urras.sr.restapp.controller;

import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.urras.sr.core.entity.SeoDomain;
import com.chuang.urras.sr.core.enums.PlatformType;
import com.chuang.urras.sr.core.service.single.IPlatformCheckContentService;
import com.chuang.urras.sr.core.service.single.ISeoDomainService;
import com.chuang.urras.sr.restapp.HttpUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

@Slf4j
@Controller
public class PublishController {

    @Resource private HttpServletResponse response;
    @Resource private ISeoDomainService seoDomainService;
    @Resource private IPlatformCheckContentService platformCheckContentService;


    @RequestMapping("/shenma-site-verification")
    @ResponseBody
    public String shenmaCheck() {
        return checkFile(PlatformType.SHENMA);
    }

    @RequestMapping("/sogousiteverification")
    @ResponseBody
    public String sogouCheck() {
        return checkFile(PlatformType.SOGOU);
    }

    @RequestMapping("/ByteDanceVerify")
    @ResponseBody
    public String toutiaoCheck() {
        return checkFile(PlatformType.TOU_TIAO);
    }


    private String checkFile(PlatformType type) {
        String server = HttpUtils.getServer();
        SeoDomain domain = seoDomainService.findByDomain(server)
                .orElseThrow(() -> new BusinessException(404, "服务器暂时没有绑定UI"));
        response.setCharacterEncoding("UTF-8");
        response.setContentType("text/plain");
        return platformCheckContentService.findCheckContent(type, domain.getDomain())
                .orElseThrow(() -> new BusinessException(404, "没有验证文件"));
    }

}
