package com.chuang.urras.sr.restapp.controller;

import com.chuang.eden.template.fetch.handler.ISelectorFinder;
import com.chuang.tauceti.support.Result;
import com.chuang.tauceti.support.SystemConfigPolicy;
import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.support.vg.OperatorManager;
import com.chuang.tauceti.tools.basic.StringKit;
import com.chuang.urras.sr.core.entity.SeoTemplate;
import com.chuang.urras.sr.core.entity.TemplateUploadUser;
import com.chuang.urras.sr.core.service.single.ITargetDomainService;
import com.chuang.urras.sr.core.service.single.ITemplateUploadUserService;
import com.chuang.urras.sr.restapp.CachedSelectorFinder;
import com.chuang.urras.sr.restapp.model.SiteFetchSetting;
import com.chuang.urras.sr.restapp.service.SeoTemplateFetchService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Mono;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.concurrent.CompletableFuture;

@RequestMapping("/setting")
@RestController
@Slf4j
@CrossOrigin(origins = "*")
public class SettingController {
    @Resource private SeoTemplateFetchService seoTemplateFetchService;
    @Resource private HttpServletRequest request;
    @Resource private SystemConfigPolicy systemSettingPolicy;
    @Resource private ISelectorFinder finder;
    @Resource private OperatorManager operatorManager;
    @Resource private ITemplateUploadUserService uploadUserService;
    @Resource private ITargetDomainService targetDomainService;


    @PostMapping(value = "/target/site")
    @ResponseBody
    public Result<?> setting(String[] urls, Boolean xinHuanOnly) {
        targetDomainService.add(Arrays.asList(urls), xinHuanOnly);
        return Result.success();
    }

    @GetMapping("/open/query/{type}")
    public Result<Map<String, Object>> queryOpen(@PathVariable("type") String type) {
        Map<String, Object> map = new LinkedHashMap<>();
        map.put("open", systemSettingPolicy.getBoolean(type).orElse(true));
        map.put("server", request.getServerName());
        return Result.success(map);
    }



    @PostMapping(value = "/{page}")
    @ResponseBody
    public Mono<Result<?>> setting(@PathVariable String page, @RequestBody SiteFetchSetting setting) {

        checkUser(setting);

        return Mono.create(monoSink -> {

                ((CachedSelectorFinder) finder).push(setting);

                CompletableFuture<SeoTemplate> template = operatorManager.temp(setting.getUsername(),
                        () -> seoTemplateFetchService.fetch(setting, page, request.getHeader("user-agent")));

                template.whenComplete((seoTemplate, throwable) -> {
                    if (null == throwable) {
                        monoSink.success(Result.success());
                    } else {
                        monoSink.success(Result.fail(throwable.getMessage()));
                    }
                    ((CachedSelectorFinder) finder).release(setting.getWebsite());
                });

        });
    }

    private void checkUser(SiteFetchSetting setting) {

        if(StringKit.isBlank(setting.getWebsite())) {
            throw new BusinessException(-1,"website 不能为空");
        }

        long count = uploadUserService.lambdaQuery()
                .eq(TemplateUploadUser::getUsername, setting.getUsername())
                .eq(TemplateUploadUser::getPassword, setting.getPassword())
                .count();

        if(count < 1) {
            throw new BusinessException(-1, "没有权限");
        }
    }
}

