package com.chuang.urras.sr.restapp.handlers;

import com.chuang.eden.template.fetch.PageInfo;
import com.chuang.eden.template.fetch.handler.IPreHandler;
import com.chuang.eden.template.fetch.handler.ISelectorFinder;
import com.chuang.urras.sr.restapp.Handler;
import org.apache.commons.lang3.RandomStringUtils;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;


/**
 * 设定是 a 标签作为文章，其他a标签直接转成 无法点击的JS
 */
@Component
public class FreeMarkATagHandler implements IPreHandler {

    private static final int MIN_SIZE = 3;

    private static final String ATAG_ATTR_KEY = "select-atag";


    @Resource private ISelectorFinder finder;

//    @Resource private ISeoTemplateService seoTemplateService;

    @Override
    public String name() {
        return Handler.free_mark_a_tag_od.name();
    }

    @Override
    public CompletableFuture<Document> hand(PageInfo info, Document doc, Map<String, Object> context) {

        Element body = getBody(info.getWebsite(), doc);

        List<String> listSelectors = finder.getSelectorQuery(info, name());

        listSelectors.forEach(query -> {
            Element e = body.selectFirst(query);
            if(null != e) {
                e.select("a").forEach(element -> tagArticleATag(element, ATAG_ATTR_KEY, MIN_SIZE));
            }
        });

        Elements as = body.select("a");

        int num = 0;
        for (int i = 0; i < as.size(); i++) {
            Element element = as.get(i);
            if(element.hasAttr(ATAG_ATTR_KEY)) { // 替换
                handArticleATag(element, ATAG_ATTR_KEY, "articles", num);
                ++num;
            } else if(!element.hasAttr("gohome")) { //失效
                if(!element.text().contains("$")) {
                    element.attr("href", "javascript:" + RandomStringUtils.randomAlphanumeric(10) + "();");
                    element.attr("target", "");
                }
            }

        }

//        saveATagNum(info, num);
        context.put("links", num);

        return CompletableFuture.completedFuture(doc);
    }

//    private void saveATagNum(PageInfo info, int num) {
//        seoTemplateService.findByWebsite(info.getWebsite()).ifPresent(seoTemplate -> {
//            if("index".equalsIgnoreCase(info.getPageTag())) {
//                seoTemplate.setIndexATagSize(num);
//            } else if("list".equalsIgnoreCase(info.getPageTag())) {
//                seoTemplate.setListATagSize(num);
//            } else if("detail".equalsIgnoreCase(info.getPageTag())) {
//                seoTemplate.setArticleATagSize(num);
//            }
//            seoTemplateService.updateById(seoTemplate);
//        });
//    }

}
