package com.chuang.urras.sr.restapp.handlers;

import com.chuang.eden.template.fetch.PageInfo;
import com.chuang.eden.template.fetch.handler.IPreHandler;
import com.chuang.eden.template.fetch.handler.ISelectorFinder;
import com.chuang.tauceti.tools.basic.collection.CollectionKit;
import com.chuang.urras.sr.restapp.Handler;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;

@Component
public class FreeMarkKeywordsHandler implements IPreHandler {

//    @Resource private ISeoTemplateService seoTemplateService;
    @Resource private ISelectorFinder finder;

    @Override
    public String name() {
        return Handler.free_mark_keywords.name();
    }

    @Override
    public CompletableFuture<Document> hand(PageInfo info, Document doc, Map<String, Object> context) {

        Element body = getBody(info.getWebsite(), doc);

        doc.title("${title}");//title
        Elements elements = doc.select("head meta[property=og:title]");
        if(elements.isEmpty()) {
            doc.head().append("<meta property=\"og:title\" content=\"${title}\">");
        } else {
            elements.attr("content", "${title}");
        }

        // og:sitename
        elements = doc.select("head meta[property=og:site_name]");
        if(elements.isEmpty()) {
            doc.head().append("<meta property=\"og:site_name\" content=\"${title}\">");
        } else {
            elements.attr("content", "${title}");
        }

        //og:type
        elements = doc.select("head meta[property=og:type]");
        if(elements.isEmpty()) {
            doc.head().append("<meta property=\"og:type\" content=\"website\">");
        } else {
            elements.attr("content", "website");
        }

        // keywords
        elements = doc.select("head meta[name=keywords]");
        if(elements.isEmpty()) {
            doc.head().append("<meta name=\"keywords\" content=\"${keywords}\">");
        } else {
            elements.attr("content", "${keywords}");
        }

        // des
        elements = doc.select("head meta[name=description]");
        if(elements.isEmpty()) {
            doc.head().append("<meta name=\"description\" content=\"${description}\">");
        } else {
            elements.attr("content", "${description}");
        }
        // og
        elements = doc.select("head meta[property=og:description]");
        if(elements.isEmpty()) {
            doc.head().append("<meta property=\"og:description\" content=\"${description}\">");
        } else {
            elements.attr("content", "${description}");
        }


        // h1
        elements = body.select("h1");
        if(elements.isEmpty()) {
            Element h1 = new Element("h1");
            h1.text("${mainKey}");
            body.children().first().before(h1);
        } else {
            elements.forEach(element -> element.text("${mainKey}"));
        }

        // random H
        elements = body.select("h2,h3,h4,h5,h6");
        int hsize = 0;
        for (int i = 0; i < elements.size(); i++) {
            Element element = elements.get(i);
            if (element.select("a").size() == 0) {
                element.text("${randomKey[" + hsize + "]}");
                hsize++;
            }
        }


        // 修改ALT
        Elements imgs = body.select("img");
        for (int i = 0; i < imgs.size(); i++) {
            Element img = imgs.get(i);
            img.attr("alt", "${subKey[" + i + "]}");
        }
//        saveImgTagAndHTagNum(info, imgs.size(), hsize);
        context.put("hsize", hsize);
        context.put("imgs", imgs.size());

        // logo
        List<String> logoSelectors = finder.getSelectorQuery(info, name() + ":logo");
        logoSelectors.forEach(query -> body.select(query).forEach(element -> {
            Element ele;
            if(element.tagName().equalsIgnoreCase("a")) {
                ele = element;
            } else {
                ele = element.selectFirst("a");
            }
            if(null != ele) {
                ele.attr("href", "/")
                        .attr("gohome", "");
            }

            // 如果是图片
            if(element.tagName().equalsIgnoreCase("img")) {
                ele = element;
            } else {
                ele = element.selectFirst("img");
            }
            if(null != ele) {
                ele.attr("alt", "${mainKey}");
                return;
            }

            // 如果是H1
            if(element.tagName().equalsIgnoreCase("H1")) {
                ele = element;
            } else {
                ele = element.selectFirst("H1");
            }
            if(null != ele) {
                ele.text("${mainKey}");
                return;
            }
            element.text("${mainKey}");
        }));


        // 版号
        elements = body.select("*:containsOwn(copyrights)");
        if(elements.isEmpty()) {
            elements = body.select("*:containsOwn(©)");
        }

        String html = "<a href=\"/\" gohome=\"\">${mainKey}</a>" + CollectionKit.randomOne("Copyrights", "©", "版权所有");
        if(elements.isEmpty()) {
            body.append(html);
        } else {
            elements.forEach(element -> element.html(html));
        }

        return CompletableFuture.completedFuture(doc);
    }

//    private void saveImgTagAndHTagNum(PageInfo info, int imgNum, int hNum) {
//        seoTemplateService.findByWebsite(info.getWebsite()).ifPresent(seoTemplate -> {
//            if("index".equalsIgnoreCase(info.getPageTag())) {
//                seoTemplate.setIndexSubKeySize(imgNum);
//                seoTemplate.setIndexHSize(hNum);
//            } else if("list".equalsIgnoreCase(info.getPageTag())) {
//                seoTemplate.setListSubKeySize(imgNum);
//                seoTemplate.setListHSize(hNum);
//            } else if("article".equalsIgnoreCase(info.getPageTag())) {
//                seoTemplate.setArticleSubKeySize(imgNum);
//                seoTemplate.setArticleHSize(hNum);
//            }
//            seoTemplateService.updateById(seoTemplate);
//        });
//    }


}
