package com.chuang.urras.sr.restapp.handlers;

import com.chuang.eden.template.fetch.PageInfo;
import com.chuang.eden.template.fetch.handler.IPreHandler;
import com.chuang.eden.template.fetch.handler.ISaveBeforeHandler;
import com.chuang.eden.template.fetch.handler.ISelectorFinder;
import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.urras.sr.restapp.Handler;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;

@Component
public class FreeMarkNavHandler implements IPreHandler, ISaveBeforeHandler {

    @Resource private ISelectorFinder finder;

    @Override
    public String name() {
        return Handler.free_mark_nav.name();
    }

    public CompletableFuture<Document> hand(PageInfo info, Document doc, Map<String, Object> context) {

        List<String> selectors = finder.getSelectorQuery(info, name());
        selectors.forEach(s -> {
            Element menu = doc.selectFirst(s);
            if(null == menu) {
                return;
            }
            if(menu.childrenSize() < 2) {
                throw new BusinessException(-1, "选择的菜单必须包含子元素");
            }

            Element item = menu.child(1);

            Element a = item;
            if(!"a".equalsIgnoreCase(a.tagName())) {
                a = item.selectFirst("a");
            }
            if(null == a) {
                //移除所有子节点，并重新创建a标签的子节点
                item.children().remove();
                item.after("<a href=\"${item.href}\">${item.name}</a>");
            } else {
                a.attr("href", "${item.href}").text("${item.name}");
            }
            menu.children().remove();
            menu.append("\r\n<!-- <#list menus as item> -->\r\n" + item.toString() + "\r\n<!-- </#list> -->");
        });
        return CompletableFuture.completedFuture(doc);
    }

    @Override
    public String hand(String doc) {
        return doc.replaceAll("<!-- <#list menus as item> -->", "<#list menus as item>")
                .replaceAll("<!-- </#list> -->", "</#list>");
    }
}
