package com.chuang.urras.sr.restapp.monitor;

import com.chuang.urras.sr.core.enums.PlatformType;
import io.micrometer.core.instrument.Counter;
import io.micrometer.core.instrument.Gauge;
import io.micrometer.core.instrument.MeterRegistry;
import io.micrometer.core.instrument.binder.MeterBinder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

@Component
@Slf4j
public class RestMetrics implements MeterBinder {

    private Counter allSpiderCounter;

    private Counter indexCounter;
    private Counter listCounter;
    private Counter articleCounter;
    private Counter sentenceCounter;
    private Counter pathErrorCounter;
    private Counter landCounter;
    private Counter blockSpiderCounter;

    private final Map<PlatformType, Counter> spiderCounter = new ConcurrentHashMap<>();

    private final Map<String, Double> map = new ConcurrentHashMap<>();

    @Override
    public void bindTo(MeterRegistry meterRegistry) {
        allSpiderCounter = Counter.builder("rest.spider.counter")
                .tags("name", "spider-all-counter")
                .description("总蜘蛛计数器").register(meterRegistry);

        for(PlatformType platform: PlatformType.values()) {
            spiderCounter.put(platform, Counter.builder("rest.spider.counter")
                            .tag("name", "spider-" + platform.name() + "-counter")
                            .description(platform.name() + "蜘蛛")
                            .register(meterRegistry)
            );
        }

        indexCounter = Counter.builder("rest.star-ring.page.counter")
                .tags("name", "star-ring-index")
                .description("成功进入Index页面次数")
                .register(meterRegistry);
        listCounter = Counter.builder("rest.star-ring.page.counter")
                .tags("name", "star-ring-list")
                .description("成功进入List页面次数")
                .register(meterRegistry);
        articleCounter = Counter.builder("rest.star-ring.page.counter")
                .tags("name", "star-ring-article")
                .description("成功进入文章页面次数")
                .register(meterRegistry);
        sentenceCounter = Counter.builder("rest.star-ring.page.counter")
                .tags("name", "star-ring-sentence")
                .description("成功进入句子页面次数")
                .register(meterRegistry);
        pathErrorCounter = Counter.builder("rest.star-ring.page.counter")
                .tags("name", "star-ring-pathError")
                .description("路径错误次数")
                .register(meterRegistry);
        landCounter = Counter.builder("rest.star-ring.page.counter")
                .tags("name", "star-ring-land")
                .description("成功进入落地页次数")
                .register(meterRegistry);
        blockSpiderCounter = Counter.builder("rest.star-ring.page.counter")
                .tags("name", "star-ring-blockSpider")
                .description("阻挡错误蜘蛛次数")
                .register(meterRegistry);

        Gauge.builder("rest.gauge", map, value -> value.get(value.toString())).tags("name", "gauge-01").description("测试Gauge").register(meterRegistry);
    }

    public void addSpider(PlatformType platform, int c) {
        spiderCounter.get(platform).increment(c);
        allSpiderCounter.increment(c);
    }

    public void addSpider(PlatformType platform) {
        spiderCounter.get(platform).increment();
        allSpiderCounter.increment();
    }

    public void addIndexCounter() {
        indexCounter.increment();
    }
    public void addListCounter() {
        listCounter.increment();
    }
    public void addArticleCounter() {
        articleCounter.increment();
    }

    public void addSentenceCounter() {
        sentenceCounter.increment();
    }

    public void addPathErrorCounter() {
        pathErrorCounter.increment();
    }
    public void addLandCounter() {
        landCounter.increment();
    }
    public void addBlockSpiderCounter() {
        blockSpiderCounter.increment();
    }
}
