package com.chuang.urras.sr.restapp.service.impl;

import com.chuang.eden.template.fetch.*;
import com.chuang.tauceti.support.exception.BusinessException;
import com.chuang.tauceti.support.vg.OperatorManager;
import com.chuang.tauceti.tools.basic.FutureKit;
import com.chuang.tauceti.tools.third.servlet.HttpKit;
import com.chuang.urras.common.events.OfficeMsgEvent;
import com.chuang.urras.common.events.RedisEventManager;
import com.chuang.urras.sr.core.entity.DownloadTask;
import com.chuang.urras.sr.core.entity.SeoTemplate;
import com.chuang.urras.sr.core.enums.CheckedStatus;
import com.chuang.urras.sr.core.enums.TaskStatus;
import com.chuang.urras.sr.core.service.single.IDownloadTaskService;
import com.chuang.urras.sr.core.service.single.ISeoTemplateService;
import com.chuang.urras.sr.restapp.model.SiteFetchSetting;
import com.chuang.urras.sr.restapp.service.SeoTemplateFetchService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationListener;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.CompletableFuture;

/**
 * @author Ath
 */
@Service
@Slf4j
public class SeoTemplateFetchServiceImpl implements SeoTemplateFetchService, ApplicationListener<DownloadResourceEvent> {

    @Resource private ISeoTemplateService seoTemplateService;
    @Resource private OperatorManager operatorManager;
    @Resource private FetchProcessor fetchProcessor;
    @Resource private IDownloadTaskService downloadTaskService;
    @Resource private RedisEventManager redisEventManager;

    @Override
    public CompletableFuture<SeoTemplate> fetch(SiteFetchSetting setting, String page, String userAgent) {

        String operator = operatorManager.get().orElse("non");

        Optional<SeoTemplate> opt = seoTemplateService.findByWebsite(setting.getWebsite());

        SeoTemplate template;
        if(opt.isPresent()) {
            template = opt.get();
            if(template.state(page) == TaskStatus.ENDED) {
                return FutureKit.error(new BusinessException(-1, page + "已抓取过"));
            }
        } else {
            template = new SeoTemplate()
                .setChecked(CheckedStatus.NO_CHECK)
                .setWebsite(setting.getWebsite())
                .setArticleATagSize(0)
                .setIndexATagSize(0)
                .setListATagSize(0)
                .setTpType(setting.getTpType())
                .setListState(TaskStatus.NONE)
                .setIndexState(TaskStatus.NONE)
                .setArticleState(TaskStatus.NONE)
                .setInterlinkSize(0)
                .setIndexSubKeySize(0)
                .setListSubKeySize(0)
                .setArticleHSize(0)
                .setIndexSentenceSize(0)
                .setListSentenceSize(0)
                .setArticleSentenceSize(0)
                .setIndexHSize(0)
                .setListHSize(0)
                .setGroupId(0)
                .setUseTimes(0)
                .setArticleSubKeySize(0);

            template.changeState(page, TaskStatus.CREATED);
            seoTemplateService.save(template);
        }

        PageInfo info = new PageInfo()
                .setWebsite(setting.getWebsite())
                .setPageTag(page)
                .setPageUrl(setting.getPageUrl())
                .setHtml(setting.getHtml()
                        .replaceAll("\\$\\{", "")
                        .replaceAll("#\\{", "")
                );

        CompletableFuture<SeoTemplate> result = new CompletableFuture<>();

        CompletableFuture<FetchResult> pageFuture = fetchProcessor.fetch(userAgent, info);

        long begin = System.currentTimeMillis();
        //noinspection unchecked
        pageFuture.thenAccept(fetchResult -> this.operatorManager.temp(operator, () -> {
            updateSeoTemplateStatus(fetchResult.getInfo(), TaskStatus.ENDED, fetchResult.getContext());
            long usedTime = System.currentTimeMillis() - begin;
            if(usedTime > 1500) {

                log.warn("=================================== 插件分析时间时间花费{} =======================\r\n{}\r\n{}",
                        usedTime,
                        fetchResult.getHandTimeStr(),
                        HttpKit.requestDebugText());
            }

            // EVENT
            redisEventManager.fireAdminMsgEvent(new OfficeMsgEvent("template-seo", OfficeMsgEvent.MsgType.INFO, setting.getUsername() + "创建了模板 -> " + setting.getWebsite()));
            result.complete(template);
        })).exceptionally(throwable -> {
            updateSeoTemplateStatus(info, TaskStatus.ERROR, null);
            log.error("模板下载错误", throwable);
            result.completeExceptionally(throwable);
            return null;
        });

        return result;
    }

    private void updateSeoTemplateStatus(PageInfo info, TaskStatus status, Map<String, Object> context) {
        seoTemplateService.findByWebsite(info.getWebsite()).ifPresent(seoTemplate -> {
            if(status == TaskStatus.ENDED) {
                Integer links = (Integer)context.get("links");
                Integer imgs = (Integer) context.get("imgs");
                Integer hsize = (Integer) context.get("hsize");
                Integer interlinks = (Integer) context.get("interlinks");
                Integer subLinks = (Integer)context.get("subLinks");
                if("index".equalsIgnoreCase(info.getPageTag())) {
                    if(null != links) {
                        seoTemplate.setIndexATagSize(links);
                    }
                    if(null != imgs) {
                        seoTemplate.setIndexSubKeySize(imgs);
                    }
                    if(null != hsize) {
                        seoTemplate.setIndexHSize(hsize);
                    }
                    if(null != subLinks) {
                        seoTemplate.setIndexSentenceSize(subLinks);
                    }
                } else if("list".equalsIgnoreCase(info.getPageTag())) {
                    if(null != links) {
                        seoTemplate.setListATagSize(links);
                    }
                    if(null != imgs) {
                        seoTemplate.setListSubKeySize(imgs);
                    }
                    if(null != hsize) {
                        seoTemplate.setListHSize(hsize);
                    }
                    if(null != subLinks) {
                        seoTemplate.setListSentenceSize(subLinks);
                    }
                } else if("article".equalsIgnoreCase(info.getPageTag())) {
                    if(null != links) {
                        seoTemplate.setArticleATagSize(links);
                    }
                    if(null != imgs) {
                        seoTemplate.setArticleSubKeySize(imgs);
                    }
                    if(null != hsize) {
                        seoTemplate.setArticleHSize(hsize);
                    }
                    if(null != subLinks) {
                        seoTemplate.setArticleSentenceSize(subLinks);
                    }
                }

                if(null != interlinks) {
                    seoTemplate.setInterlinkSize(interlinks);
                }
            }
            seoTemplate.changeState(info.getPageTag(), status);
            seoTemplateService.updateById(seoTemplate);
        });
    }



    @Override
    public void onApplicationEvent(DownloadResourceEvent event) {
        operatorManager.temp("event", () -> {
            switch (event.getState()) {
                case CREATED:
                    createTaskByDownloadEvent(event);
                    break;
                case ERROR:
                    downloadTaskService.findByUrlAndWebSite(event.getDownloadPath(), event.getWebsite()).ifPresent(downloadTask -> {
                        downloadTask.setError(event.getMessage());

                        downloadTask.setState(TaskStatus.valueOf(event.getState().name()));

                        if(event.getType() == FetchErrorType.HTTP_CODE || event.getType() == FetchErrorType.WRITE_FILE) {
                            downloadTask.setState(TaskStatus.ENDED);
                            downloadTask.setFailTimes(1000);
                        }

                        downloadTaskService.updateById(downloadTask);
                    });
                    break;
                case ENDED:
                    downloadTaskService.findByUrlAndWebSite(event.getDownloadPath(), event.getWebsite()).ifPresent(downloadTask -> {
                        downloadTask.setState(TaskStatus.valueOf(event.getState().name()));
                        downloadTaskService.updateById(downloadTask);
                    });

                default:
                    break;
            }
        });

    }



    private void createTaskByDownloadEvent(DownloadResourceEvent event) {
        TaskStatus status = TaskStatus.valueOf(event.getState().name());
        if(status == TaskStatus.CREATED) {
            downloadTaskService.findByUrlAndWebSite(event.getDownloadPath(), event.getWebsite())
                    .orElseGet(() -> {
                        DownloadTask task = new DownloadTask()
                                .setError("")
                                .setState(status)
                                .setSavePath(event.getSavePath())
                                .setUrl(event.getDownloadPath())
                                .setWebsite(event.getWebsite());
                        try {
                            downloadTaskService.save(task);
                        } catch (Exception e) {
                            log.error("保存任务错误：=> " + task.toString());
                            throw e;
                        }
                        return task;
                    });
        }

    }

}
